﻿namespace Microsoft.Xna.GameStudio
{
    using EnvDTE;
    using System;
    using System.Diagnostics.CodeAnalysis;
    using System.Runtime.InteropServices;

    [ComVisible(true), SuppressMessage("Microsoft.Interoperability", "CA1409:ComVisibleTypesShouldBeCreatable"), ClassInterface(ClassInterfaceType.None), Guid("396545A7-2E65-4ECE-A582-1A53C7DC64FA")]
    public abstract class AutomationExtender : IFilterProperties, IDisposable
    {
        private int cookie;
        private object extendeeObject;
        private IExtenderSite extenderSite;

        internal event EventHandler Disposed;

        public AutomationExtender(object extendeeObject, IExtenderSite extenderSite, int cookie)
        {
            if (extendeeObject == null)
            {
                throw new ArgumentNullException("extendeeObject");
            }
            if (extenderSite == null)
            {
                throw new ArgumentNullException("extenderSite");
            }
            this.extendeeObject = extendeeObject;
            this.extenderSite = extenderSite;
            this.cookie = cookie;
        }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (this.extenderSite != null)
                {
                    this.extenderSite.NotifyDelete(this.cookie);
                }
                if (this.disposedHandlers != null)
                {
                    this.disposedHandlers(this, EventArgs.Empty);
                }
            }
            this.extenderSite = null;
            this.extendeeObject = null;
        }

        ~AutomationExtender()
        {
            this.Dispose(false);
        }

        public virtual vsFilterProperties IsPropertyHidden(string PropertyName)
        {
            return vsFilterProperties.vsFilterPropertiesNone;
        }

        internal int Cookie
        {
            get
            {
                return this.cookie;
            }
        }

        protected object Object
        {
            get
            {
                return this.extendeeObject;
            }
        }
    }
}

