﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio.SmartDevice.ProjectSystem;
    using System;
    using System.Reflection;

    internal class CentralDeviceSelectorProxy
    {
        private object centralDeviceSelectorService;
        private Assembly centralSelectorAssembly;
        private const string centralSelectorAssemblyName = "Microsoft.VisualStudio.SmartDevice.ProjectSystem.Base";
        private object[] centralSelectorHandlerArgs;
        private MethodInfo centralSelectorRemoveHandler;
        private Type centralSelectorType;
        private MethodInfo dropdownDisableMethod;
        private MethodInfo dropdownEnableMethod;
        private MethodInfo getSelectedDeviceMethod;
        private const string getSelectedDeviceMethodName = "GetSelectedDeviceID";
        private bool isAvailable;
        private bool isConnectable;
        private const string selectionChangedEventName = "DeviceSelectionChanged";
        private const string selectionDisableMethodName = "Disable";
        private const string selectionEnableMethodName = "Enable";
        private const string toolbarInterfaceName = "Microsoft.VisualStudio.SmartDevice.ProjectSystem.IDeviceToolbarService";

        internal event EventHandler SelectionChanged;

        internal CentralDeviceSelectorProxy()
        {
            try
            {
                this.centralSelectorAssembly = Assembly.Load("Microsoft.VisualStudio.SmartDevice.ProjectSystem.Base");
                if (this.centralSelectorAssembly != null)
                {
                    this.centralSelectorType = this.centralSelectorAssembly.GetType("Microsoft.VisualStudio.SmartDevice.ProjectSystem.IDeviceToolbarService");
                    this.isConnectable = true;
                    this.isAvailable = false;
                }
            }
            catch
            {
                this.isConnectable = false;
                this.isAvailable = false;
            }
        }

        internal bool Connect(object centralSelectorService)
        {
            if ((this.isAvailable || !this.isConnectable) || (centralSelectorService == null))
            {
                return false;
            }
            this.isAvailable = false;
            EventInfo info = this.centralSelectorType.GetEvent("DeviceSelectionChanged");
            if (info != null)
            {
                this.centralDeviceSelectorService = centralSelectorService;
                Type eventHandlerType = info.EventHandlerType;
                MethodInfo method = base.GetType().GetMethod("OnCentralComboSelectionChanged", BindingFlags.NonPublic | BindingFlags.Instance);
                Delegate delegate2 = Delegate.CreateDelegate(eventHandlerType, this, method);
                this.dropdownEnableMethod = this.CentralDeviceSelectorServiceType.GetMethod("Enable");
                this.dropdownDisableMethod = this.CentralDeviceSelectorServiceType.GetMethod("Disable");
                this.getSelectedDeviceMethod = this.CentralDeviceSelectorServiceType.GetMethod("GetSelectedDeviceID");
                MethodInfo addMethod = info.GetAddMethod();
                this.centralSelectorHandlerArgs = new object[] { delegate2 };
                addMethod.Invoke(this.centralDeviceSelectorService, this.centralSelectorHandlerArgs);
                this.centralSelectorRemoveHandler = info.GetRemoveMethod();
                this.isAvailable = true;
            }
            return this.isAvailable;
        }

        internal void Disable()
        {
            if (this.isAvailable && (this.dropdownDisableMethod != null))
            {
                this.dropdownDisableMethod.Invoke(this.centralDeviceSelectorService, null);
            }
        }

        internal void Disconnect()
        {
            if (this.isAvailable)
            {
                this.centralSelectorRemoveHandler.Invoke(this.centralDeviceSelectorService, this.centralSelectorHandlerArgs);
                this.isAvailable = false;
            }
        }

        internal void Enable()
        {
            if (this.isAvailable && (this.dropdownEnableMethod != null))
            {
                this.dropdownEnableMethod.Invoke(this.centralDeviceSelectorService, null);
            }
        }

        private void OnCentralComboSelectionChanged(object sender, DeviceSelectionChangedEventArgs e)
        {
            if (e != null)
            {
                this.selectionChangedEvent(sender, new CentralDeviceEventArgs(e.DeviceID));
            }
        }

        public Type CentralDeviceSelectorServiceType
        {
            get
            {
                return this.centralSelectorType;
            }
        }

        public bool IsAvailable
        {
            get
            {
                return this.isAvailable;
            }
        }

        public bool IsConnectable
        {
            get
            {
                return this.isConnectable;
            }
        }

        public string SelectedDeviceId
        {
            get
            {
                if (this.isAvailable && (this.getSelectedDeviceMethod != null))
                {
                    object obj2 = this.getSelectedDeviceMethod.Invoke(this.centralDeviceSelectorService, null);
                    if (obj2 is string)
                    {
                        return (string) obj2;
                    }
                }
                return string.Empty;
            }
        }

        internal class CentralDeviceEventArgs : EventArgs
        {
            internal string DeviceId;

            internal CentralDeviceEventArgs(string DeviceId)
            {
                this.DeviceId = DeviceId;
            }
        }
    }
}

