﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Runtime.InteropServices;
    using VSLangProj;

    internal class AddContentReferenceCommand : AddReferenceCommand
    {
        internal AddContentReferenceCommand(IXnaCodeProject xnaProject) : base(xnaProject, CmdIDConstants.AddContentReferenceCmdID)
        {
        }

        protected override void OnAddReference()
        {
            VSCOMPONENTSELECTORTABINIT[] tabInit = new VSCOMPONENTSELECTORTABINIT[1];
            tabInit[0].dwSize = (uint) Marshal.SizeOf(typeof(VSCOMPONENTSELECTORTABINIT));
            tabInit[0].varTabInitInfo = -999000001;
            tabInit[0].guidTab = VSConstants.GUID_SolutionPage;
            IVsUIHierarchy vsHierarchy = (IVsUIHierarchy) base.XnaProject.GetVsHierarchy();
            string helpTopic = "XNA.ContentProjectReferences.Add";
            bool flag = false;
            IVsUIHierarchyWindow vsService = (IVsUIHierarchyWindow) base.XnaProject.GetVsService(typeof(IVsUIHierarchyWindow));
            uint itemid = base.XnaProject.HierarchyItemManager.ReferencesFolderNode.Itemid;
            if (vsService != null)
            {
                uint dwStateMask = 0x20;
                uint pdwState = 0;
                if (ErrorHandler.Succeeded(vsService.GetItemState(vsHierarchy, itemid, dwStateMask, out pdwState)))
                {
                    flag = pdwState == 0x20;
                }
            }
            if (ErrorHandler.Succeeded(base.ShowComponentSelectorDialog(tabInit, XnaProjectResource.AddContentReferenceDialogTitle, null, null, helpTopic)) && (vsService != null))
            {
                if (!flag)
                {
                    ErrorHandler.ThrowOnFailure(vsService.ExpandItem(vsHierarchy, itemid, EXPANDFLAGS.EXPF_CollapseFolder));
                }
                uint itemId = base.XnaProject.HierarchyItemManager.ContentReferencesFolderNode.GetItemId();
                ErrorHandler.ThrowOnFailure(vsService.ExpandItem(vsHierarchy, itemId, EXPANDFLAGS.EXPF_ExpandFolder));
            }
        }

        protected override void OnAfterAddReference(Reference reference)
        {
            if (reference == null)
            {
                throw new ArgumentNullException("reference");
            }
            uint referenceItemid = Microsoft.Xna.GameStudio.ProjectUtilities.GetReferenceItemid(base.XnaProject, reference);
            if (referenceItemid != uint.MaxValue)
            {
                ErrorHandler.ThrowOnFailure(base.XnaProject.InnerVsBuildPropertyStorage.SetItemAttribute(referenceItemid, "XnaReferenceType", "Content"));
                HierarchyItemManager hierarchyItemManager = base.XnaProject.HierarchyItemManager;
                bool flag = hierarchyItemManager.IsContentReferencesFolderNodeVisible();
                hierarchyItemManager.OnHierarchyChanged();
                IVsHierarchyEvents vsHierarchy = base.XnaProject.GetVsHierarchy() as IVsHierarchyEvents;
                if (vsHierarchy != null)
                {
                    vsHierarchy.OnInvalidateItems(hierarchyItemManager.ReferencesFolderNode.Itemid);
                    ContentReferencesFolderNode contentReferencesFolderNode = hierarchyItemManager.ContentReferencesFolderNode;
                    if (flag)
                    {
                        ContentReferenceNode managedItem = hierarchyItemManager.GetManagedItem(referenceItemid) as ContentReferenceNode;
                        if (managedItem != null)
                        {
                            vsHierarchy.OnItemAdded(contentReferencesFolderNode.GetItemId(), managedItem.PrevContentReferenceItem, referenceItemid);
                        }
                    }
                    else
                    {
                        uint itemidSiblingPrev = hierarchyItemManager.IsNestedProjectFirstChildOfRoot ? uint.MaxValue : hierarchyItemManager.ItemBeforeContentReferencesFolder;
                        vsHierarchy.OnItemAdded(0xfffffffe, itemidSiblingPrev, contentReferencesFolderNode.GetItemId());
                    }
                }
            }
        }
    }
}

