﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.Build.Construction;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Projects.LanguageAdapters;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using System.Threading;
    using VSLangProj;

    internal abstract class BaseProjectLanguageAdapter : IProjectLanguageAdapter
    {
        protected IProjectPlatformAdapter adapter;
        protected IVsHierarchy innerHierarchy;
        protected IServiceProvider serviceProvider;

        protected BaseProjectLanguageAdapter()
        {
        }

        public virtual string GetCapability(string capabilityName)
        {
            return null;
        }

        private IEnumerable<uint> GetChildrenOfNode(uint itemid)
        {
            object pvar = null;
            ErrorHandler.ThrowOnFailure(this.innerHierarchy.GetProperty(itemid, -2041, out pvar));
            itemid = (uint) ((int) pvar);
            while (true)
            {
                if (itemid == uint.MaxValue)
                {
                    yield break;
                }
                yield return itemid;
                ErrorHandler.ThrowOnFailure(this.innerHierarchy.GetProperty(itemid, -2042, out pvar));
                itemid = (uint) ((int) pvar);
            }
        }

        public virtual Guid GetItemTemplatesGuid(Guid guid)
        {
            return guid;
        }

        public virtual List<string> GetLanguageCaps()
        {
            return new List<string>();
        }

        public virtual IEnumerable<uint> GetReferenceNodes()
        {
            return this.GetChildrenOfNode(this.GetReferencesFolderNode());
        }

        public virtual uint GetReferencesFolderNode()
        {
            foreach (uint num in this.GetChildrenOfNode(0xfffffffe))
            {
                object obj2;
                object obj3;
                Guid empty = Guid.Empty;
                ErrorHandler.ThrowOnFailure(this.innerHierarchy.GetGuidProperty(num, -1004, out empty));
                if (((empty == VSConstants.GUID_ItemType_VirtualFolder) && (ErrorHandler.Failed(this.innerHierarchy.GetProperty(num, -2062, out obj2)) || !((bool) obj2))) && ErrorHandler.Succeeded(this.innerHierarchy.GetProperty(num, -1001, out obj3)))
                {
                    for (uint i = (uint) ((int) obj3); i != uint.MaxValue; i = (uint) ((int) obj3))
                    {
                        if (this.IsReferenceItem(i))
                        {
                            return num;
                        }
                        if (ErrorHandler.Failed(this.innerHierarchy.GetProperty(i, -1002, out obj3)))
                        {
                            break;
                        }
                    }
                }
            }
            return uint.MaxValue;
        }

        public virtual IEnumerable<uint> GetVirtualFolders()
        {
            foreach (uint iteratorVariable0 in this.GetChildrenOfNode(0xfffffffe))
            {
                Guid empty = Guid.Empty;
                ErrorHandler.ThrowOnFailure(this.innerHierarchy.GetGuidProperty(iteratorVariable0, -1004, out empty));
                if (empty == VSConstants.GUID_ItemType_VirtualFolder)
                {
                    yield return iteratorVariable0;
                }
            }
        }

        public virtual void Initialize(IVsHierarchy innerHierarchy, IProjectPlatformAdapter adapter, IServiceProvider serviceProvider)
        {
            this.innerHierarchy = innerHierarchy;
            this.adapter = adapter;
            this.serviceProvider = serviceProvider;
        }

        private bool IsReferenceItem(uint itemid)
        {
            object obj2;
            if (ErrorHandler.Succeeded(this.innerHierarchy.GetProperty(itemid, -2027, out obj2)))
            {
                Reference reference = obj2 as Reference;
                return (reference != null);
            }
            return false;
        }

        public virtual void ModifyProjectOnCopy(ProjectRootElement projectRoot, IProjectPlatformAdapter targetAdapter)
        {
        }

        public virtual bool QueryService(ref Guid guidService, ref Guid riid, out object service)
        {
            service = null;
            return false;
        }

        public virtual string TranslateProjectDefines(string defines)
        {
            return defines;
        }

        public virtual Guid LanguageGuid
        {
            get
            {
                return new Guid("FAE04EC0-301F-11D3-BF4B-00C04F79EFBC");
            }
        }

        public virtual string Name
        {
            get
            {
                return "CSharp";
            }
        }

        public virtual string ProjectBrowseObjectGuidString
        {
            get
            {
                return "{4EF9F003-DE95-4d60-96B0-212979F2A857}";
            }
        }

        public virtual string ProjectConfigBrowseObjectGuidString
        {
            get
            {
                return "{A12CE10A-227F-4963-ADB6-3A43388513CA}";
            }
        }

        public virtual IPropertyPageProvider PropertyPageHandler
        {
            get
            {
                return new BasePropertyPageHandler();
            }
        }


    }
}

