﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.PlatformTools;
    using Microsoft.Xna.GameStudio.Resources;
    using Microsoft.Xna.GameStudio.Wizards;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel.Design;
    using System.Globalization;

    internal class CloneProjectCommand : OleMenuCommand, IServiceProvider
    {
        private string assemblyLocations;
        private bool isContextCommand;
        private IXnaCodeProject xnaProject;

        internal CloneProjectCommand(IXnaCodeProject xnaProject, CommandID commandID, bool isContextCommand) : base(new EventHandler(CloneProjectCommand.OnCloneProject), commandID)
        {
            if (xnaProject == null)
            {
                throw new ArgumentNullException("xnaProject");
            }
            this.xnaProject = xnaProject;
            this.isContextCommand = isContextCommand;
            base.BeforeQueryStatus += new EventHandler(this.OnBeforeQueryStatus);
            string[] sdkPaths = this.xnaProject.SdkPaths;
            this.assemblyLocations = string.Join(";", sdkPaths);
        }

        private bool ClonedProjectExists()
        {
            IXnaCodeProject project = null;
            CrossPlatformGroupManager service = this.GetService(typeof(CrossPlatformGroupManager)) as CrossPlatformGroupManager;
            if (service != null)
            {
                string crossPlatformGroupID = this.xnaProject.CrossPlatformGroupID;
                if (crossPlatformGroupID != null)
                {
                    project = service.GetProject(crossPlatformGroupID, this.TargetPlatform);
                }
            }
            return (project != null);
        }

        public object GetService(Type serviceType)
        {
            return this.xnaProject.GetVsService(serviceType);
        }

        private void OnBeforeQueryStatus(object sender, EventArgs e)
        {
            OleMenuCommand command = sender as OleMenuCommand;
            if (command != null)
            {
                IList<IPlatform> platforms = PlatformManager.GetPlatforms();
                bool flag = false;
                foreach (IPlatform platform in platforms)
                {
                    if (string.Equals(platform.Name, this.TargetPlatform, StringComparison.OrdinalIgnoreCase))
                    {
                        foreach (KeyValuePair<string, string> pair in this.xnaProject.KnownProjectPlatformAdapters)
                        {
                            if (pair.Value.Equals(this.TargetPlatform, StringComparison.OrdinalIgnoreCase))
                            {
                                flag = true;
                                break;
                            }
                        }
                        break;
                    }
                }
                command.Supported = true;
                command.Visible = flag;
                command.Enabled = Microsoft.Xna.GameStudio.ProjectUtilities.IsDesignMode() && !this.ClonedProjectExists();
                if (!this.isContextCommand)
                {
                    string projectName = Microsoft.Xna.GameStudio.ProjectUtilities.GetProjectName(this.xnaProject.GetVsHierarchy());
                    command.Text = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.XnaConvertPlatformCommand, new object[] { projectName, this.TargetPlatform });
                }
            }
        }

        private static void OnCloneProject(object sender, EventArgs args)
        {
            CloneProjectCommand serviceProvider = sender as CloneProjectCommand;
            if (serviceProvider != null)
            {
                int hr = Microsoft.Xna.GameStudio.ProjectUtilities.GetRequiredService<IVsSolution>(serviceProvider).SaveSolutionElement(1, null, 0);
                if (hr != -2147221492)
                {
                    ErrorHandler.ThrowOnFailure(hr);
                    CrossPlatformProjectWizard wizard = new CrossPlatformProjectWizard(serviceProvider.xnaProject, serviceProvider.TargetPlatform);
                    wizard.Execute();
                    GameStudioWindowPane.WriteLine(wizard.GetFeedback(), new object[0]);
                }
            }
        }

        private string TargetPlatform
        {
            get
            {
                string str = null;
                if ((this.CommandID == CmdIDConstants.XnaCloneForWindowsCmdID) || (this.CommandID == CmdIDConstants.XnaCloneForWindowsCtxCmdID))
                {
                    return "Windows";
                }
                if ((this.CommandID == CmdIDConstants.XnaCloneForXbox360CmdID) || (this.CommandID == CmdIDConstants.XnaCloneForXbox360CtxCmdID))
                {
                    return "Xbox 360";
                }
                if ((this.CommandID != CmdIDConstants.XnaCloneForWindowsPhoneCmdID) && (this.CommandID != CmdIDConstants.XnaCloneForWindowsPhoneCtxCmdID))
                {
                    return str;
                }
                return "Windows Phone";
            }
        }
    }
}

