﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    [ComVisible(true), Guid("D7F828FC-6CDF-4F04-8A48-7DDE7481EA16"), ClassInterface(ClassInterfaceType.None)]
    internal class ContentCompressionPropertyPage : ProjectDesignerPropertyPage
    {
        private IVsHierarchy activeHierarchy;
        private bool alwaysDisabled;
        private IVsBuildPropertyStorage buildPropertyStorage;
        private Dictionary<string, string> configProperties;
        private Label errorLabel;
        private List<string> projectConfigNames;
        private List<IVsCfg> projectConfigs;
        private static readonly string[] propertyNames = new string[] { "XnaCompressContent" };
        private ContentCompressionPropertyPageView view;

        public ContentCompressionPropertyPage() : base(XnaProjectResource.ContentBuildPropertyPageTabTitle, "XNA.ContentBuildPropertiesPage")
        {
        }

        private string ConvertCheckStateToString(CheckState state)
        {
            if (state == CheckState.Checked)
            {
                return bool.TrueString;
            }
            if (state == CheckState.Unchecked)
            {
                return bool.FalseString;
            }
            return null;
        }

        public override bool FinishPendingValidations()
        {
            return true;
        }

        public override object GetProperty(string propertyName)
        {
            return this.GetPropertyForConfiguration(propertyName, this.projectConfigNames[0]);
        }

        private object GetPropertyForConfiguration(string propertyName, string configurationName)
        {
            string pbstrPropValue = null;
            if (this.buildPropertyStorage != null)
            {
                uint storage = 1;
                ErrorHandler.ThrowOnFailure(this.buildPropertyStorage.GetPropertyValue(propertyName, configurationName, storage, out pbstrPropValue), new int[] { -2147170504 });
            }
            return pbstrPropValue;
        }

        public override bool GetPropertyMultipleValues(string propertyName, out object[] objects, out object[] values)
        {
            objects = null;
            values = null;
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            string[] strArray = new string[this.projectConfigs.Count];
            IVsCfg[] cfgArray = new IVsCfg[this.projectConfigs.Count];
            int index = 0;
            foreach (string str in this.projectConfigNames)
            {
                strArray[index] = this.GetPropertyForConfiguration(propertyName, str) as string;
                cfgArray[index] = this.projectConfigs[index];
                index++;
            }
            objects = cfgArray;
            values = strArray;
            return true;
        }

        protected override void OnCommitPropertyEdit()
        {
            CheckState contentCompression = this.view.ContentCompression;
            string falseString = this.ConvertCheckStateToString(contentCompression);
            if (string.IsNullOrEmpty(falseString))
            {
                falseString = bool.FalseString;
            }
            if (this.buildPropertyStorage != null)
            {
                foreach (string str2 in this.projectConfigNames)
                {
                    uint storage = 1;
                    ErrorHandler.ThrowOnFailure(this.buildPropertyStorage.SetPropertyValue("XnaCompressContent", str2, storage, falseString), new int[] { -2147170504 });
                    this.configProperties["XnaCompressContent"] = falseString;
                }
            }
        }

        private void OnPropertyChanged(object sender, EventArgs args)
        {
            string str = this.ConvertCheckStateToString(this.view.ContentCompression);
            string str2 = this.configProperties["XnaCompressContent"];
            if ((!string.IsNullOrEmpty(str) || !string.IsNullOrEmpty(str2)) && (str != str2))
            {
                base.OnPropertyEdited("XnaCompressContent", typeof(string));
            }
        }

        private void RefreshConfigProperties()
        {
            this.configProperties = new Dictionary<string, string>(propertyNames.Length);
            foreach (string str in propertyNames)
            {
                foreach (string str2 in this.projectConfigNames)
                {
                    string str3 = null;
                    string propertyForConfiguration = this.GetPropertyForConfiguration(str, str2) as string;
                    if (this.configProperties.TryGetValue(str, out str3))
                    {
                        if (!string.Equals(str3, propertyForConfiguration, StringComparison.InvariantCultureIgnoreCase))
                        {
                            this.configProperties[str] = "indeterminate";
                        }
                    }
                    else
                    {
                        this.configProperties[str] = propertyForConfiguration;
                    }
                }
            }
        }

        private void RefreshView()
        {
            this.RefreshConfigProperties();
            if (this.view != null)
            {
                this.view.RefreshView(this.activeHierarchy, this.configProperties);
            }
        }

        public override void SetObjects(uint cObjects, object[] ppunk)
        {
            this.projectConfigNames = new List<string>((int) cObjects);
            this.projectConfigs = new List<IVsCfg>((int) cObjects);
            this.activeHierarchy = null;
            if ((ppunk != null) && (cObjects <= ppunk.Length))
            {
                IXnaCodeProject hierarchyObject = null;
                for (int i = 0; i < cObjects; i++)
                {
                    IVsCfgBrowseObject obj2 = ppunk[i] as IVsCfgBrowseObject;
                    if (obj2 != null)
                    {
                        if (this.activeHierarchy == null)
                        {
                            uint maxValue = uint.MaxValue;
                            ErrorHandler.ThrowOnFailure(obj2.GetProjectItem(out this.activeHierarchy, out maxValue));
                            hierarchyObject = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>(this.activeHierarchy as IHierarchyObjectProvider);
                            if (hierarchyObject == null)
                            {
                                break;
                            }
                            IVsBuildPropertyStorage activeHierarchy = this.activeHierarchy as IVsBuildPropertyStorage;
                            IVsProject project = this.activeHierarchy as IVsProject;
                            if ((activeHierarchy != null) && (project != null))
                            {
                                this.buildPropertyStorage = new VsBuildPropertyStorageSccWrapper(activeHierarchy, project);
                            }
                            this.alwaysDisabled = !hierarchyObject.Adapter.SupportsContentCompression;
                        }
                        IVsCfg ppCfg = null;
                        ErrorHandler.ThrowOnFailure(obj2.GetCfg(out ppCfg));
                        if (ppCfg != null)
                        {
                            string pbstrDisplayName = null;
                            ErrorHandler.ThrowOnFailure(ppCfg.get_DisplayName(out pbstrDisplayName));
                            if (pbstrDisplayName != null)
                            {
                                this.projectConfigNames.Add(pbstrDisplayName);
                                this.projectConfigs.Add(ppCfg);
                            }
                        }
                    }
                }
            }
            base.SetObjects(cObjects, ppunk);
            this.RefreshView();
        }

        public override void SetProperty(string propertyName, object value)
        {
            this.SetPropertyForConfiguration(propertyName, value, this.projectConfigNames[0]);
        }

        private void SetPropertyForConfiguration(string propertyName, object value, string configurationName)
        {
            string strA = value as string;
            if (strA == null)
            {
                strA = string.Empty;
            }
            string property = this.GetProperty(propertyName) as string;
            if (property == null)
            {
                property = string.Empty;
            }
            if ((string.CompareOrdinal(strA, property) != 0) && (this.buildPropertyStorage != null))
            {
                uint storage = 1;
                ErrorHandler.ThrowOnFailure(this.buildPropertyStorage.SetPropertyValue(propertyName, configurationName, storage, strA));
            }
            this.RefreshView();
        }

        public override void SetPropertyMultipleValues(string propertyName, object[] objects, object[] values)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            if (objects == null)
            {
                throw new ArgumentNullException("objects");
            }
            if (values == null)
            {
                throw new ArgumentNullException("values");
            }
            if (objects.Length == values.Length)
            {
                for (int i = 0; i < objects.Length; i++)
                {
                    IVsCfg cfg = (IVsCfg) objects[i];
                    string pbstrDisplayName = null;
                    ErrorHandler.ThrowOnFailure(cfg.get_DisplayName(out pbstrDisplayName));
                    string str2 = values[i] as string;
                    if (cfg != null)
                    {
                        this.SetPropertyForConfiguration(propertyName, str2, pbstrDisplayName);
                    }
                }
                this.RefreshView();
            }
        }

        protected override void SetPropertyPageEnabled()
        {
            if (!this.alwaysDisabled)
            {
                base.SetPropertyPageEnabled();
            }
        }

        public override bool SupportsMultipleValueUndo(string propertyName)
        {
            return (this.projectConfigs.Count > 1);
        }

        protected override Control PageView
        {
            get
            {
                if (this.view == null)
                {
                    if ((this.projectConfigs == null) || (this.projectConfigs.Count == 0))
                    {
                        if (this.errorLabel == null)
                        {
                            this.errorLabel = new Label();
                            this.errorLabel.AutoSize = true;
                            this.errorLabel.TextAlign = ContentAlignment.MiddleCenter;
                            this.errorLabel.Dock = DockStyle.Fill;
                            this.errorLabel.Text = XnaProjectResource.BuildPropertyPageInitializationError;
                        }
                        return this.errorLabel;
                    }
                    this.view = new ContentCompressionPropertyPageView(this.activeHierarchy, this.configProperties);
                    this.view.PropertyChanged += new EventHandler(this.OnPropertyChanged);
                    this.view.AlwaysDisabled = this.alwaysDisabled;
                }
                return this.view;
            }
        }
    }
}

