﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Windows.Forms;

    internal class ContentCompressionPropertyPageView : UserControl
    {
        private bool alwaysDisabled;
        private IContainer components;
        private CheckBox compressContentCheckbox;
        private IVsHierarchy hierarchy;
        private bool inRefresh;
        private Label sectionLabel;

        internal event EventHandler PropertyChanged;

        internal ContentCompressionPropertyPageView(IVsHierarchy hierarchy, Dictionary<string, string> properties)
        {
            if (properties == null)
            {
                throw new ArgumentNullException("properties");
            }
            this.InitializeComponent();
            this.RefreshView(hierarchy, properties);
        }

        private void compressContentCheckbox_CheckedChanged(object sender, EventArgs e)
        {
            if (!this.inRefresh && (this.propertyChangedEvent != null))
            {
                this.propertyChangedEvent(this, EventArgs.Empty);
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void InitializeComponent()
        {
            ComponentResourceManager manager = new ComponentResourceManager(typeof(ContentCompressionPropertyPageView));
            this.sectionLabel = new Label();
            this.compressContentCheckbox = new CheckBox();
            base.SuspendLayout();
            manager.ApplyResources(this.sectionLabel, "sectionLabel");
            this.sectionLabel.Name = "sectionLabel";
            manager.ApplyResources(this.compressContentCheckbox, "compressContentCheckbox");
            this.compressContentCheckbox.Name = "compressContentCheckbox";
            this.compressContentCheckbox.UseVisualStyleBackColor = true;
            this.compressContentCheckbox.CheckedChanged += new EventHandler(this.compressContentCheckbox_CheckedChanged);
            manager.ApplyResources(this, "$this");
            base.AutoScaleMode = AutoScaleMode.Font;
            base.Controls.Add(this.compressContentCheckbox);
            base.Controls.Add(this.sectionLabel);
            base.Name = "ContentCompressionPropertyPageView";
            base.ResumeLayout(false);
            base.PerformLayout();
        }

        internal void RefreshView(IVsHierarchy activeHierarchy, Dictionary<string, string> properties)
        {
            this.hierarchy = activeHierarchy;
            bool inRefresh = this.inRefresh;
            this.inRefresh = true;
            try
            {
                if ((properties == null) || (properties.Count == 0))
                {
                    base.Enabled = false;
                    this.compressContentCheckbox.CheckState = CheckState.Unchecked;
                }
                else
                {
                    base.Enabled = !this.alwaysDisabled;
                    string a = properties["XnaCompressContent"];
                    if (a == null)
                    {
                        this.compressContentCheckbox.CheckState = CheckState.Unchecked;
                    }
                    else if (a == "indeterminate")
                    {
                        this.compressContentCheckbox.CheckState = CheckState.Indeterminate;
                    }
                    else
                    {
                        this.compressContentCheckbox.CheckState = string.Equals(a, bool.TrueString, StringComparison.InvariantCultureIgnoreCase) ? CheckState.Checked : CheckState.Unchecked;
                    }
                }
            }
            finally
            {
                this.inRefresh = inRefresh;
            }
        }

        internal bool AlwaysDisabled
        {
            get
            {
                return this.alwaysDisabled;
            }
            set
            {
                this.alwaysDisabled = value;
                if (this.alwaysDisabled)
                {
                    base.Enabled = false;
                }
            }
        }

        internal CheckState ContentCompression
        {
            get
            {
                return this.compressContentCheckbox.CheckState;
            }
        }
    }
}

