﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.Xna.GameStudio;
    using System;
    using System.IO;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using VSLangProj;
    using VSLangProj80;

    internal class ContentReferenceNode : HierarchyItem
    {
        private IXnaCodeProject codeProject;
        private uint itemid;
        private uint nextSiblingItemid;

        internal ContentReferenceNode(uint itemid, uint itemidParent, IXnaCodeProject codeProject, uint prevNonContentReferenceItem, uint prevContentReferenceItem) : base(codeProject.InnerIVsUIHierarchy)
        {
            this.itemid = itemid;
            this.NextContentReferenceItem = uint.MaxValue;
            this.PrevContentReferenceItem = prevContentReferenceItem;
            this.PrevNonContentReferenceItem = prevNonContentReferenceItem;
            this.codeProject = codeProject;
            base.OverrideProperty(-2005, new HierarchyItem.GetPropertyDelegate(this.GetIconIndex));
            base.OverrideProperty(-2015, new HierarchyItem.GetPropertyDelegate(this.GetIconIndex));
            base.OverrideProperty(-2003, new HierarchyItem.GetPropertyDelegate(this.GetCaption));
            base.OverrideProperty(-1002, -1);
            base.OverrideProperty(-2042, -1);
            base.OverrideProperty(-1000, (int) itemidParent);
        }

        private int GetCaption(uint itemid, int propid, out object property)
        {
            object obj2;
            if (ErrorHandler.Succeeded(base.InnerHierarchy.GetProperty(itemid, -2027, out obj2)))
            {
                Reference reference = obj2 as Reference;
                if ((reference != null) && (reference.SourceProject != null))
                {
                    try
                    {
                        property = Path.GetFileNameWithoutExtension(reference.SourceProject.FileName);
                    }
                    catch (ArgumentException)
                    {
                        property = null;
                    }
                    if (property != null)
                    {
                        return 0;
                    }
                }
            }
            return base.InnerHierarchy.GetProperty(itemid, propid, out property);
        }

        private int GetIconIndex(uint itemid, int propid, out object property)
        {
            object obj2;
            if (ErrorHandler.Succeeded(base.InnerHierarchy.GetProperty(itemid, -2027, out obj2)))
            {
                Reference3 reference = obj2 as Reference3;
                if ((reference != null) && !reference.Resolved)
                {
                    property = this.codeProject.HierarchyItemManager.ProjectNode.ContentReferenceMissingIconIndex;
                    return 0;
                }
            }
            property = this.codeProject.HierarchyItemManager.ProjectNode.ContentReferenceIconIndex;
            return 0;
        }

        internal override int QueryStatusCommand(uint itemid, ref Guid pguidCmdGroup, uint cCmds, OLECMD[] prgCmds, IntPtr pCmdText)
        {
            int hr = base.QueryStatusCommand(itemid, ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
            if (ErrorHandler.Succeeded(hr) && (pguidCmdGroup == VSConstants.VSStd2K))
            {
                for (int i = 0; (i < prgCmds.Length) && (i < cCmds); i++)
                {
                    if (prgCmds[i].cmdID == 0x45f)
                    {
                        prgCmds[i].cmdf = 0x11;
                    }
                }
            }
            return hr;
        }

        internal uint Itemid
        {
            get
            {
                return this.itemid;
            }
        }

        internal uint NextContentReferenceItem
        {
            get
            {
                return this.nextSiblingItemid;
            }
            set
            {
                this.nextSiblingItemid = value;
                base.OverrideProperty(-1002, (int) value);
                base.OverrideProperty(-2042, (int) value);
            }
        }

        internal uint PrevContentReferenceItem { get; private set; }

        internal uint PrevNonContentReferenceItem { get; private set; }
    }
}

