﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using System;
    using System.Collections.Generic;
    using System.Runtime.InteropServices;

    internal class ComEnumerator<T>
    {
        private ICollection<T> collection;
        private IEnumerator<T> enumerator;

        protected ComEnumerator(ComEnumerator<T> original)
        {
            if (original == null)
            {
                throw new ArgumentNullException("original");
            }
            this.collection = original.collection;
            IEnumerator<T> enumerator = null;
            if (original.enumerator != null)
            {
                enumerator = this.collection.GetEnumerator();
                while (!enumerator.MoveNext() && !object.ReferenceEquals(enumerator.Current, this.enumerator.Current))
                {
                }
                if (!object.ReferenceEquals(enumerator.Current, enumerator.Current))
                {
                    throw new InvalidOperationException();
                }
                this.enumerator = enumerator;
            }
        }

        internal ComEnumerator(ICollection<T> collection) : this(collection, null)
        {
        }

        private ComEnumerator(ICollection<T> collection, IEnumerator<T> enumerator)
        {
            if (collection == null)
            {
                throw new ArgumentNullException("collection");
            }
            this.collection = collection;
            if ((enumerator == null) && (collection.Count > 0))
            {
                enumerator = collection.GetEnumerator();
                enumerator.MoveNext();
            }
            this.enumerator = enumerator;
        }

        public int GetCount(out uint pcelt)
        {
            pcelt = (uint) this.collection.Count;
            return 0;
        }

        public int Next(uint length, T[] elements, ref uint fetched)
        {
            fetched = 0;
            if (elements == null)
            {
                throw new ArgumentNullException("elements");
            }
            if (elements.Length < length)
            {
                throw new ArgumentOutOfRangeException("length");
            }
            uint num = 0;
            while ((num < length) && (this.enumerator != null))
            {
                elements[num++] = this.enumerator.Current;
                if (!this.enumerator.MoveNext())
                {
                    this.enumerator = null;
                }
            }
            fetched = num;
            if (num >= length)
            {
                return 0;
            }
            return 1;
        }

        public int Reset()
        {
            if (this.enumerator != null)
            {
                this.enumerator.Reset();
                this.enumerator.MoveNext();
            }
            else if (this.collection.Count > 0)
            {
                this.enumerator = this.collection.GetEnumerator();
                this.enumerator.MoveNext();
            }
            return 0;
        }

        public int Skip(uint count)
        {
            while ((count > 0) && (this.enumerator != null))
            {
                if (!this.enumerator.MoveNext())
                {
                    this.enumerator = null;
                }
                count--;
            }
            if (count != 0)
            {
                return 1;
            }
            return 0;
        }
    }
}

