﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using Microsoft.VisualStudio.Debugger.Interop;
    using Microsoft.VisualStudio.OLE.Interop;
    using System;
    using System.Collections.Generic;
    using System.Runtime.InteropServices;

    internal class DebugProcess : IDebugProcess2
    {
        private DateTime creationTime;
        private System.Guid guid;
        private uint pid;
        private IDebugPort2 port;
        private IDebugProgram2 program;
        private const uint SupportedFields = 0xef;
        private string titleExeName;
        private string titleName;

        internal DebugProcess(uint pid, string titleName, string titleExeName, IDebugPort2 port)
        {
            if (titleName == null)
            {
                throw new ArgumentNullException("name");
            }
            if (titleExeName == null)
            {
                throw new ArgumentNullException("title");
            }
            if (port == null)
            {
                throw new ArgumentNullException("port");
            }
            this.titleExeName = titleExeName;
            this.titleName = titleName;
            this.port = port;
            this.creationTime = DateTime.Now;
            this.pid = pid;
            this.guid = System.Guid.NewGuid();
        }

        int IDebugProcess2.Attach(IDebugEventCallback2 pCallback, System.Guid[] rgguidSpecificEngines, uint celtSpecificEngines, int[] rghrEngineAttach)
        {
            throw new NotImplementedException();
        }

        int IDebugProcess2.CanDetach()
        {
            return 1;
        }

        int IDebugProcess2.CauseBreak()
        {
            throw new NotImplementedException();
        }

        int IDebugProcess2.Detach()
        {
            throw new NotImplementedException();
        }

        int IDebugProcess2.EnumPrograms(out IEnumDebugPrograms2 ppEnum)
        {
            ppEnum = new EnumDebugPrograms(new IDebugProgram2[] { this.Program });
            return 0;
        }

        int IDebugProcess2.EnumThreads(out IEnumDebugThreads2 ppEnum)
        {
            ppEnum = new EnumDebugThreads(new List<IDebugThread2>());
            return 0;
        }

        int IDebugProcess2.GetAttachedSessionName(out string pbstrSessionName)
        {
            pbstrSessionName = null;
            throw new NotImplementedException();
        }

        int IDebugProcess2.GetInfo(uint Fields, PROCESS_INFO[] pProcessInfo)
        {
            if ((pProcessInfo == null) || (pProcessInfo.Length == 0))
            {
                throw new ArgumentNullException("pProcessInfo");
            }
            PROCESS_INFO process_info = new PROCESS_INFO {
                Fields = Fields & 0xef
            };
            if ((Fields & 0x20) != 0)
            {
                process_info.bstrAttachedSessionName = null;
            }
            if ((Fields & 2) != 0)
            {
                process_info.bstrBaseName = this.titleExeName;
            }
            if ((Fields & 0x40) != 0)
            {
                process_info.CreationTime = new Microsoft.VisualStudio.OLE.Interop.FILETIME();
                process_info.CreationTime.dwHighDateTime = 0;
                process_info.CreationTime.dwLowDateTime = (uint) this.creationTime.ToFileTime();
            }
            if ((Fields & 1) != 0)
            {
                process_info.bstrFileName = this.titleExeName;
            }
            if ((Fields & 8) != 0)
            {
                process_info.ProcessId = new AD_PROCESS_ID();
                process_info.ProcessId.ProcessIdType = 0;
                process_info.ProcessId.dwProcessId = this.pid;
            }
            if ((Fields & 0x10) != 0)
            {
                process_info.dwSessionId = 0;
            }
            if ((Fields & 4) != 0)
            {
                process_info.bstrTitle = this.titleName;
            }
            pProcessInfo[0] = process_info;
            return 0;
        }

        int IDebugProcess2.GetName(uint gnType, out string pbstrName)
        {
            pbstrName = null;
            switch (gnType)
            {
                case 0:
                    pbstrName = this.titleExeName;
                    break;

                case 5:
                    pbstrName = this.titleName;
                    break;

                default:
                    pbstrName = string.Empty;
                    break;
            }
            return 0;
        }

        int IDebugProcess2.GetPhysicalProcessId(AD_PROCESS_ID[] pProcessId)
        {
            if ((pProcessId == null) || (pProcessId.Length < 1))
            {
                throw new ArgumentNullException("pProcessId");
            }
            pProcessId[0] = new AD_PROCESS_ID();
            pProcessId[0].ProcessIdType = 0;
            pProcessId[0].dwProcessId = this.pid;
            return 0;
        }

        int IDebugProcess2.GetPort(out IDebugPort2 ppPort)
        {
            ppPort = this.port;
            return 0;
        }

        int IDebugProcess2.GetProcessId(out System.Guid pguidProcessId)
        {
            pguidProcessId = this.guid;
            return 0;
        }

        int IDebugProcess2.GetServer(out IDebugCoreServer2 ppServer)
        {
            ppServer = null;
            throw new NotImplementedException();
        }

        int IDebugProcess2.Terminate()
        {
            throw new NotImplementedException();
        }

        public System.Guid Guid
        {
            get
            {
                return this.guid;
            }
        }

        public uint Pid
        {
            get
            {
                return this.pid;
            }
        }

        public IDebugProgram2 Program
        {
            get
            {
                if (this.program == null)
                {
                    this.program = new DebugProgram(this.titleName, this);
                }
                return this.program;
            }
        }
    }
}

