﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Debugger.Interop;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Runtime.InteropServices;

    internal class DebugProgram : IDebugProgram2, IDebugProgramEx2, IDebugProgramNode2, IDebugCOMPlusProgramNode2
    {
        private string name;
        private IDebugProcess2 process;
        private Guid programId;

        internal DebugProgram(string name, IDebugProcess2 process)
        {
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (process == null)
            {
                throw new ArgumentNullException("process");
            }
            this.name = name;
            this.process = process;
            this.programId = Guid.NewGuid();
        }

        public int GetAppDomainId(out uint pul32Id)
        {
            pul32Id = 1;
            return 0;
        }

        int IDebugProgram2.Attach(IDebugEventCallback2 pCallback)
        {
            return 1;
        }

        int IDebugProgram2.CanDetach()
        {
            return 1;
        }

        int IDebugProgram2.CauseBreak()
        {
            throw new NotImplementedException();
        }

        int IDebugProgram2.Continue(IDebugThread2 pThread)
        {
            throw new NotImplementedException();
        }

        int IDebugProgram2.Detach()
        {
            return -2147467259;
        }

        int IDebugProgram2.EnumCodeContexts(IDebugDocumentPosition2 pDocPos, out IEnumDebugCodeContexts2 ppEnum)
        {
            ppEnum = null;
            throw new NotImplementedException();
        }

        int IDebugProgram2.EnumCodePaths(string pszHint, IDebugCodeContext2 pStart, IDebugStackFrame2 pFrame, int fSource, out IEnumCodePaths2 ppEnum, out IDebugCodeContext2 ppSafety)
        {
            ppEnum = null;
            ppSafety = null;
            throw new NotImplementedException();
        }

        int IDebugProgram2.EnumModules(out IEnumDebugModules2 ppEnum)
        {
            ppEnum = null;
            throw new NotImplementedException();
        }

        int IDebugProgram2.EnumThreads(out IEnumDebugThreads2 ppEnum)
        {
            ppEnum = new EnumDebugThreads(new List<IDebugThread2>());
            return 0;
        }

        int IDebugProgram2.Execute()
        {
            throw new NotImplementedException();
        }

        int IDebugProgram2.GetDebugProperty(out IDebugProperty2 ppProperty)
        {
            ppProperty = null;
            throw new NotImplementedException();
        }

        int IDebugProgram2.GetDisassemblyStream(uint dwScope, IDebugCodeContext2 pCodeContext, out IDebugDisassemblyStream2 ppDisassemblyStream)
        {
            ppDisassemblyStream = null;
            throw new NotImplementedException();
        }

        int IDebugProgram2.GetENCUpdate(out object ppUpdate)
        {
            ppUpdate = null;
            throw new NotImplementedException();
        }

        int IDebugProgram2.GetEngineInfo(out string pbstrEngine, out Guid pguidEngine)
        {
            pbstrEngine = XnaProjectResource.XnaFrameworkDebugEngine;
            pguidEngine = DebugConstants.DebugEngineGuid;
            return 0;
        }

        int IDebugProgram2.GetMemoryBytes(out IDebugMemoryBytes2 ppMemoryBytes)
        {
            ppMemoryBytes = null;
            throw new NotImplementedException();
        }

        int IDebugProgram2.GetName(out string pbstrName)
        {
            pbstrName = this.name;
            return 0;
        }

        int IDebugProgram2.GetProcess(out IDebugProcess2 ppProcess)
        {
            ppProcess = this.process;
            return 0;
        }

        int IDebugProgram2.GetProgramId(out Guid pguidProgramId)
        {
            pguidProgramId = this.programId;
            return 0;
        }

        int IDebugProgram2.Step(IDebugThread2 pThread, uint sk, uint Step)
        {
            throw new NotImplementedException();
        }

        int IDebugProgram2.Terminate()
        {
            throw new NotImplementedException();
        }

        int IDebugProgram2.WriteDump(uint DUMPTYPE, string pszDumpUrl)
        {
            throw new NotImplementedException();
        }

        int IDebugProgramEx2.Attach(IDebugEventCallback2 pCallback, uint dwReason, IDebugSession2 pSession)
        {
            return 1;
        }

        int IDebugProgramEx2.GetProgramNode(out IDebugProgramNode2 ppProgramNode)
        {
            ppProgramNode = this;
            return 0;
        }

        int IDebugProgramNode2.Attach_V7(IDebugProgram2 pMDMProgram, IDebugEventCallback2 pCallback, uint dwReason)
        {
            return 0;
        }

        int IDebugProgramNode2.DetachDebugger_V7()
        {
            return 0;
        }

        int IDebugProgramNode2.GetEngineInfo(out string pbstrEngine, out Guid pguidEngine)
        {
            return ((IDebugProgram2) this).GetEngineInfo(out pbstrEngine, out pguidEngine);
        }

        int IDebugProgramNode2.GetHostMachineName_V7(out string pbstrHostMachineName)
        {
            IDebugPort2 port;
            ErrorHandler.ThrowOnFailure(this.process.GetPort(out port));
            ErrorHandler.ThrowOnFailure(port.GetPortName(out pbstrHostMachineName));
            return 0;
        }

        int IDebugProgramNode2.GetHostName(uint dwHostNameType, out string pbstrHostName)
        {
            pbstrHostName = null;
            if (dwHostNameType == 0)
            {
                return this.process.GetName(0, out pbstrHostName);
            }
            if (dwHostNameType != 1)
            {
                throw new ArgumentOutOfRangeException("dwHostNameType");
            }
            PROCESS_INFO[] pProcessInfo = new PROCESS_INFO[1];
            ErrorHandler.ThrowOnFailure(this.process.GetInfo(1, pProcessInfo));
            if ((pProcessInfo[0].Fields & 1) != 0)
            {
                pbstrHostName = pProcessInfo[0].bstrFileName;
            }
            return 0;
        }

        int IDebugProgramNode2.GetHostPid(AD_PROCESS_ID[] pHostProcessId)
        {
            if (pHostProcessId == null)
            {
                throw new ArgumentNullException("pHostProcessId");
            }
            return this.process.GetPhysicalProcessId(pHostProcessId);
        }

        int IDebugProgramNode2.GetProgramName(out string pbstrProgramName)
        {
            pbstrProgramName = this.name;
            return 0;
        }
    }
}

