﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using EnvDTE;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.PlatformTools;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Globalization;
    using System.Runtime.InteropServices;

    internal abstract class DebuggableProjectCfg : IVsDebuggableProjectCfg2, IVsDebuggableProjectCfg, IVsProjectCfg, IVsCfg, IDisposable
    {
        private IVsDebuggableProjectCfg2 innerDebuggableProjectCfg2;
        protected PlatformFlavorCfg parentCfg;

        internal DebuggableProjectCfg(PlatformFlavorCfg parentCfg)
        {
            if (parentCfg == null)
            {
                throw new ArgumentNullException("parentCfg");
            }
            this.parentCfg = parentCfg;
            this.innerDebuggableProjectCfg2 = this.parentCfg.GetInnerCfgType(typeof(IVsDebuggableProjectCfg2)) as IVsDebuggableProjectCfg2;
        }

        public int DebugLaunch(uint grfLaunch)
        {
            IVsDebugger2 vsService = this.parentCfg.Project.GetVsService(typeof(SVsShellDebugger)) as IVsDebugger2;
            if (vsService == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.MissingService, new object[] { typeof(IVsDebugger2).FullName }));
            }
            VsDebugTargetInfo2 debugTarget = new VsDebugTargetInfo2();
            IntPtr zero = IntPtr.Zero;
            try
            {
                IPlatform platform = PlatformManager.GetPlatform(this.parentCfg.Project.Adapter.XnaPlatform, this.parentCfg.XnaFrameworkRuntimeProfile);
                if (platform == null)
                {
                    throw new InvalidOperationException(XnaProjectResource.DebugNoPlatformAvailable);
                }
                ITitlePlayer titlePlayer = platform.GetTitlePlayer(this.parentCfg.TargetConsole);
                if (titlePlayer == null)
                {
                    throw new InvalidOperationException(string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.DebugNoTargetAvailable, new object[] { this.parentCfg.TargetConsole }));
                }
                if (titlePlayer.GetRuntimeSupport(this.parentCfg.XnaFrameworkRuntimeProfile) != RuntimeSupport.Supported)
                {
                    throw new InvalidOperationException(string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.DebugNoCompatibleRuntime, new object[0]));
                }
                ILaunchService service = titlePlayer.GetService(typeof(ILaunchService)) as ILaunchService;
                if (service.IsTitleRunning())
                {
                    string error = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.Xbox360LaunchErrorTitleAlreadyExecuting, new object[] { this.parentCfg.TargetConsole });
                    this.parentCfg.Project.ErrorProvider.ReportError(error);
                    throw new COMException(error, -2147467259);
                }
                Project dteProject = Microsoft.Xna.GameStudio.ProjectUtilities.GetDteProject(this.parentCfg.Project.GetVsHierarchy());
                if ((dteProject == null) || (dteProject.Properties == null))
                {
                    throw new InvalidOperationException(XnaProjectResource.Xbox360LaunchDataUnavailable);
                }
                Guid assemblyGuid = Microsoft.Xna.GameStudio.ProjectUtilities.GetAssemblyGuid(dteProject);
                if (assemblyGuid == Guid.Empty)
                {
                    throw new InvalidOperationException(XnaProjectResource.Xbox360DeployFailedMissingTitleID);
                }
                string commandLine = null;
                try
                {
                    if (((dteProject.ConfigurationManager != null) && (dteProject.ConfigurationManager.ActiveConfiguration != null)) && (dteProject.ConfigurationManager.ActiveConfiguration.Properties != null))
                    {
                        Property property = dteProject.ConfigurationManager.ActiveConfiguration.Properties.Item("StartArguments");
                        if (property != null)
                        {
                            commandLine = property.Value as string;
                        }
                    }
                }
                catch (ArgumentException)
                {
                }
                if ((grfLaunch & 4) != 0)
                {
                    service.LaunchTitle(assemblyGuid.ToString("N", CultureInfo.InvariantCulture), commandLine, null);
                }
                else
                {
                    this.GetDebugTargetInfo(ref debugTarget, grfLaunch, assemblyGuid, commandLine);
                    zero = Marshal.AllocHGlobal(Marshal.SizeOf(debugTarget));
                    Marshal.StructureToPtr(debugTarget, zero, false);
                    int hr = vsService.LaunchDebugTargets2(1, zero);
                    if (hr == -1989083114)
                    {
                        throw new InvalidOperationException(XnaProjectResource.DebugCannotLaunchTarget);
                    }
                    ErrorHandler.ThrowOnFailure(hr);
                }
            }
            finally
            {
                if (zero != IntPtr.Zero)
                {
                    Marshal.DestroyStructure(zero, debugTarget.GetType());
                    Marshal.FreeHGlobal(zero);
                    zero = IntPtr.Zero;
                }
                if (debugTarget.pDebugEngines != IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(debugTarget.pDebugEngines);
                    debugTarget.pDebugEngines = IntPtr.Zero;
                }
            }
            return 0;
        }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            this.parentCfg = null;
            this.innerDebuggableProjectCfg2 = null;
        }

        public int EnumOutputs(out IVsEnumOutputs ppIVsEnumOutputs)
        {
            throw new NotImplementedException();
        }

        ~DebuggableProjectCfg()
        {
            this.Dispose(false);
        }

        public int get_BuildableProjectCfg(out IVsBuildableProjectCfg ppIVsBuildableProjectCfg)
        {
            throw new NotImplementedException();
        }

        public int get_CanonicalName(out string pbstrCanonicalName)
        {
            throw new NotImplementedException();
        }

        public int get_DisplayName(out string pbstrDisplayName)
        {
            throw new NotImplementedException();
        }

        public int get_IsDebugOnly(out int pfIsDebugOnly)
        {
            throw new NotImplementedException();
        }

        public int get_IsPackaged(out int pfIsPackaged)
        {
            throw new NotImplementedException();
        }

        public int get_IsReleaseOnly(out int pfIsReleaseOnly)
        {
            throw new NotImplementedException();
        }

        public int get_IsSpecifyingOutputSupported(out int pfIsSpecifyingOutputSupported)
        {
            throw new NotImplementedException();
        }

        public int get_Platform(out Guid pguidPlatform)
        {
            throw new NotImplementedException();
        }

        public int get_ProjectCfgProvider(out IVsProjectCfgProvider ppIVsProjectCfgProvider)
        {
            throw new NotImplementedException();
        }

        public int get_RootURL(out string pbstrRootURL)
        {
            throw new NotImplementedException();
        }

        public int get_TargetCodePage(out uint puiTargetCodePage)
        {
            throw new NotImplementedException();
        }

        public int get_UpdateSequenceNumber(ULARGE_INTEGER[] puliUSN)
        {
            throw new NotImplementedException();
        }

        protected abstract void GetDebugTargetInfo(ref VsDebugTargetInfo2 debugTarget, uint grfLaunch, Guid titleId, string args);
        public int OnBeforeDebugLaunch(uint grfLaunch)
        {
            if (this.innerDebuggableProjectCfg2 != null)
            {
                return this.innerDebuggableProjectCfg2.OnBeforeDebugLaunch(grfLaunch);
            }
            return 0;
        }

        public int OpenOutput(string szOutputCanonicalName, out IVsOutput ppIVsOutput)
        {
            throw new NotImplementedException();
        }

        public int QueryDebugLaunch(uint grfLaunch, out int pfCanLaunch)
        {
            if (!this.parentCfg.Project.IsEntryProject)
            {
                pfCanLaunch = 0;
                return 0;
            }
            if ((grfLaunch & 0x100) != 0)
            {
                pfCanLaunch = 0;
            }
            else
            {
                pfCanLaunch = 1;
            }
            return 0;
        }

        protected static void SetDebugEngine(ref VsDebugTargetInfo2 debugTarget, Guid debugEngineGuid)
        {
            debugTarget.guidLaunchDebugEngine = debugEngineGuid;
            debugTarget.pDebugEngines = Marshal.AllocHGlobal(Marshal.SizeOf(debugTarget.guidLaunchDebugEngine));
            Marshal.StructureToPtr(debugTarget.guidLaunchDebugEngine, debugTarget.pDebugEngines, false);
            debugTarget.dwDebugEngineCount = 1;
        }
    }
}

