﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.Collections.Generic;

    internal class DeployParams
    {
        private Microsoft.Xna.GameStudio.CodeProject.Debugging.DeployTransaction deployTransaction;
        private string outputDirectory;
        private IVsOutputWindowPane outputWindow;
        internal static readonly string[] ProjectProperties = new string[] { "Title", "Description", "AssemblyName" };
        private Dictionary<string, string> properties;

        internal DeployParams(IVsOutputWindowPane outputWindow)
        {
            if (outputWindow == null)
            {
                throw new ArgumentNullException("outputWindow");
            }
            this.outputWindow = outputWindow;
        }

        internal string GetProperty(string propertyName)
        {
            string str;
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            if ((this.properties != null) && this.properties.TryGetValue(propertyName, out str))
            {
                return str;
            }
            return string.Empty;
        }

        internal void SetProperty(string propertyName, string value)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            if (value == null)
            {
                if ((this.properties != null) && this.properties.ContainsKey(propertyName))
                {
                    this.properties.Remove(propertyName);
                }
            }
            else
            {
                if (this.properties == null)
                {
                    this.properties = new Dictionary<string, string>();
                }
                this.properties[propertyName] = value;
            }
        }

        internal Microsoft.Xna.GameStudio.CodeProject.Debugging.DeployTransaction DeployTransaction
        {
            get
            {
                return this.deployTransaction;
            }
            set
            {
                this.deployTransaction = value;
            }
        }

        internal string Description
        {
            get
            {
                return this.GetProperty("Description");
            }
        }

        internal string OutputDirectory
        {
            get
            {
                if (this.outputDirectory == null)
                {
                    this.outputDirectory = string.Empty;
                }
                return this.outputDirectory;
            }
            set
            {
                this.outputDirectory = value;
            }
        }

        internal IVsOutputWindowPane OutputWindow
        {
            get
            {
                return this.outputWindow;
            }
        }

        internal string RuntimeProfile
        {
            get
            {
                return this.GetProperty("RuntimeProfile");
            }
            set
            {
                this.SetProperty("RuntimeProfile", value);
            }
        }

        internal string StartupAssembly
        {
            get
            {
                return (this.GetProperty("AssemblyName") + ".exe");
            }
        }

        internal string TargetName
        {
            get
            {
                return this.GetProperty("TargetName");
            }
            set
            {
                this.SetProperty("TargetName", value);
            }
        }

        internal string Thumbnail
        {
            get
            {
                return this.GetProperty("Thumbnail");
            }
            set
            {
                this.SetProperty("Thumbnail", value);
            }
        }

        internal string Title
        {
            get
            {
                return this.GetProperty("Title");
            }
        }

        internal string TitleID
        {
            get
            {
                return this.GetProperty("TitleID");
            }
            set
            {
                this.SetProperty("TitleID", value);
            }
        }
    }
}

