﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using System;
    using System.Collections.Generic;

    internal abstract class DeployTransaction : IDisposable
    {
        private bool cancelled;
        private bool cleanDeploy;
        private bool committed;
        private List<string> filesNotFound;
        private List<string> filesToDeploy;
        private List<string> filesUpToDate;
        private bool reset;
        private object tag;

        protected DeployTransaction(bool cleanDeploy)
        {
            this.cleanDeploy = cleanDeploy;
            this.filesToDeploy = new List<string>();
            this.filesUpToDate = new List<string>();
            this.filesNotFound = new List<string>();
        }

        internal void Cancel()
        {
            if ((!this.reset && !this.cancelled) && !this.committed)
            {
                this.cancelled = true;
                this.OnCancel();
            }
        }

        internal void Commit()
        {
            bool flag = false;
            if ((!this.reset && !this.cancelled) && !this.committed)
            {
                this.committed = true;
                try
                {
                    this.OnCommit();
                    flag = true;
                }
                finally
                {
                    this.committed = flag;
                }
            }
        }

        public void Dispose()
        {
            if ((!this.reset && !this.committed) && !this.cancelled)
            {
                this.Cancel();
            }
        }

        protected abstract void OnCancel();
        protected abstract void OnCommit();
        protected abstract void OnReset();
        internal void Reset()
        {
            if ((!this.reset && !this.cancelled) && !this.committed)
            {
                this.reset = true;
                this.OnReset();
            }
        }

        internal bool Cancelled
        {
            get
            {
                return this.cancelled;
            }
        }

        internal bool CleanDeploy
        {
            get
            {
                return this.cleanDeploy;
            }
        }

        internal bool Committed
        {
            get
            {
                return this.committed;
            }
        }

        internal ICollection<string> FilesNotFound
        {
            get
            {
                return this.filesNotFound;
            }
        }

        internal ICollection<string> FilesToDeploy
        {
            get
            {
                return this.filesToDeploy;
            }
        }

        internal ICollection<string> FilesUpToDate
        {
            get
            {
                return this.filesUpToDate;
            }
        }

        internal virtual object Tag
        {
            get
            {
                return this.tag;
            }
            set
            {
                this.tag = value;
            }
        }
    }
}

