﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Globalization;
    using System.Text;

    internal static class TitlePropertiesHelper
    {
        internal static void DisplayTitleProperties(IVsOutputWindowPane outputWindow, DeployParams deployParams)
        {
            if (deployParams == null)
            {
                throw new ArgumentNullException("deployParams");
            }
            if (outputWindow != null)
            {
                LabelValuePair[] pairArray = new LabelValuePair[] { new LabelValuePair(XnaProjectResource.TitlePropertiesTitleID, deployParams.TitleID), new LabelValuePair(XnaProjectResource.TitlePropertiesTitle, deployParams.Title), new LabelValuePair(XnaProjectResource.TitlePropertiesDescription, deployParams.Description), new LabelValuePair(XnaProjectResource.TitlePropertiesStartupAssembly, deployParams.StartupAssembly), new LabelValuePair(XnaProjectResource.TitlePropertiesRuntimeProfile, deployParams.RuntimeProfile) };
                StringBuilder builder = new StringBuilder();
                foreach (LabelValuePair pair in pairArray)
                {
                    string str = string.Format(CultureInfo.InvariantCulture, "    {0,-24} '{1}'", new object[] { pair.Label, pair.Value });
                    builder.AppendLine(str);
                }
                builder.Append("\n");
                outputWindow.OutputStringThreadSafe(builder.ToString());
            }
        }

        private class LabelValuePair
        {
            private string label;
            private string value;

            public LabelValuePair(string label, string value)
            {
                if (label == null)
                {
                    throw new ArgumentNullException("label");
                }
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                this.label = label;
                this.value = value;
            }

            public string Label
            {
                get
                {
                    return this.label;
                }
            }

            public string Value
            {
                get
                {
                    return this.value;
                }
            }
        }
    }
}

