﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Globalization;
    using System.Runtime.InteropServices;

    internal class XnaDebuggableProjectCfg : DebuggableProjectCfg
    {
        private static uint debugEngineCmdUICookie;

        public XnaDebuggableProjectCfg(PlatformFlavorCfg parentCfg) : base(parentCfg)
        {
            this.ActivateDebugEngine();
        }

        private void ActivateDebugEngine()
        {
            if (debugEngineCmdUICookie == 0)
            {
                IVsMonitorSelection vsService = base.parentCfg.Project.GetVsService(typeof(SVsShellMonitorSelection)) as IVsMonitorSelection;
                if (vsService == null)
                {
                    throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.MissingService, new object[] { typeof(IVsMonitorSelection).FullName }));
                }
                Guid debugEngineGuid = DebugConstants.DebugEngineGuid;
                int cmdUIContextCookie = vsService.GetCmdUIContextCookie(ref debugEngineGuid, out debugEngineCmdUICookie);
                if (ErrorHandler.Succeeded(cmdUIContextCookie))
                {
                    ErrorHandler.ThrowOnFailure(vsService.SetCmdUIContext(debugEngineCmdUICookie, 1));
                }
                else
                {
                    IVsShell shell = base.parentCfg.Project.GetVsService(typeof(SVsShell)) as IVsShell;
                    if (shell != null)
                    {
                        object obj2;
                        ErrorHandler.ThrowOnFailure(shell.GetProperty(-9022, out obj2));
                        if ((obj2 is bool) && ((bool) obj2))
                        {
                            cmdUIContextCookie = 0;
                        }
                    }
                    ErrorHandler.ThrowOnFailure(cmdUIContextCookie);
                }
            }
        }

        protected override void GetDebugTargetInfo(ref VsDebugTargetInfo2 debugTarget, uint grfLaunch, Guid titleId, string args)
        {
            debugTarget.cbSize = (uint) Marshal.SizeOf((VsDebugTargetInfo2) debugTarget);
            debugTarget.dlo = 1;
            debugTarget.LaunchFlags = grfLaunch;
            IProjectPlatformAdapter adapter = base.parentCfg.Project.Adapter;
            debugTarget.guidPortSupplier = typeof(XnaPortSupplier).GUID;
            debugTarget.bstrPortName = XnaPortSupplier.CreatePortName(adapter.XnaPlatform, base.parentCfg.XnaFrameworkRuntimeProfile, base.parentCfg.TargetConsole);
            DebuggableProjectCfg.SetDebugEngine(ref debugTarget, DebugConstants.DebugEngineGuid);
            debugTarget.bstrExe = titleId.ToString("N", CultureInfo.InvariantCulture);
            debugTarget.bstrArg = args;
        }
    }
}

