﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Debugger.Interop;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Runtime.InteropServices;
    using System.Text.RegularExpressions;

    [Guid("2121371D-97BC-4138-A641-69FEA2764D27"), ComVisible(true), ClassInterface(ClassInterfaceType.None)]
    public class XnaPortSupplier : IDebugPortSupplier2, IDebugPortSupplierDescription2
    {
        private Dictionary<Guid, IDebugPort2> mapPortIdToPort = new Dictionary<Guid, IDebugPort2>();
        private Dictionary<string, IDebugPort2> mapPortNameToPort = new Dictionary<string, IDebugPort2>(StringComparer.Ordinal);

        public static string CreatePortName(string platform, string runtimeToken, string targetName)
        {
            if (platform == null)
            {
                throw new ArgumentNullException("platform");
            }
            if (runtimeToken == null)
            {
                throw new ArgumentNullException("runtimeToken");
            }
            if (targetName == null)
            {
                throw new ArgumentNullException("targetName");
            }
            if (platform.Contains(";"))
            {
                throw new ArgumentException();
            }
            if (runtimeToken.Contains(";"))
            {
                throw new ArgumentException();
            }
            return string.Format(CultureInfo.InvariantCulture, "[{0}];[{1}];[{2}]", new object[] { platform, runtimeToken, targetName });
        }

        int IDebugPortSupplier2.AddPort(IDebugPortRequest2 pRequest, out IDebugPort2 ppPort)
        {
            ppPort = null;
            if (pRequest == null)
            {
                throw new ArgumentNullException("pRequest");
            }
            string pbstrPortName = null;
            int portName = pRequest.GetPortName(out pbstrPortName);
            if (ErrorHandler.Failed(portName))
            {
                return portName;
            }
            if (this.mapPortNameToPort.ContainsKey(pbstrPortName))
            {
                ppPort = this.mapPortNameToPort[pbstrPortName];
                return 0;
            }
            IDebugPort2 port = new DebugPort(pRequest, this);
            Guid empty = Guid.Empty;
            ErrorHandler.ThrowOnFailure(port.GetPortId(out empty));
            this.mapPortNameToPort[pbstrPortName] = port;
            this.mapPortIdToPort[empty] = port;
            ppPort = port;
            return 0;
        }

        int IDebugPortSupplier2.CanAddPort()
        {
            return 0;
        }

        int IDebugPortSupplier2.EnumPorts(out IEnumDebugPorts2 ppEnum)
        {
            ppEnum = new EnumDebugPorts(this.mapPortIdToPort.Values);
            return 0;
        }

        int IDebugPortSupplier2.GetPort(ref Guid guidPort, out IDebugPort2 ppPort)
        {
            ppPort = null;
            if (this.mapPortIdToPort.TryGetValue(guidPort, out ppPort))
            {
                return 0;
            }
            return -2147221376;
        }

        int IDebugPortSupplier2.GetPortSupplierId(out Guid pguidPortSupplier)
        {
            pguidPortSupplier = base.GetType().GUID;
            return 0;
        }

        int IDebugPortSupplier2.GetPortSupplierName(out string pbstrName)
        {
            pbstrName = XnaProjectResource.DebugPortName;
            return 0;
        }

        int IDebugPortSupplier2.RemovePort(IDebugPort2 pPort)
        {
            Guid guid;
            string str;
            IDebugPort2 port;
            IDebugPort2 port2;
            if (pPort == null)
            {
                throw new ArgumentNullException("pPort");
            }
            int hr = 0;
            hr = pPort.GetPortId(out guid);
            if (ErrorHandler.Failed(hr))
            {
                return hr;
            }
            hr = pPort.GetPortName(out str);
            if (ErrorHandler.Failed(hr))
            {
                return hr;
            }
            if ((this.mapPortIdToPort.TryGetValue(guid, out port) && object.ReferenceEquals(pPort, port)) && (this.mapPortNameToPort.TryGetValue(str, out port2) && object.ReferenceEquals(pPort, port2)))
            {
                this.mapPortIdToPort.Remove(guid);
                this.mapPortNameToPort.Remove(str);
                return 0;
            }
            return -2147467259;
        }

        int IDebugPortSupplierDescription2.GetDescription(out uint pdwFlags, out string pbstrText)
        {
            pdwFlags = 0;
            pbstrText = XnaProjectResource.DebugPortDescription;
            return 0;
        }

        public static bool TryParsePortName(string portName, out string platform, out string runtimeToken, out string targetName)
        {
            if (portName == null)
            {
                throw new ArgumentNullException("portName");
            }
            platform = null;
            runtimeToken = null;
            targetName = null;
            Match match = Regex.Match(portName, @"\[(?<platform>[^;]+)\];\[(?<runtimeToken>[^;]+)\];\[(?<targetName>.+)\]", RegexOptions.CultureInvariant);
            bool success = match.Success;
            if (success)
            {
                platform = match.Groups["platform"].Value;
                runtimeToken = match.Groups["runtimeToken"].Value;
                targetName = match.Groups["targetName"].Value;
            }
            return success;
        }
    }
}

