﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using EnvDTE;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using System;
    using System.Collections.Generic;
    using System.IO;

    internal class DelayedRenameEventManager
    {
        private Dictionary<string, string> childToParentFileMap = new Dictionary<string, string>();
        private Dictionary<string, ProjectDocumentsRenamedEventArgs> origRenameEvents = new Dictionary<string, ProjectDocumentsRenamedEventArgs>();
        private Dictionary<string, List<string>> parentToChildrenFilesMap = new Dictionary<string, List<string>>();
        private List<ProjectDocumentsRenamedEventArgs> readyToFireEvents = new List<ProjectDocumentsRenamedEventArgs>();

        internal DelayedRenameEventManager()
        {
        }

        internal bool AddRenameEvent(ProjectItem projItem, ProjectDocumentsRenamedEventArgs renameEventArgs)
        {
            if (projItem == null)
            {
                throw new ArgumentNullException("projItem");
            }
            if (renameEventArgs == null)
            {
                throw new ArgumentNullException("renameEventArgs");
            }
            if ((projItem.ProjectItems.Count <= 0) || !File.Exists(projItem.get_FileNames(0)))
            {
                return false;
            }
            List<string> list = new List<string>();
            foreach (ProjectItem item in projItem.ProjectItems)
            {
                string str = item.get_FileNames(0);
                list.Add(str);
                this.childToParentFileMap.Add(str, renameEventArgs.MkDocumentOld);
            }
            this.parentToChildrenFilesMap.Add(renameEventArgs.MkDocumentOld, list);
            this.origRenameEvents.Add(renameEventArgs.MkDocumentOld, renameEventArgs);
            return true;
        }

        internal bool HandleFileRename(ProjectDocumentsRenamedEventArgs renameEventArgs)
        {
            if (renameEventArgs == null)
            {
                throw new ArgumentNullException("renameEventArgs");
            }
            string item = null;
            if (this.childToParentFileMap.ContainsKey(renameEventArgs.MkDocumentNew))
            {
                item = renameEventArgs.MkDocumentNew;
            }
            else if (this.childToParentFileMap.ContainsKey(renameEventArgs.MkDocumentOld))
            {
                item = renameEventArgs.MkDocumentOld;
            }
            if (item == null)
            {
                return false;
            }
            string key = this.childToParentFileMap[item];
            List<string> list = this.parentToChildrenFilesMap[key];
            list.Remove(item);
            if (list.Count == 0)
            {
                this.readyToFireEvents.Add(this.origRenameEvents[key]);
                this.parentToChildrenFilesMap.Remove(key);
                this.origRenameEvents.Remove(key);
            }
            this.childToParentFileMap.Remove(item);
            return true;
        }

        internal List<ProjectDocumentsRenamedEventArgs> ReadyEvents
        {
            get
            {
                return this.readyToFireEvents;
            }
        }
    }
}

