﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using EnvDTE;
    using EnvDTE80;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Windows.Forms;

    internal class GameStartupTypeSelector : UserControl, IServiceProvider
    {
        private Dictionary<string, bool> aggregateStatus = new Dictionary<string, bool>();
        private CodeModelEvents codeModelEvents;
        private IContainer components;
        private Project dteProject;
        private Events2 eventsContainer;
        private bool isDirty;
        private bool isRescanNeeded;
        private bool isUpdating;
        private static string MicrosoftXnaFrameworkGameClass = typeof(Game).FullName;
        private ComboBox pickerComboGameStartupType;
        private Label pickerLabel;
        private IXnaCodeProject project;
        private TableLayoutPanel tableLayoutPanel1;

        internal event EventHandler PropertyChanged;

        internal GameStartupTypeSelector(IXnaCodeProject project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            this.InitializeComponent();
            this.project = project;
            this.dteProject = ProjectUtilities.GetDteProject(this.project.InnerIVsHierarchy);
            this.eventsContainer = (Events2) this.dteProject.DTE.Events;
            this.codeModelEvents = this.eventsContainer.get_CodeModelEvents(null);
            this.codeModelEvents.add_ElementAdded(new _dispCodeModelEvents_ElementAddedEventHandler(this.OnCodeModelChanged));
            this.codeModelEvents.add_ElementChanged(new _dispCodeModelEvents_ElementChangedEventHandler(this.OnCodeModelChanged));
            this.codeModelEvents.add_ElementDeleted(new _dispCodeModelEvents_ElementDeletedEventHandler(this.OnCodeModelChanged));
            this.isRescanNeeded = true;
            this.RefreshList();
            this.pickerComboGameStartupType.SelectedIndexChanged += new EventHandler(this.OnSelectedIndexChanged);
            this.pickerComboGameStartupType.DropDown += new EventHandler(this.OnDropdown);
        }

        private void ClearCandidacy()
        {
            this.aggregateStatus.Clear();
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void InitializeComponent()
        {
            ComponentResourceManager manager = new ComponentResourceManager(typeof(GameStartupTypeSelector));
            this.tableLayoutPanel1 = new TableLayoutPanel();
            this.pickerComboGameStartupType = new ComboBox();
            this.pickerLabel = new Label();
            this.tableLayoutPanel1.SuspendLayout();
            base.SuspendLayout();
            manager.ApplyResources(this.tableLayoutPanel1, "tableLayoutPanel1");
            this.tableLayoutPanel1.Controls.Add(this.pickerComboGameStartupType, 0, 1);
            this.tableLayoutPanel1.Controls.Add(this.pickerLabel, 0, 0);
            this.tableLayoutPanel1.Name = "tableLayoutPanel1";
            this.pickerComboGameStartupType.AutoCompleteMode = AutoCompleteMode.Append;
            this.pickerComboGameStartupType.AutoCompleteSource = AutoCompleteSource.ListItems;
            manager.ApplyResources(this.pickerComboGameStartupType, "pickerComboGameStartupType");
            this.pickerComboGameStartupType.DropDownStyle = ComboBoxStyle.DropDownList;
            this.pickerComboGameStartupType.FormattingEnabled = true;
            this.pickerComboGameStartupType.Name = "pickerComboGameStartupType";
            manager.ApplyResources(this.pickerLabel, "pickerLabel");
            this.pickerLabel.Name = "pickerLabel";
            manager.ApplyResources(this, "$this");
            base.AutoScaleMode = AutoScaleMode.Font;
            base.Controls.Add(this.tableLayoutPanel1);
            base.Name = "GameStartupTypeSelector";
            this.tableLayoutPanel1.ResumeLayout(false);
            this.tableLayoutPanel1.PerformLayout();
            base.ResumeLayout(false);
            base.PerformLayout();
        }

        internal void OnCodeModelChanged(CodeElement element)
        {
            this.isRescanNeeded = true;
        }

        internal void OnCodeModelChanged(CodeElement element, vsCMChangeKind change)
        {
            this.isRescanNeeded = true;
        }

        internal void OnCodeModelChanged(object Parent, CodeElement Element)
        {
            this.isRescanNeeded = true;
        }

        internal void OnDropdown(object sender, EventArgs e)
        {
            if (this.isRescanNeeded)
            {
                this.RefreshList();
            }
        }

        protected override void OnFontChanged(EventArgs e)
        {
            base.OnFontChanged(e);
            if (this.pickerComboGameStartupType != null)
            {
                ComboBoxUtils.SetComboBoxDropdownWidth(this.pickerComboGameStartupType);
            }
        }

        private void OnSelectedIndexChanged(object sender, EventArgs args)
        {
            if (!this.isUpdating)
            {
                this.Dirty = true;
            }
        }

        private List<string> ProduceCandidateList()
        {
            List<string> list = new List<string>();
            foreach (string str in this.aggregateStatus.Keys)
            {
                if (this.aggregateStatus[str] && !list.Contains(str))
                {
                    list.Add(str);
                }
            }
            list.Sort();
            return list;
        }

        public void ProjectChanged()
        {
            this.isRescanNeeded = true;
        }

        internal void RefreshList()
        {
            bool isUpdating = this.isUpdating;
            this.isUpdating = true;
            try
            {
                string text = this.pickerComboGameStartupType.Text;
                if (string.IsNullOrEmpty(text))
                {
                    text = XnaProjectResource.DefaultGameStartupType;
                }
                this.pickerComboGameStartupType.Items.Clear();
                this.pickerComboGameStartupType.Items.Add(XnaProjectResource.DefaultGameStartupType);
                this.ClearCandidacy();
                int count = this.dteProject.ProjectItems.Count;
                for (int i = 1; i <= count; i++)
                {
                    FileCodeModel fileCodeModel = this.dteProject.ProjectItems.Item(i).FileCodeModel;
                    FileCodeModel2 model2 = (FileCodeModel2) fileCodeModel;
                    if ((model2 != null) && (model2.ParseStatus != vsCMParseStatus.vsCMParseStatusComplete))
                    {
                        model2.Synchronize();
                    }
                    if ((fileCodeModel != null) && (fileCodeModel.CodeElements != null))
                    {
                        this.ScanFileCodeModel(fileCodeModel);
                    }
                }
                this.pickerComboGameStartupType.Items.AddRange(this.ProduceCandidateList().ToArray());
                this.pickerComboGameStartupType.Text = text;
                ComboBoxUtils.SetComboBoxDropdownWidth(this.pickerComboGameStartupType);
                this.isRescanNeeded = false;
            }
            finally
            {
                this.isUpdating = isUpdating;
            }
            this.Text = this.pickerComboGameStartupType.Text;
        }

        private void RegisterCandidacy(bool isCandidate, string fullName)
        {
            if (!this.aggregateStatus.ContainsKey(fullName))
            {
                this.aggregateStatus.Add(fullName, isCandidate);
            }
            else if (!isCandidate)
            {
                this.aggregateStatus[fullName] = false;
            }
        }

        internal void ReleaseEvents()
        {
            if (this.codeModelEvents != null)
            {
                this.codeModelEvents.remove_ElementAdded(new _dispCodeModelEvents_ElementAddedEventHandler(this.OnCodeModelChanged));
                this.codeModelEvents.remove_ElementChanged(new _dispCodeModelEvents_ElementChangedEventHandler(this.OnCodeModelChanged));
                this.codeModelEvents.remove_ElementDeleted(new _dispCodeModelEvents_ElementDeletedEventHandler(this.OnCodeModelChanged));
                this.codeModelEvents = null;
            }
            if (this.eventsContainer != null)
            {
                this.eventsContainer = null;
            }
            if (this.dteProject != null)
            {
                this.dteProject = null;
            }
        }

        private void ScanFileCodeModel(FileCodeModel target)
        {
            if ((target != null) && (target.CodeElements != null))
            {
                this.TraverseElts(target.CodeElements);
            }
        }

        object IServiceProvider.GetService(System.Type serviceType)
        {
            return this.project.GetVsService(serviceType);
        }

        private void TraverseElts(CodeElements elts)
        {
            if (elts == null)
            {
                throw new ArgumentNullException("elts");
            }
            for (int i = 1; i <= elts.Count; i++)
            {
                CodeElement targetElement = elts.Item(i);
                if (targetElement.Kind == vsCMElement.vsCMElementNamespace)
                {
                    CodeNamespace namespace2 = (CodeNamespace) targetElement;
                    this.TraverseElts(namespace2.Members);
                }
                else if (targetElement.Kind == vsCMElement.vsCMElementClass)
                {
                    this.RegisterCandidacy(this.ValidateClass(targetElement), targetElement.FullName);
                }
            }
        }

        private bool ValidateClass(CodeElement targetElement)
        {
            if ((((targetElement != null) && !string.IsNullOrEmpty(targetElement.FullName)) && (targetElement.IsCodeType && (targetElement.Kind == vsCMElement.vsCMElementClass))) && (string.Compare(targetElement.FullName, MicrosoftXnaFrameworkGameClass, StringComparison.InvariantCulture) != 0))
            {
                CodeClass class2 = (CodeClass) targetElement;
                if ((!class2.IsAbstract && (class2.Access != vsCMAccess.vsCMAccessPrivate)) && ((class2.Access != vsCMAccess.vsCMAccessAssemblyOrFamily) && class2.get_IsDerivedFrom(MicrosoftXnaFrameworkGameClass)))
                {
                    bool flag = false;
                    int count = class2.Members.Count;
                    for (int i = 1; i <= count; i++)
                    {
                        CodeElement element = class2.Members.Item(i);
                        if (element.Kind == vsCMElement.vsCMElementFunction)
                        {
                            CodeFunction function = (CodeFunction) element;
                            if (function.FunctionKind == vsCMFunction.vsCMFunctionConstructor)
                            {
                                flag = true;
                                if (function.Parameters.Count == 0)
                                {
                                    if (function.Access != vsCMAccess.vsCMAccessPublic)
                                    {
                                        return false;
                                    }
                                    return true;
                                }
                            }
                        }
                    }
                    return !flag;
                }
            }
            return false;
        }

        internal bool Dirty
        {
            get
            {
                return this.isDirty;
            }
            set
            {
                this.isDirty = value;
                if (value)
                {
                    this.propertyChangedEvent(this, null);
                }
            }
        }

        public override string Text
        {
            get
            {
                string text = this.pickerComboGameStartupType.Text;
                if (text == XnaProjectResource.DefaultGameStartupType)
                {
                    text = string.Empty;
                }
                return text;
            }
            set
            {
                bool isUpdating = this.isUpdating;
                this.isUpdating = true;
                try
                {
                    if (string.IsNullOrEmpty(value))
                    {
                        value = XnaProjectResource.DefaultGameStartupType;
                    }
                    this.pickerComboGameStartupType.Text = value;
                    if (!this.pickerComboGameStartupType.Text.Equals(value))
                    {
                        this.pickerComboGameStartupType.Text = XnaProjectResource.DefaultGameStartupType;
                        this.Dirty = true;
                    }
                }
                finally
                {
                    this.isUpdating = isUpdating;
                }
            }
        }
    }
}

