﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using System;
    using System.ComponentModel;
    using System.Windows.Forms;

    internal class GraphicsProfileSelector : FlowLayoutPanel, IServiceProvider
    {
        private IContainer components;
        private bool isDirty;
        private bool isUpdating;
        private Label pickerLabel;
        private IXnaCodeProject project;
        private RadioButton radioButtonHiDef;
        private RadioButton radioButtonReach;
        private string setting;

        internal event EventHandler PropertyChanged;

        internal GraphicsProfileSelector(IXnaCodeProject project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            this.InitializeComponent();
            this.project = project;
            this.radioButtonHiDef.CheckedChanged += new EventHandler(this.OnHiDefChanged);
            this.radioButtonReach.CheckedChanged += new EventHandler(this.OnReachChanged);
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void InitializeComponent()
        {
            ComponentResourceManager manager = new ComponentResourceManager(typeof(GraphicsProfileSelector));
            this.radioButtonHiDef = new RadioButton();
            this.radioButtonReach = new RadioButton();
            this.pickerLabel = new Label();
            base.SuspendLayout();
            manager.ApplyResources(this.radioButtonHiDef, "radioButtonHiDef");
            base.SetFlowBreak(this.radioButtonHiDef, true);
            this.radioButtonHiDef.Name = "radioButtonHiDef";
            this.radioButtonHiDef.TabStop = true;
            this.radioButtonHiDef.UseVisualStyleBackColor = true;
            manager.ApplyResources(this.radioButtonReach, "radioButtonReach");
            this.radioButtonReach.Checked = true;
            base.SetFlowBreak(this.radioButtonReach, true);
            this.radioButtonReach.Name = "radioButtonReach";
            this.radioButtonReach.TabStop = true;
            this.radioButtonReach.UseVisualStyleBackColor = true;
            manager.ApplyResources(this.pickerLabel, "pickerLabel");
            base.SetFlowBreak(this.pickerLabel, true);
            this.pickerLabel.Name = "pickerLabel";
            manager.ApplyResources(this, "$this");
            base.Controls.Add(this.pickerLabel);
            base.Controls.Add(this.radioButtonReach);
            base.Controls.Add(this.radioButtonHiDef);
            base.ResumeLayout(false);
            base.PerformLayout();
        }

        internal void OnHiDefChanged(object sender, EventArgs args)
        {
            if (!this.isUpdating)
            {
                RadioButton button = sender as RadioButton;
                if (button.Checked)
                {
                    this.setting = "HiDef";
                    this.Dirty = true;
                }
            }
        }

        internal void OnReachChanged(object sender, EventArgs args)
        {
            if (!this.isUpdating)
            {
                RadioButton button = sender as RadioButton;
                if (button.Checked)
                {
                    this.setting = "Reach";
                    this.Dirty = true;
                }
            }
        }

        object IServiceProvider.GetService(System.Type serviceType)
        {
            return this.project.GetVsService(serviceType);
        }

        internal bool Dirty
        {
            get
            {
                return this.isDirty;
            }
            set
            {
                this.isDirty = value;
                if (value)
                {
                    this.propertyChangedEvent(this, null);
                }
            }
        }

        public override string Text
        {
            get
            {
                return this.setting;
            }
            set
            {
                bool isUpdating = this.isUpdating;
                this.isUpdating = true;
                try
                {
                    string strA = value;
                    if (string.IsNullOrEmpty(value))
                    {
                        strA = "Reach";
                    }
                    if (string.CompareOrdinal(strA, "HiDef") == 0)
                    {
                        this.radioButtonHiDef.Checked = true;
                        this.radioButtonReach.Checked = false;
                    }
                    else
                    {
                        this.radioButtonHiDef.Checked = false;
                        this.radioButtonReach.Checked = true;
                    }
                    this.setting = strA;
                }
                finally
                {
                    this.isUpdating = isUpdating;
                }
            }
        }
    }
}

