﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.CodeProject.Debugging;
    using System;
    using System.IO;

    internal class PlatformFlavorCfg : ProjectFlavorCfg, IVsUpdateSolutionEvents2, IVsUpdateSolutionEvents
    {
        private uint buildEvents2Cookie;
        private IVsDebuggableProjectCfg2 debuggableProjectCfg;
        private IVsDeployableProjectCfg deployableProjectCfg;
        private Microsoft.Xna.GameStudio.CodeProject.Debugging.DeployManager deployManager;
        private IVsSolutionBuildManager2 solutionBuildManager2;
        private IXnaCodeProject xnaCodeProject;

        internal PlatformFlavorCfg(IXnaCodeProject project, IVsCfg baseCfg, IVsProjectFlavorCfg innerCfg, string deploymentF1HelpKeyword) : base(baseCfg, innerCfg, project.XnaFrameworkVersion)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            this.xnaCodeProject = project;
            this.debuggableProjectCfg = this.CreateDebuggableProjectCfg();
            this.deployableProjectCfg = this.CreateDeployableProjectCfg(deploymentF1HelpKeyword);
            base.AddCfgType(typeof(IVsDebuggableProjectCfg2), this.debuggableProjectCfg);
            base.AddCfgType(typeof(IVsDebuggableProjectCfg), this.debuggableProjectCfg);
            base.AddCfgType(typeof(IVsDeployableProjectCfg), this.deployableProjectCfg);
            this.solutionBuildManager2 = Microsoft.Xna.GameStudio.ProjectUtilities.GetRequiredService<IVsSolutionBuildManager2>((IServiceProvider) this.xnaCodeProject, typeof(SVsSolutionBuildManager));
            ErrorHandler.ThrowOnFailure(this.solutionBuildManager2.AdviseUpdateSolutionEvents(this, out this.buildEvents2Cookie));
        }

        public override int Close()
        {
            if (this.debuggableProjectCfg != null)
            {
                IDisposable debuggableProjectCfg = this.debuggableProjectCfg as IDisposable;
                this.debuggableProjectCfg = null;
                base.RemoveCfgTypeObject(typeof(IVsDebuggableProjectCfg2));
                base.RemoveCfgTypeObject(typeof(IVsDebuggableProjectCfg));
                if (debuggableProjectCfg != null)
                {
                    debuggableProjectCfg.Dispose();
                }
            }
            if (this.deployableProjectCfg != null)
            {
                this.deployableProjectCfg = null;
                base.RemoveCfgTypeObject(typeof(IVsDeployableProjectCfg));
            }
            if ((this.buildEvents2Cookie != 0) && (this.solutionBuildManager2 != null))
            {
                this.solutionBuildManager2.UnadviseUpdateSolutionEvents(this.buildEvents2Cookie);
            }
            this.solutionBuildManager2 = null;
            this.buildEvents2Cookie = 0;
            return base.Close();
        }

        internal virtual IVsDebuggableProjectCfg2 CreateDebuggableProjectCfg()
        {
            return new XnaDebuggableProjectCfg(this);
        }

        internal virtual IVsDeployableProjectCfg CreateDeployableProjectCfg(string deploymentF1HelpKeyword)
        {
            return new XnaDeployableProjectCfg(this, deploymentF1HelpKeyword);
        }

        public int OnActiveProjectCfgChange(IVsHierarchy pIVsHierarchy)
        {
            return 0;
        }

        internal void OnBuildActionClean()
        {
            DirectoryInfo info = new DirectoryInfo(Microsoft.Xna.GameStudio.CodeProject.Debugging.DeployManager.GetDeployCacheDirectory(this.Project));
            if (info.Exists)
            {
                string str2;
                ErrorHandler.ThrowOnFailure(this.Project.InnerVsProject.GetMkDocument(0xfffffffe, out str2));
                string searchPattern = Path.GetFileName(str2) + ".*.DeployCache.txt";
                foreach (FileInfo info2 in info.GetFiles(searchPattern))
                {
                    try
                    {
                        if (!info2.IsReadOnly)
                        {
                            info2.Delete();
                        }
                    }
                    catch (IOException)
                    {
                    }
                    catch (UnauthorizedAccessException)
                    {
                    }
                }
            }
        }

        public int UpdateProjectCfg_Begin(IVsHierarchy pHierProj, IVsCfg pCfgProj, IVsCfg pCfgSln, uint dwAction, ref int pfCancel)
        {
            if (((ComUtilities.IsSameComObject(pHierProj, this.Project.GetVsHierarchy()) && (pCfgProj != null)) && (ComUtilities.IsSameComObject(pCfgProj, base.BaseCfg) && (pfCancel == 0))) && (dwAction == 0x100000))
            {
                this.OnBuildActionClean();
            }
            return 0;
        }

        public int UpdateProjectCfg_Done(IVsHierarchy pHierProj, IVsCfg pCfgProj, IVsCfg pCfgSln, uint dwAction, int fSuccess, int fCancel)
        {
            return 0;
        }

        public int UpdateSolution_Begin(ref int pfCancelUpdate)
        {
            return 0;
        }

        public int UpdateSolution_Cancel()
        {
            return 0;
        }

        public int UpdateSolution_Done(int fSucceeded, int fModified, int fCancelCommand)
        {
            return 0;
        }

        public int UpdateSolution_StartUpdate(ref int pfCancelUpdate)
        {
            return 0;
        }

        internal Microsoft.Xna.GameStudio.CodeProject.Debugging.DeployManager DeployManager
        {
            get
            {
                if (this.deployManager == null)
                {
                    this.deployManager = new Microsoft.Xna.GameStudio.CodeProject.Debugging.DeployManager(this.xnaCodeProject);
                }
                return this.deployManager;
            }
        }

        internal IXnaCodeProject Project
        {
            get
            {
                return this.xnaCodeProject;
            }
        }

        internal string TargetConsole
        {
            get
            {
                IXnaDeviceDeploymentRegistration vsService = this.Project.GetVsService(typeof(IXnaDeviceDeploymentRegistration)) as IXnaDeviceDeploymentRegistration;
                if (vsService != null)
                {
                    return vsService.GetDeployDevice(this.xnaCodeProject.Adapter.XnaPlatform, base.XnaFrameworkRuntimeProfile);
                }
                return null;
            }
        }
    }
}

