﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using System;
    using System.ComponentModel;
    using System.Windows.Forms;

    internal class TombstoneEnable : UserControl, IServiceProvider
    {
        private CheckBox checkboxTombstoneEnable;
        private IContainer components;
        private bool isDirty;
        private bool isUpdating;
        private IXnaCodeProject project;
        private TableLayoutPanel tableLayoutPanel1;

        internal event EventHandler PropertyChanged;

        internal TombstoneEnable(IXnaCodeProject project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            this.InitializeComponent();
            this.project = project;
            this.RefreshList();
            this.checkboxTombstoneEnable.CheckedChanged += new EventHandler(this.OnTombstoneCheckChanged);
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void InitializeComponent()
        {
            ComponentResourceManager manager = new ComponentResourceManager(typeof(TombstoneEnable));
            this.tableLayoutPanel1 = new TableLayoutPanel();
            this.checkboxTombstoneEnable = new CheckBox();
            this.tableLayoutPanel1.SuspendLayout();
            base.SuspendLayout();
            manager.ApplyResources(this.tableLayoutPanel1, "tableLayoutPanel1");
            this.tableLayoutPanel1.Controls.Add(this.checkboxTombstoneEnable, 0, 0);
            this.tableLayoutPanel1.Name = "tableLayoutPanel1";
            manager.ApplyResources(this.checkboxTombstoneEnable, "checkboxTombstoneEnable");
            this.checkboxTombstoneEnable.Name = "checkboxTombstoneEnable";
            manager.ApplyResources(this, "$this");
            base.AutoScaleMode = AutoScaleMode.Font;
            base.Controls.Add(this.tableLayoutPanel1);
            base.Name = "TombstoneEnable";
            this.tableLayoutPanel1.ResumeLayout(false);
            base.ResumeLayout(false);
            base.PerformLayout();
        }

        private void OnTombstoneCheckChanged(object sender, EventArgs args)
        {
            if (!this.isUpdating)
            {
                this.Dirty = true;
            }
        }

        internal void RefreshList()
        {
            bool isUpdating = this.isUpdating;
            this.isUpdating = true;
            try
            {
                string buildProperty = this.project.GetBuildProperty("WPTombstoneEnabled", null);
                bool result = false;
                bool.TryParse(buildProperty, out result);
                this.checkboxTombstoneEnable.Checked = result;
            }
            finally
            {
                this.isUpdating = isUpdating;
            }
        }

        object IServiceProvider.GetService(System.Type serviceType)
        {
            return this.project.GetVsService(serviceType);
        }

        internal bool Dirty
        {
            get
            {
                return this.isDirty;
            }
            set
            {
                this.isDirty = value;
                if (value)
                {
                    this.propertyChangedEvent(this, null);
                }
            }
        }

        public override string Text
        {
            get
            {
                return this.checkboxTombstoneEnable.Checked.ToString();
            }
            set
            {
                bool isUpdating = this.isUpdating;
                this.isUpdating = true;
                try
                {
                    bool result = false;
                    if (!string.IsNullOrEmpty(value) && !bool.TryParse(value, out result))
                    {
                        result = false;
                    }
                    this.checkboxTombstoneEnable.Checked = result;
                }
                finally
                {
                    this.isUpdating = isUpdating;
                }
            }
        }
    }
}

