﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.Build.Construction;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Win32;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Projects.LanguageAdapters;
    using Microsoft.Xna.GameStudio.RegistrationAttributes;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.IO;
    using System.Linq;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using System.Threading;
    using VSLangProj;

    [Guid("F1BF3526-21A3-44F3-BEEE-54AAC3A3264E"), ProjectLanguageAdapter("F184B08F-C81C-45F6-A57F-5ABD9991F28F", typeof(VisualBasicLanguageAdapter), typeof(XnaPackage))]
    internal sealed class VisualBasicLanguageAdapter : BaseProjectLanguageAdapter, IVbCompilerHost
    {
        public override string GetCapability(string capabilityName)
        {
            if (!string.IsNullOrWhiteSpace(capabilityName) && capabilityName.Equals("defaultAppSettings"))
            {
                return bool.TrueString;
            }
            return null;
        }

        private IEnumerable<uint> GetChildrenOfNode(uint itemid, bool visibleOnly = true)
        {
            __VSHPROPID iteratorVariable0 = visibleOnly ? __VSHPROPID.VSHPROPID_FirstVisibleChild : __VSHPROPID.VSHPROPID_FirstChild;
            __VSHPROPID iteratorVariable1 = visibleOnly ? __VSHPROPID.VSHPROPID_NextVisibleSibling : __VSHPROPID.VSHPROPID_NextSibling;
            object pvar = null;
            ErrorHandler.ThrowOnFailure(this.innerHierarchy.GetProperty(itemid, (int) iteratorVariable0, out pvar));
            itemid = (uint) ((int) pvar);
            while (true)
            {
                if (itemid == uint.MaxValue)
                {
                    yield break;
                }
                yield return itemid;
                ErrorHandler.ThrowOnFailure(this.innerHierarchy.GetProperty(itemid, (int) iteratorVariable1, out pvar));
                itemid = (uint) ((int) pvar);
            }
        }

        public override Guid GetItemTemplatesGuid(Guid guid)
        {
            if (guid.Equals(typeof(WindowsPlatformAdapter).GUID))
            {
                return typeof(WindowsVisualBasicTemplatesRegistration).GUID;
            }
            if (guid.Equals(typeof(WindowsPhonePlatformAdapter).GUID))
            {
                return typeof(WindowsPhoneVisualBasicTemplatesRegistration).GUID;
            }
            if (guid.Equals(typeof(Xbox360PlatformAdapter).GUID))
            {
                return typeof(XboxVisualBasicTemplatesRegistration).GUID;
            }
            return guid;
        }

        public override List<string> GetLanguageCaps()
        {
            return new List<string> { "defaultAppSettings" };
        }

        public override IEnumerable<uint> GetReferenceNodes()
        {
            return this.GetChildrenOfNode(this.GetReferencesFolderNode(), false);
        }

        public override uint GetReferencesFolderNode()
        {
            foreach (uint num in this.GetChildrenOfNode(0xfffffffe, false))
            {
                object obj2;
                object obj3;
                Guid empty = Guid.Empty;
                ErrorHandler.ThrowOnFailure(base.innerHierarchy.GetGuidProperty(num, -1004, out empty));
                if (((empty == VSConstants.GUID_ItemType_VirtualFolder) && (ErrorHandler.Failed(base.innerHierarchy.GetProperty(num, -2062, out obj2)) || !((bool) obj2))) && ErrorHandler.Succeeded(base.innerHierarchy.GetProperty(num, -1001, out obj3)))
                {
                    for (uint i = (uint) ((int) obj3); i != uint.MaxValue; i = (uint) ((int) obj3))
                    {
                        if (this.IsReferenceItem(i))
                        {
                            return num;
                        }
                        if (ErrorHandler.Failed(base.innerHierarchy.GetProperty(i, -1002, out obj3)))
                        {
                            break;
                        }
                    }
                }
            }
            return uint.MaxValue;
        }

        public int GetSdkPath(out string sdkPath)
        {
            IXnaCodeProject serviceProvider = base.serviceProvider as IXnaCodeProject;
            string str = null;
            bool flag = this.GetServicePackLevel().Equals("0");
            string[] strArray = (serviceProvider != null) ? serviceProvider.SdkPaths : null;
            sdkPath = null;
            if (strArray != null)
            {
                if (CS$<>9__CachedAnonymousMethodDelegate11 == null)
                {
                    CS$<>9__CachedAnonymousMethodDelegate11 = new Func<string, bool>(null, (IntPtr) <GetSdkPath>b__f);
                }
                sdkPath = Enumerable.FirstOrDefault<string>(strArray, CS$<>9__CachedAnonymousMethodDelegate11);
                if (flag)
                {
                    if (CS$<>9__CachedAnonymousMethodDelegate12 == null)
                    {
                        CS$<>9__CachedAnonymousMethodDelegate12 = new Func<string, bool>(null, (IntPtr) <GetSdkPath>b__10);
                    }
                    str = Enumerable.FirstOrDefault<string>(strArray, CS$<>9__CachedAnonymousMethodDelegate12);
                }
            }
            if ((sdkPath != null) && (!flag || (str != null)))
            {
                return 0;
            }
            int num = -2147467259;
            sdkPath = string.Empty;
            Microsoft.VisualStudio.Shell.Interop.IVsFrameworkMultiTargeting service = base.serviceProvider.GetService(typeof(SVsFrameworkMultiTargeting)) as Microsoft.VisualStudio.Shell.Interop.IVsFrameworkMultiTargeting;
            if (service != null)
            {
                string pbstrResolvedAssemblyPath = string.Empty;
                num = service.ResolveAssemblyPath("mscorlib.dll", this.TargetFrameworkMoniker, out pbstrResolvedAssemblyPath);
                try
                {
                    sdkPath = Path.GetDirectoryName(pbstrResolvedAssemblyPath);
                }
                catch (Exception)
                {
                }
            }
            return num;
        }

        private string GetServicePackLevel()
        {
            string str = string.Empty;
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\DevDiv\VS\Servicing\10.0");
            if (key != null)
            {
                try
                {
                    str = ((int) key.GetValue("SP")).ToString();
                }
                catch
                {
                }
            }
            return str;
        }

        public int GetTargetLibraryType(out VBTargetLibraryType targetLibraryType)
        {
            if (base.adapter.XnaPlatform != "Windows")
            {
                targetLibraryType = VBTargetLibraryType.TLB_Starlite;
            }
            else
            {
                targetLibraryType = VBTargetLibraryType.TLB_Desktop;
            }
            return 0;
        }

        public override IEnumerable<uint> GetVirtualFolders()
        {
            foreach (uint iteratorVariable0 in this.GetChildrenOfNode(0xfffffffe, true))
            {
                Guid empty = Guid.Empty;
                ErrorHandler.ThrowOnFailure(this.innerHierarchy.GetGuidProperty(iteratorVariable0, -1004, out empty));
                if (empty == VSConstants.GUID_ItemType_VirtualFolder)
                {
                    yield return iteratorVariable0;
                }
            }
        }

        private bool IsReferenceItem(uint itemid)
        {
            object obj2;
            if (ErrorHandler.Succeeded(base.innerHierarchy.GetProperty(itemid, -2027, out obj2)))
            {
                Reference reference = obj2 as Reference;
                return (reference != null);
            }
            return false;
        }

        public override void ModifyProjectOnCopy(ProjectRootElement projectRoot, IProjectPlatformAdapter targetAdapter)
        {
            if (targetAdapter.IsDevicePlatform)
            {
                projectRoot.AddProperty("TargetCompactFramework", "true");
            }
            else
            {
                projectRoot.AddProperty("TargetCompactFramework", "false");
            }
            base.ModifyProjectOnCopy(projectRoot, targetAdapter);
        }

        public int OutputString(string strTxt)
        {
            return 0;
        }

        public override bool QueryService(ref Guid guidService, ref Guid riid, out object service)
        {
            return base.QueryService(ref guidService, ref riid, out service);
        }

        public override string TranslateProjectDefines(string defines)
        {
            string[] strArray = defines.Split(new char[] { ';' });
            if (CS$<>9__CachedAnonymousMethodDelegate8 == null)
            {
                CS$<>9__CachedAnonymousMethodDelegate8 = new Func<string, bool>(null, (IntPtr) <TranslateProjectDefines>b__7);
            }
            return string.Join(",", Enumerable.Where<string>(strArray, CS$<>9__CachedAnonymousMethodDelegate8));
        }

        public override Guid LanguageGuid
        {
            get
            {
                return new Guid("F184B08F-C81C-45F6-A57F-5ABD9991F28F");
            }
        }

        public override string Name
        {
            get
            {
                return "VisualBasic";
            }
        }

        public override string ProjectBrowseObjectGuidString
        {
            get
            {
                return "{E0FDC879-C32A-4751-A3D3-0B3824BD575F}";
            }
        }

        public override string ProjectConfigBrowseObjectGuidString
        {
            get
            {
                return "{67F8DD11-14EB-489b-87F0-F01C52AF3870}";
            }
        }

        public override IPropertyPageProvider PropertyPageHandler
        {
            get
            {
                return new VisualBasicPropertyPageHandler();
            }
        }

        public string TargetFrameworkMoniker
        {
            get
            {
                object obj2;
                ErrorHandler.ThrowOnFailure(base.innerHierarchy.GetProperty(0xfffffffe, -2102, out obj2));
                return (string) obj2;
            }
        }


    }
}

