﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.Build.Construction;
    using Microsoft.Build.Evaluation;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.RegistrationAttributes;
    using Microsoft.Xna.GameStudio.Resources;
    using Microsoft.Xna.GameStudio.WindowsPhone;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Diagnostics;
    using System.Drawing;
    using System.IO;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using System.Threading;

    [ProjectFileFilter(typeof(WindowsPhonePlatformAdapter), "Executable Files", 0x79, 0x5dc), ProjectType(typeof(WindowsPhonePlatformAdapter), typeof(XnaPackage), null, null, null, null, null, LanguageVsTemplate="CSharp", ShowOnlySpecifiedTemplatesVsTemplate=true, TemplateGroupIDsVsTemplate="XnaFramework-v4.0,XnaFramework-v4.0-WindowsPhone,XnaFramework-v4.0-Devices", TemplateIDsVsTemplate="Microsoft.CSharp.CodeFile,Microsoft.CSharp.Resource,Microsoft.CSharp.TextFile,Microsoft.CSharp.XmlFile,Microsoft.CSharp.ClassDiagram,Microsoft.CSharp.XMLSchema"), ProjectFileFilter(typeof(WindowsPhonePlatformAdapter), "Audio Files", 0x77, 0x514), ProjectFileFilter(typeof(WindowsPhonePlatformAdapter), "Common Files", 0x7b, 100, CommonFindFilesFilter=true, NotOpenFileFilter=true, NotAddExistingItemFilter=true), ProjectFileFilter(typeof(WindowsPhonePlatformAdapter), "Data Files", 0x75, 0x44c), Guid("A41D24BA-B416-43a8-A6CF-BC925916EF94"), ProjectFileFilter(typeof(WindowsPhonePlatformAdapter), "Image Files", 0x76, 0x4b0), ProjectFileFilter(typeof(WindowsPhonePlatformAdapter), "Script Files", 0x7a, 0x640), ProjectFileFilter(typeof(WindowsPhonePlatformAdapter), "Video Files", 120, 0x578), ProjectFileFilter(typeof(WindowsPhonePlatformAdapter), "Visual C# Files", 0x74, 0x3e8), ProjectPlatformAdapter(typeof(WindowsPhonePlatformAdapter), typeof(XnaPackage))]
    public sealed class WindowsPhonePlatformAdapter : BasePlatformAdapter, IProjectPlatformAdapterCaps, IXnaCreationOptionsFilter, IXnaRefreshLevel, IProjectSkuSupport
    {
        private IList<INamedExtenderProvider> extenderProviders;
        private Microsoft.Xna.GameStudio.CodeProject.HelpContextFilters helpContextFilters;

        public WindowsPhonePlatformAdapter()
        {
            List<INamedExtenderProvider> list = new List<INamedExtenderProvider> {
                new WindowsPhoneProjectPropertiesExtenderProvider()
            };
            this.extenderProviders = new ReadOnlyCollection<INamedExtenderProvider>(list);
        }

        [CompilerGenerated]
        private IEnumerable<string> <>n__FabricatedMethod4()
        {
            return base.FilteredPropertyPages();
        }

        private void AddRefreshLevelSelection(XnaProjectCreationOption creationOption, bool useMaximum)
        {
            if (!useMaximum)
            {
                creationOption.Selections.Add(new XnaProjectCreationOption.Selection(XnaProjectResource.WPXnaRefreshLevelZero, "0", XnaProjectResource.WPXnaRefreshLevelZeroDescription, false));
            }
            else
            {
                creationOption.Selections.Add(new XnaProjectCreationOption.Selection(XnaProjectResource.WPXnaRefreshLevelOne, "1", XnaProjectResource.WPXnaRefreshLevelOneDescription, true));
            }
        }

        public bool CanDowngradeRefreshLevel(string targetRefreshLevel)
        {
            return false;
        }

        public bool CanUpgradeRefreshLevel(string sourceRefreshLevel)
        {
            if (!"0".Equals(sourceRefreshLevel, StringComparison.OrdinalIgnoreCase) && !string.IsNullOrEmpty(sourceRefreshLevel))
            {
                return false;
            }
            return true;
        }

        public override IVsProjectFlavorCfg CreateProjectFlavorCfg(IServiceProvider serviceProvider, IVsCfg baseProjectCfg, IVsProjectFlavorCfg innerFlavorCfg)
        {
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            if (baseProjectCfg == null)
            {
                throw new ArgumentNullException("baseProjectCfg");
            }
            IXnaCodeProject requiredService = Microsoft.Xna.GameStudio.ProjectUtilities.GetRequiredService<IXnaCodeProject>(serviceProvider);
            if (requiredService != null)
            {
                return new WindowsPhoneFlavorCfg(requiredService, baseProjectCfg, innerFlavorCfg, "Microsoft.Xna.VisualStudio.DeployingWindowsPhoneGame");
            }
            return innerFlavorCfg;
        }

        public bool DowngradeRefreshLevel(IVsHierarchy hierarchy)
        {
            throw new NotImplementedException();
        }

        public override IEnumerable<string> FilteredPropertyPages()
        {
            foreach (string iteratorVariable0 in this.<>n__FabricatedMethod4())
            {
                yield return iteratorVariable0;
            }
            yield return "{6185191F-1008-4FB2-A715-3A4E4F27E610}";
        }

        public override string[] GetDeviceSdkPaths()
        {
            return SdkPaths.GetAllSdkPaths(SdkPathGroup.Mobile, this.XnaFrameworkVersion, null);
        }

        public override string[] GetFilterProperties()
        {
            return new string[] { 
                "AllowUnsafeBlocks", "AssemblyFileVersion", "ComVisible", "EnableASPDebugging", "EnableASPXDebugging", "EnableSecurityDebugging", "EnableSQLServerDebugging", "EnableUnmanagedDebugging", "NoStdLib", "OutputType", "PlatformTarget", "RegisterForComInterop", "RemoteDebugEnabled", "RemoteDebugMachine", "SignManifests", "StartAction", 
                "StartupObject", "StartWorkingDirectory", "TargetFrameworkMoniker", "UseVSHostingProcess"
             };
        }

        public string[] GetRefreshSdkPaths(string refreshLevel)
        {
            return SdkPaths.GetAllSdkPaths(SdkPathGroup.Mobile, this.XnaFrameworkVersion, (string.IsNullOrWhiteSpace(refreshLevel) || (refreshLevel == "0")) ? null : "1");
        }

        public string MaximumRefreshLevel()
        {
            return "1";
        }

        string IProjectPlatformAdapterCaps.GetCapabilityValue(string capabilityName)
        {
            if (capabilityName == null)
            {
                throw new ArgumentNullException("capabilityName");
            }
            if (capabilityName.Equals("listDefaultDevice", StringComparison.OrdinalIgnoreCase))
            {
                return "false";
            }
            if (capabilityName.Equals("tombstoneDebugging", StringComparison.OrdinalIgnoreCase))
            {
                return "true";
            }
            if (capabilityName.Equals("centralDeviceSelector", StringComparison.OrdinalIgnoreCase))
            {
                return "true";
            }
            return null;
        }

        bool IProjectPlatformAdapterCaps.SetCapabilityValue(string capabilityName, string capabilityValue)
        {
            return false;
        }

        internal override bool ProcessFile(string sourcePath, string destPath, ProjectRootElement project)
        {
            if (string.IsNullOrEmpty(sourcePath))
            {
                throw new ArgumentNullException("sourcePath");
            }
            if (string.IsNullOrEmpty(destPath))
            {
                throw new ArgumentNullException("destPath");
            }
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            if (!File.Exists(sourcePath))
            {
                throw new ArgumentException("filePath");
            }
            if (string.Equals(Path.GetFileName(sourcePath), "WMAppManifest.xml", StringComparison.OrdinalIgnoreCase))
            {
                using (StreamReader reader = new StreamReader(sourcePath))
                {
                    using (StreamWriter writer = new StreamWriter(destPath))
                    {
                        Project project2 = new Project(project);
                        try
                        {
                            string str4;
                            string propertyValue = project2.GetPropertyValue("AssemblyName");
                            string str2 = project2.GetPropertyValue("ProjectGuid");
                            string str3 = project2.GetPropertyValue("Thumbnail");
                            while ((str4 = reader.ReadLine()) != null)
                            {
                                if (str4.Contains("$safeprojectname$") && !string.IsNullOrEmpty(propertyValue))
                                {
                                    str4 = str4.Replace("$safeprojectname$", propertyValue);
                                }
                                if (str4.Contains("$guid1$") && !string.IsNullOrEmpty(str2))
                                {
                                    str2 = str2.TrimStart(new char[] { '{' }).TrimEnd(new char[] { '}' });
                                    str4 = str4.Replace("$guid1$", str2);
                                }
                                if (str4.Contains("GameThumbnail.png") && !string.IsNullOrEmpty(str3))
                                {
                                    str4 = str4.Replace("GameThumbnail.png", str3);
                                }
                                writer.WriteLine(str4);
                            }
                        }
                        finally
                        {
                            ProjectCollection.GlobalProjectCollection.UnloadProject(project2);
                        }
                        writer.Flush();
                        return true;
                    }
                }
            }
            return false;
        }

        public List<XnaProjectCreationOption> ProjectCreationOptions(object dte, string templateName, Dictionary<string, string> replacementsDictionary)
        {
            List<XnaProjectCreationOption> list = new List<XnaProjectCreationOption>();
            if ((replacementsDictionary != null) && replacementsDictionary.ContainsKey("XnaRefreshLevel"))
            {
                XnaProjectCreationOption creationOption = new XnaProjectCreationOption(XnaProjectResource.WPOptionTitle, "XnaRefreshLevel", XnaProjectResource.WPOptionTitleDescription, null);
                string str = string.Empty;
                bool flag = false;
                int num = 0;
                XnaPackage instance = XnaPackage.Instance;
                if (instance != null)
                {
                    IServiceProvider provider = instance;
                    IVsSolution service = null;
                    if (provider != null)
                    {
                        service = provider.GetService(typeof(SVsSolution)) as IVsSolution;
                    }
                    if (service != null)
                    {
                        IEnumHierarchies ppenum = null;
                        Guid empty = Guid.Empty;
                        service.GetProjectEnum(0x1b, ref empty, out ppenum);
                        if (ppenum != null)
                        {
                            IVsHierarchy[] rgelt = new IVsHierarchy[1];
                            uint pceltFetched = 0;
                            ppenum.Reset();
                            while ((ppenum.Next(1, rgelt, out pceltFetched) == 0) && (pceltFetched == 1))
                            {
                                if (rgelt[0] != null)
                                {
                                    IVsBuildPropertyStorage storage = rgelt[0] as IVsBuildPropertyStorage;
                                    string pbstrPropValue = string.Empty;
                                    if (storage != null)
                                    {
                                        string str3 = string.Empty;
                                        if (!ErrorHandler.Succeeded(storage.GetPropertyValue("XnaPlatform", null, 1, out str3)) || !str3.Equals("Windows Phone", StringComparison.OrdinalIgnoreCase))
                                        {
                                            continue;
                                        }
                                        num++;
                                        if (!ErrorHandler.Succeeded(storage.GetPropertyValue("XnaRefreshLevel", null, 1, out pbstrPropValue)) || string.IsNullOrWhiteSpace(pbstrPropValue))
                                        {
                                            pbstrPropValue = "0";
                                        }
                                    }
                                    if (string.IsNullOrWhiteSpace(str))
                                    {
                                        str = pbstrPropValue;
                                    }
                                    else if (!str.Equals(pbstrPropValue))
                                    {
                                        flag = true;
                                    }
                                }
                            }
                        }
                    }
                }
                if (flag || (num == 0))
                {
                    this.AddRefreshLevelSelection(creationOption, false);
                    this.AddRefreshLevelSelection(creationOption, true);
                }
                else if (str.Equals("0", StringComparison.OrdinalIgnoreCase))
                {
                    this.AddRefreshLevelSelection(creationOption, false);
                }
                else
                {
                    this.AddRefreshLevelSelection(creationOption, true);
                }
                list.Add(creationOption);
            }
            return list;
        }

        public SkuSupportLevel SkuSupport(object skuOrCustom, int subSku, string localRegPath)
        {
            if (skuOrCustom == null)
            {
                throw new ArgumentNullException("skuOrCustom");
            }
            if (string.IsNullOrEmpty(localRegPath))
            {
                throw new ArgumentNullException("localRegPath");
            }
            if (!(skuOrCustom is int) || (((int) skuOrCustom) != 500))
            {
                return SkuSupportLevel.Supported;
            }
            if (subSku == 0x1000)
            {
                return SkuSupportLevel.Supported;
            }
            return SkuSupportLevel.Warn;
        }

        public bool UpgradeRefreshLevel(IVsHierarchy hierarchy)
        {
            if (hierarchy == null)
            {
                throw new ArgumentNullException("hierarchy");
            }
            IVsBuildPropertyStorage storage = hierarchy as IVsBuildPropertyStorage;
            if (storage == null)
            {
                return false;
            }
            ErrorHandler.ThrowOnFailure(storage.SetPropertyValue("XnaRefreshLevel", null, 1, "1"));
            return true;
        }

        public override IList<INamedExtenderProvider> ExtenderProviders
        {
            get
            {
                return this.extenderProviders;
            }
        }

        public override string[] GetDisabledPropertyList
        {
            get
            {
                return new string[] { "XnaProfile" };
            }
        }

        public override Microsoft.Xna.GameStudio.CodeProject.HelpContextFilters HelpContextFilters
        {
            get
            {
                if (this.helpContextFilters == null)
                {
                    this.helpContextFilters = new Microsoft.Xna.GameStudio.CodeProject.HelpContextFilters("XNA", "XNA_40", "XNA Platform Windows Phone");
                }
                return this.helpContextFilters;
            }
        }

        public override bool IsDevicePlatform
        {
            get
            {
                return true;
            }
        }

        List<string> IProjectPlatformAdapterCaps.Capabilities
        {
            get
            {
                return new List<string> { "listDefaultDevice", "tombstoneDebugging", "centralDeviceSelector" };
            }
        }

        public override string PlatformConstants
        {
            get
            {
                return "WINDOWS_PHONE";
            }
        }

        public override Bitmap ProjectNodeIcon
        {
            get
            {
                return XnaProjectResource._302;
            }
        }

        public override bool SupportsContentCompression
        {
            get
            {
                return false;
            }
        }

        public override bool SupportsDeviceCenter
        {
            get
            {
                return false;
            }
        }

        public override bool SupportsPackaging
        {
            get
            {
                return true;
            }
        }

        public override bool SupportsServiceReferences
        {
            get
            {
                return true;
            }
        }

        public override string XnaFrameworkRuntimeProfile
        {
            get
            {
                return "Windows Phone.v4.0.Reach;Windows Phone.v4.0R1.Reach";
            }
        }

        public override string XnaFrameworkVersion
        {
            get
            {
                return "v4.0";
            }
        }

        public override string XnaPlatform
        {
            get
            {
                return "Windows Phone";
            }
        }

    }
}

