﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.RegistrationAttributes;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Drawing;
    using System.Runtime.InteropServices;

    [ProjectType(typeof(WindowsPlatformAdapter), typeof(XnaPackage), null, null, null, null, null, LanguageVsTemplate="CSharp", TemplateGroupIDsVsTemplate="XnaFramework-v4.0,XnaFramework-v4.0-Windows"), ProjectFileFilter(typeof(WindowsPlatformAdapter), "Audio Files", 0x77, 0x514), ProjectFileFilter(typeof(WindowsPlatformAdapter), "Data Files", 0x75, 0x44c), ProjectFileFilter(typeof(WindowsPlatformAdapter), "Executable Files", 0x79, 0x5dc), ProjectFileFilter(typeof(WindowsPlatformAdapter), "Image Files", 0x76, 0x4b0), ProjectFileFilter(typeof(WindowsPlatformAdapter), "Script Files", 0x7a, 0x640), ProjectFileFilter(typeof(WindowsPlatformAdapter), "Video Files", 120, 0x578), ProjectFileFilter(typeof(WindowsPlatformAdapter), "Visual C# Files", 0x74, 0x3e8), ProjectFileFilter(typeof(WindowsPlatformAdapter), "Common Files", 0x7b, 100, CommonFindFilesFilter=true, NotOpenFileFilter=true, NotAddExistingItemFilter=true), Guid("17B8375C-1E54-49ad-8D94-471ECEE9AC00"), ProjectPlatformAdapter(typeof(WindowsPlatformAdapter), typeof(XnaPackage))]
    public class WindowsPlatformAdapter : BasePlatformAdapter, IProjectPlatformAdapterCaps
    {
        private IList<INamedExtenderProvider> extenderProviders;
        private Microsoft.Xna.GameStudio.CodeProject.HelpContextFilters helpContextFilters;

        public WindowsPlatformAdapter()
        {
            List<INamedExtenderProvider> list = new List<INamedExtenderProvider> {
                new ClickOnceProjectExtenderProvider("Microsoft.Xna.Framework.4.0")
            };
            this.extenderProviders = new ReadOnlyCollection<INamedExtenderProvider>(list);
        }

        string IProjectPlatformAdapterCaps.GetCapabilityValue(string capabilityName)
        {
            if (capabilityName == null)
            {
                throw new ArgumentNullException("capabilityName");
            }
            if (capabilityName.Equals("bootstrapperClickOnce", StringComparison.OrdinalIgnoreCase))
            {
                return "true";
            }
            return null;
        }

        bool IProjectPlatformAdapterCaps.SetCapabilityValue(string capabilityName, string capabilityValue)
        {
            return false;
        }

        public override IList<INamedExtenderProvider> ExtenderProviders
        {
            get
            {
                return this.extenderProviders;
            }
        }

        public override string[] GetDisabledPropertyList
        {
            get
            {
                return new string[] { "GameStartupType", "TileImage", "TileTitle" };
            }
        }

        public override Microsoft.Xna.GameStudio.CodeProject.HelpContextFilters HelpContextFilters
        {
            get
            {
                if (this.helpContextFilters == null)
                {
                    this.helpContextFilters = new Microsoft.Xna.GameStudio.CodeProject.HelpContextFilters("XNA", "XNA_40", "XNA Platform Windows");
                }
                return this.helpContextFilters;
            }
        }

        public override bool IsDevicePlatform
        {
            get
            {
                return false;
            }
        }

        List<string> IProjectPlatformAdapterCaps.Capabilities
        {
            get
            {
                return new List<string> { "bootstrapperClickOnce" };
            }
        }

        public override string PlatformConfigurationName
        {
            get
            {
                return "x86";
            }
        }

        public override string PlatformConstants
        {
            get
            {
                return "WINDOWS";
            }
        }

        public override Bitmap ProjectNodeIcon
        {
            get
            {
                return XnaProjectResource._300;
            }
        }

        public override bool SupportsServiceReferences
        {
            get
            {
                return true;
            }
        }

        public override string XnaFrameworkRuntimeProfile
        {
            get
            {
                return "Windows.v4.0.HiDef;Windows.v4.0.Reach";
            }
        }

        public override string XnaFrameworkVersion
        {
            get
            {
                return "v4.0";
            }
        }

        public override string XnaPlatform
        {
            get
            {
                return "Windows";
            }
        }
    }
}

