﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.RegistrationAttributes;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Drawing;
    using System.Runtime.InteropServices;

    [ProjectFileFilter(typeof(Xbox360PlatformAdapter), "Audio Files", 0x77, 0x514), ProjectFileFilter(typeof(Xbox360PlatformAdapter), "Data Files", 0x75, 0x44c), ProjectFileFilter(typeof(Xbox360PlatformAdapter), "Executable Files", 0x79, 0x5dc), ProjectFileFilter(typeof(Xbox360PlatformAdapter), "Image Files", 0x76, 0x4b0), ProjectFileFilter(typeof(Xbox360PlatformAdapter), "Script Files", 0x7a, 0x640), ProjectFileFilter(typeof(Xbox360PlatformAdapter), "Video Files", 120, 0x578), ProjectFileFilter(typeof(Xbox360PlatformAdapter), "Visual C# Files", 0x74, 0x3e8), ProjectPlatformAdapter(typeof(Xbox360PlatformAdapter), typeof(XnaPackage)), ProjectType(typeof(Xbox360PlatformAdapter), typeof(XnaPackage), null, null, null, null, null, LanguageVsTemplate="CSharp", ShowOnlySpecifiedTemplatesVsTemplate=true, TemplateGroupIDsVsTemplate="XnaFramework-v4.0,XnaFramework-v4.0-Xbox360,XnaFramework-v4.0-Devices", TemplateIDsVsTemplate="Microsoft.CSharp.CodeFile,Microsoft.CSharp.Resource,Microsoft.CSharp.TextFile,Microsoft.CSharp.XmlFile,Microsoft.CSharp.ClassDiagram,Microsoft.CSharp.XMLSchema"), Guid("14C1F6C6-984C-4aee-BFA4-3226C06AAFF9"), ProjectFileFilter(typeof(Xbox360PlatformAdapter), "Common Files", 0x7b, 100, CommonFindFilesFilter=true, NotOpenFileFilter=true, NotAddExistingItemFilter=true)]
    public sealed class Xbox360PlatformAdapter : BasePlatformAdapter
    {
        private Microsoft.Xna.GameStudio.CodeProject.HelpContextFilters helpContextFilters;

        public override IVsProjectFlavorCfg CreateProjectFlavorCfg(IServiceProvider serviceProvider, IVsCfg baseProjectCfg, IVsProjectFlavorCfg innerFlavorCfg)
        {
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            if (baseProjectCfg == null)
            {
                throw new ArgumentNullException("baseProjectCfg");
            }
            IXnaCodeProject requiredService = ProjectUtilities.GetRequiredService<IXnaCodeProject>(serviceProvider);
            if (requiredService != null)
            {
                return new PlatformFlavorCfg(requiredService, baseProjectCfg, innerFlavorCfg, "Microsoft.Xna.VisualStudio.DeployingXbox360Game");
            }
            return innerFlavorCfg;
        }

        public override string[] GetDeviceSdkPaths()
        {
            return SdkPaths.GetAllSdkPaths(SdkPathGroup.Xbox360, this.XnaFrameworkVersion, null);
        }

        public override string[] GetFilterProperties()
        {
            return new string[] { 
                "AssemblyFileVersion", "ComVisible", "EnableASPDebugging", "EnableASPXDebugging", "EnableSecurityDebugging", "EnableSQLServerDebugging", "EnableUnmanagedDebugging", "NoStdLib", "PlatformTarget", "RegisterForComInterop", "RemoteDebugEnabled", "RemoteDebugMachine", "SignManifests", "StartAction", "StartWorkingDirectory", "TargetFrameworkMoniker", 
                "UseVSHostingProcess"
             };
        }

        public override string[] GetDisabledPropertyList
        {
            get
            {
                return new string[] { "GameStartupType", "TileImage", "TileTitle" };
            }
        }

        public override Microsoft.Xna.GameStudio.CodeProject.HelpContextFilters HelpContextFilters
        {
            get
            {
                if (this.helpContextFilters == null)
                {
                    this.helpContextFilters = new Microsoft.Xna.GameStudio.CodeProject.HelpContextFilters("XNA", "XNA_40", "XNA Platform Xbox 360");
                }
                return this.helpContextFilters;
            }
        }

        public override bool IsDevicePlatform
        {
            get
            {
                return true;
            }
        }

        public override string PlatformConstants
        {
            get
            {
                return "XBOX;XBOX360";
            }
        }

        public override Bitmap ProjectNodeIcon
        {
            get
            {
                return XnaProjectResource._301;
            }
        }

        public override string XnaFrameworkRuntimeProfile
        {
            get
            {
                return "Xbox 360.v4.0.HiDef;Xbox 360.v4.0.Reach";
            }
        }

        public override string XnaFrameworkVersion
        {
            get
            {
                return "v4.0";
            }
        }

        public override string XnaPlatform
        {
            get
            {
                return "Xbox 360";
            }
        }
    }
}

