﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.VSHelp;
    using Microsoft.VisualStudio.VSHelp80;
    using System;
    using System.Reflection;

    internal class XnaErrorProvider : ErrorListProvider
    {
        private object criticalSection;
        private bool isDisposed;

        internal XnaErrorProvider(IServiceProvider serviceProvider) : base(serviceProvider)
        {
            this.criticalSection = new object();
        }

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
            this.isDisposed = true;
        }

        private static void OnHelp(object sender, EventArgs args)
        {
            Task task = sender as Task;
            if ((task != null) && !string.IsNullOrEmpty(task.HelpKeyword))
            {
                Help2 globalService = Package.GetGlobalService(typeof(SVsHelp)) as Help2;
                if (globalService != null)
                {
                    globalService.DisplayTopicFromF1Keyword(task.HelpKeyword);
                }
            }
        }

        internal void ReportError(object error)
        {
            this.ReportError(error, null);
        }

        internal void ReportError(object error, string helpKeyword)
        {
            this.ReportError(error, helpKeyword, TaskPriority.High);
        }

        private void ReportError(object error, string helpKeyword, TaskPriority priority)
        {
            if ((error != null) && !this.isDisposed)
            {
                Task task;
                Exception innerException = error as Exception;
                if (innerException != null)
                {
                    if ((error is TargetInvocationException) && (innerException.InnerException != null))
                    {
                        innerException = innerException.InnerException;
                    }
                    task = new Task(innerException);
                }
                else
                {
                    task = new Task();
                }
                if (task.Text.Length == 0)
                {
                    task.Text = error.ToString();
                }
                task.ImageIndex = 0;
                task.Category = TaskCategory.BuildCompile;
                task.Priority = priority;
                if (!string.IsNullOrEmpty(helpKeyword))
                {
                    task.HelpKeyword = helpKeyword;
                    task.Help += new EventHandler(XnaErrorProvider.OnHelp);
                }
                lock (this.criticalSection)
                {
                    base.Tasks.Add(task);
                }
            }
        }

        internal void ReportWarning(object warning)
        {
            this.ReportError(warning, null);
        }

        internal void ReportWarning(object warning, string helpKeyword)
        {
            this.ReportError(warning, helpKeyword, TaskPriority.Low);
        }
    }
}

