﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Flavor;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    [Guid("0CC9EF4B-1729-49fe-A2D3-791F60A45E0C"), ClassInterface(ClassInterfaceType.None), ComVisible(true)]
    internal class XnaGameStudioPropPage : ProjectDesignerPropertyPage
    {
        private bool alwaysDisabled;
        private IVsBuildPropertyStorage buildPropertyStorage;
        private IXnaCodeProject codeProject;
        private Dictionary<string, string> configProperties;
        private List<string> disabledAdapterProperties;
        private Label errorLabel;
        private FlowLayoutPanel pageView;
        private GraphicsProfileSelector profileSelector;
        private static readonly string[] propertyNames = new string[] { "Thumbnail", "GameStartupType", "XnaProfile", "TileImage", "TileTitle", "WPTombstoneEnabled" };
        private GameStartupTypeSelector startupSelector;
        private ThumbnailPicker thumbnailPicker;
        private TilePicker tilePicker;
        private TombstoneEnable tombstoneEnable;

        public XnaGameStudioPropPage() : base(XnaProjectResource.XGSPropertyPageTabTitle, "XNA.GameStudioPropertiesPage")
        {
        }

        private void CreatePropertyControls()
        {
            if (this.profileSelector == null)
            {
                this.profileSelector = new GraphicsProfileSelector(this.codeProject);
                this.profileSelector.PropertyChanged += new EventHandler(this.OnControlChangedProfile);
            }
            this.profileSelector.Enabled = !this.disabledAdapterProperties.Contains("XnaProfile");
            if (this.thumbnailPicker == null)
            {
                this.thumbnailPicker = new ThumbnailPicker(this.codeProject);
                this.thumbnailPicker.PropertyChanged += new EventHandler(this.OnControlChangedThumbnail);
            }
            this.thumbnailPicker.Enabled = !this.disabledAdapterProperties.Contains("Thumbnail");
            if (!this.disabledAdapterProperties.Contains("GameStartupType"))
            {
                if (this.startupSelector == null)
                {
                    this.startupSelector = new GameStartupTypeSelector(this.codeProject);
                    this.startupSelector.PropertyChanged += new EventHandler(this.OnControlChangedStartup);
                }
            }
            else
            {
                this.startupSelector = null;
            }
            if (!this.disabledAdapterProperties.Contains("TileImage") && !this.disabledAdapterProperties.Contains("TileTitle"))
            {
                if (this.tilePicker == null)
                {
                    this.tilePicker = new TilePicker(this.codeProject);
                    this.tilePicker.PropertyChanged += new EventHandler(this.OnControlChangedTile);
                }
            }
            else
            {
                this.tilePicker = null;
            }
            if (((this.tombstoneEnable == null) && string.Equals((string) this.GetProperty("XnaOutputType"), "Game", StringComparison.OrdinalIgnoreCase)) && ((this.codeProject.Adapter is IProjectPlatformAdapterCaps) && bool.TrueString.Equals(((IProjectPlatformAdapterCaps) this.codeProject.Adapter).GetCapabilityValue("tombstoneDebugging"), StringComparison.OrdinalIgnoreCase)))
            {
                this.tombstoneEnable = new TombstoneEnable(this.codeProject);
                this.tombstoneEnable.PropertyChanged += new EventHandler(this.OnControlChangedTombstoneEnable);
                this.tombstoneEnable.Enabled = false;
                if ((this.codeProject.Adapter is IXnaRefreshLevel) && string.Equals((string) this.GetProperty("XnaRefreshLevel"), ((IXnaRefreshLevel) this.codeProject.Adapter).MaximumRefreshLevel(), StringComparison.OrdinalIgnoreCase))
                {
                    this.tombstoneEnable.Enabled = true;
                }
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (this.tombstoneEnable != null)
                {
                    this.tombstoneEnable.PropertyChanged -= new EventHandler(this.OnControlChangedTombstoneEnable);
                    this.tombstoneEnable.Dispose();
                }
                if (this.tilePicker != null)
                {
                    this.tilePicker.PropertyChanged -= new EventHandler(this.OnControlChangedTile);
                    this.tilePicker.Dispose();
                }
                if (this.thumbnailPicker != null)
                {
                    this.thumbnailPicker.PropertyChanged -= new EventHandler(this.OnControlChangedThumbnail);
                    this.thumbnailPicker.Dispose();
                }
                if (this.profileSelector != null)
                {
                    this.profileSelector.PropertyChanged -= new EventHandler(this.OnControlChangedProfile);
                    this.profileSelector.Dispose();
                }
                if (this.startupSelector != null)
                {
                    this.startupSelector.PropertyChanged -= new EventHandler(this.OnControlChangedStartup);
                    this.startupSelector.ReleaseEvents();
                    this.startupSelector.Dispose();
                }
                if (this.codeProject != null)
                {
                    this.codeProject.ThumbnailChanged -= new EventHandler(this.OnThumbnailChanged);
                    this.codeProject.FileAdded -= new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.OnProjectItemChanged);
                    this.codeProject.FileRemoved -= new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.OnProjectItemChanged);
                    this.codeProject.FileRenamed -= new System.EventHandler<ProjectDocumentsRenamedEventArgs>(this.OnProjectItemChanged);
                }
                if (this.pageView != null)
                {
                    this.pageView.Dispose();
                    this.pageView = null;
                }
            }
            base.Dispose(disposing);
        }

        public override bool FinishPendingValidations()
        {
            return true;
        }

        public override object GetProperty(string propertyName)
        {
            string pbstrPropValue = null;
            if (this.buildPropertyStorage != null)
            {
                uint storage = 1;
                ErrorHandler.ThrowOnFailure(this.buildPropertyStorage.GetPropertyValue(propertyName, string.Empty, storage, out pbstrPropValue), new int[] { -2147170504 });
            }
            return pbstrPropValue;
        }

        public override bool GetPropertyMultipleValues(string propertyName, out object[] objects, out object[] values)
        {
            objects = null;
            values = null;
            return false;
        }

        protected override void OnCommitPropertyEdit()
        {
            if (this.buildPropertyStorage != null)
            {
                uint storage = 1;
                if (this.profileSelector.Dirty)
                {
                    this.buildPropertyStorage.SetPropertyValue("XnaProfile", string.Empty, storage, this.profileSelector.Text);
                    this.configProperties["XnaProfile"] = this.profileSelector.Text;
                    this.profileSelector.Dirty = false;
                }
                if (this.thumbnailPicker.Dirty)
                {
                    this.buildPropertyStorage.SetPropertyValue("Thumbnail", string.Empty, storage, this.thumbnailPicker.Text);
                    string text = this.thumbnailPicker.Text;
                    this.configProperties["Thumbnail"] = text;
                    this.codeProject.Thumbnail = text;
                    this.thumbnailPicker.Dirty = false;
                }
                if ((this.startupSelector != null) && this.startupSelector.Dirty)
                {
                    this.buildPropertyStorage.SetPropertyValue("GameStartupType", string.Empty, storage, this.startupSelector.Text);
                    this.configProperties["GameStartupType"] = this.startupSelector.Text;
                    this.startupSelector.Dirty = false;
                }
                if ((this.tilePicker != null) && this.tilePicker.Dirty)
                {
                    this.buildPropertyStorage.SetPropertyValue("TileImage", string.Empty, storage, this.tilePicker.Text);
                    this.configProperties["TileImage"] = this.tilePicker.Text;
                    this.buildPropertyStorage.SetPropertyValue("TileTitle", string.Empty, storage, this.tilePicker.Title);
                    this.configProperties["TileTitle"] = this.tilePicker.Title;
                    this.tilePicker.Dirty = false;
                }
                if ((this.tombstoneEnable != null) && this.tombstoneEnable.Dirty)
                {
                    this.buildPropertyStorage.SetPropertyValue("WPTombstoneEnabled", string.Empty, storage, this.tombstoneEnable.Text);
                    this.configProperties["WPTombstoneEnabled"] = this.tombstoneEnable.Text;
                }
            }
        }

        private void OnControlChangedProfile(object sender, EventArgs args)
        {
            GraphicsProfileSelector selector = sender as GraphicsProfileSelector;
            if (selector != null)
            {
                string text = selector.Text;
                string str2 = this.configProperties["XnaProfile"];
                if ((!string.IsNullOrEmpty(text) || !string.IsNullOrEmpty(str2)) && (text != str2))
                {
                    base.OnPropertyEdited("XnaProfile", typeof(string));
                }
            }
        }

        private void OnControlChangedStartup(object sender, EventArgs args)
        {
            GameStartupTypeSelector selector = sender as GameStartupTypeSelector;
            if (selector != null)
            {
                string text = selector.Text;
                string str2 = this.configProperties["GameStartupType"];
                if ((!string.IsNullOrEmpty(text) || !string.IsNullOrEmpty(str2)) && (text != str2))
                {
                    base.OnPropertyEdited("GameStartupType", typeof(string));
                }
            }
        }

        private void OnControlChangedThumbnail(object sender, EventArgs args)
        {
            ThumbnailPicker picker = sender as ThumbnailPicker;
            if (picker != null)
            {
                string text = picker.Text;
                string str2 = this.configProperties["Thumbnail"];
                if ((!string.IsNullOrEmpty(text) || !string.IsNullOrEmpty(str2)) && (text != str2))
                {
                    base.OnPropertyEdited("Thumbnail", typeof(string));
                }
            }
        }

        private void OnControlChangedTile(object sender, EventArgs args)
        {
            TilePicker picker = sender as TilePicker;
            if (picker != null)
            {
                string text = picker.Text;
                string str2 = this.configProperties["TileImage"];
                if ((!string.IsNullOrEmpty(text) || !string.IsNullOrEmpty(str2)) && (text != str2))
                {
                    base.OnPropertyEdited("TileImage", typeof(string));
                }
                text = picker.Title;
                str2 = this.configProperties["TileTitle"];
                if ((!string.IsNullOrEmpty(text) || !string.IsNullOrEmpty(str2)) && (text != str2))
                {
                    base.OnPropertyEdited("TileTitle", typeof(string));
                }
            }
        }

        private void OnControlChangedTombstoneEnable(object sender, EventArgs args)
        {
            TombstoneEnable enable = sender as TombstoneEnable;
            if (enable != null)
            {
                string text = enable.Text;
                string str2 = this.configProperties["WPTombstoneEnabled"];
                if ((!string.IsNullOrEmpty(text) || !string.IsNullOrEmpty(str2)) && (text != str2))
                {
                    base.OnPropertyEdited("WPTombstoneEnabled", typeof(string));
                }
            }
        }

        private void OnProjectItemChanged(object sender, ProjectDocumentsChangeEventArgs args)
        {
            if (this.thumbnailPicker != null)
            {
                this.thumbnailPicker.RefreshList();
            }
            if (this.startupSelector != null)
            {
                this.startupSelector.ProjectChanged();
            }
            if (this.tilePicker != null)
            {
                this.tilePicker.RefreshList();
            }
            if (this.tombstoneEnable != null)
            {
                this.tombstoneEnable.RefreshList();
            }
        }

        private void OnProjectItemChanged(object sender, ProjectDocumentsRenamedEventArgs args)
        {
            if (this.thumbnailPicker != null)
            {
                this.thumbnailPicker.RefreshList();
            }
            if (this.startupSelector != null)
            {
                this.startupSelector.ProjectChanged();
            }
            if (this.tilePicker != null)
            {
                this.tilePicker.RefreshList();
            }
            if (this.tombstoneEnable != null)
            {
                this.tombstoneEnable.RefreshList();
            }
        }

        private void OnThumbnailChanged(object sender, EventArgs args)
        {
            if (this.thumbnailPicker != null)
            {
                this.thumbnailPicker.RefreshList();
            }
        }

        private void RefreshConfigProperties()
        {
            this.configProperties = new Dictionary<string, string>(propertyNames.Length);
            foreach (string str in propertyNames)
            {
                this.configProperties[str] = (string) this.GetProperty(str);
            }
        }

        private void RefreshView()
        {
            this.RefreshConfigProperties();
            if (this.thumbnailPicker != null)
            {
                this.thumbnailPicker.Text = this.configProperties["Thumbnail"];
            }
            if (this.profileSelector != null)
            {
                this.profileSelector.Text = this.configProperties["XnaProfile"];
            }
            if (this.startupSelector != null)
            {
                this.startupSelector.Text = this.configProperties["GameStartupType"];
            }
            if (this.tilePicker != null)
            {
                this.tilePicker.Text = this.configProperties["TileImage"];
                this.tilePicker.Title = this.configProperties["TileTitle"];
            }
            if (this.tombstoneEnable != null)
            {
                this.tombstoneEnable.Text = this.tombstoneEnable.Enabled ? this.configProperties["WPTombstoneEnabled"] : "true";
            }
        }

        public override void SetObjects(uint cObjects, object[] ppunk)
        {
            if ((cObjects == 1) && (ppunk != null))
            {
                IVsBrowseObject obj2 = ppunk[0] as IVsBrowseObject;
                if (obj2 != null)
                {
                    IVsHierarchy pHier = null;
                    uint maxValue = uint.MaxValue;
                    ErrorHandler.ThrowOnFailure(obj2.GetProjectItem(out pHier, out maxValue));
                    this.codeProject = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>((IHierarchyObjectProvider) pHier);
                    if (this.codeProject != null)
                    {
                        this.alwaysDisabled = false;
                        this.disabledAdapterProperties = new List<string>((this.codeProject.Adapter.GetDisabledPropertyList == null) ? ((IEnumerable<string>) new string[0]) : ((IEnumerable<string>) this.codeProject.Adapter.GetDisabledPropertyList));
                        IVsBuildPropertyStorage propertyStorage = pHier as IVsBuildPropertyStorage;
                        IVsProject project = pHier as IVsProject;
                        if ((propertyStorage != null) && (project != null))
                        {
                            this.buildPropertyStorage = new VsBuildPropertyStorageSccWrapper(propertyStorage, project);
                        }
                        this.CreatePropertyControls();
                        this.codeProject.ThumbnailChanged += new EventHandler(this.OnThumbnailChanged);
                        this.codeProject.FileAdded += new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.OnProjectItemChanged);
                        this.codeProject.FileRemoved += new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.OnProjectItemChanged);
                        this.codeProject.FileRenamed += new System.EventHandler<ProjectDocumentsRenamedEventArgs>(this.OnProjectItemChanged);
                        this.RefreshView();
                    }
                }
            }
            base.SetObjects(cObjects, ppunk);
        }

        public override void SetProperty(string propertyName, object value)
        {
            string strA = value as string;
            if (strA == null)
            {
                strA = string.Empty;
            }
            string property = this.GetProperty(propertyName) as string;
            if (property == null)
            {
                property = string.Empty;
            }
            if ((string.CompareOrdinal(strA, property) != 0) && (this.buildPropertyStorage != null))
            {
                uint storage = 1;
                ErrorHandler.ThrowOnFailure(this.buildPropertyStorage.SetPropertyValue(propertyName, string.Empty, storage, strA));
            }
            this.RefreshView();
        }

        public override void SetPropertyMultipleValues(string propertyName, object[] objects, object[] values)
        {
        }

        protected override void SetPropertyPageEnabled()
        {
            if (!this.alwaysDisabled)
            {
                base.SetPropertyPageEnabled();
            }
        }

        public override bool SupportsMultipleValueUndo(string propertyName)
        {
            return false;
        }

        protected override Control PageView
        {
            get
            {
                if (this.pageView == null)
                {
                    if (((propertyNames.Length == 0) || (this.profileSelector == null)) || (this.thumbnailPicker == null))
                    {
                        if (this.errorLabel == null)
                        {
                            this.errorLabel = new Label();
                            this.errorLabel.AutoSize = true;
                            this.errorLabel.TextAlign = ContentAlignment.MiddleCenter;
                            this.errorLabel.Dock = DockStyle.Fill;
                            this.errorLabel.Text = XnaProjectResource.XGSPropertyPageInitializationError;
                        }
                        return this.errorLabel;
                    }
                    FlowLayoutPanel panel = new FlowLayoutPanel();
                    panel.SuspendLayout();
                    panel.FlowDirection = FlowDirection.LeftToRight;
                    panel.WrapContents = true;
                    panel.AutoScroll = true;
                    panel.AutoSize = true;
                    panel.Padding = Padding.Empty;
                    panel.Dock = DockStyle.Fill;
                    if (this.profileSelector != null)
                    {
                        panel.Controls.Add(this.profileSelector);
                        panel.SetFlowBreak(this.profileSelector, true);
                    }
                    if (this.thumbnailPicker != null)
                    {
                        panel.Controls.Add(this.thumbnailPicker);
                        panel.SetFlowBreak(this.thumbnailPicker, true);
                    }
                    if (this.tilePicker != null)
                    {
                        panel.Controls.Add(this.tilePicker);
                        panel.SetFlowBreak(this.tilePicker, true);
                    }
                    if (this.startupSelector != null)
                    {
                        panel.Controls.Add(this.startupSelector);
                        panel.SetFlowBreak(this.startupSelector, true);
                    }
                    if (this.tombstoneEnable != null)
                    {
                        panel.Controls.Add(this.tombstoneEnable);
                        panel.SetFlowBreak(this.tombstoneEnable, true);
                    }
                    panel.TabIndex = 0;
                    panel.ResumeLayout();
                    panel.PerformLayout();
                    this.pageView = panel;
                }
                return this.pageView;
            }
        }
    }
}

