﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using EnvDTE;
    using EnvDTE80;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.ServicesProxy;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Flavor;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.VSDesigner.WCFModel;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Projects;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel.Design;
    using System.Diagnostics;
    using System.Diagnostics.CodeAnalysis;
    using System.Globalization;
    using System.IO;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using System.Text;
    using System.Threading;
    using VSLangProj;

    [SuppressMessage("Microsoft.Design", "CA1001:TypesThatOwnDisposableFieldsShouldBeDisposable"), Guid("D4646D23-7D26-42d2-AC0C-4AF99E818F68"), ComVisible(false)]
    internal sealed class XnaProject : Microsoft.Xna.GameStudio.FlavoredProject.FlavoredProjectBase, IVsUpdateSolutionEvents2, IVsUpdateSolutionEvents, IVsProjectFlavorCfgProvider, IInternalExtenderProvider, IVsDependencyProvider, IVsPersistHierarchyItem2, IVsPersistHierarchyItem, IVsProjectFlavorReferences, IVsHierarchyDeleteHandler, IVsProjectSpecialFiles, Microsoft.VisualStudio.OLE.Interop.IServiceProvider, IVbCompilerHost
    {
        private uint buildEvents2Cookie;
        private DelayedRenameEventManager delayedRenameEventManager;
        private XnaErrorProvider errorProvider;
        private Dictionary<string, IExtenderProvider> extenderProviders = new Dictionary<string, IExtenderProvider>();
        private HierarchyItemManager hierarchyItemManager;
        private bool inDeleteItem;
        private IVsHierarchyDeleteHandler innerDeleteHandler;
        private IVsBuildPropertyStorage innerVsBuildPropertyStorage;
        private IVsDependencyProvider innerVsDependencyProvider;
        private Microsoft.VisualStudio.OLE.Interop.IServiceProvider innerVsHierarchySite;
        private IVsPersistHierarchyItem innerVsPersistHierarchyItem;
        private IVsProject2 innerVsProject2;
        private IVsProjectBuildSystem innerVsProjectBuildSystem;
        private IVsProjectFlavorCfgProvider innerVsProjectFlavorCfgProvider;
        private bool inSyncDirectoryRemove;
        private bool inSyncDirectoryRename;
        private bool inSyncFileAdd;
        private bool inSyncFileRemove;
        private bool inSyncFileRename;
        private bool isAggregationComplete;
        private bool isBuilding;
        private bool isClosed;
        private bool isNewProject;
        private IProjectLanguageAdapter languageAdapter;
        private MenuCommandFilter menuCommandFilter;
        private MenuCommandHandlers menuCommands;
        private Package package;
        private IProjectPlatformAdapter platformAdapter;
        private Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> platformAdapters;
        private bool showProjectInSolutionPage = true;
        private IVsSolutionBuildManager2 solutionBuildManager2;
        private string thumbnail;
        private WcfSilverlightClientGenerationExtension wcfSilverlightClientGenerationExtension;
        private IXnaCodeProject xnaCodeProject;
        private XnaShouldPackage xnaShouldPackage;

        private event EventHandler BuildStatusChanged;

        private event EventHandler ThumbnailChanged;

        internal XnaProject(Package package, Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> adapters)
        {
            if (package == null)
            {
                throw new ArgumentNullException("package");
            }
            if (adapters == null)
            {
                throw new ArgumentNullException("adapters");
            }
            this.package = package;
            this.platformAdapters = adapters;
            this.xnaCodeProject = new XnaCodeProject(this);
            base.FileAdded += new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.FileAddedHandler);
            base.FileRenamed += new System.EventHandler<ProjectDocumentsRenamedEventArgs>(this.FileRenamedHandler);
            base.FileRemoved += new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.FileRemovedHandler);
            base.DirectoryRemoved += new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.DirectoryRemovedHandler);
            this.delayedRenameEventManager = new DelayedRenameEventManager();
        }

        [Conditional("DEBUG")]
        private static void AssertUnexpectedPropId(int propId)
        {
        }

        public bool CanExtend(string ExtenderCATID, string ExtenderName, object ExtendeeObject)
        {
            IExtenderProvider provider = null;
            return (this.extenderProviders.TryGetValue(ExtenderName, out provider) && provider.CanExtend(ExtenderCATID, ExtenderName, ExtendeeObject));
        }

        protected override void Close()
        {
            if (!this.isClosed)
            {
                CrossPlatformGroupManager vsService = this.GetVsService(typeof(CrossPlatformGroupManager)) as CrossPlatformGroupManager;
                if (vsService != null)
                {
                    vsService.UnregisterProject(this.xnaCodeProject);
                }
                base.FileRemoved -= new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.OnFileRemovedFromProject);
                base.FileRenamed -= new System.EventHandler<ProjectDocumentsRenamedEventArgs>(this.OnFileRenamedInProject);
                base.Close();
                IXnaDeviceDeploymentRegistration registration = this.GetVsService(typeof(IXnaDeviceDeploymentRegistration)) as IXnaDeviceDeploymentRegistration;
                if (registration != null)
                {
                    registration.UnregisterProject(this);
                }
                if (this.menuCommandFilter != null)
                {
                    this.menuCommandFilter.Enable(false);
                    this.menuCommandFilter = null;
                }
                if (this.menuCommands != null)
                {
                    this.menuCommands.Enable(false);
                    this.menuCommands = null;
                }
                if ((this.buildEvents2Cookie != 0) && (this.solutionBuildManager2 != null))
                {
                    this.solutionBuildManager2.UnadviseUpdateSolutionEvents(this.buildEvents2Cookie);
                }
                this.solutionBuildManager2 = null;
                this.buildEvents2Cookie = 0;
                if (this.hierarchyItemManager != null)
                {
                    this.hierarchyItemManager.Dispose();
                }
                if (this.errorProvider != null)
                {
                    this.errorProvider.Dispose();
                    this.errorProvider = null;
                }
                this.isClosed = true;
                this.innerVsProjectFlavorCfgProvider = null;
                this.innerVsBuildPropertyStorage = null;
                this.innerVsDependencyProvider = null;
                this.innerVsPersistHierarchyItem = null;
                this.innerVsProject2 = null;
                this.innerVsProjectBuildSystem = null;
            }
        }

        private void CreateLocationForFolderItem(EnvDTE.ProjectItem folderItem)
        {
            if ((folderItem != null) && (folderItem.Kind == "{6BB5F8EF-4483-11D3-8BCF-00C04F8EC28C}"))
            {
                string str = folderItem.get_FileNames(0);
                if (!string.IsNullOrEmpty(str) && !Directory.Exists(str))
                {
                    Directory.CreateDirectory(str);
                }
            }
        }

        public int CreateProjectFlavorCfg(IVsCfg pBaseProjectCfg, out IVsProjectFlavorCfg ppFlavorCfg)
        {
            ppFlavorCfg = null;
            this.InitializeItemManager();
            IVsProjectFlavorCfg cfg = null;
            if (pBaseProjectCfg != null)
            {
                string str;
                if (this.innerVsProjectFlavorCfgProvider != null)
                {
                    this.innerVsProjectFlavorCfgProvider.CreateProjectFlavorCfg(pBaseProjectCfg, out cfg);
                }
                ErrorHandler.ThrowOnFailure(pBaseProjectCfg.get_DisplayName(out str));
                IVsProjectFlavorCfg cfg2 = null;
                if (this.platformAdapter == null)
                {
                    throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.InvalidXnaPlatform, new object[] { str }));
                }
                cfg2 = this.platformAdapter.CreateProjectFlavorCfg((System.IServiceProvider) this.xnaCodeProject, pBaseProjectCfg, cfg);
                if (cfg2 == null)
                {
                    cfg2 = new ProjectFlavorCfg(pBaseProjectCfg, cfg, this.platformAdapter.XnaFrameworkVersion);
                }
                ppFlavorCfg = cfg2;
            }
            return 0;
        }

        public int DeleteItem(uint dwDelItemOp, uint itemid)
        {
            if ((this.hierarchyItemManager.GetContentReferenceNode(itemid) != null) && (dwDelItemOp != 2))
            {
                return -2147024809;
            }
            return this.innerDeleteHandler.DeleteItem(dwDelItemOp, itemid);
        }

        private void DirectoryRemovedHandler(object sender, ProjectDocumentsChangeEventArgs eventArgs)
        {
            if (sender == null)
            {
                throw new ArgumentNullException("sender");
            }
            if (eventArgs == null)
            {
                throw new ArgumentNullException("eventArgs");
            }
            if (!this.inSyncDirectoryRemove && !Directory.Exists(eventArgs.MkDocument))
            {
                CrossPlatformGroupManager vsService = this.GetVsService(typeof(CrossPlatformGroupManager)) as CrossPlatformGroupManager;
                if (vsService != null)
                {
                    Dictionary<string, IXnaCodeProject> projects = vsService.GetProjects(this.xnaCodeProject.CrossPlatformGroupID);
                    if ((projects != null) && (projects.Count > 1))
                    {
                        string xnaPlatform = this.platformAdapter.XnaPlatform;
                        foreach (string str2 in projects.Keys)
                        {
                            if (str2 != xnaPlatform)
                            {
                                projects[str2].SyncDirectoryRemove(eventArgs.MkDocument);
                            }
                        }
                    }
                }
            }
        }

        private void DoPackaging(bool shouldPackage)
        {
            if (this.xnaShouldPackage == null)
            {
                this.xnaShouldPackage = new XnaShouldPackage(base.ServiceProvider);
            }
            this.xnaShouldPackage.ShouldPackage = shouldPackage;
            this.xnaShouldPackage.Thumbnail = this.GetFullThumbnailPath();
        }

        public int EnumDependencies(out IVsEnumDependencies ppIVsEnumDependencies)
        {
            ppIVsEnumDependencies = null;
            List<IVsDependency> collection = new List<IVsDependency>();
            if (this.innerVsDependencyProvider != null)
            {
                IVsDependency[] rgpIVsDependency = new IVsDependency[1];
                IVsEnumDependencies dependencies = null;
                if (ErrorHandler.Succeeded(this.innerVsDependencyProvider.EnumDependencies(out dependencies)))
                {
                    uint pcElementsFetched = 0;
                    while (ErrorHandler.Succeeded(dependencies.Next(1, rgpIVsDependency, out pcElementsFetched)))
                    {
                        if (pcElementsFetched == 0)
                        {
                            break;
                        }
                        collection.Add(rgpIVsDependency[0]);
                    }
                }
            }
            ppIVsEnumDependencies = new VsEnumDependencies(collection);
            return 0;
        }

        protected override int ExecCommand(uint itemid, ref Guid pguidCmdGroup, uint nCmdID, uint nCmdexecopt, IntPtr pvaIn, IntPtr pvaOut)
        {
            ContentReferencesFolderNode contentReferencesFolderNode = this.hierarchyItemManager.ContentReferencesFolderNode;
            if (itemid == contentReferencesFolderNode.GetItemId())
            {
                return contentReferencesFolderNode.ExecCommand(itemid, ref pguidCmdGroup, nCmdID, nCmdexecopt, pvaIn, pvaOut);
            }
            ContentReferenceNode contentReferenceNode = this.hierarchyItemManager.GetContentReferenceNode(itemid);
            if (contentReferenceNode != null)
            {
                return contentReferenceNode.ExecCommand(itemid, ref pguidCmdGroup, nCmdID, nCmdexecopt, pvaIn, pvaOut);
            }
            if ((this.platformAdapter.IsDevicePlatform && ((pguidCmdGroup == Guid.Empty) || (pguidCmdGroup == CmdIDConstants.AddReferenceCmdID.Guid))) && ((nCmdID == CmdIDConstants.AddReferenceCmdID.ID) || (nCmdID == CmdIDConstants.AddReferenceCtxCmdID.ID)))
            {
                this.menuCommands.OnAddReference();
                return 0;
            }
            return base.ExecCommand(itemid, ref pguidCmdGroup, nCmdID, nCmdexecopt, pvaIn, pvaOut);
        }

        private void FileAddedHandler(object sender, ProjectDocumentsChangeEventArgs eventArgs)
        {
            if (sender == null)
            {
                throw new ArgumentNullException("sender");
            }
            if (eventArgs == null)
            {
                throw new ArgumentNullException("eventArgs");
            }
            if (!this.inSyncFileAdd)
            {
                CrossPlatformGroupManager vsService = this.GetVsService(typeof(CrossPlatformGroupManager)) as CrossPlatformGroupManager;
                if (vsService != null)
                {
                    Dictionary<string, IXnaCodeProject> projects = vsService.GetProjects(this.xnaCodeProject.CrossPlatformGroupID);
                    if ((projects != null) && (projects.Count > 1))
                    {
                        int num;
                        uint num2;
                        VSDOCUMENTPRIORITY[] pdwPriority = new VSDOCUMENTPRIORITY[1];
                        ErrorHandler.ThrowOnFailure(this.InnerVsProject2.IsDocumentInProject(eventArgs.MkDocument, out num, pdwPriority, out num2));
                        string xnaPlatform = this.platformAdapter.XnaPlatform;
                        if (num != 0)
                        {
                            object obj2;
                            ErrorHandler.ThrowOnFailure(this.GetProperty(num2, -2074, out obj2));
                            if (!((bool) obj2))
                            {
                                Dictionary<string, object> syncProperties = this.GetSyncProperties(num2);
                                foreach (KeyValuePair<string, IXnaCodeProject> pair in projects)
                                {
                                    if (pair.Key != xnaPlatform)
                                    {
                                        pair.Value.SyncFileAdd(eventArgs.MkDocument, syncProperties);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        private void FileRemovedHandler(object sender, ProjectDocumentsChangeEventArgs eventArgs)
        {
            if (sender == null)
            {
                throw new ArgumentNullException("sender");
            }
            if (eventArgs == null)
            {
                throw new ArgumentNullException("eventArgs");
            }
            if (!this.inSyncFileRemove && !File.Exists(eventArgs.MkDocument))
            {
                CrossPlatformGroupManager vsService = this.GetVsService(typeof(CrossPlatformGroupManager)) as CrossPlatformGroupManager;
                if (vsService != null)
                {
                    Dictionary<string, IXnaCodeProject> projects = vsService.GetProjects(this.xnaCodeProject.CrossPlatformGroupID);
                    if ((projects != null) && (projects.Count > 1))
                    {
                        string xnaPlatform = this.platformAdapter.XnaPlatform;
                        foreach (string str2 in projects.Keys)
                        {
                            if (str2 != xnaPlatform)
                            {
                                projects[str2].SyncFileRemove(eventArgs.MkDocument);
                            }
                        }
                    }
                }
            }
        }

        private void FileRenamedHandler(object sender, ProjectDocumentsRenamedEventArgs renameEventArgs)
        {
            if (sender == null)
            {
                throw new ArgumentNullException("sender");
            }
            if (renameEventArgs == null)
            {
                throw new ArgumentNullException("renameEventArgs");
            }
            if (!this.inSyncFileRename)
            {
                CrossPlatformGroupManager vsService = this.GetVsService(typeof(CrossPlatformGroupManager)) as CrossPlatformGroupManager;
                if (vsService != null)
                {
                    string crossPlatformGroupID = this.xnaCodeProject.CrossPlatformGroupID;
                    Dictionary<string, IXnaCodeProject> projects = vsService.GetProjects(crossPlatformGroupID);
                    if ((projects != null) && (projects.Count > 1))
                    {
                        int num;
                        uint num2;
                        VSDOCUMENTPRIORITY[] pdwPriority = new VSDOCUMENTPRIORITY[1];
                        ErrorHandler.ThrowOnFailure(this.InnerVsProject2.IsDocumentInProject(renameEventArgs.MkDocumentNew, out num, pdwPriority, out num2));
                        if (num2 != 0xfffffffe)
                        {
                            string xnaPlatform = this.platformAdapter.XnaPlatform;
                            if (num != 0)
                            {
                                object property = null;
                                ErrorHandler.ThrowOnFailure(this.GetProperty(num2, -2027, out property));
                                EnvDTE.ProjectItem projItem = (EnvDTE.ProjectItem) property;
                                if (!this.delayedRenameEventManager.AddRenameEvent(projItem, renameEventArgs))
                                {
                                    bool flag = false;
                                    if (this.delayedRenameEventManager.HandleFileRename(renameEventArgs))
                                    {
                                        if (this.delayedRenameEventManager.ReadyEvents.Count == 0)
                                        {
                                            return;
                                        }
                                        flag = true;
                                    }
                                    if (flag)
                                    {
                                        foreach (ProjectDocumentsRenamedEventArgs args in this.delayedRenameEventManager.ReadyEvents)
                                        {
                                            vsService.SyncRenameComponent.NotifyFileRenamed(crossPlatformGroupID, xnaPlatform, args.MkDocumentOld, args.MkDocumentNew);
                                        }
                                    }
                                    else
                                    {
                                        vsService.SyncRenameComponent.NotifyFileRenamed(crossPlatformGroupID, xnaPlatform, renameEventArgs.MkDocumentOld, renameEventArgs.MkDocumentNew);
                                    }
                                    if (flag)
                                    {
                                        this.delayedRenameEventManager.ReadyEvents.Clear();
                                    }
                                }
                            }
                            else if (Directory.Exists(renameEventArgs.MkDocumentNew))
                            {
                                vsService.SyncRenameComponent.NotifyDirectoryRenamed(crossPlatformGroupID, xnaPlatform, renameEventArgs.MkDocumentOld, renameEventArgs.MkDocumentNew);
                            }
                        }
                    }
                }
            }
        }

        private EnvDTE.ProjectItem FindFolder(string folderName, ProjectItems items)
        {
            if (folderName == null)
            {
                throw new ArgumentNullException("folderName");
            }
            if (items == null)
            {
                throw new ArgumentNullException("items");
            }
            foreach (EnvDTE.ProjectItem item in items)
            {
                if (string.Compare(folderName, item.get_FileNames(0), true, CultureInfo.InvariantCulture) == 0)
                {
                    return item;
                }
                if (item.ProjectItems.Count > 0)
                {
                    EnvDTE.ProjectItem item2 = this.FindFolder(folderName, item.ProjectItems);
                    if (item2 != null)
                    {
                        return item2;
                    }
                }
            }
            return null;
        }

        private void GenerateSyncFileRename(string oldFolderPath, string newFolderPath, EnvDTE.ProjectItem item)
        {
            if (oldFolderPath == null)
            {
                throw new ArgumentNullException("oldFolderPath");
            }
            if (newFolderPath == null)
            {
                throw new ArgumentNullException("newFolderPath");
            }
            if (item == null)
            {
                throw new ArgumentNullException("item");
            }
            if (item.ProjectItems.Count > 0)
            {
                bool flag = File.Exists(Path.Combine(newFolderPath, item.Name));
                foreach (EnvDTE.ProjectItem item2 in item.ProjectItems)
                {
                    if (flag)
                    {
                        this.SyncFileRename(Path.Combine(oldFolderPath, item.Name), Path.Combine(newFolderPath, item.Name));
                    }
                    else
                    {
                        this.GenerateSyncFileRename(Path.Combine(oldFolderPath, item.Name), Path.Combine(newFolderPath, item.Name), item2);
                    }
                }
            }
            else
            {
                int num;
                uint num2;
                VSDOCUMENTPRIORITY[] pdwPriority = new VSDOCUMENTPRIORITY[1];
                ErrorHandler.ThrowOnFailure(this.InnerVsProject2.IsDocumentInProject(item.get_FileNames(0), out num, pdwPriority, out num2));
                if (num != 0)
                {
                    this.SyncFileRename(Path.Combine(oldFolderPath, item.Name), Path.Combine(newFolderPath, item.Name));
                }
                else
                {
                    this.SyncDirectoryRemove(Path.Combine(oldFolderPath, item.Name));
                }
            }
        }

        private string GetBuildProperty(string propertyName)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            return this.GetBuildProperty(propertyName, null);
        }

        private string GetBuildProperty(string propertyName, string configurationName)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            string pbstrPropValue = null;
            uint storage = 1;
            ErrorHandler.ThrowOnFailure(this.innerVsBuildPropertyStorage.GetPropertyValue(propertyName, configurationName, storage, out pbstrPropValue), new int[] { -2147170504 });
            return pbstrPropValue;
        }

        public object GetExtender(string ExtenderCATID, string ExtenderName, object ExtendeeObject, IExtenderSite ExtenderSite, int Cookie)
        {
            IExtenderProvider provider = null;
            if (!this.extenderProviders.TryGetValue(ExtenderName, out provider))
            {
                throw new ArgumentException(string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.UnableToFindExtenderProvider, new object[] { ExtenderName }));
            }
            return provider.GetExtender(ExtenderCATID, ExtenderName, ExtendeeObject, ExtenderSite, Cookie);
        }

        public object GetExtenderNames(string ExtenderCATID, object ExtendeeObject)
        {
            List<string> list = new List<string>();
            foreach (KeyValuePair<string, IExtenderProvider> pair in this.extenderProviders)
            {
                if (pair.Value.CanExtend(ExtenderCATID, pair.Key, ExtendeeObject))
                {
                    list.Add(pair.Key);
                }
            }
            return list.ToArray();
        }

        public int GetFile(int fileID, uint grfFlags, out uint pitemid, out string pbstrFilename)
        {
            int num = -2147467263;
            pitemid = uint.MaxValue;
            pbstrFilename = null;
            if (this.InnerIVsHierarchy != null)
            {
                if (((fileID == -1005) || (fileID == -1006)) && !bool.TrueString.Equals(this.languageAdapter.GetCapability("defaultAppSettings")))
                {
                    return num;
                }
                if (base.ImpersonateSilverlightProject && (fileID == -1009))
                {
                    return this.GetWcfServiceReferenceSpecialFile(grfFlags, out pitemid, out pbstrFilename);
                }
                IVsProjectSpecialFiles innerIVsHierarchy = this.InnerIVsHierarchy as IVsProjectSpecialFiles;
                if (innerIVsHierarchy != null)
                {
                    num = innerIVsHierarchy.GetFile(fileID, grfFlags, out pitemid, out pbstrFilename);
                }
            }
            return num;
        }

        private string GetFullThumbnailPath()
        {
            return this.GetFullThumbnailPath(false);
        }

        private string GetFullThumbnailPath(bool reportError)
        {
            string fullPath = string.Empty;
            if (!string.IsNullOrEmpty(this.thumbnail))
            {
                bool flag = false;
                try
                {
                    flag = Path.IsPathRooted(this.thumbnail);
                }
                catch (ArgumentException exception)
                {
                    if (reportError)
                    {
                        string str2 = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.Xbox360ThumbnailInvalidPath, new object[] { exception.Message });
                        this.ErrorProvider.ReportWarning(str2, "XNA.GameStudioPropertiesPage");
                        this.ErrorProvider.BringToFront();
                        return fullPath;
                    }
                }
                if (flag)
                {
                    return this.thumbnail;
                }
                object pvar = null;
                ErrorHandler.ThrowOnFailure(((IVsHierarchy) this).GetProperty(0xfffffffe, -2021, out pvar));
                string str3 = pvar as string;
                string warning = null;
                try
                {
                    fullPath = Path.GetFullPath(Path.Combine(str3, this.thumbnail));
                }
                catch (ArgumentException exception2)
                {
                    warning = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.Xbox360ThumbnailInvalidPath, new object[] { exception2.Message });
                }
                catch (NotSupportedException exception3)
                {
                    warning = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.Xbox360ThumbnailInvalidPath, new object[] { exception3.Message });
                }
                catch (PathTooLongException exception4)
                {
                    warning = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.Xbox360ThumbnailInvalidPath, new object[] { exception4.Message });
                }
                if (reportError && (warning != null))
                {
                    this.ErrorProvider.ReportWarning(warning, "XNA.GameStudioPropertiesPage");
                    this.ErrorProvider.BringToFront();
                }
            }
            return fullPath;
        }

        protected override Guid GetGuidProperty(uint itemId, int propId)
        {
            if (!this.isAggregationComplete)
            {
                throw new InvalidOperationException(XnaProjectResource.InternalErrorProjectInitialization);
            }
            HierarchyItem managedItem = this.hierarchyItemManager.GetManagedItem(itemId);
            if (managedItem != null)
            {
                Guid empty = Guid.Empty;
                ErrorHandler.ThrowOnFailure(managedItem.GetGuidProperty(itemId, propId, out empty));
                return empty;
            }
            return base.GetGuidProperty(itemId, propId);
        }

        private uint GetItemIdForPath(string path)
        {
            if ((this.InnerVsProject2 != null) && !string.IsNullOrEmpty(path))
            {
                uint maxValue = uint.MaxValue;
                int pfFound = 0;
                VSDOCUMENTPRIORITY[] pdwPriority = new VSDOCUMENTPRIORITY[1];
                if ((ErrorHandler.Succeeded(this.InnerVsProject2.IsDocumentInProject(path, out pfFound, pdwPriority, out maxValue)) && (pfFound != 0)) && (maxValue != uint.MaxValue))
                {
                    return maxValue;
                }
            }
            return uint.MaxValue;
        }

        protected override object GetObject(Type objectType)
        {
            if (objectType == typeof(IXnaCodeProject))
            {
                return this.xnaCodeProject;
            }
            if (objectType == typeof(ISolutionBuildStatusProvider))
            {
                return base.ServiceProvider.GetService(typeof(ISolutionBuildStatusProvider));
            }
            object hierarchyObject = this.platformAdapter.GetHierarchyObject(objectType);
            if (hierarchyObject != null)
            {
                return hierarchyObject;
            }
            return base.GetObject(objectType);
        }

        private string GetProjectDirectory()
        {
            object obj2;
            this.GetProperty(0xfffffffe, -2021, out obj2);
            return Microsoft.Xna.GameStudio.ProjectUtilities.EnsureTrailingBackSlash((string) obj2);
        }

        protected override int GetProperty(uint itemId, int propId, out object property)
        {
            if (!this.isAggregationComplete)
            {
                return this.GetPropertyBeforeAggregationComplete(itemId, propId, out property);
            }
            HierarchyItem managedItem = this.hierarchyItemManager.GetManagedItem(itemId);
            if (managedItem != null)
            {
                return managedItem.GetProperty(itemId, propId, out property);
            }
            if (!this.hierarchyItemManager.IsContentReferencesFolderNodeVisible() || ((propId != -1002) && (propId != -2042)))
            {
                return base.GetProperty(itemId, propId, out property);
            }
            if (itemId == this.hierarchyItemManager.ItemBeforeContentReferencesFolder)
            {
                property = (int) this.hierarchyItemManager.ContentReferencesFolderNode.GetItemId();
                return 0;
            }
            int hr = base.GetProperty(itemId, propId, out property);
            if (ErrorHandler.Succeeded(hr))
            {
                uint itemid = (uint) ((int) property);
                if (this.hierarchyItemManager.GetManagedItem(itemid) is ContentReferenceNode)
                {
                    property = (int) this.hierarchyItemManager.GetNextNonContentReferenceItem(itemid, propId);
                }
            }
            return hr;
        }

        private int GetPropertyBeforeAggregationComplete(uint itemId, int propId, out object property)
        {
            if ((propId == -2095) || (propId == -2094))
            {
                property = false;
                return 0;
            }
            if (((propId != -2077) && (propId != -2027)) && ((propId != -2102) && ((propId != -2003) || (itemId != 0xfffffffe))))
            {
                return this.InnerIVsHierarchy.GetProperty(itemId, propId, out property);
            }
            return this.InnerIVsHierarchy.GetProperty(itemId, propId, out property);
        }

        public int GetSdkPath(out string sdkPath)
        {
            IVbCompilerHost languageAdapter = this.languageAdapter as IVbCompilerHost;
            if (languageAdapter != null)
            {
                return languageAdapter.GetSdkPath(out sdkPath);
            }
            sdkPath = string.Empty;
            return -2147467259;
        }

        private string GetSolutionDir()
        {
            object obj2;
            ErrorHandler.ThrowOnFailure(this.GetProperty(0xfffffffe, -2027, out obj2));
            Project project = obj2 as Project;
            Solution solution = project.DTE.Solution;
            if ((solution != null) && !string.IsNullOrEmpty(solution.FullName))
            {
                return Microsoft.Xna.GameStudio.ProjectUtilities.EnsureTrailingBackSlash(Path.GetDirectoryName(solution.FullName));
            }
            return null;
        }

        private Dictionary<string, object> GetSyncProperties(EnvDTE.ProjectItem projItem)
        {
            if (projItem == null)
            {
                throw new ArgumentNullException("projItem");
            }
            Dictionary<string, object> dictionary = new Dictionary<string, object>();
            foreach (string str in Microsoft.Xna.GameStudio.ProjectConstants.SupportedSyncedProperties)
            {
                Property property = projItem.Properties.Item(str);
                dictionary.Add(property.Name, property.Value);
            }
            return dictionary;
        }

        private Dictionary<string, object> GetSyncProperties(uint itemid)
        {
            object property = null;
            ErrorHandler.ThrowOnFailure(this.GetProperty(itemid, -2027, out property));
            EnvDTE.ProjectItem projItem = (EnvDTE.ProjectItem) property;
            return this.GetSyncProperties(projItem);
        }

        public int GetTargetLibraryType(out VBTargetLibraryType targetLibraryType)
        {
            IVbCompilerHost languageAdapter = this.languageAdapter as IVbCompilerHost;
            if (languageAdapter != null)
            {
                return languageAdapter.GetTargetLibraryType(out targetLibraryType);
            }
            targetLibraryType = VBTargetLibraryType.TLB_AutoDetect;
            return -2147467259;
        }

        internal string GetUserProjectProperty(string propertyName)
        {
            string pbstrPropValue = null;
            this.innerVsBuildPropertyStorage.GetPropertyValue(propertyName, string.Empty, 2, out pbstrPropValue);
            return pbstrPropValue;
        }

        private IEnumerable<uint> GetVisibleItems()
        {
            uint itemId = 0xfffffffe;
            Stack<uint> iteratorVariable1 = new Stack<uint>();
            object property = null;
            ErrorHandler.ThrowOnFailure(this.GetProperty(itemId, -2041, out property));
            itemId = (uint) ((int) property);
            while (itemId != uint.MaxValue)
            {
                if (Microsoft.Xna.GameStudio.ProjectUtilities.IsNestedProjectNode(this, itemId))
                {
                    iteratorVariable1.Push(itemId);
                }
                else
                {
                    do
                    {
                        iteratorVariable1.Push(itemId);
                        ErrorHandler.ThrowOnFailure(this.GetProperty(itemId, -2041, out property));
                        itemId = (uint) ((int) property);
                    }
                    while (itemId != uint.MaxValue);
                }
                do
                {
                    itemId = iteratorVariable1.Pop();
                    yield return itemId;
                    ErrorHandler.ThrowOnFailure(this.GetProperty(itemId, -2042, out property));
                    itemId = (uint) ((int) property);
                }
                while ((itemId == uint.MaxValue) && (iteratorVariable1.Count > 0));
            }
        }

        private object GetVsService(Type service)
        {
            return base.ServiceProvider.GetService(service);
        }

        private int GetWcfServiceReferenceSpecialFile(uint grfFlags, out uint pitemid, out string strFilename)
        {
            string path = Path.Combine(this.GetProjectDirectory(), "ServiceReferences.ClientConfig");
            int num = 0;
            pitemid = uint.MaxValue;
            uint itemIdForPath = this.GetItemIdForPath(path);
            if (itemIdForPath != uint.MaxValue)
            {
                pitemid = itemIdForPath;
            }
            else if ((grfFlags & 1) != 0)
            {
                object obj2;
                ErrorHandler.ThrowOnFailure(this.GetProperty(0xfffffffe, -2027, out obj2));
                Project project = obj2 as Project;
                try
                {
                    if (!File.Exists(path))
                    {
                        using (StreamWriter writer = new StreamWriter(path, false, new UTF8Encoding(true)))
                        {
                            writer.WriteLine("<configuration />");
                            writer.Flush();
                        }
                    }
                    project.ProjectItems.AddFromFileCopy(path).Properties.Item("CopyToOutputDirectory").Value = 2;
                    itemIdForPath = this.GetItemIdForPath(path);
                    if (itemIdForPath != uint.MaxValue)
                    {
                        pitemid = itemIdForPath;
                    }
                }
                catch
                {
                    num = -2147467259;
                }
            }
            if ((grfFlags & 2) != 0)
            {
                strFilename = path;
                return num;
            }
            strFilename = "ServiceReferences.ClientConfig";
            return num;
        }

        public int IgnoreItemFileChanges(uint itemid, int fIgnore)
        {
            throw new NotImplementedException();
        }

        protected override void InitializeForOuter(string fileName, string location, string name, uint flags, ref Guid guidProject, out bool cancel)
        {
            cancel = false;
            this.solutionBuildManager2 = this.GetVsService(typeof(IVsSolutionBuildManager)) as IVsSolutionBuildManager2;
            if (this.solutionBuildManager2 == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.MissingService, new object[] { typeof(IVsSolutionBuildManager).FullName }));
            }
            ErrorHandler.ThrowOnFailure(this.solutionBuildManager2.AdviseUpdateSolutionEvents(this, out this.buildEvents2Cookie));
            XnaFilterPropertiesExtenderProvider provider = new XnaFilterPropertiesExtenderProvider(this.xnaCodeProject);
            XnaProjectPropertiesExtenderProvider provider2 = new XnaProjectPropertiesExtenderProvider(this.xnaCodeProject);
            this.extenderProviders.Add(provider.Name, provider);
            this.extenderProviders.Add(provider2.Name, provider2);
            if ((flags & 1) == 1)
            {
                this.isNewProject = true;
            }
            base.InitializeForOuter(fileName, location, name, flags, ref guidProject, out cancel);
        }

        private void InitializeItemManager()
        {
            if (this.hierarchyItemManager == null)
            {
                string buildProperty = this.GetBuildProperty("XnaFrameworkVersion");
                string str2 = this.GetBuildProperty("XnaPlatform");
                if ("Windows Phone".Equals(str2, StringComparison.OrdinalIgnoreCase))
                {
                    base.ImpersonateSilverlightProject = true;
                }
                KeyValuePair<string, string> key = new KeyValuePair<string, string>(buildProperty, str2);
                if (!this.platformAdapters.TryGetValue(key, out this.platformAdapter))
                {
                    this.platformAdapter = new UnsupportedVersionAdapter(str2, buildProperty);
                    Microsoft.Xna.GameStudio.ProjectUtilities.ShowErrorMessage(Package.GetGlobalService(typeof(SVsUIShell)) as IVsUIShell, null, string.Format(XnaProjectResource.UnsupportedPlatform, str2), string.Empty);
                }
                if (this.xnaCodeProject != null)
                {
                    this.languageAdapter = LanguageAdapterFactory.CreateAdapter(this.xnaCodeProject);
                }
                this.hierarchyItemManager = new HierarchyItemManager(this.InnerIVsHierarchy, this.xnaCodeProject);
            }
        }

        public int IsItemDirty(uint itemid, IntPtr punkDocData, out int pfDirty)
        {
            ContentReferencesFolderNode contentReferencesFolderNode = this.hierarchyItemManager.ContentReferencesFolderNode;
            if (itemid == contentReferencesFolderNode.GetItemId())
            {
                pfDirty = 0;
                return 0;
            }
            return this.innerVsPersistHierarchyItem.IsItemDirty(itemid, punkDocData, out pfDirty);
        }

        public int IsItemReloadable(uint itemid, out int pfReloadable)
        {
            pfReloadable = 0;
            return 0;
        }

        int Microsoft.VisualStudio.OLE.Interop.IServiceProvider.QueryService(ref Guid guidService, ref Guid riid, out IntPtr ppvObject)
        {
            int num = 0;
            ppvObject = IntPtr.Zero;
            object service = null;
            if (base.ImpersonateSilverlightProject && typeof(IWcfClientGenerationExtension).GUID.Equals((Guid) guidService))
            {
                if (this.wcfSilverlightClientGenerationExtension == null)
                {
                    this.wcfSilverlightClientGenerationExtension = new WcfSilverlightClientGenerationExtension(this, Microsoft.Xna.GameStudio.ProjectUtilities.GetSilverlightServiceReferenceAssemblyPath());
                }
                service = this.wcfSilverlightClientGenerationExtension;
            }
            if (service == null)
            {
                if (this.languageAdapter.QueryService(ref guidService, ref riid, out service))
                {
                    IntPtr pUnk = Marshal.GetIUnknownForObject(service);
                    num = Marshal.QueryInterface(pUnk, ref riid, out ppvObject);
                    ComUtilities.SmartRelease(ref pUnk);
                    return num;
                }
                if ((this.innerVsHierarchySite == null) && (this.InnerIVsHierarchy != null))
                {
                    this.InnerIVsHierarchy.GetSite(out this.innerVsHierarchySite);
                }
                if (this.innerVsHierarchySite != null)
                {
                    return this.innerVsHierarchySite.QueryService(ref guidService, ref riid, out ppvObject);
                }
                return -2147467259;
            }
            if (riid.Equals(VSConstants.IID_IUnknown))
            {
                ppvObject = Marshal.GetIUnknownForObject(service);
                return 0;
            }
            IntPtr iUnknownForObject = Marshal.GetIUnknownForObject(service);
            num = Marshal.QueryInterface(iUnknownForObject, ref riid, out ppvObject);
            ComUtilities.SmartRelease(ref iUnknownForObject);
            return num;
        }

        public int OnActiveProjectCfgChange(IVsHierarchy pIVsHierarchy)
        {
            return 0;
        }

        protected override void OnAggregationComplete()
        {
            this.InitializeItemManager();
            this.isAggregationComplete = true;
            OleMenuCommandService menuService = ((System.IServiceProvider) this).GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
            if (menuService != null)
            {
                this.menuCommandFilter = new MenuCommandFilter(menuService, CmdIDConstants.OnlyRootCommandsToFilter);
                if (this.platformAdapter.IsDevicePlatform)
                {
                    this.menuCommandFilter.AddRange(CmdIDConstants.DeviceProjectCommandsToFilter);
                    IXnaDeviceDeploymentRegistration registration = this.GetVsService(typeof(IXnaDeviceDeploymentRegistration)) as IXnaDeviceDeploymentRegistration;
                    if (registration != null)
                    {
                        string xnaFrameworkRuntimeProfile = this.platformAdapter.XnaFrameworkRuntimeProfile;
                        if (!string.IsNullOrEmpty(xnaFrameworkRuntimeProfile))
                        {
                            char[] separator = new char[] { ';' };
                            foreach (string str2 in xnaFrameworkRuntimeProfile.Split(separator, StringSplitOptions.RemoveEmptyEntries))
                            {
                                registration.RegisterProject(this, this.platformAdapter.XnaPlatform, str2);
                            }
                        }
                    }
                }
                this.menuCommandFilter.Enable(true);
                this.menuCommands = new MenuCommandHandlers(menuService, this.xnaCodeProject);
                this.menuCommands.Enable(true);
            }
            foreach (INamedExtenderProvider provider in this.platformAdapter.ExtenderProviders)
            {
                this.extenderProviders[provider.Name] = provider;
            }
            this.thumbnail = this.GetBuildProperty("Thumbnail");
            CrossPlatformGroupManager vsService = this.GetVsService(typeof(CrossPlatformGroupManager)) as CrossPlatformGroupManager;
            if (vsService != null)
            {
                vsService.RegisterProject(this.xnaCodeProject);
            }
            base.OnAggregationComplete();
            base.FileRemoved += new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.OnFileRemovedFromProject);
            base.FileRenamed += new System.EventHandler<ProjectDocumentsRenamedEventArgs>(this.OnFileRenamedInProject);
        }

        private void OnFileRemovedFromProject(object sender, ProjectDocumentsChangeEventArgs args)
        {
            if (((args != null) && !string.IsNullOrEmpty(args.MkDocument)) && (string.Compare(this.GetFullThumbnailPath(), args.MkDocument) == 0))
            {
                this.Thumbnail = string.Empty;
            }
        }

        private void OnFileRenamedInProject(object sender, ProjectDocumentsRenamedEventArgs args)
        {
            if ((((args != null) && !string.IsNullOrEmpty(args.MkDocumentOld)) && !string.IsNullOrEmpty(args.MkDocumentNew)) && (string.Compare(this.GetFullThumbnailPath(), args.MkDocumentOld) == 0))
            {
                this.Thumbnail = ProjectAssemblyPath.MakeProjectRelativePath(args.MkDocumentNew, this.xnaCodeProject.GetVsHierarchy());
            }
        }

        public override int OnInvalidateItems(uint itemidParent)
        {
            if (this.hierarchyItemManager != null)
            {
                this.hierarchyItemManager.OnHierarchyChanged();
            }
            return base.OnInvalidateItems(itemidParent);
        }

        public override int OnItemAdded(uint itemidParent, uint itemidSiblingPrev, uint itemidAdded)
        {
            if (((this.hierarchyItemManager != null) && this.hierarchyItemManager.IsManagedItem(itemidParent)) && (itemidAdded != this.hierarchyItemManager.ContentReferencesFolderNode.GetItemId()))
            {
                this.hierarchyItemManager.OnHierarchyChanged();
                if ((this.hierarchyItemManager.IsContentReferencesFolderNodeVisible() && (itemidSiblingPrev != uint.MaxValue)) && (itemidSiblingPrev == this.hierarchyItemManager.ItemBeforeContentReferencesFolder))
                {
                    itemidSiblingPrev = this.hierarchyItemManager.ContentReferencesFolderNode.GetItemId();
                }
                else
                {
                    HierarchyItem managedItem = this.hierarchyItemManager.GetManagedItem(itemidAdded);
                    if (managedItem != null)
                    {
                        object obj2;
                        ErrorHandler.ThrowOnFailure(managedItem.GetProperty(itemidAdded, -1000, out obj2));
                        itemidParent = (uint) ((int) obj2);
                        ContentReferenceNode node = managedItem as ContentReferenceNode;
                        if (node != null)
                        {
                            itemidSiblingPrev = node.PrevContentReferenceItem;
                        }
                    }
                }
            }
            return base.OnItemAdded(itemidParent, itemidSiblingPrev, itemidAdded);
        }

        public override int OnItemDeleted(uint itemid)
        {
            int num;
            bool inDeleteItem = this.inDeleteItem;
            try
            {
                this.inDeleteItem = true;
                if (this.hierarchyItemManager != null)
                {
                    this.hierarchyItemManager.OnItemDeleted(itemid);
                }
                num = base.OnItemDeleted(itemid);
            }
            finally
            {
                this.inDeleteItem = inDeleteItem;
            }
            return num;
        }

        public override int OnItemsAppended(uint itemidParent)
        {
            if (this.hierarchyItemManager != null)
            {
                this.hierarchyItemManager.OnHierarchyChanged();
                ReferencesFolderNode referencesFolderNode = this.hierarchyItemManager.ReferencesFolderNode;
                if (itemidParent == referencesFolderNode.Itemid)
                {
                    base.OnItemsAppended(this.hierarchyItemManager.ContentReferencesFolderNode.GetItemId());
                }
            }
            return base.OnItemsAppended(itemidParent);
        }

        public int OpenDependency(string szDependencyCanonicalName, out IVsDependency ppIVsDependency)
        {
            ppIVsDependency = null;
            return 0;
        }

        public int OutputString(string strTxt)
        {
            IVbCompilerHost languageAdapter = this.languageAdapter as IVbCompilerHost;
            if (languageAdapter != null)
            {
                return languageAdapter.OutputString(strTxt);
            }
            return -2147467259;
        }

        public int QueryAddProjectReference(object pReferencedProject, out int pbCanAdd)
        {
            pbCanAdd = 1;
            IXnaCodeProject hierarchyObject = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>(pReferencedProject as IHierarchyObjectProvider);
            if (hierarchyObject != null)
            {
                string xnaPlatform = hierarchyObject.Adapter.XnaPlatform;
                string xnaFrameworkVersion = hierarchyObject.Adapter.XnaFrameworkVersion;
                if (xnaPlatform != this.platformAdapter.XnaPlatform)
                {
                    string projectName = Microsoft.Xna.GameStudio.ProjectUtilities.GetProjectName(pReferencedProject as IVsHierarchy);
                    string text = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.CrossPlatformReferenceNotSupported, new object[] { projectName, xnaPlatform, this.platformAdapter.XnaPlatform });
                    VsShellMessageBox.Show(null, text, null, 0, OLEMSGBUTTON.OLEMSGBUTTON_OK, OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_FIRST, OLEMSGICON.OLEMSGICON_CRITICAL);
                    pbCanAdd = 0;
                }
                else if (((XnaPackage.Instance != null) && !XnaPackage.Instance.IsProjectUpgradeInProgress) && !string.Equals(xnaFrameworkVersion, this.platformAdapter.XnaFrameworkVersion, StringComparison.OrdinalIgnoreCase))
                {
                    string str5 = Microsoft.Xna.GameStudio.ProjectUtilities.GetProjectName(pReferencedProject as IVsHierarchy);
                    string str6 = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.CrossVersionReferenceNotSupported, new object[] { str5, xnaFrameworkVersion, this.platformAdapter.XnaFrameworkVersion });
                    VsShellMessageBox.Show(null, str6, null, 0, OLEMSGBUTTON.OLEMSGBUTTON_OK, OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_FIRST, OLEMSGICON.OLEMSGICON_CRITICAL);
                    pbCanAdd = 0;
                }
                else
                {
                    string buildProperty = this.GetBuildProperty("XnaRefreshLevel");
                    string b = hierarchyObject.GetBuildProperty("XnaRefreshLevel", null);
                    if (string.IsNullOrWhiteSpace(buildProperty))
                    {
                        buildProperty = "0";
                    }
                    if (string.IsNullOrWhiteSpace(b))
                    {
                        b = "0";
                    }
                    if (!string.Equals(buildProperty, b, StringComparison.OrdinalIgnoreCase))
                    {
                        string str9 = Microsoft.Xna.GameStudio.ProjectUtilities.GetProjectName(pReferencedProject as IVsHierarchy);
                        string str10 = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.CrossRefreshReferenceNotSupported, new object[] { str9 });
                        VsShellMessageBox.Show(null, str10, null, 0, OLEMSGBUTTON.OLEMSGBUTTON_OK, OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_FIRST, OLEMSGICON.OLEMSGICON_CRITICAL);
                        pbCanAdd = 0;
                    }
                }
            }
            return 0;
        }

        public int QueryCanBeReferenced(object pReferencingProject, out int pbAllowReferenced)
        {
            pbAllowReferenced = 1;
            return 0;
        }

        public int QueryDeleteItem(uint dwDelItemOp, uint itemid, out int pfCanDelete)
        {
            pfCanDelete = 0;
            if ((this.hierarchyItemManager.GetContentReferenceNode(itemid) != null) && (dwDelItemOp != 2))
            {
                return 0;
            }
            return this.innerDeleteHandler.QueryDeleteItem(dwDelItemOp, itemid, out pfCanDelete);
        }

        public int QueryRefreshReferences(uint Reason, out int pbUpdate)
        {
            pbUpdate = 1;
            return 0;
        }

        protected override int QueryStatusCommand(uint itemid, ref Guid pguidCmdGroup, uint cCmds, OLECMD[] prgCmds, IntPtr pCmdText)
        {
            if (prgCmds == null)
            {
                throw new ArgumentNullException("prgCmds");
            }
            ContentReferencesFolderNode contentReferencesFolderNode = this.hierarchyItemManager.ContentReferencesFolderNode;
            if (itemid == contentReferencesFolderNode.GetItemId())
            {
                return contentReferencesFolderNode.QueryStatusCommand(itemid, ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
            }
            ContentReferenceNode contentReferenceNode = this.hierarchyItemManager.GetContentReferenceNode(itemid);
            if (contentReferenceNode != null)
            {
                return contentReferenceNode.QueryStatusCommand(itemid, ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
            }
            if (((this.menuCommandFilter != null) && (prgCmds != null)) && (prgCmds.Length > 0))
            {
                bool flag = false;
                for (int i = 0; (i < prgCmds.Length) && (i < cCmds); i++)
                {
                    if (this.menuCommandFilter.IsCommandFiltered(ref pguidCmdGroup, prgCmds[i].cmdID) || this.menuCommands.ShouldFilterCommand(itemid, ref pguidCmdGroup, prgCmds[i].cmdID))
                    {
                        prgCmds[i].cmdf = 0x11;
                        flag = true;
                    }
                }
                if (flag)
                {
                    return 0;
                }
            }
            return base.QueryStatusCommand(itemid, ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
        }

        public int ReloadItem(uint itemid, uint dwReserved)
        {
            throw new InvalidOperationException();
        }

        public int SaveItem(VSSAVEFLAGS dwSave, string pszSilentSaveAsName, uint itemid, IntPtr punkDocData, out int pfCanceled)
        {
            ContentReferencesFolderNode contentReferencesFolderNode = this.hierarchyItemManager.ContentReferencesFolderNode;
            if (itemid == contentReferencesFolderNode.GetItemId())
            {
                pfCanceled = 0;
                return 0;
            }
            return this.innerVsPersistHierarchyItem.SaveItem(dwSave, pszSilentSaveAsName, itemid, punkDocData, out pfCanceled);
        }

        private void SetBuildProperty(string propertyName, string propertyValue)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            if (propertyValue == null)
            {
                propertyValue = string.Empty;
            }
            uint storage = 1;
            ErrorHandler.ThrowOnFailure(this.innerVsBuildPropertyStorage.SetPropertyValue(propertyName, null, storage, propertyValue));
        }

        protected override void SetGuidProperty(uint itemId, int propId, ref Guid guid)
        {
            if (!this.isAggregationComplete)
            {
                throw new InvalidOperationException(XnaProjectResource.InternalErrorProjectInitialization);
            }
            HierarchyItem managedItem = this.hierarchyItemManager.GetManagedItem(itemId);
            if (managedItem != null)
            {
                ErrorHandler.ThrowOnFailure(managedItem.SetGuidProperty(itemId, propId, ref guid));
            }
            else
            {
                base.SetGuidProperty(itemId, propId, ref guid);
            }
        }

        protected override void SetInnerProject(IntPtr innerIUnknown)
        {
            if (innerIUnknown == IntPtr.Zero)
            {
                throw new ArgumentNullException("innerIUnknown");
            }
            object objectForIUnknown = Marshal.GetObjectForIUnknown(innerIUnknown);
            if (base.ServiceProvider == null)
            {
                base.ServiceProvider = this.package;
            }
            base.SetInnerProject(innerIUnknown);
            this.innerVsProjectFlavorCfgProvider = objectForIUnknown as IVsProjectFlavorCfgProvider;
            this.innerVsBuildPropertyStorage = (IVsBuildPropertyStorage) objectForIUnknown;
            this.innerVsProject2 = objectForIUnknown as IVsProject2;
            if (this.innerVsProject2 != null)
            {
                this.innerVsBuildPropertyStorage = new VsBuildPropertyStorageSccWrapper(this.innerVsBuildPropertyStorage, this.innerVsProject2);
            }
            this.innerVsProjectBuildSystem = (IVsProjectBuildSystem) objectForIUnknown;
            this.innerVsDependencyProvider = objectForIUnknown as IVsDependencyProvider;
            this.innerVsPersistHierarchyItem = (IVsPersistHierarchyItem) objectForIUnknown;
            this.innerDeleteHandler = (IVsHierarchyDeleteHandler) objectForIUnknown;
        }

        protected override int SetProperty(uint itemId, int propId, object property)
        {
            if (!this.isAggregationComplete)
            {
                throw new InvalidOperationException(XnaProjectResource.InternalErrorProjectInitialization);
            }
            HierarchyItem managedItem = this.hierarchyItemManager.GetManagedItem(itemId);
            if (managedItem != null)
            {
                return managedItem.SetProperty(itemId, propId, property);
            }
            return base.SetProperty(itemId, propId, property);
        }

        internal void SetUserProjectProperty(string propertyName, string propertyValue)
        {
            this.innerVsBuildPropertyStorage.SetPropertyValue(propertyName, string.Empty, 2, propertyValue);
        }

        internal void SyncDirectoryRemove(string dirPath)
        {
            if (dirPath == null)
            {
                throw new ArgumentNullException("dirPath");
            }
            bool inSyncDirectoryRemove = this.inSyncDirectoryRemove;
            try
            {
                object obj2;
                this.inSyncDirectoryRemove = true;
                ErrorHandler.ThrowOnFailure(this.GetProperty(0xfffffffe, -2027, out obj2));
                Project project = obj2 as Project;
                EnvDTE.ProjectItem item = this.FindFolder(dirPath, project.ProjectItems);
                if (item != null)
                {
                    item.Remove();
                }
            }
            finally
            {
                this.inSyncDirectoryRemove = inSyncDirectoryRemove;
            }
        }

        internal void SyncDirectoryRename(string oldDocumentPath, string newDocumentPath)
        {
            if (oldDocumentPath == null)
            {
                throw new ArgumentNullException("oldDocumentPath");
            }
            if (newDocumentPath == null)
            {
                throw new ArgumentNullException("newDocumentPath");
            }
            string str = oldDocumentPath.TrimEnd(new char[] { Path.DirectorySeparatorChar });
            string str2 = newDocumentPath.TrimEnd(new char[] { Path.DirectorySeparatorChar });
            if (!str.Equals(str2, StringComparison.InvariantCultureIgnoreCase))
            {
                bool inSyncDirectoryRename = this.inSyncDirectoryRename;
                try
                {
                    this.inSyncDirectoryRename = true;
                    object property = null;
                    ErrorHandler.ThrowOnFailure(this.GetProperty(0xfffffffe, -2027, out property));
                    Project project = (Project) property;
                    EnvDTE.ProjectItem folderItem = this.FindFolder(oldDocumentPath, project.ProjectItems);
                    if (folderItem != null)
                    {
                        List<EnvDTE.ProjectItem> list = new List<EnvDTE.ProjectItem>();
                        foreach (EnvDTE.ProjectItem item2 in folderItem.ProjectItems)
                        {
                            list.Add(item2);
                        }
                        foreach (EnvDTE.ProjectItem item3 in list)
                        {
                            this.GenerateSyncFileRename(oldDocumentPath, newDocumentPath, item3);
                        }
                        if (folderItem.ProjectItems.Count == 0)
                        {
                            bool inSyncDirectoryRemove = this.inSyncDirectoryRemove;
                            try
                            {
                                this.inSyncDirectoryRemove = true;
                                folderItem.Remove();
                                return;
                            }
                            finally
                            {
                                this.inSyncDirectoryRemove = inSyncDirectoryRemove;
                            }
                        }
                        this.CreateLocationForFolderItem(folderItem);
                    }
                }
                finally
                {
                    this.inSyncDirectoryRename = inSyncDirectoryRename;
                }
            }
        }

        internal void SyncFileAdd(string documentPath, Dictionary<string, object> properties)
        {
            int num;
            uint num2;
            if (documentPath == null)
            {
                throw new ArgumentNullException("documentPath");
            }
            if (properties == null)
            {
                throw new ArgumentNullException("properties");
            }
            VSDOCUMENTPRIORITY[] pdwPriority = new VSDOCUMENTPRIORITY[1];
            ErrorHandler.ThrowOnFailure(this.InnerVsProject2.IsDocumentInProject(documentPath, out num, pdwPriority, out num2));
            if (num == 0)
            {
                bool inSyncFileAdd = this.inSyncFileAdd;
                try
                {
                    this.inSyncFileAdd = true;
                    object property = null;
                    ErrorHandler.ThrowOnFailure(this.GetProperty(0xfffffffe, -2027, out property));
                    Project project = (Project) property;
                    EnvDTE.ProjectItem item = project.ProjectItems.AddFromFile(documentPath);
                    foreach (string str in properties.Keys)
                    {
                        item.Properties.Item(str).Value = properties[str];
                    }
                }
                finally
                {
                    this.inSyncFileAdd = inSyncFileAdd;
                }
            }
        }

        internal void SyncFileRemove(string documentPath)
        {
            if (documentPath == null)
            {
                throw new ArgumentNullException("documentPath");
            }
            bool inSyncFileRemove = this.inSyncFileRemove;
            try
            {
                int num;
                uint num2;
                this.inSyncFileRemove = true;
                VSDOCUMENTPRIORITY[] pdwPriority = new VSDOCUMENTPRIORITY[1];
                ErrorHandler.ThrowOnFailure(this.InnerVsProject2.IsDocumentInProject(documentPath, out num, pdwPriority, out num2));
                if (num != 0)
                {
                    object property = null;
                    ErrorHandler.ThrowOnFailure(this.GetProperty(num2, -2027, out property));
                    ((EnvDTE.ProjectItem) property).Remove();
                }
            }
            finally
            {
                this.inSyncFileRemove = inSyncFileRemove;
            }
        }

        internal void SyncFileRename(string oldDocumentPath, string newDocumentPath)
        {
            if (oldDocumentPath == null)
            {
                throw new ArgumentNullException("oldDocumentPath");
            }
            if (newDocumentPath == null)
            {
                throw new ArgumentNullException("newDocumentPath");
            }
            bool inSyncFileRename = this.inSyncFileRename;
            try
            {
                int num;
                uint num2;
                this.inSyncFileRename = true;
                VSDOCUMENTPRIORITY[] pdwPriority = new VSDOCUMENTPRIORITY[1];
                ErrorHandler.ThrowOnFailure(this.InnerVsProject2.IsDocumentInProject(oldDocumentPath, out num, pdwPriority, out num2));
                if (num != 0)
                {
                    if (ProjectAssemblyPath.MakeProjectRelativePath(oldDocumentPath, this.xnaCodeProject.GetVsHierarchy()).Equals(this.Thumbnail, StringComparison.InvariantCultureIgnoreCase))
                    {
                        this.Thumbnail = ProjectAssemblyPath.MakeProjectRelativePath(newDocumentPath, this.xnaCodeProject.GetVsHierarchy());
                    }
                    Dictionary<string, object> syncProperties = this.GetSyncProperties(num2);
                    this.SyncFileRemove(oldDocumentPath);
                    this.SyncFileAdd(newDocumentPath, syncProperties);
                    Microsoft.Xna.GameStudio.ProjectUtilities.GetVSProject(this).Refresh();
                }
            }
            finally
            {
                this.inSyncFileRename = inSyncFileRename;
            }
        }

        public int UpdateProjectCfg_Begin(IVsHierarchy pHierProj, IVsCfg pCfgProj, IVsCfg pCfgSln, uint dwAction, ref int pfCancel)
        {
            return 0;
        }

        public int UpdateProjectCfg_Done(IVsHierarchy pHierProj, IVsCfg pCfgProj, IVsCfg pCfgSln, uint dwAction, int fSuccess, int fCancel)
        {
            return 0;
        }

        public int UpdateSolution_Begin(ref int pfCancelUpdate)
        {
            if (this.xnaShouldPackage != null)
            {
                this.xnaShouldPackage.SetAsHostObject(this.innerVsProjectBuildSystem);
            }
            this.isBuilding = true;
            if (this.BuildStatusChanged != null)
            {
                this.BuildStatusChanged(this, EventArgs.Empty);
            }
            return 0;
        }

        public int UpdateSolution_Cancel()
        {
            return 0;
        }

        public int UpdateSolution_Done(int fSucceeded, int fModified, int fCancelCommand)
        {
            this.isBuilding = false;
            try
            {
                if (((this.xnaShouldPackage != null) && (fSucceeded == 0)) && (fCancelCommand == 0))
                {
                    IVsShell vsService = this.GetVsService(typeof(SVsShell)) as IVsShell;
                    if (vsService != null)
                    {
                        object pvar = null;
                        if (ErrorHandler.Succeeded(vsService.GetProperty(-9024, out pvar)) && Convert.ToBoolean(pvar, CultureInfo.InvariantCulture))
                        {
                            this.ErrorProvider.BringToFront();
                        }
                    }
                }
            }
            finally
            {
                this.DoPackaging(false);
            }
            if (this.BuildStatusChanged != null)
            {
                this.BuildStatusChanged(this, EventArgs.Empty);
            }
            return 0;
        }

        public int UpdateSolution_StartUpdate(ref int pfCancelUpdate)
        {
            return 0;
        }

        public string CrossPlatformGroupID
        {
            get
            {
                return this.GetBuildProperty("XnaCrossPlatformGroupID", null);
            }
            set
            {
                CrossPlatformGroupManager vsService = this.GetVsService(typeof(CrossPlatformGroupManager)) as CrossPlatformGroupManager;
                if (vsService != null)
                {
                    vsService.UnregisterProject(this.xnaCodeProject);
                }
                ErrorHandler.ThrowOnFailure(this.xnaCodeProject.InnerVsBuildPropertyStorage.SetPropertyValue("XnaCrossPlatformGroupID", null, 1, value));
                if (vsService != null)
                {
                    vsService.RegisterProject(this.xnaCodeProject);
                }
            }
        }

        private XnaErrorProvider ErrorProvider
        {
            get
            {
                if (this.errorProvider == null)
                {
                    this.errorProvider = new XnaErrorProvider(base.ServiceProvider);
                }
                return this.errorProvider;
            }
        }

        private IVsHierarchy InnerIVsHierarchy
        {
            get
            {
                return base.InnerVsHierarchy;
            }
        }

        private IVsUIHierarchy InnerIVsUIHierarchy
        {
            get
            {
                return base.InnerVsUIHierarchy;
            }
        }

        private IVsBuildPropertyStorage InnerVsBuildPropertyStorage
        {
            get
            {
                return this.innerVsBuildPropertyStorage;
            }
        }

        private IVsProject2 InnerVsProject2
        {
            get
            {
                return this.innerVsProject2;
            }
        }

        private bool IsBuilding
        {
            get
            {
                return this.isBuilding;
            }
        }

        private bool IsEntryProject
        {
            get
            {
                object obj2;
                ErrorHandler.ThrowOnFailure(this.GetProperty(0xfffffffe, -2027, out obj2));
                Project project = obj2 as Project;
                if ((project != null) && (project.Properties != null))
                {
                    try
                    {
                        Property property = project.Properties.Item("OutputType");
                        if ((property != null) && (property.Value != null))
                        {
                            switch (((prjOutputType) property.Value))
                            {
                                case prjOutputType.prjOutputTypeWinExe:
                                case prjOutputType.prjOutputTypeExe:
                                    return true;
                            }
                        }
                    }
                    catch (ArgumentException)
                    {
                    }
                    catch (InvalidCastException)
                    {
                    }
                    if (!string.IsNullOrEmpty(this.GetBuildProperty("XapFilename")))
                    {
                        return true;
                    }
                }
                return false;
            }
        }

        internal IEnumerable<KeyValuePair<string, string>> KnownProjectPlatformAdapters
        {
            get
            {
                foreach (KeyValuePair<string, string> iteratorVariable0 in this.platformAdapters.Keys)
                {
                    yield return iteratorVariable0;
                }
            }
        }

        private bool ShowProjectInSolutionPage
        {
            get
            {
                return this.showProjectInSolutionPage;
            }
            set
            {
                this.showProjectInSolutionPage = value;
            }
        }

        internal bool SupportsCentralDeviceDropdown
        {
            get
            {
                return (((this.platformAdapter != null) && (this.platformAdapter is IProjectPlatformAdapterCaps)) && bool.TrueString.Equals(((IProjectPlatformAdapterCaps) this.platformAdapter).GetCapabilityValue("centralDeviceSelector"), StringComparison.OrdinalIgnoreCase));
            }
        }

        private string Thumbnail
        {
            get
            {
                return this.thumbnail;
            }
            set
            {
                if (this.thumbnail != value)
                {
                    this.SetBuildProperty("Thumbnail", value);
                    this.thumbnail = value;
                    if (this.ThumbnailChanged != null)
                    {
                        this.ThumbnailChanged(this, EventArgs.Empty);
                    }
                }
            }
        }



        private sealed class XnaCodeProject : IXnaCodeProject, System.IServiceProvider
        {
            private XnaProject project;

            event EventHandler IXnaCodeProject.BuildStatusChanged
            {
                add
                {
                    this.project.BuildStatusChanged += value;
                }
                remove
                {
                    this.project.BuildStatusChanged -= value;
                }
            }

            event System.EventHandler<ProjectDocumentsChangeEventArgs> IXnaCodeProject.FileAdded
            {
                add
                {
                    this.project.FileAdded += value;
                }
                remove
                {
                    this.project.FileAdded -= value;
                }
            }

            event System.EventHandler<ProjectDocumentsChangeEventArgs> IXnaCodeProject.FileRemoved
            {
                add
                {
                    this.project.FileRemoved += value;
                }
                remove
                {
                    this.project.FileRemoved -= value;
                }
            }

            event System.EventHandler<ProjectDocumentsRenamedEventArgs> IXnaCodeProject.FileRenamed
            {
                add
                {
                    this.project.FileRenamed += value;
                }
                remove
                {
                    this.project.FileRenamed -= value;
                }
            }

            event EventHandler IXnaCodeProject.ThumbnailChanged
            {
                add
                {
                    this.project.ThumbnailChanged += value;
                }
                remove
                {
                    this.project.ThumbnailChanged -= value;
                }
            }

            internal XnaCodeProject(XnaProject project)
            {
                if (project == null)
                {
                    throw new ArgumentNullException("project");
                }
                this.project = project;
            }

            void IXnaCodeProject.DoPackaging(bool shouldPackage)
            {
                this.project.DoPackaging(shouldPackage);
            }

            string IXnaCodeProject.GetBuildProperty(string propertyName, string configurationName)
            {
                if (propertyName == null)
                {
                    throw new ArgumentNullException("propertyName");
                }
                return this.project.GetBuildProperty(propertyName, configurationName);
            }

            string IXnaCodeProject.GetFullThumbnailPath()
            {
                return this.project.GetFullThumbnailPath();
            }

            string IXnaCodeProject.GetSolutionDir()
            {
                return this.project.GetSolutionDir();
            }

            IEnumerable<uint> IXnaCodeProject.GetVisibleItems()
            {
                return this.project.GetVisibleItems();
            }

            IVsHierarchy IXnaCodeProject.GetVsHierarchy()
            {
                return Microsoft.Xna.GameStudio.ProjectUtilities.GetVsHierarchy(this.project);
            }

            object IXnaCodeProject.GetVsService(Type service)
            {
                return this.project.GetVsService(service);
            }

            bool IXnaCodeProject.InDeleteItem()
            {
                return this.project.inDeleteItem;
            }

            public void SyncDirectoryRemove(string dirPath)
            {
                this.project.SyncDirectoryRemove(dirPath);
            }

            public void SyncDirectoryRename(string oldDirpath, string newDirpath)
            {
                this.project.SyncDirectoryRename(oldDirpath, newDirpath);
            }

            public void SyncFileAdd(string filepath, Dictionary<string, object> properties)
            {
                this.project.SyncFileAdd(filepath, properties);
            }

            public void SyncFileRemove(string filepath)
            {
                this.project.SyncFileRemove(filepath);
            }

            public void SyncFileRename(string oldFilepath, string newFilepath)
            {
                this.project.SyncFileRename(oldFilepath, newFilepath);
            }

            object System.IServiceProvider.GetService(Type serviceType)
            {
                if (serviceType == typeof(IXnaCodeProject))
                {
                    return this;
                }
                return ((IXnaCodeProject) this).GetVsService(serviceType);
            }

            public string CrossPlatformGroupID
            {
                get
                {
                    return this.project.CrossPlatformGroupID;
                }
                set
                {
                    this.project.CrossPlatformGroupID = value;
                }
            }

            public bool IsNewProject
            {
                get
                {
                    return this.project.isNewProject;
                }
            }

            IProjectPlatformAdapter IXnaCodeProject.Adapter
            {
                get
                {
                    return this.project.platformAdapter;
                }
            }

            XnaErrorProvider IXnaCodeProject.ErrorProvider
            {
                get
                {
                    return this.project.ErrorProvider;
                }
            }

            HierarchyItemManager IXnaCodeProject.HierarchyItemManager
            {
                get
                {
                    return this.project.hierarchyItemManager;
                }
            }

            IVsHierarchy IXnaCodeProject.InnerIVsHierarchy
            {
                get
                {
                    return this.project.InnerIVsHierarchy;
                }
            }

            IVsUIHierarchy IXnaCodeProject.InnerIVsUIHierarchy
            {
                get
                {
                    return this.project.InnerIVsUIHierarchy;
                }
            }

            IVsBuildPropertyStorage IXnaCodeProject.InnerVsBuildPropertyStorage
            {
                get
                {
                    return this.project.InnerVsBuildPropertyStorage;
                }
            }

            IVsProject IXnaCodeProject.InnerVsProject
            {
                get
                {
                    return this.project.InnerVsProject2;
                }
            }

            bool IXnaCodeProject.IsBuilding
            {
                get
                {
                    return this.project.IsBuilding;
                }
            }

            bool IXnaCodeProject.IsEntryProject
            {
                get
                {
                    return this.project.IsEntryProject;
                }
            }

            IEnumerable<KeyValuePair<string, string>> IXnaCodeProject.KnownProjectPlatformAdapters
            {
                get
                {
                    return this.project.KnownProjectPlatformAdapters;
                }
            }

            IProjectLanguageAdapter IXnaCodeProject.LanguageAdapter
            {
                get
                {
                    return this.project.languageAdapter;
                }
            }

            bool IXnaCodeProject.ShowProjectInSolutionPage
            {
                get
                {
                    return this.project.ShowProjectInSolutionPage;
                }
                set
                {
                    this.project.ShowProjectInSolutionPage = value;
                }
            }

            string IXnaCodeProject.Thumbnail
            {
                get
                {
                    return this.project.Thumbnail;
                }
                set
                {
                    this.project.Thumbnail = value;
                }
            }

            string IXnaCodeProject.XnaFrameworkVersion
            {
                get
                {
                    return this.project.platformAdapter.XnaFrameworkVersion;
                }
            }

            public string[] SdkPaths
            {
                get
                {
                    if (this.project.platformAdapter == null)
                    {
                        return null;
                    }
                    if (this.project.platformAdapter is IXnaRefreshLevel)
                    {
                        return ((IXnaRefreshLevel) this.project.platformAdapter).GetRefreshSdkPaths(this.project.GetBuildProperty("XnaRefreshLevel"));
                    }
                    return this.project.platformAdapter.GetDeviceSdkPaths();
                }
            }

            public string TargetFrameworkMoniker
            {
                get
                {
                    object obj2;
                    this.project.GetProperty(0xfffffffe, -2102, out obj2);
                    return (string) obj2;
                }
            }
        }
    }
}

