﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Text.RegularExpressions;

    [ComVisible(false), Guid("2D60D749-60A7-4138-AD75-387F2D223103")]
    internal sealed class XnaProjectFactory : FlavoredProjectFactoryBase, IVsProjectFlavorUpgradeViaFactory
    {
        private static readonly int[] allowedUpgradeFailures = new int[] { -2147170504 };
        private Package package;
        private Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> platformAdapters;

        public XnaProjectFactory(Package package, Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> platformAdapters) : base(false)
        {
            if (package == null)
            {
                throw new ArgumentNullException("package");
            }
            if (platformAdapters == null)
            {
                throw new ArgumentNullException("platformAdapters");
            }
            this.package = package;
            this.platformAdapters = platformAdapters;
        }

        private string DetermineDefaultGraphicsProfile(string platform)
        {
            if (string.IsNullOrEmpty(platform))
            {
                throw new ArgumentNullException("platform");
            }
            if (platform.Equals("Windows Phone", StringComparison.InvariantCultureIgnoreCase))
            {
                return "Reach";
            }
            if (platform.Equals("Windows", StringComparison.InvariantCultureIgnoreCase))
            {
                return "Reach";
            }
            if (!platform.Equals("Xbox 360", StringComparison.InvariantCultureIgnoreCase))
            {
                throw new Exception("Invalid platform " + platform);
            }
            return "HiDef";
        }

        public int OnUpgradeProjectFlavorCancelled(string pszFileName)
        {
            return -2147467263;
        }

        protected override object PreCreateForOuter(IntPtr outerProjectIUnknown)
        {
            if (outerProjectIUnknown != IntPtr.Zero)
            {
                throw new COMException(XnaProjectResource.InvalidOperationProjectNotFlavorable, -2147221232);
            }
            return new XnaProject(this.package, this.platformAdapters);
        }

        private bool ShouldUpgradeProject(IVsUpgradeBuildPropertyStorage pUpgradeBuildPropStg)
        {
            string str;
            ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.GetPropertyValue("XnaFrameworkVersion", null, 1, out str), allowedUpgradeFailures);
            if (string.IsNullOrEmpty(str) || (!(str == "v3.0") && !(str == "v3.1")))
            {
                return false;
            }
            return true;
        }

        public int UpgradeProjectFlavor(string pszFileName, IVsUpgradeBuildPropertyStorage pUpgradeBuildPropStg, string pszProjFileXMLFragment, string pszUserFileXMLFragment, IVsUpgradeLogger pLogger, out int pUpgradeRequired, string[] pbstrUpgradedProjFileXMLFragment, string[] pbstrUpgradedUserFileXMLFragment, out Guid pguidNewProjectFactory)
        {
            string str2;
            Array array;
            Array array2;
            string str5;
            string fileNameWithoutExtension = Path.GetFileNameWithoutExtension(pszFileName);
            pguidNewProjectFactory = Guid.Empty;
            if (!this.ShouldUpgradeProject(pUpgradeBuildPropStg))
            {
                pUpgradeRequired = 0;
                return 0;
            }
            pUpgradeRequired = 1;
            ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.GetPropertyValue("XnaPlatform", null, 1, out str2), allowedUpgradeFailures);
            if (!string.IsNullOrEmpty(str2) && str2.Equals("Zune", StringComparison.InvariantCultureIgnoreCase))
            {
                ErrorHandler.ThrowOnFailure(pLogger.LogMessage(1, fileNameWithoutExtension, pszFileName, XnaProjectResource.ErrorNoZuneUpgrade));
                pUpgradeRequired = 0;
                return 0;
            }
            ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.SetPropertyValue("XnaUpgrade", null, 1, "1"));
            ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.GetImports(out array, out array2));
            int index = 0;
            Regex regex = new Regex(@"(.*\\Microsoft\\XNA Game Studio).*Microsoft.Xna.GameStudio.Common.targets", RegexOptions.CultureInvariant | RegexOptions.IgnoreCase);
            Regex regex2 = new Regex(@".*\\Microsoft\\XNA Game Studio.*Microsoft.Xna.GameStudio.NestedContent.targets", RegexOptions.CultureInvariant | RegexOptions.IgnoreCase);
            foreach (string str3 in array)
            {
                string pszOldCondition = (string) array2.GetValue(index);
                Match match = regex.Match(str3);
                if (match.Success && (match.Captures.Count > 0))
                {
                    ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.ReplaceImport(str3, pszOldCondition, ((match.Groups.Count > 0) ? match.Groups[1].Value : @"$(MSBuildExtensionsPath)\Microsoft\XNA Game Studio") + @"\Microsoft.Xna.GameStudio.targets", pszOldCondition));
                }
                if (regex2.IsMatch(str3))
                {
                    ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.RemoveImport(str3, pszOldCondition));
                }
                index++;
            }
            ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.GetPropertyValue("OutputType", null, 1, out str5), allowedUpgradeFailures);
            if (str5.Equals("WinExe", StringComparison.InvariantCultureIgnoreCase) || str5.Equals("Exe", StringComparison.InvariantCultureIgnoreCase))
            {
                str5 = "Game";
            }
            ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.SetPropertyValue("XnaOutputType", null, 1, str5));
            string pszPropValue = this.DetermineDefaultGraphicsProfile(str2);
            ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.SetPropertyValue("XnaProfile", null, 1, pszPropValue));
            ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.SetPropertyValue("XnaFrameworkVersion", null, 1, "v4.0"));
            ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.SetPropertyValue("TargetFrameworkVersion", null, 1, "v4.0"));
            ErrorHandler.ThrowOnFailure(pUpgradeBuildPropStg.SetPropertyValue("TargetFrameworkProfile", null, 1, "Client"));
            ErrorHandler.ThrowOnFailure(pLogger.LogMessage(0, fileNameWithoutExtension, pszFileName, XnaProjectResource.UpgradedXnaProject));
            return 0;
        }

        public int UpgradeProjectFlavor_CheckOnly(string pszFileName, IVsUpgradeBuildPropertyStorage pUpgradeBuildPropStg, string pszProjFileXMLFragment, string pszUserFileXMLFragment, IVsUpgradeLogger pLogger, out int pUpgradeRequired, out Guid pguidNewProjectFactory)
        {
            pguidNewProjectFactory = Guid.Empty;
            pUpgradeRequired = this.ShouldUpgradeProject(pUpgradeBuildPropStg) ? 1 : 0;
            return 0;
        }

        internal sealed class AdapterKeyComparer : IEqualityComparer<KeyValuePair<string, string>>
        {
            public bool Equals(KeyValuePair<string, string> x, KeyValuePair<string, string> y)
            {
                return ((x.Key == y.Key) && (x.Value == y.Value));
            }

            public int GetHashCode(KeyValuePair<string, string> obj)
            {
                return (obj.Key + ";" + obj.Value).GetHashCode();
            }
        }
    }
}

