﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Runtime.InteropServices;

    internal class XnaProjectNode : ProjectHierarchyItem
    {
        private int contentReferenceIconIndex;
        private int contentReferenceMissingIconIndex;
        private IXnaCodeProject project;
        private int projectContentReferenceFolderIconIndex;
        private IProjectPlatformAdapter projectExtender;
        private int projectIconIndex;
        private IProjectLanguageAdapter projectLanguage;
        private IVsUserContext userContext;

        internal XnaProjectNode(IXnaCodeProject project) : base(project.InnerIVsUIHierarchy)
        {
            this.projectIconIndex = -1;
            this.projectContentReferenceFolderIconIndex = -1;
            this.contentReferenceIconIndex = -1;
            this.contentReferenceMissingIconIndex = -1;
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            if (project.InnerIVsHierarchy == null)
            {
                throw new ArgumentException(XnaProjectResource.ProjectNotInitialized, "project");
            }
            this.project = project;
            this.projectLanguage = project.LanguageAdapter;
            this.projectExtender = project.Adapter;
            if (this.projectExtender.IsDevicePlatform)
            {
                base.OverrideProperty(-2083, false);
                base.OverrideProperty(-2055, new HierarchyItem.GetPropertyDelegate(this.GetShowProjInSolutionPage));
            }
            if (this.projectExtender.SupportsServiceReferences)
            {
                base.OverrideProperty(-2095, true);
            }
            base.OverrideProperty(-1001, new HierarchyItem.GetPropertyDelegate(this.GetFirstChild));
            base.OverrideProperty(-2041, new HierarchyItem.GetPropertyDelegate(this.GetFirstChild));
            base.OverrideProperty(-2004, new HierarchyItem.GetPropertyDelegate(this.GetIconImgList));
            base.OverrideProperty(-2005, new HierarchyItem.GetPropertyDelegate(this.GetIconIndex));
            base.OverrideProperty(-2015, new HierarchyItem.GetPropertyDelegate(this.GetIconIndex));
            base.OverrideProperty(-2065, new HierarchyItem.GetPropertyDelegate(this.GetPropertyPagesCLSIDList));
            base.OverrideProperty(-2066, new HierarchyItem.GetPropertyDelegate(this.GetCfgPropertyPagesCLSIDList));
            base.OverrideProperty(-2089, new HierarchyItem.GetPropertyDelegate(this.GetPriorityPropertyPagesCLSIDList));
            base.OverrideProperty(-2023, new HierarchyItem.GetPropertyDelegate(this.GetUserContext));
            base.OverrideProperty(-2090, true);
            Guid addItemTemplatesGuid = this.AddItemTemplatesGuid;
            base.OverrideGuidProperty(-2070, ref addItemTemplatesGuid);
        }

        private string AdditionalXnaCfgPropertyPages()
        {
            return (string.Join(";", this.project.LanguageAdapter.PropertyPageHandler.NewCfgPropertyPages) + ";" + typeof(ContentCompressionPropertyPage).GUID.ToString("B"));
        }

        private string AdditionalXnaPropertyPages()
        {
            return (string.Join(";", this.project.LanguageAdapter.PropertyPageHandler.NewPropertyPages) + ";" + typeof(XnaGameStudioPropPage).GUID.ToString("B"));
        }

        private void EnsureImageListInitialized()
        {
            if (this.projectIconIndex < 0)
            {
                this.contentReferenceMissingIconIndex = base.AddImageListIcon(XnaProjectResource.MissingContentReference);
                this.contentReferenceIconIndex = base.AddImageListIcon(XnaProjectResource.ContentReference1);
                this.projectIconIndex = base.AddImageListIcon(this.projectExtender.ProjectNodeIcon);
                this.projectContentReferenceFolderIconIndex = base.AddImageListIcon(XnaProjectResource.ContentReferencesFolderImage);
            }
        }

        private string FilterPagesByAdapterAndProgrammingLanguage(string propertyPagesList)
        {
            foreach (string str in this.project.LanguageAdapter.PropertyPageHandler.FilterAndTranslatePropertyPages(this.projectExtender.FilteredPropertyPages()))
            {
                propertyPagesList = propertyPagesList.Replace(str, "");
            }
            propertyPagesList = string.Join(";", propertyPagesList.Split(new char[] { ';' }, StringSplitOptions.RemoveEmptyEntries));
            return propertyPagesList;
        }

        private int GetCfgPropertyPagesCLSIDList(uint itemid, int propid, out object property)
        {
            property = null;
            string propertyPagesList = this.InnerHierarchyPropertyPageList(itemid, propid, ref property);
            property = this.FilterPagesByAdapterAndProgrammingLanguage(propertyPagesList) + ";" + this.AdditionalXnaCfgPropertyPages();
            return 0;
        }

        private int GetFirstChild(uint itemid, int propid, out object property)
        {
            HierarchyItemManager hierarchyItemManager = this.project.HierarchyItemManager;
            if ((hierarchyItemManager != null) && hierarchyItemManager.IsNestedProjectFirstChildOfRoot)
            {
                property = (int) hierarchyItemManager.ContentReferencesFolderNode.GetItemId();
                return 0;
            }
            return base.InnerHierarchy.GetProperty(itemid, propid, out property);
        }

        private int GetIconImgList(uint itemid, int propid, out object property)
        {
            this.EnsureImageListInitialized();
            property = base.IconImageList.Handle;
            return 0;
        }

        private int GetIconIndex(uint itemid, int propid, out object property)
        {
            this.EnsureImageListInitialized();
            property = this.projectIconIndex;
            return 0;
        }

        private int GetPriorityPropertyPagesCLSIDList(uint itemid, int propid, out object property)
        {
            property = null;
            string propertyPagesList = this.InnerHierarchyPropertyPageList(itemid, propid, ref property);
            List<string> list = (this.FilterPagesByAdapterAndProgrammingLanguage(propertyPagesList) + ";" + this.AdditionalXnaPropertyPages() + ";" + this.AdditionalXnaCfgPropertyPages()).Split(new char[] { ';' }, StringSplitOptions.RemoveEmptyEntries).ToList<string>();
            string item = typeof(XnaGameStudioPropPage).GUID.ToString("B");
            int index = list.IndexOf(item);
            if (index > 0)
            {
                list.RemoveAt(index);
                list.Insert(0, item);
                index = 0;
            }
            List<string> propertyPages = new List<string> { "{5E9A8AC2-4F34-4521-858F-4C248BA31532}" };
            item = string.Join(";", this.project.LanguageAdapter.PropertyPageHandler.TranslatePropertyPages(propertyPages));
            int num3 = -1;
            if (!string.IsNullOrWhiteSpace(item))
            {
                num3 = list.IndexOf(item);
                if (num3 > 1)
                {
                    list.Remove(item);
                    list.Insert(1, item);
                    num3 = 1;
                }
            }
            string str3 = typeof(ContentCompressionPropertyPage).GUID.ToString("B");
            if (list.IndexOf(str3) >= 0)
            {
                list.Remove(str3);
                int num5 = -1;
                string buildPageGuidString = this.project.LanguageAdapter.PropertyPageHandler.BuildPageGuidString;
                if (!string.IsNullOrWhiteSpace(buildPageGuidString))
                {
                    num5 = list.IndexOf(buildPageGuidString);
                }
                list.Insert((num5 >= 0) ? (num5 + 1) : ((num3 >= 0) ? (num3 + 1) : ((index >= 0) ? (index + 1) : 0)), str3);
            }
            property = string.Join(";", list);
            return 0;
        }

        private int GetPropertyPagesCLSIDList(uint itemid, int propid, out object property)
        {
            property = null;
            string propertyPagesList = this.InnerHierarchyPropertyPageList(itemid, propid, ref property);
            property = this.FilterPagesByAdapterAndProgrammingLanguage(propertyPagesList) + ";" + this.AdditionalXnaPropertyPages();
            return 0;
        }

        private int GetShowProjInSolutionPage(uint itemid, int propid, out object property)
        {
            property = null;
            property = this.project.ShowProjectInSolutionPage;
            return 0;
        }

        private int GetUserContext(uint itemid, int propid, out object property)
        {
            property = null;
            if (this.userContext == null)
            {
                IVsMonitorUserContext vsService = this.project.GetVsService(typeof(SVsMonitorUserContext)) as IVsMonitorUserContext;
                if (vsService == null)
                {
                    throw new InvalidOperationException(XnaProjectResource.NoMonitorUserContext);
                }
                ErrorHandler.ThrowOnFailure(vsService.CreateEmptyContext(out this.userContext));
                ErrorHandler.ThrowOnFailure(this.userContext.AddAttribute(VSUSERCONTEXTATTRIBUTEUSAGE.VSUC_Usage_Filter, "Product", this.projectExtender.HelpContextFilters.Product));
                ErrorHandler.ThrowOnFailure(this.userContext.AddAttribute(VSUSERCONTEXTATTRIBUTEUSAGE.VSUC_Usage_Filter, "ProjType", this.projectExtender.HelpContextFilters.ProjType));
                ErrorHandler.ThrowOnFailure(this.userContext.AddAttribute(VSUSERCONTEXTATTRIBUTEUSAGE.VSUC_Usage_Filter, "Project", this.projectExtender.HelpContextFilters.Project));
            }
            property = this.userContext;
            return 0;
        }

        private string InnerHierarchyPropertyPageList(uint itemid, int propid, ref object property)
        {
            ErrorHandler.ThrowOnFailure(base.InnerHierarchy.GetProperty(itemid, propid, out property));
            if (property != null)
            {
                return ((string) property).ToUpperInvariant();
            }
            return string.Empty;
        }

        public virtual Guid AddItemTemplatesGuid
        {
            get
            {
                if (this.projectLanguage == null)
                {
                    return this.projectExtender.AddItemTemplatesGuid;
                }
                return this.projectLanguage.GetItemTemplatesGuid(this.projectExtender.GetType().GUID);
            }
        }

        internal int ContentReferenceFolderIconIndex
        {
            get
            {
                this.EnsureImageListInitialized();
                return this.projectContentReferenceFolderIconIndex;
            }
        }

        internal int ContentReferenceIconIndex
        {
            get
            {
                this.EnsureImageListInitialized();
                return this.contentReferenceIconIndex;
            }
        }

        internal int ContentReferenceMissingIconIndex
        {
            get
            {
                this.EnsureImageListInitialized();
                return this.contentReferenceMissingIconIndex;
            }
        }
    }
}

