﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.Build.Framework;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Diagnostics.CodeAnalysis;

    public sealed class XnaShouldPackage : ITaskHost
    {
        private IServiceProvider serviceProvider;
        private bool shouldPackage;
        private const string TargetName = "XnaPackageBuild";
        private const string TaskName = "XnaClubPackageTask";
        private string thumbnail;

        internal XnaShouldPackage(IServiceProvider serviceProvider)
        {
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            this.serviceProvider = serviceProvider;
        }

        [SuppressMessage("Microsoft.Security", "CA2109:ReviewVisibleEventHandlers")]
        public void OnBeginPackaging(object sender, EventArgs args)
        {
            this.UpdateStatusText(null, XnaProjectResource.CreateXnaPackageStatus);
        }

        [SuppressMessage("Microsoft.Security", "CA2109:ReviewVisibleEventHandlers")]
        public void OnEndPackaging(object sender, EventArgs args)
        {
            this.UpdateStatusText(XnaProjectResource.CreateXnaPackageStatus, XnaProjectResource.FinishXnaPackageStatus);
        }

        public void SetAsHostObject(IVsProjectBuildSystem projectBuildSystem)
        {
            if (projectBuildSystem == null)
            {
                throw new ArgumentNullException("projectBuildSystem");
            }
            ErrorHandler.ThrowOnFailure(projectBuildSystem.SetHostObject("XnaPackageBuild", "XnaClubPackageTask", this));
        }

        private void UpdateStatusText(string oldStatus, string newStatus)
        {
            if (newStatus == null)
            {
                throw new ArgumentNullException("newStatus");
            }
            IVsStatusbar service = this.serviceProvider.GetService(typeof(IVsStatusbar)) as IVsStatusbar;
            if (service != null)
            {
                if (oldStatus != null)
                {
                    string pszText = null;
                    if (ErrorHandler.Succeeded(service.GetText(out pszText)) && (string.CompareOrdinal(oldStatus, pszText) == 0))
                    {
                        service.SetText(newStatus);
                    }
                }
                else
                {
                    service.SetText(newStatus);
                }
            }
        }

        public bool ShouldPackage
        {
            get
            {
                return this.shouldPackage;
            }
            set
            {
                this.shouldPackage = value;
            }
        }

        public string Thumbnail
        {
            get
            {
                return this.thumbnail;
            }
            set
            {
                this.thumbnail = value;
            }
        }
    }
}

