﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using Microsoft.Xna.GameStudio;
    using System;
    using System.ComponentModel;
    using System.Globalization;

    internal class ContentImporterConverter : StringConverter
    {
        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            ContentPipelineManager pipelineManagerFromContext = ProjectUtilities.GetPipelineManagerFromContext(context);
            if (pipelineManagerFromContext != null)
            {
                return pipelineManagerFromContext.GetImporterName(value as string);
            }
            return base.ConvertFrom(context, culture, value);
        }

        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
            ContentPipelineManager pipelineManagerFromContext = ProjectUtilities.GetPipelineManagerFromContext(context);
            if (pipelineManagerFromContext != null)
            {
                return pipelineManagerFromContext.GetImporterDisplayName(value as string);
            }
            return base.ConvertTo(context, culture, value, destinationType);
        }

        public override TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            ContentPipelineManager pipelineManagerFromContext = ProjectUtilities.GetPipelineManagerFromContext(context);
            if (pipelineManagerFromContext == null)
            {
                return new TypeConverter.StandardValuesCollection(new object[0]);
            }
            return new TypeConverter.StandardValuesCollection(pipelineManagerFromContext.GetImporterNames());
        }

        public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
        {
            return true;
        }

        public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
        {
            return true;
        }
    }
}

