﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using EnvDTE;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.PlatformTools.ContentPipeline.Views;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Diagnostics.CodeAnalysis;
    using System.Globalization;
    using System.Runtime.InteropServices;

    [SuppressMessage("Microsoft.Interoperability", "CA1409:ComVisibleTypesShouldBeCreatable"), ClassInterface(ClassInterfaceType.None), ComVisible(true), Guid("A8C35011-300C-439d-A65A-9911AC6D44D8")]
    public sealed class ContentPipelineExtender : AutomationExtender
    {
        private string assetName;
        private string importer;
        private bool initialized;
        private string processor;

        internal ContentPipelineExtender(object extendeeObject, IExtenderSite extenderSite, int cookie) : base(extendeeObject, extenderSite, cookie)
        {
        }

        internal static bool CanExtend(string extenderCATID, object ExtendeeObject)
        {
            IVsBrowseObject obj2 = ExtendeeObject as IVsBrowseObject;
            if ((obj2 != null) && (string.Compare(extenderCATID, "{8D58E6AF-ED4E-48B0-8C7B-C74EF0735451}", StringComparison.OrdinalIgnoreCase) == 0))
            {
                IVsHierarchy pHier = null;
                uint maxValue = uint.MaxValue;
                object pvar = null;
                ErrorHandler.ThrowOnFailure(obj2.GetProjectItem(out pHier, out maxValue));
                ErrorHandler.ThrowOnFailure(pHier.GetProperty(maxValue, -2044, out pvar));
                return !Convert.ToBoolean(pvar, CultureInfo.InvariantCulture);
            }
            return false;
        }

        private void EnsureInitialized()
        {
            IVsBrowseObject obj2 = base.Object as IVsBrowseObject;
            if (!this.initialized && (obj2 != null))
            {
                this.initialized = true;
                IVsHierarchy pHier = null;
                uint maxValue = uint.MaxValue;
                ErrorHandler.ThrowOnFailure(obj2.GetProjectItem(out pHier, out maxValue));
                IVsBuildPropertyStorage buildPropertyStorage = pHier as IVsBuildPropertyStorage;
                if (buildPropertyStorage != null)
                {
                    LoadAttribute(buildPropertyStorage, maxValue, "Name", out this.assetName);
                    LoadAttribute(buildPropertyStorage, maxValue, "Importer", out this.importer);
                    LoadAttribute(buildPropertyStorage, maxValue, "Processor", out this.processor);
                }
            }
        }

        public override vsFilterProperties IsPropertyHidden(string PropertyName)
        {
            if ((string.CompareOrdinal(PropertyName, "CustomTool") != 0) && (string.CompareOrdinal(PropertyName, "CustomToolNamespace") != 0))
            {
                return base.IsPropertyHidden(PropertyName);
            }
            return vsFilterProperties.vsFilterPropertiesAll;
        }

        private static void LoadAttribute(IVsBuildPropertyStorage buildPropertyStorage, uint itemid, string propertyName, out string value)
        {
            ErrorHandler.ThrowOnFailure(buildPropertyStorage.GetItemAttribute(itemid, propertyName, out value), new int[] { -2147170504 });
            if (value != null)
            {
                value.Trim();
            }
        }

        private void SetContentPipelineProperty(string propertyName, string propertyValue)
        {
            propertyValue = propertyValue.Trim();
            IVsBrowseObject obj2 = base.Object as IVsBrowseObject;
            if (obj2 != null)
            {
                IVsHierarchy pHier = null;
                uint maxValue = uint.MaxValue;
                ErrorHandler.ThrowOnFailure(obj2.GetProjectItem(out pHier, out maxValue));
                IVsBuildPropertyStorage propertyStorage = pHier as IVsBuildPropertyStorage;
                if (propertyStorage != null)
                {
                    IVsProject project = pHier as IVsProject;
                    if (project != null)
                    {
                        propertyStorage = new VsBuildPropertyStorageSccWrapper(propertyStorage, project);
                    }
                    ErrorHandler.ThrowOnFailure(propertyStorage.SetItemAttribute(maxValue, propertyName, propertyValue));
                }
            }
        }

        [XnaDisplayName("AssetName"), HelpKeyword("XNA.ContentProject.Properties"), XnaDescription("AssetNameDescription"), Category("XNA Framework Content Pipeline")]
        public string AssetName
        {
            get
            {
                this.EnsureInitialized();
                return this.assetName;
            }
            set
            {
                if (value != null)
                {
                    value = value.Trim();
                }
                if (string.IsNullOrEmpty(value))
                {
                    throw new ArgumentNullException("value");
                }
                if (string.CompareOrdinal(this.assetName, value) != 0)
                {
                    this.SetContentPipelineProperty("Name", value);
                    this.assetName = value;
                }
            }
        }

        [HelpKeyword("XNA.ContentProject.Properties"), XnaDescription("ContentImporterDescription"), Category("XNA Framework Content Pipeline"), TypeConverter(typeof(ContentImporterConverter)), XnaDisplayName("ContentImporter")]
        public string Importer
        {
            get
            {
                this.EnsureInitialized();
                return this.importer;
            }
            set
            {
                if (string.CompareOrdinal(this.importer, value) != 0)
                {
                    this.SetContentPipelineProperty("Importer", value);
                    this.importer = value;
                }
            }
        }

        internal static string Name
        {
            get
            {
                return typeof(ContentPipelineExtender).FullName;
            }
        }

        [RefreshProperties(RefreshProperties.Repaint), XnaDisplayName("ContentProcessor"), XnaDescription("ContentProcessorDescription"), Category("XNA Framework Content Pipeline"), TypeConverter(typeof(ContentProcessorConverter)), HelpKeyword("XNA.ContentProject.Properties")]
        public string Processor
        {
            get
            {
                this.EnsureInitialized();
                return this.processor;
            }
            set
            {
                if (string.CompareOrdinal(this.processor, value) != 0)
                {
                    if (!string.IsNullOrEmpty(this.processor))
                    {
                        ContentPipelineManager pipelineManagerFromContext = Microsoft.Xna.GameStudio.ProjectUtilities.GetPipelineManagerFromContext(base.Object);
                        if (pipelineManagerFromContext != null)
                        {
                            IList<IParameterDescriptor> processorParameters = pipelineManagerFromContext.GetProcessorParameters(this.processor);
                            if (processorParameters != null)
                            {
                                foreach (IParameterDescriptor descriptor in processorParameters)
                                {
                                    ParameterPropertyDescriptor.Create(descriptor, base.Object as IVsBrowseObject).RemoveMetadata();
                                }
                            }
                        }
                    }
                    this.SetContentPipelineProperty("Processor", value);
                    this.processor = value;
                }
            }
        }
    }
}

