﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.PlatformTools.ContentPipeline.Views;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Diagnostics.CodeAnalysis;
    using System.Globalization;

    internal class ContentProcessorConverter : StringConverter
    {
        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            ContentPipelineManager pipelineManagerFromContext = ProjectUtilities.GetPipelineManagerFromContext(context);
            if (pipelineManagerFromContext != null)
            {
                return pipelineManagerFromContext.GetProcessorName(value as string);
            }
            return base.ConvertFrom(context, culture, value);
        }

        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
            ContentPipelineManager pipelineManagerFromContext = ProjectUtilities.GetPipelineManagerFromContext(context);
            if (pipelineManagerFromContext != null)
            {
                return pipelineManagerFromContext.GetProcessorDisplayName(value as string);
            }
            return base.ConvertTo(context, culture, value, destinationType);
        }

        private static IList<IParameterDescriptor> GetProcessorParameters(ITypeDescriptorContext context)
        {
            IList<IParameterDescriptor> processorParameters = null;
            ContentPipelineManager pipelineManagerFromContext = ProjectUtilities.GetPipelineManagerFromContext(context);
            if (((pipelineManagerFromContext != null) && (context != null)) && ((context.PropertyDescriptor != null) && (context.Instance != null)))
            {
                string str = context.PropertyDescriptor.GetValue(context.Instance) as string;
                if (!string.IsNullOrEmpty(str))
                {
                    processorParameters = pipelineManagerFromContext.GetProcessorParameters(str);
                }
            }
            return processorParameters;
        }

        public override PropertyDescriptorCollection GetProperties(ITypeDescriptorContext context, object value, Attribute[] attributes)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            Array instance = context.Instance as Array;
            IList<IParameterDescriptor> processorParameters = GetProcessorParameters(context);
            if ((processorParameters != null) && (processorParameters.Count > 0))
            {
                List<PropertyDescriptor> list2 = new List<PropertyDescriptor>(processorParameters.Count);
                if (instance != null)
                {
                    foreach (IParameterDescriptor descriptor in processorParameters)
                    {
                        PropertyDescriptor[] descriptors = new PropertyDescriptor[instance.Length];
                        for (int i = 0; i < instance.Length; i++)
                        {
                            descriptors[i] = ParameterPropertyDescriptor.Create(descriptor, instance.GetValue(i) as IVsBrowseObject);
                        }
                        list2.Add(new SimpleMergePropertyDescriptor(descriptors));
                    }
                    return new PropertyDescriptorCollection(list2.ToArray());
                }
                IVsBrowseObject browseObject = context.Instance as IVsBrowseObject;
                if (browseObject != null)
                {
                    foreach (IParameterDescriptor descriptor2 in processorParameters)
                    {
                        list2.Add(ParameterPropertyDescriptor.Create(descriptor2, browseObject));
                    }
                    return new PropertyDescriptorCollection(list2.ToArray());
                }
            }
            return new PropertyDescriptorCollection(new PropertyDescriptor[0]);
        }

        public override bool GetPropertiesSupported(ITypeDescriptorContext context)
        {
            IList<IParameterDescriptor> processorParameters = GetProcessorParameters(context);
            return ((processorParameters != null) && (processorParameters.Count > 0));
        }

        [SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId="System.ArgumentException.#ctor(System.String)"), SuppressMessage("Microsoft.Usage", "CA2208:InstantiateArgumentExceptionsCorrectly")]
        public override TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            ContentPipelineManager pipelineManagerFromContext = ProjectUtilities.GetPipelineManagerFromContext(context);
            if (pipelineManagerFromContext == null)
            {
                return new TypeConverter.StandardValuesCollection(new object[0]);
            }
            return new TypeConverter.StandardValuesCollection(pipelineManagerFromContext.GetProcessorNames());
        }

        public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
        {
            return true;
        }

        public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
        {
            return true;
        }
    }
}

