﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.RegistrationAttributes;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Globalization;
    using System.IO;
    using System.Reflection;
    using System.Runtime.InteropServices;
    using System.Xml;

    [ProjectFileFilter(typeof(ContentProjectFactory), "Audio Files", 300, 0x44c), ProjectFileFilter(typeof(ContentProjectFactory), "Model Files", 0x12e, 0x514), Guid("96E2B04D-8817-42c6-938A-82C39BA4D311"), ProjectType(typeof(ContentProjectFactory), typeof(XnaPackage), null, "#200", "contentproj", "contentproj", null, LanguageVsTemplate="CSharp", ShowOnlySpecifiedTemplatesVsTemplate=true, TemplateGroupIDsVsTemplate="XnaFramework-v4.0-Content", TemplateIDsVsTemplate="Microsoft.CSharp.Bitmap"), ProjectFileFilter(typeof(ContentProjectFactory), "XNA Content Files", 0x132, 100), ProjectFileFilter(typeof(ContentProjectFactory), "Texture Files", 0x12d, 0x4b0), ComVisible(false), ProjectFileFilter(typeof(ContentProjectFactory), "Effect Files", 0x12f, 0x578), ProjectFileFilter(typeof(ContentProjectFactory), "Font Files", 0x130, 0x5dc), ProjectFileFilter(typeof(ContentProjectFactory), "XML Files", 0x131, 0x640), ProjectFileFilter(typeof(ContentProjectFactory), "Video Files", 0x135, 0x6a4)]
    internal sealed class ContentProjectFactory : FlavoredProjectFactoryBase, IVsProjectUpgradeViaFactory
    {
        private const string CSharpProjectGuid = "{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}";
        private Package package;
        private static readonly string ThisProjectGuid = typeof(ContentProjectFactory).GUID.ToString("B");

        public ContentProjectFactory(Package package) : base(true)
        {
            if (package == null)
            {
                throw new ArgumentNullException("package");
            }
            this.package = package;
        }

        protected override void CreateProject(string fileName, string location, string name, uint flags, ref Guid projectGuid, out IntPtr project, out int canceled)
        {
            project = IntPtr.Zero;
            canceled = 0;
            string pszProjectTypeGuids = this.ProjectTypeGuids(fileName);
            int hr = ((IVsCreateAggregateProject) ((IServiceProvider) this.package).GetService(typeof(SVsCreateAggregateProject))).CreateAggregateProject(pszProjectTypeGuids, fileName, location, name, flags, ref projectGuid, out project);
            if (hr == -2147467260)
            {
                canceled = 1;
            }
            ErrorHandler.ThrowOnFailure(hr, new int[] { -2146233088 });
            if (hr == -2146233088)
            {
                throw new COMException(Marshal.GetExceptionForHR(hr).Message, -2147467259);
            }
        }

        public int GetSccInfo(string bstrProjectFileName, out string pbstrSccProjectName, out string pbstrSccAuxPath, out string pbstrSccLocalPath, out string pbstrProvider)
        {
            pbstrSccProjectName = null;
            pbstrSccLocalPath = null;
            pbstrSccAuxPath = null;
            pbstrProvider = null;
            XmlDocument document = new XmlDocument();
            document.Load(bstrProjectFileName);
            XmlNamespaceManager nsmgr = new XmlNamespaceManager(document.NameTable);
            nsmgr.AddNamespace("msbuild", "http://schemas.microsoft.com/developer/msbuild/2003");
            XmlNode node = document.SelectSingleNode("//msbuild:SccProjectName", nsmgr);
            if (node != null)
            {
                pbstrSccProjectName = node.InnerText;
            }
            XmlNode node2 = document.SelectSingleNode("//msbuild:SccLocalPath", nsmgr);
            if (node2 != null)
            {
                pbstrSccLocalPath = node2.InnerText;
            }
            XmlNode node3 = document.SelectSingleNode("//msbuild:SccAuxPath", nsmgr);
            if (node3 != null)
            {
                pbstrSccAuxPath = node3.InnerText;
            }
            XmlNode node4 = document.SelectSingleNode("//msbuild:SccProvider", nsmgr);
            if (node4 != null)
            {
                pbstrProvider = node4.InnerText;
            }
            return 0;
        }

        protected override object PreCreateForOuter(IntPtr outerProjectIUnknown)
        {
            if (outerProjectIUnknown != IntPtr.Zero)
            {
                throw new COMException(XnaProjectResource.InvalidOperationProjectNotFlavorable, -2147221232);
            }
            return new ProjectHierarchy(this.package);
        }

        protected override string ProjectTypeGuids(string file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }
            return (ThisProjectGuid + ";{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}");
        }

        public int UpgradeProject(string bstrFileName, uint fUpgradeFlag, string bstrCopyLocation, out string pbstrUpgradedFullyQualifiedFileName, IVsUpgradeLogger pLogger, out int pUpgradeRequired, out Guid pguidNewProjectFactory)
        {
            pbstrUpgradedFullyQualifiedFileName = null;
            pUpgradeRequired = 1;
            pguidNewProjectFactory = Guid.Empty;
            XmlDocument document = new XmlDocument();
            document.Load(bstrFileName);
            XmlNamespaceManager nsmgr = new XmlNamespaceManager(document.NameTable);
            nsmgr.AddNamespace("msbuild", "http://schemas.microsoft.com/developer/msbuild/2003");
            XmlElement element = document.SelectSingleNode("//msbuild:Project", nsmgr) as XmlElement;
            if (element == null)
            {
                ErrorHandler.ThrowOnFailure(pLogger.LogMessage(2, Path.GetFileNameWithoutExtension(bstrFileName), bstrFileName, XnaProjectResource.UpgradeNestedContentFailure));
                return -2147213339;
            }
            XmlNode node = document.SelectSingleNode("//msbuild:PropertyGroup[not(@Condition)]", nsmgr);
            if (node == null)
            {
                ErrorHandler.ThrowOnFailure(pLogger.LogMessage(2, Path.GetFileNameWithoutExtension(bstrFileName), bstrFileName, XnaProjectResource.UpgradeNestedContentFailure));
                return -2147213339;
            }
            XmlNode node2 = document.SelectSingleNode("//msbuild:XnaFrameworkVersion", nsmgr);
            if ((node2 != null) && (node2.InnerText == "v4.0"))
            {
                pUpgradeRequired = 0;
                return 0;
            }
            IVsQueryEditQuerySave2 globalService = Package.GetGlobalService(typeof(SVsQueryEditQuerySave)) as IVsQueryEditQuerySave2;
            if (globalService != null)
            {
                string[] rgpszMkDocuments = new string[] { bstrFileName };
                uint rgfQueryEdit = 0xc2;
                uint pfEditVerdict = 0;
                uint prgfMoreInfo = 0;
                ErrorHandler.ThrowOnFailure(globalService.QueryEditFiles(rgfQueryEdit, 1, rgpszMkDocuments, new uint[1], new VSQEQS_FILE_ATTRIBUTE_DATA[1], out pfEditVerdict, out prgfMoreInfo));
                if (pfEditVerdict != 0)
                {
                    return -2147213339;
                }
                if (element.Attributes["ToolsVersion"] == null)
                {
                    XmlAttribute attribute = document.CreateAttribute("ToolsVersion");
                    element.Attributes.Append(attribute);
                }
                element.SetAttribute("ToolsVersion", "4.0");
                XmlNode node3 = document.SelectSingleNode(@"//msbuild:Import[@Project='$(MSBuildExtensionsPath)\Microsoft\XNA Game Studio\v3.0\Microsoft.Xna.GameStudio.ContentPipeline.targets']", nsmgr);
                if (node3 != null)
                {
                    node3.Attributes["Project"].Value = @"$(MSBuildExtensionsPath)\Microsoft\XNA Game Studio\$(XnaFrameworkVersion)\Microsoft.Xna.GameStudio.ContentPipeline.targets";
                }
                node3 = document.SelectSingleNode(@"//msbuild:Import[@Project='$(MSBuildExtensionsPath)\Microsoft\XNA Game Studio\v3.1\Microsoft.Xna.GameStudio.ContentPipeline.targets']", nsmgr);
                if (node3 != null)
                {
                    node3.Attributes["Project"].Value = @"$(MSBuildExtensionsPath)\Microsoft\XNA Game Studio\$(XnaFrameworkVersion)\Microsoft.Xna.GameStudio.ContentPipeline.targets";
                }
                XmlNodeList list = document.SelectNodes("//msbuild:Reference[starts-with(@Include, 'Microsoft.Xna')]", nsmgr);
                bool flag = true;
                bool flag2 = true;
                foreach (XmlNode node4 in list)
                {
                    AssemblyName name = new AssemblyName(node4.Attributes["Include"].Value);
                    node4.Attributes["Include"].Value = string.Format("{0}, Version=4.0.0.0, Culture=neutral, PublicKeyToken={1}, processorArchitecture={2}", name.Name, "842cf8be1de50553", name.ProcessorArchitecture);
                    if (name.Name == "Microsoft.Xna.Framework.Content.Pipeline.VideoImporters")
                    {
                        flag = false;
                    }
                    if (name.Name == "Microsoft.Xna.Framework.Content.Pipeline.AudioImporters")
                    {
                        flag2 = false;
                    }
                }
                XmlNode node5 = document.SelectSingleNode("//msbuild:ItemGroup[msbuild:Reference]", nsmgr);
                if (node5 != null)
                {
                    XmlElement element2;
                    if (flag2)
                    {
                        element2 = document.CreateElement("Reference", "http://schemas.microsoft.com/developer/msbuild/2003");
                        element2.SetAttribute("Include", "Microsoft.Xna.Framework.Content.Pipeline.AudioImporters, Version=4.0.0.0, Culture=neutral, PublicKeyToken=842cf8be1de50553, processorArchitecture=MSIL");
                        node5.AppendChild(element2);
                    }
                    if (flag)
                    {
                        element2 = document.CreateElement("Reference", "http://schemas.microsoft.com/developer/msbuild/2003");
                        element2.SetAttribute("Include", string.Format(CultureInfo.InvariantCulture, "Microsoft.Xna.Framework.Content.Pipeline.VideoImporters, Version=4.0.0.0, Culture=neutral, PublicKeyToken={0}", new object[] { "842cf8be1de50553" }));
                        node5.AppendChild(element2);
                    }
                }
                if (node2 != null)
                {
                    node2.InnerText = "v4.0";
                }
                else
                {
                    XmlElement newChild = document.CreateElement("XnaFrameworkVersion", "http://schemas.microsoft.com/developer/msbuild/2003");
                    newChild.InnerText = "v4.0";
                    node.AppendChild(newChild);
                }
                XmlNode node6 = document.SelectSingleNode("//msbuild:TargetFrameworkVersion", nsmgr);
                if (node6 != null)
                {
                    node6.InnerText = "v4.0";
                }
                else
                {
                    XmlElement element4 = document.CreateElement("TargetFrameworkVersion", "http://schemas.microsoft.com/developer/msbuild/2003");
                    element4.InnerText = "v4.0";
                    node.AppendChild(element4);
                }
                if (document.SelectSingleNode("//msbuild:ContentRootDirectory", nsmgr) == null)
                {
                    XmlElement element5 = document.CreateElement("ContentRootDirectory", "http://schemas.microsoft.com/developer/msbuild/2003");
                    element5.InnerText = "Content";
                    node.AppendChild(element5);
                }
                ErrorHandler.ThrowOnFailure(globalService.QuerySaveFile(bstrFileName, 0, null, out pfEditVerdict));
                if (pfEditVerdict != 0)
                {
                    return -2147213339;
                }
                document.Save(bstrFileName);
                ErrorHandler.ThrowOnFailure(pLogger.LogMessage(0, Path.GetFileNameWithoutExtension(bstrFileName), bstrFileName, XnaProjectResource.UpgradedNestedContentProjects));
            }
            return 0;
        }

        public int UpgradeProject_CheckOnly(string bstrFileName, IVsUpgradeLogger pLogger, out int pUpgradeRequired, out Guid pguidNewProjectFactory, out uint pUpgradeProjectCapabilityFlags)
        {
            pUpgradeRequired = 0;
            pUpgradeProjectCapabilityFlags = 0xc0;
            pguidNewProjectFactory = Guid.Empty;
            XmlDocument document = new XmlDocument();
            document.Load(bstrFileName);
            XmlNamespaceManager nsmgr = new XmlNamespaceManager(document.NameTable);
            nsmgr.AddNamespace("msbuild", "http://schemas.microsoft.com/developer/msbuild/2003");
            XmlNode node = document.SelectSingleNode("//msbuild:XnaFrameworkVersion", nsmgr);
            if (((node == null) || string.IsNullOrEmpty(node.InnerText)) || (node.InnerText.Equals("v3.0") || node.InnerText.Equals("v3.1")))
            {
                pUpgradeRequired = 1;
            }
            return 0;
        }
    }
}

