﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using EnvDTE;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Diagnostics.CodeAnalysis;
    using System.IO;
    using System.Reflection;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using System.Security;
    using System.Windows.Forms;

    [ComVisible(true), ClassInterface(ClassInterfaceType.AutoDispatch), SuppressMessage("Microsoft.Interoperability", "CA1409:ComVisibleTypesShouldBeCreatable"), Guid("664B7105-EA83-4599-9E05-AE799AF3B8EC")]
    public sealed class ContentRootDirectoryExtender : AutomationExtender
    {
        private IXnaContentProject extendeeContentProject;
        private Delegate getPropertyDelegate;
        private Delegate setPropertyDelegate;

        internal ContentRootDirectoryExtender(IXnaContentProject extendeeContentProject, object extendeeObject, IExtenderSite extenderSite, int cookie) : base(extendeeObject, extenderSite, cookie)
        {
            if (extendeeContentProject == null)
            {
                throw new ArgumentNullException("extendeeContentProject");
            }
            this.extendeeContentProject = extendeeContentProject;
            MethodInfo method = typeof(ContentRootDirectoryExtender).GetMethod("GetContentRootDirectory", BindingFlags.GetProperty | BindingFlags.NonPublic | BindingFlags.Instance);
            this.getPropertyDelegate = Delegate.CreateDelegate(typeof(GetStringMethod), this, method);
            MethodInfo info2 = typeof(ContentRootDirectoryExtender).GetMethod("SetContentRootDirectory", BindingFlags.SetProperty | BindingFlags.NonPublic | BindingFlags.Instance);
            this.setPropertyDelegate = Delegate.CreateDelegate(typeof(SetStringMethod), this, info2);
        }

        internal static bool CanExtend(string extenderCATID)
        {
            if (string.Compare(extenderCATID, "{4EF9F003-DE95-4d60-96B0-212979F2A857}", StringComparison.OrdinalIgnoreCase) != 0)
            {
                return (string.Compare(extenderCATID, "{610D4614-D0D5-11D2-8599-006097C68E81}", StringComparison.OrdinalIgnoreCase) == 0);
            }
            return true;
        }

        private string GetContentRootDirectory()
        {
            return this.ContentRootDirectory;
        }

        private static void InvalidateCaption(IVsHierarchy hierarchy, uint itemid)
        {
            IXnaContentProject hierarchyObject = ProjectUtilities.GetHierarchyObject<IXnaContentProject>(hierarchy as IHierarchyObjectProvider);
            if (hierarchyObject != null)
            {
                IVsHierarchyEvents vsHierarchy = hierarchyObject.GetVsHierarchy() as IVsHierarchyEvents;
                if (vsHierarchy != null)
                {
                    vsHierarchy.OnPropertyChanged(itemid, -2003, 0);
                }
            }
        }

        private void SetContentRootDirectory(string value)
        {
            this.ContentRootDirectory = value;
        }

        private static void ValidateRelativePath(string value)
        {
            if (!string.IsNullOrEmpty(value))
            {
                if (Path.IsPathRooted(value))
                {
                    throw new ArgumentException(XnaProjectResource.ContentRootDirectoryMustBeRelative, "value");
                }
                Exception exception = null;
                try
                {
                    Path.GetFullPath(value);
                }
                catch (ArgumentException exception2)
                {
                    exception = exception2;
                }
                catch (NotSupportedException exception3)
                {
                    exception = exception3;
                }
                catch (PathTooLongException exception4)
                {
                    exception = exception4;
                }
                if (exception != null)
                {
                    throw new ArgumentException(exception.Message, "value");
                }
                Path.Combine(Path.GetTempPath(), value);
                try
                {
                    DirectoryInfo info = new DirectoryInfo(Path.GetTempPath());
                    info.CreateSubdirectory(value).Delete();
                }
                catch (SecurityException)
                {
                    throw new ArgumentException(XnaProjectResource.ContentRootDirectoryInvalidPath, "value");
                }
                catch (IOException)
                {
                    throw new ArgumentException(XnaProjectResource.ContentRootDirectoryInvalidPath, "value");
                }
                catch (NotSupportedException)
                {
                    throw new ArgumentException(XnaProjectResource.ContentRootDirectoryInvalidPath, "value");
                }
                catch (ArgumentException)
                {
                    throw new ArgumentException(XnaProjectResource.ContentRootDirectoryInvalidPath, "value");
                }
            }
        }

        [XnaDescription("ContentRootDirectoryPropertyDescription"), Category("XNA Framework Content Pipeline"), HelpKeyword("XNA.ContentProject.Overview"), XnaDisplayName("ContentRootDirectoryPropertyName")]
        public string ContentRootDirectory
        {
            get
            {
                Control dispatcher = this.extendeeContentProject.Dispatcher;
                if (dispatcher.InvokeRequired)
                {
                    return (string) dispatcher.Invoke(this.getPropertyDelegate);
                }
                return this.extendeeContentProject.GetBuildProperty("ContentRootDirectory", string.Empty);
            }
            set
            {
                Control dispatcher = this.extendeeContentProject.Dispatcher;
                if (dispatcher.InvokeRequired)
                {
                    dispatcher.Invoke(this.setPropertyDelegate, new object[] { value });
                }
                if (string.IsNullOrEmpty(value))
                {
                    value = ".";
                }
                if (this.ContentRootDirectory != value)
                {
                    if (value != ".")
                    {
                        ValidateRelativePath(value);
                    }
                    this.extendeeContentProject.SetBuildProperty("ContentRootDirectory", string.Empty, value);
                    InvalidateCaption(this.extendeeContentProject.GetVsHierarchy(), 0xfffffffe);
                }
            }
        }

        private delegate string GetStringMethod();

        private delegate void SetStringMethod(string value);
    }
}

