﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using EnvDTE;
    using EnvDTE80;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.ComponentModel;
    using System.Diagnostics.CodeAnalysis;
    using System.Runtime.InteropServices;
    using VSLangProj;

    [ClassInterface(ClassInterfaceType.AutoDispatch), Guid("DB4DD591-D076-4da6-B019-B7CBBE51846B"), SuppressMessage("Microsoft.Interoperability", "CA1409:ComVisibleTypesShouldBeCreatable"), ComVisible(true)]
    public sealed class ProjectBrowseObject : ICustomTypeDescriptor, IVsGetCfgProvider, IVsSpecifyProjectDesignerPages, IVsBrowseObject, IInternalExtenderProvider, ProjectProperties, IConnectionPointContainer
    {
        private ProjectCfgProvider cfgProvider;
        private IConnectionPointContainer innerBrowseCPC;
        private object innerBrowseObject;
        private ProjectProperties innerBrowseProjectProperties;
        private IVsProjectCfgProvider innerProjectCfgProvider;
        private IInternalExtenderProvider outerExtenderProvider;
        private IVsHierarchy outerHierarchy;

        internal ProjectBrowseObject(IVsProjectCfgProvider innerProjectCfgProvider, object innerBrowseObject, IVsHierarchy outerHierarchy)
        {
            if (innerProjectCfgProvider == null)
            {
                throw new ArgumentNullException("innerProjectCfgProvider");
            }
            if (innerBrowseObject == null)
            {
                throw new ArgumentNullException("innerBrowseObject");
            }
            if (outerHierarchy == null)
            {
                throw new ArgumentNullException("outerHierarchy");
            }
            this.innerProjectCfgProvider = innerProjectCfgProvider;
            this.innerBrowseObject = innerBrowseObject;
            this.cfgProvider = new ProjectCfgProvider(this.innerProjectCfgProvider);
            this.outerHierarchy = outerHierarchy;
            this.outerExtenderProvider = this.outerHierarchy as IInternalExtenderProvider;
            this.innerBrowseProjectProperties = innerBrowseObject as ProjectProperties;
            this.innerBrowseCPC = innerBrowseObject as IConnectionPointContainer;
            if ((this.innerBrowseProjectProperties == null) && (this.innerBrowseCPC == null))
            {
                throw new InvalidOperationException("Target of proxy must provide IConnectionPointContainer and ProjectProperties interfaces.");
            }
        }

        public bool CanExtend(string ExtenderCATID, string ExtenderName, object ExtendeeObject)
        {
            return ((this.outerExtenderProvider != null) && this.outerExtenderProvider.CanExtend(ExtenderCATID, ExtenderName, ExtendeeObject));
        }

        public void EnumConnectionPoints(out IEnumConnectionPoints ppEnum)
        {
            this.innerBrowseCPC.EnumConnectionPoints(out ppEnum);
        }

        public void FindConnectionPoint(ref Guid riid, out IConnectionPoint ppCP)
        {
            this.innerBrowseCPC.FindConnectionPoint(ref riid, out ppCP);
        }

        public object get_Extender(string ExtenderName)
        {
            if (this.innerBrowseProjectProperties != null)
            {
                return this.innerBrowseProjectProperties.get_Extender(ExtenderName);
            }
            return null;
        }

        public AttributeCollection GetAttributes()
        {
            return TypeDescriptor.GetAttributes(this.innerBrowseObject);
        }

        public int GetCfgProvider(out IVsCfgProvider ppCfgProvider)
        {
            ppCfgProvider = this.cfgProvider;
            return 0;
        }

        public string GetClassName()
        {
            return TypeDescriptor.GetClassName(this.innerBrowseObject);
        }

        public string GetComponentName()
        {
            return TypeDescriptor.GetComponentName(this.innerBrowseObject);
        }

        public TypeConverter GetConverter()
        {
            return TypeDescriptor.GetConverter(this.innerBrowseObject);
        }

        public EventDescriptor GetDefaultEvent()
        {
            return TypeDescriptor.GetDefaultEvent(this.innerBrowseObject);
        }

        public PropertyDescriptor GetDefaultProperty()
        {
            return TypeDescriptor.GetDefaultProperty(this.innerBrowseObject);
        }

        public object GetEditor(Type editorBaseType)
        {
            return TypeDescriptor.GetEditor(this.innerBrowseObject, editorBaseType);
        }

        public EventDescriptorCollection GetEvents()
        {
            return TypeDescriptor.GetEvents(this.innerBrowseObject);
        }

        public EventDescriptorCollection GetEvents(Attribute[] attributes)
        {
            return TypeDescriptor.GetEvents(this.innerBrowseObject, attributes);
        }

        public object GetExtender(string ExtenderCATID, string ExtenderName, object ExtendeeObject, IExtenderSite ExtenderSite, int Cookie)
        {
            if (this.outerExtenderProvider != null)
            {
                return this.outerExtenderProvider.GetExtender(ExtenderCATID, ExtenderName, ExtendeeObject, ExtenderSite, Cookie);
            }
            return null;
        }

        public object GetExtenderNames(string ExtenderCATID, object ExtendeeObject)
        {
            if (this.outerExtenderProvider != null)
            {
                return this.outerExtenderProvider.GetExtenderNames(ExtenderCATID, ExtendeeObject);
            }
            return null;
        }

        public int GetProjectDesignerPages(CAUUID[] pPages)
        {
            IVsSpecifyProjectDesignerPages innerBrowseObject = this.innerBrowseObject as IVsSpecifyProjectDesignerPages;
            if (innerBrowseObject != null)
            {
                return innerBrowseObject.GetProjectDesignerPages(pPages);
            }
            if (pPages != null)
            {
                pPages[0].cElems = 0;
                pPages[0].pElems = IntPtr.Zero;
            }
            return 0;
        }

        public int GetProjectItem(out IVsHierarchy pHier, out uint pItemid)
        {
            pHier = this.outerHierarchy;
            pItemid = 0xfffffffe;
            return 0;
        }

        public PropertyDescriptorCollection GetProperties()
        {
            return TypeDescriptor.GetProperties(this.innerBrowseObject);
        }

        public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
        {
            return TypeDescriptor.GetProperties(this.innerBrowseObject, attributes);
        }

        public object GetPropertyOwner(PropertyDescriptor pd)
        {
            return this.innerBrowseObject;
        }

        public string __id
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.__id;
                }
                return null;
            }
        }

        public object __project
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.__project;
                }
                return null;
            }
        }

        public string AbsoluteProjectDirectory
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.AbsoluteProjectDirectory;
                }
                return string.Empty;
            }
        }

        public ProjectConfigurationProperties ActiveConfigurationSettings
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.ActiveConfigurationSettings;
                }
                return null;
            }
        }

        public string ActiveFileSharePath
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.ActiveFileSharePath;
                }
                return string.Empty;
            }
        }

        public prjWebAccessMethod ActiveWebAccessMethod
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.ActiveWebAccessMethod;
                }
                return prjWebAccessMethod.prjWebAccessMethodFileShare;
            }
        }

        public string ApplicationIcon
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.ApplicationIcon;
                }
                return string.Empty;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.ApplicationIcon = value;
                }
            }
        }

        public string AssemblyKeyContainerName
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.AssemblyKeyContainerName;
                }
                return string.Empty;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.AssemblyKeyContainerName = value;
                }
            }
        }

        public string AssemblyName
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.AssemblyName;
                }
                return string.Empty;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.AssemblyName = value;
                }
            }
        }

        public string AssemblyOriginatorKeyFile
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.AssemblyOriginatorKeyFile;
                }
                return string.Empty;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.AssemblyOriginatorKeyFile = value;
                }
            }
        }

        public prjOriginatorKeyMode AssemblyOriginatorKeyMode
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.AssemblyOriginatorKeyMode;
                }
                return prjOriginatorKeyMode.prjOriginatorKeyModeNone;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.AssemblyOriginatorKeyMode = value;
                }
            }
        }

        public prjScriptLanguage DefaultClientScript
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.DefaultClientScript;
                }
                return prjScriptLanguage.prjScriptLanguageJScript;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.DefaultClientScript = value;
                }
            }
        }

        public prjHTMLPageLayout DefaultHTMLPageLayout
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.DefaultHTMLPageLayout;
                }
                return prjHTMLPageLayout.prjHTMLPageLayoutFlow;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.DefaultHTMLPageLayout = value;
                }
            }
        }

        public string DefaultNamespace
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.DefaultNamespace;
                }
                return string.Empty;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.DefaultNamespace = value;
                }
            }
        }

        public prjTargetSchema DefaultTargetSchema
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.DefaultTargetSchema;
                }
                return prjTargetSchema.prjTargetSchemaIE32Nav30;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.DefaultTargetSchema = value;
                }
            }
        }

        public bool DelaySign
        {
            get
            {
                return ((this.innerBrowseProjectProperties != null) && this.innerBrowseProjectProperties.DelaySign);
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.DelaySign = value;
                }
            }
        }

        public string ExtenderCATID
        {
            get
            {
                return "{4EF9F003-DE95-4d60-96B0-212979F2A857}";
            }
        }

        public object ExtenderNames
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.ExtenderNames;
                }
                return null;
            }
        }

        public string FileName
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.FileName;
                }
                return string.Empty;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.FileName = value;
                }
            }
        }

        public string FileSharePath
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.FileSharePath;
                }
                return string.Empty;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.FileSharePath = value;
                }
            }
        }

        public string FullPath
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.FullPath;
                }
                return string.Empty;
            }
        }

        public bool LinkRepair
        {
            get
            {
                return ((this.innerBrowseProjectProperties != null) && this.innerBrowseProjectProperties.LinkRepair);
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.LinkRepair = value;
                }
            }
        }

        public string LocalPath
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.LocalPath;
                }
                return string.Empty;
            }
        }

        public string OfflineURL
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.OfflineURL;
                }
                return string.Empty;
            }
        }

        public prjCompare OptionCompare
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.OptionCompare;
                }
                return prjCompare.prjCompareBinary;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.OptionCompare = value;
                }
            }
        }

        public prjOptionExplicit OptionExplicit
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.OptionExplicit;
                }
                return prjOptionExplicit.prjOptionExplicitOff;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.OptionExplicit = value;
                }
            }
        }

        public prjOptionStrict OptionStrict
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.OptionStrict;
                }
                return prjOptionStrict.prjOptionStrictOff;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.OptionStrict = value;
                }
            }
        }

        public string OutputFileName
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.OutputFileName;
                }
                return null;
            }
        }

        public prjOutputType OutputType
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.OutputType;
                }
                return prjOutputType.prjOutputTypeWinExe;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.OutputType = value;
                }
            }
        }

        public prjProjectType ProjectType
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.ProjectType;
                }
                return prjProjectType.prjProjectTypeLocal;
            }
        }

        public string ReferencePath
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.ReferencePath;
                }
                return string.Empty;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.ReferencePath = value;
                }
            }
        }

        public string RootNamespace
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.RootNamespace;
                }
                return string.Empty;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.RootNamespace = value;
                }
            }
        }

        public string ServerExtensionsVersion
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.ServerExtensionsVersion;
                }
                return string.Empty;
            }
        }

        public string StartupObject
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.StartupObject;
                }
                return string.Empty;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.StartupObject = value;
                }
            }
        }

        public string URL
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.URL;
                }
                return string.Empty;
            }
        }

        public prjWebAccessMethod WebAccessMethod
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.WebAccessMethod;
                }
                return prjWebAccessMethod.prjWebAccessMethodFileShare;
            }
            set
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    this.innerBrowseProjectProperties.WebAccessMethod = value;
                }
            }
        }

        public string WebServer
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.WebServer;
                }
                return string.Empty;
            }
        }

        public string WebServerVersion
        {
            get
            {
                if (this.innerBrowseProjectProperties != null)
                {
                    return this.innerBrowseProjectProperties.WebServerVersion;
                }
                return string.Empty;
            }
        }
    }
}

