﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using EnvDTE;
    using EnvDTE80;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Flavor;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using Microsoft.Xna.PlatformTools.ContentPipeline.Views;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel.Design;
    using System.Diagnostics;
    using System.Diagnostics.CodeAnalysis;
    using System.Globalization;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Threading;
    using System.Windows.Forms;
    using VSLangProj;

    [ComVisible(false), SuppressMessage("Microsoft.Design", "CA1001:TypesThatOwnDisposableFieldsShouldBeDisposable"), Guid("3DF8988D-C7CA-4e81-9554-27AA91F00B71")]
    internal sealed class ProjectHierarchy : Microsoft.Xna.GameStudio.FlavoredProject.FlavoredProjectBase, IVsUpdateSolutionEvents2, IVsUpdateSolutionEvents, IVsProjectFlavorCfgProvider, IInternalExtenderProvider, IVsProjectSpecialFiles, IVsProjectFlavorReferences
    {
        private uint buildEvents2Cookie;
        private static EventWaitHandle cancelContentBuild;
        private Microsoft.Xna.GameStudio.ContentProject.ContentPipelineManager contentPipelineManager;
        private Dictionary<string, IExtenderProvider> extenderProviders = new Dictionary<string, IExtenderProvider>();
        private IVsBuildPropertyStorage innerVsBuildPropertyStorage;
        private IVsProjectBuildSystem innerVsProjectBuildSystem;
        private IVsProjectFlavorCfgProvider innerVsProjectFlavorCfgProvider;
        private IXnaContentProject internalProxy;
        private Dictionary<IVsCfg, ProjectFlavorCfg> mapIVsCfgToProjectFlavorCfg = new Dictionary<IVsCfg, ProjectFlavorCfg>();
        private MenuCommandFilter menuCommandFilter;
        private Package package;
        private ContentProjectNode projectNode;
        private ReferencesEvents referencesEvents;
        private IVsSolutionBuildManager2 solutionBuildManager2;
        private string xnaFrameworkVersion;

        internal ProjectHierarchy(Package package)
        {
            if (package == null)
            {
                throw new ArgumentNullException("package");
            }
            this.package = package;
            this.internalProxy = new XnaContentProject(this);
        }

        public bool CanExtend(string ExtenderCATID, string ExtenderName, object ExtendeeObject)
        {
            IExtenderProvider provider = null;
            return (this.extenderProviders.TryGetValue(ExtenderName, out provider) && provider.CanExtend(ExtenderCATID, ExtenderName, ExtendeeObject));
        }

        protected override void Close()
        {
            base.Close();
            base.FileAdded -= new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.OnFileAddedToProject);
            base.FileRenamed -= new System.EventHandler<ProjectDocumentsRenamedEventArgs>(this.OnFileRenamedInProject);
            if ((this.buildEvents2Cookie != 0) && (this.solutionBuildManager2 != null))
            {
                this.solutionBuildManager2.UnadviseUpdateSolutionEvents(this.buildEvents2Cookie);
            }
            this.solutionBuildManager2 = null;
            this.buildEvents2Cookie = 0;
            if (this.menuCommandFilter != null)
            {
                this.menuCommandFilter.Enable(false);
                this.menuCommandFilter = null;
            }
            if ((this.referencesEvents != null) && (this.contentPipelineManager != null))
            {
                this.referencesEvents.remove_ReferenceAdded(new _dispReferencesEvents_ReferenceAddedEventHandler(this.OnReferenceCollectionModified));
                this.referencesEvents.remove_ReferenceChanged(new _dispReferencesEvents_ReferenceChangedEventHandler(this.OnReferenceCollectionModified));
                this.referencesEvents.remove_ReferenceRemoved(new _dispReferencesEvents_ReferenceRemovedEventHandler(this.OnReferenceCollectionModified));
            }
            this.referencesEvents = null;
        }

        public int CreateProjectFlavorCfg(IVsCfg pBaseProjectCfg, out IVsProjectFlavorCfg ppFlavorCfg)
        {
            ppFlavorCfg = null;
            IVsProjectFlavorCfg cfg = null;
            if (pBaseProjectCfg != null)
            {
                string str;
                if (this.innerVsProjectFlavorCfgProvider != null)
                {
                    this.innerVsProjectFlavorCfgProvider.CreateProjectFlavorCfg(pBaseProjectCfg, out cfg);
                }
                ErrorHandler.ThrowOnFailure(pBaseProjectCfg.get_DisplayName(out str));
                string pbstrPropValue = null;
                if (this.innerVsBuildPropertyStorage != null)
                {
                    ErrorHandler.ThrowOnFailure(this.innerVsBuildPropertyStorage.GetPropertyValue("XnaFrameworkVersion", str, 1, out pbstrPropValue));
                }
                ProjectFlavorCfg cfg2 = new Microsoft.Xna.GameStudio.ContentProject.Configuration(pBaseProjectCfg, cfg, pbstrPropValue);
                cfg2.Disposed += new EventHandler(this.OnProjectFlavorCfgClosed);
                this.mapIVsCfgToProjectFlavorCfg.Add(pBaseProjectCfg, cfg2);
                ppFlavorCfg = cfg2;
            }
            return 0;
        }

        private string GetBuildProperty(string propertyName, string configurationName)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            string pbstrPropValue = null;
            uint storage = 1;
            ErrorHandler.ThrowOnFailure(this.innerVsBuildPropertyStorage.GetPropertyValue(propertyName, configurationName, storage, out pbstrPropValue), new int[] { -2147170504 });
            return pbstrPropValue;
        }

        public object GetExtender(string ExtenderCATID, string ExtenderName, object ExtendeeObject, IExtenderSite ExtenderSite, int Cookie)
        {
            IExtenderProvider provider = null;
            if (!this.extenderProviders.TryGetValue(ExtenderName, out provider))
            {
                throw new ArgumentException(string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.UnableToFindExtenderProvider, new object[] { ExtenderName }));
            }
            return provider.GetExtender(ExtenderCATID, ExtenderName, ExtendeeObject, ExtenderSite, Cookie);
        }

        public object GetExtenderNames(string ExtenderCATID, object ExtendeeObject)
        {
            List<string> list = new List<string>();
            foreach (KeyValuePair<string, IExtenderProvider> pair in this.extenderProviders)
            {
                if (pair.Value.CanExtend(ExtenderCATID, pair.Key, ExtendeeObject))
                {
                    list.Add(pair.Key);
                }
            }
            return list.ToArray();
        }

        public int GetFile(int fileID, uint grfFlags, out uint pitemid, out string pbstrFilename)
        {
            pitemid = uint.MaxValue;
            pbstrFilename = null;
            return -2147024809;
        }

        protected override Guid GetGuidProperty(uint itemId, int propId)
        {
            if (itemId == 0xfffffffe)
            {
                Guid empty = Guid.Empty;
                ErrorHandler.ThrowOnFailure(this.projectNode.GetGuidProperty(itemId, propId, out empty), new int[] { -2147467263 });
                return empty;
            }
            return base.GetGuidProperty(itemId, propId);
        }

        protected override object GetObject(System.Type objectType)
        {
            if (objectType == null)
            {
                return null;
            }
            if (objectType.Equals(typeof(IXnaContentProject)))
            {
                return this.internalProxy;
            }
            if (objectType.Equals(typeof(Microsoft.Xna.GameStudio.ContentProject.ContentPipelineManager)))
            {
                return this.ContentPipelineManager;
            }
            if (objectType.Equals(typeof(ISolutionBuildStatusProvider)))
            {
                return base.ServiceProvider.GetService(typeof(ISolutionBuildStatusProvider));
            }
            return base.GetObject(objectType);
        }

        private ProjectFlavorCfg GetProjectFlavorCfg(IVsCfg config)
        {
            if (config == null)
            {
                throw new ArgumentNullException("config");
            }
            ProjectFlavorCfg cfg = null;
            this.mapIVsCfgToProjectFlavorCfg.TryGetValue(config, out cfg);
            return cfg;
        }

        protected override int GetProperty(uint itemId, int propId, out object property)
        {
            if (itemId == 0xfffffffe)
            {
                return this.projectNode.GetProperty(itemId, propId, out property);
            }
            return base.GetProperty(itemId, propId, out property);
        }

        protected override void InitializeForOuter(string fileName, string location, string name, uint flags, ref Guid guidProject, out bool cancel)
        {
            this.solutionBuildManager2 = base.ServiceProvider.GetService(typeof(IVsSolutionBuildManager)) as IVsSolutionBuildManager2;
            if (this.solutionBuildManager2 == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.MissingService, new object[] { typeof(IVsSolutionBuildManager).FullName }));
            }
            ErrorHandler.ThrowOnFailure(this.solutionBuildManager2.AdviseUpdateSolutionEvents(this, out this.buildEvents2Cookie));
            ContentRootDirectoryExtenderProvider provider = new ContentRootDirectoryExtenderProvider(this.internalProxy);
            ContentPipelineExtenderProvider provider2 = new ContentPipelineExtenderProvider();
            this.extenderProviders.Add(provider.Name, provider);
            this.extenderProviders.Add(provider2.Name, provider2);
            base.InitializeForOuter(fileName, location, name, flags, ref guidProject, out cancel);
        }

        private bool IsProjectBrowseOnly()
        {
            return false;
        }

        public int OnActiveProjectCfgChange(IVsHierarchy pIVsHierarchy)
        {
            return 0;
        }

        protected override void OnAggregationComplete()
        {
            OleMenuCommandService menuService = ((System.IServiceProvider) this).GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
            if (menuService != null)
            {
                this.menuCommandFilter = new MenuCommandFilter(menuService, CmdIDConstants.ContentProjectCommandsToFilter);
                this.menuCommandFilter.Enable(true);
            }
            base.FileAdded += new System.EventHandler<ProjectDocumentsChangeEventArgs>(this.OnFileAddedToProject);
            base.FileRenamed += new System.EventHandler<ProjectDocumentsRenamedEventArgs>(this.OnFileRenamedInProject);
            base.OnAggregationComplete();
            this.SetProjectIDGuidOverride();
            Microsoft.Xna.GameStudio.ProjectUtilities.GetVSProject(this).Refresh();
        }

        private void OnFileAddedToProject(object sender, ProjectDocumentsChangeEventArgs args)
        {
            if ((args != null) && !string.IsNullOrEmpty(args.MkDocument))
            {
                IVsProject2 innerVsHierarchy = (IVsProject2) base.InnerVsHierarchy;
                int pfFound = 0;
                uint maxValue = uint.MaxValue;
                if (ErrorHandler.Succeeded(innerVsHierarchy.IsDocumentInProject(args.MkDocument, out pfFound, new VSDOCUMENTPRIORITY[1], out maxValue)) && (pfFound != 0))
                {
                    this.SetBuildActionToCompile(maxValue);
                    string pszAttributeValue = Path.GetFileNameWithoutExtension(args.MkDocument).Trim();
                    string extension = Path.GetExtension(args.MkDocument);
                    string importer = string.Empty;
                    string processor = string.Empty;
                    if (!string.IsNullOrEmpty(extension))
                    {
                        this.ContentPipelineManager.GetDefaultImporterAndProcessor(extension, ref importer, ref processor);
                    }
                    this.innerVsBuildPropertyStorage.SetItemAttribute(maxValue, "Name", pszAttributeValue);
                    if (!string.IsNullOrEmpty(importer))
                    {
                        this.innerVsBuildPropertyStorage.SetItemAttribute(maxValue, "Importer", importer);
                    }
                    if (!string.IsNullOrEmpty(processor))
                    {
                        this.innerVsBuildPropertyStorage.SetItemAttribute(maxValue, "Processor", processor);
                    }
                }
            }
        }

        private void OnFileRenamedInProject(object sender, ProjectDocumentsRenamedEventArgs args)
        {
            if (((args != null) && !string.IsNullOrEmpty(args.MkDocumentOld)) && !string.IsNullOrEmpty(args.MkDocumentNew))
            {
                IVsProject2 innerVsHierarchy = (IVsProject2) base.InnerVsHierarchy;
                int pfFound = 0;
                uint maxValue = uint.MaxValue;
                if (ErrorHandler.Succeeded(innerVsHierarchy.IsDocumentInProject(args.MkDocumentNew, out pfFound, new VSDOCUMENTPRIORITY[1], out maxValue)) && (pfFound != 0))
                {
                    string fileNameWithoutExtension = Path.GetFileNameWithoutExtension(args.MkDocumentOld);
                    string pbstrAttributeValue = null;
                    if (ErrorHandler.Succeeded(this.innerVsBuildPropertyStorage.GetItemAttribute(maxValue, "Name", out pbstrAttributeValue)) && StringComparer.Ordinal.Equals(fileNameWithoutExtension, pbstrAttributeValue))
                    {
                        string pszAttributeValue = Path.GetFileNameWithoutExtension(args.MkDocumentNew);
                        this.innerVsBuildPropertyStorage.SetItemAttribute(maxValue, "Name", pszAttributeValue);
                    }
                    string extension = Path.GetExtension(args.MkDocumentOld);
                    string str5 = Path.GetExtension(args.MkDocumentNew);
                    string importer = string.Empty;
                    string processor = string.Empty;
                    if (!string.IsNullOrEmpty(str5) && !StringComparer.OrdinalIgnoreCase.Equals(extension, str5))
                    {
                        this.ContentPipelineManager.GetDefaultImporterAndProcessor(str5, ref importer, ref processor);
                    }
                    string str8 = string.Empty;
                    this.innerVsBuildPropertyStorage.GetItemAttribute(maxValue, "Importer", out str8);
                    if (!string.IsNullOrEmpty(importer) && !StringComparer.Ordinal.Equals(str8, importer))
                    {
                        this.innerVsBuildPropertyStorage.SetItemAttribute(maxValue, "Importer", importer);
                        string str9 = string.Empty;
                        this.innerVsBuildPropertyStorage.GetItemAttribute(maxValue, "Processor", out str9);
                        IList<IParameterDescriptor> processorParameters = null;
                        if (!string.IsNullOrEmpty(str9))
                        {
                            processorParameters = this.ContentPipelineManager.GetProcessorParameters(str9);
                        }
                        if ((processorParameters != null) && !StringComparer.Ordinal.Equals(str9, processor))
                        {
                            foreach (IParameterDescriptor descriptor in processorParameters)
                            {
                                this.innerVsBuildPropertyStorage.SetItemAttribute(maxValue, "ProcessorParameters_" + descriptor.PropertyName, string.Empty);
                            }
                            this.innerVsBuildPropertyStorage.SetItemAttribute(maxValue, "Processor", processor);
                        }
                    }
                }
            }
        }

        private void OnProjectFlavorCfgClosed(object sender, EventArgs args)
        {
            ProjectFlavorCfg cfg = sender as ProjectFlavorCfg;
            if (cfg != null)
            {
                if ((this.mapIVsCfgToProjectFlavorCfg != null) && this.mapIVsCfgToProjectFlavorCfg.ContainsKey(cfg.BaseCfg))
                {
                    this.mapIVsCfgToProjectFlavorCfg.Remove(cfg.BaseCfg);
                }
                cfg.Disposed -= new EventHandler(this.OnProjectFlavorCfgClosed);
            }
        }

        private void OnReferenceCollectionModified(Reference reference)
        {
            if (reference != null)
            {
                this.ContentPipelineManager.OnReferencesChanged(reference.Collection);
            }
        }

        public int QueryAddProjectReference(object pReferencedProject, out int pbCanAdd)
        {
            pbCanAdd = 1;
            IntPtr pUnk = ComUtilities.QueryInterfaceIUnknown(pReferencedProject);
            IVsBuildPropertyStorage objectForIUnknown = null;
            try
            {
                objectForIUnknown = Marshal.GetObjectForIUnknown(pUnk) as IVsBuildPropertyStorage;
            }
            finally
            {
                ComUtilities.SmartRelease(ref pUnk);
            }
            if (objectForIUnknown != null)
            {
                string pbstrPropValue = null;
                uint storage = 1;
                ErrorHandler.ThrowOnFailure(objectForIUnknown.GetPropertyValue("XnaFrameworkVersion", null, storage, out pbstrPropValue), new int[] { -2147170504 });
                if (((XnaPackage.Instance != null) && !XnaPackage.Instance.IsProjectUpgradeInProgress) && (!string.IsNullOrEmpty(pbstrPropValue) && !string.Equals(pbstrPropValue, this.XnaFrameworkVersion, StringComparison.OrdinalIgnoreCase)))
                {
                    string projectName = Microsoft.Xna.GameStudio.ProjectUtilities.GetProjectName(pReferencedProject as IVsHierarchy);
                    string text = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.CrossVersionReferenceNotSupported, new object[] { projectName, pbstrPropValue, this.XnaFrameworkVersion });
                    VsShellMessageBox.Show(null, text, null, 0, OLEMSGBUTTON.OLEMSGBUTTON_OK, OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_FIRST, OLEMSGICON.OLEMSGICON_CRITICAL);
                    pbCanAdd = 0;
                }
            }
            return 0;
        }

        public int QueryCanBeReferenced(object pReferencingProject, out int pbAllowReferenced)
        {
            pbAllowReferenced = 1;
            return 0;
        }

        public int QueryRefreshReferences(uint Reason, out int pbUpdate)
        {
            pbUpdate = 1;
            return 0;
        }

        protected override int QueryStatusCommand(uint itemid, ref Guid pguidCmdGroup, uint cCmds, Microsoft.VisualStudio.OLE.Interop.OLECMD[] prgCmds, IntPtr pCmdText)
        {
            if ((prgCmds != null) && (prgCmds.Length > 0))
            {
                bool flag = false;
                for (int i = 0; (i < prgCmds.Length) && (i < cCmds); i++)
                {
                    if (this.menuCommandFilter.IsCommandFiltered(ref pguidCmdGroup, prgCmds[i].cmdID))
                    {
                        prgCmds[i].cmdf = 0x11;
                        flag = true;
                    }
                }
                if (flag)
                {
                    return 0;
                }
            }
            return base.QueryStatusCommand(itemid, ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
        }

        private void SetBuildActionToCompile(uint itemid)
        {
            object pvar = null;
            if (ErrorHandler.Succeeded(base.InnerVsHierarchy.GetProperty(itemid, -2027, out pvar)))
            {
                EnvDTE.ProjectItem item = pvar as EnvDTE.ProjectItem;
                try
                {
                    Property property = item.Properties.Item("BuildAction");
                    if (((prjBuildAction) property.Value) != prjBuildAction.prjBuildActionCompile)
                    {
                        property.Value = prjBuildAction.prjBuildActionCompile;
                    }
                }
                catch (ArgumentException)
                {
                }
            }
        }

        private void SetBuildProperty(string propertyName, string configurationName, string propertyValue)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            if (propertyValue == null)
            {
                propertyValue = string.Empty;
            }
            uint storage = 1;
            ErrorHandler.ThrowOnFailure(this.innerVsBuildPropertyStorage.SetPropertyValue(propertyName, configurationName, storage, propertyValue));
        }

        protected override void SetGuidProperty(uint itemId, int propId, ref Guid guid)
        {
            if (itemId == 0xfffffffe)
            {
                ErrorHandler.ThrowOnFailure(this.projectNode.SetGuidProperty(itemId, propId, ref guid));
            }
            base.SetGuidProperty(itemId, propId, ref guid);
        }

        protected override void SetInnerProject(IntPtr innerIUnknown)
        {
            if (innerIUnknown == IntPtr.Zero)
            {
                throw new ArgumentNullException("innerIUnknown");
            }
            object objectForIUnknown = Marshal.GetObjectForIUnknown(innerIUnknown);
            if (base.ServiceProvider == null)
            {
                base.ServiceProvider = this.package;
            }
            base.SetInnerProject(innerIUnknown);
            this.innerVsProjectFlavorCfgProvider = objectForIUnknown as IVsProjectFlavorCfgProvider;
            this.innerVsBuildPropertyStorage = new VsBuildPropertyStorageSccWrapper((IVsBuildPropertyStorage) objectForIUnknown, (IVsProject) objectForIUnknown);
            this.innerVsProjectBuildSystem = (IVsProjectBuildSystem) objectForIUnknown;
            this.projectNode = new ContentProjectNode(base.InnerVsUIHierarchy, base.ServiceProvider, this.internalProxy);
        }

        private void SetProjectIDGuidOverride()
        {
            string buildProperty = this.GetBuildProperty("ProjectGuid", null);
            Guid empty = Guid.Empty;
            try
            {
                empty = new Guid(buildProperty);
            }
            catch (ArgumentNullException)
            {
            }
            catch (FormatException)
            {
            }
            if (empty == Guid.Empty)
            {
                empty = Guid.NewGuid();
                this.SetBuildProperty("ProjectGuid", null, empty.ToString("B"));
            }
            this.projectNode.SetProjectID(ref empty);
        }

        protected override int SetProperty(uint itemId, int propId, object property)
        {
            if (itemId == 0xfffffffe)
            {
                return this.projectNode.SetProperty(itemId, propId, property);
            }
            return base.SetProperty(itemId, propId, property);
        }

        public int UpdateProjectCfg_Begin(IVsHierarchy pHierProj, IVsCfg pCfgProj, IVsCfg pCfgSln, uint dwAction, ref int pfCancel)
        {
            return 0;
        }

        public int UpdateProjectCfg_Done(IVsHierarchy pHierProj, IVsCfg pCfgProj, IVsCfg pCfgSln, uint dwAction, int fSuccess, int fCancel)
        {
            return 0;
        }

        public int UpdateSolution_Begin(ref int pfCancelUpdate)
        {
            int hr = 0;
            if (cancelContentBuild == null)
            {
                string name = string.Format(CultureInfo.InvariantCulture, @"Local\Microsoft.Xna.GameStudio.ContentPipeline.CancelBuildEvent+{0}", new object[] { System.Diagnostics.Process.GetCurrentProcess().Id });
                cancelContentBuild = new EventWaitHandle(false, EventResetMode.ManualReset, name);
            }
            cancelContentBuild.Reset();
            if (ErrorHandler.Failed(hr))
            {
                pfCancelUpdate = Convert.ToInt32(true);
            }
            return hr;
        }

        public int UpdateSolution_Cancel()
        {
            if (cancelContentBuild != null)
            {
                cancelContentBuild.Set();
            }
            return 0;
        }

        public int UpdateSolution_Done(int fSucceeded, int fModified, int fCancelCommand)
        {
            if (cancelContentBuild != null)
            {
                cancelContentBuild.Close();
                cancelContentBuild = null;
            }
            if (this.contentPipelineManager != null)
            {
                this.contentPipelineManager.OnReferencesChanged(Microsoft.Xna.GameStudio.ProjectUtilities.GetVSProject(this).References);
            }
            return 0;
        }

        public int UpdateSolution_StartUpdate(ref int pfCancelUpdate)
        {
            return 0;
        }

        private Microsoft.Xna.GameStudio.ContentProject.ContentPipelineManager ContentPipelineManager
        {
            get
            {
                if (this.contentPipelineManager == null)
                {
                    bool disableAssemblyLoad = this.IsProjectBrowseOnly();
                    VSProject vSProject = Microsoft.Xna.GameStudio.ProjectUtilities.GetVSProject(this);
                    this.contentPipelineManager = new Microsoft.Xna.GameStudio.ContentProject.ContentPipelineManager(vSProject.References, disableAssemblyLoad, this.XnaFrameworkVersion);
                    this.referencesEvents = vSProject.Events.ReferencesEvents;
                    this.referencesEvents.add_ReferenceAdded(new _dispReferencesEvents_ReferenceAddedEventHandler(this.OnReferenceCollectionModified));
                    this.referencesEvents.add_ReferenceChanged(new _dispReferencesEvents_ReferenceChangedEventHandler(this.OnReferenceCollectionModified));
                    this.referencesEvents.add_ReferenceRemoved(new _dispReferencesEvents_ReferenceRemovedEventHandler(this.OnReferenceCollectionModified));
                }
                return this.contentPipelineManager;
            }
        }

        private string XnaFrameworkVersion
        {
            get
            {
                if (this.xnaFrameworkVersion == null)
                {
                    this.xnaFrameworkVersion = this.GetBuildProperty("XnaFrameworkVersion", null);
                    if (this.xnaFrameworkVersion == null)
                    {
                        this.xnaFrameworkVersion = string.Empty;
                    }
                }
                return this.xnaFrameworkVersion;
            }
        }

        private class XnaContentProject : IXnaContentProject
        {
            private Control dispatcher;
            private ProjectHierarchy wrapped;

            internal XnaContentProject(ProjectHierarchy wrapped)
            {
                if (wrapped == null)
                {
                    throw new ArgumentNullException("wrapped");
                }
                this.wrapped = wrapped;
                this.dispatcher = new Control();
            }

            public string GetBuildProperty(string propertyName, string configurationName)
            {
                return this.wrapped.GetBuildProperty(propertyName, configurationName);
            }

            public ProjectFlavorCfg GetProjectFlavorCfg(IVsCfg config)
            {
                if (config == null)
                {
                    throw new ArgumentNullException("config");
                }
                return this.wrapped.GetProjectFlavorCfg(config);
            }

            public System.IServiceProvider GetServiceProvider()
            {
                return this.wrapped.ServiceProvider;
            }

            public IVsHierarchy GetVsHierarchy()
            {
                IntPtr zero = IntPtr.Zero;
                IVsHierarchy typedObjectForIUnknown = null;
                try
                {
                    zero = ComUtilities.QueryInterfaceIUnknown(this.wrapped);
                    typedObjectForIUnknown = (IVsHierarchy) Marshal.GetTypedObjectForIUnknown(zero, typeof(IVsHierarchy));
                }
                finally
                {
                    if (zero != IntPtr.Zero)
                    {
                        Marshal.Release(zero);
                        zero = IntPtr.Zero;
                    }
                }
                return typedObjectForIUnknown;
            }

            public void SetBuildProperty(string propertyName, string configurationName, string propertyValue)
            {
                this.wrapped.SetBuildProperty(propertyName, configurationName, propertyValue);
            }

            public Microsoft.Xna.GameStudio.ContentProject.ContentPipelineManager ContentPipelineManager
            {
                get
                {
                    return this.wrapped.ContentPipelineManager;
                }
            }

            public Control Dispatcher
            {
                get
                {
                    return this.dispatcher;
                }
            }
        }
    }
}

