﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using System;
    using System.ComponentModel;

    internal class SimpleMergePropertyDescriptor : PropertyDescriptor
    {
        private TriState canReset;
        private PropertyDescriptor[] descriptors;
        private TriState localizable;
        private TriState readOnly;

        public SimpleMergePropertyDescriptor(PropertyDescriptor[] descriptors) : base(descriptors[0].Name, null)
        {
            this.descriptors = descriptors;
        }

        public override bool CanResetValue(object component)
        {
            if (this.canReset == TriState.Unknown)
            {
                this.canReset = TriState.Yes;
                foreach (PropertyDescriptor descriptor in this.descriptors)
                {
                    if (!descriptor.CanResetValue(component))
                    {
                        this.canReset = TriState.No;
                        break;
                    }
                }
            }
            return (this.canReset == TriState.Yes);
        }

        protected override AttributeCollection CreateAttributeCollection()
        {
            return this.descriptors[0].Attributes;
        }

        public override object GetEditor(Type editorBaseType)
        {
            return this.descriptors[0].GetEditor(editorBaseType);
        }

        public override object GetValue(object component)
        {
            object obj2 = this.descriptors[0].GetValue(component);
            for (int i = 1; i < this.descriptors.Length; i++)
            {
                object obj3 = this.descriptors[i].GetValue(component);
                if (((obj2 == null) && (obj3 != null)) || ((obj2 != null) && !obj2.Equals(obj3)))
                {
                    return null;
                }
            }
            return obj2;
        }

        public override void ResetValue(object component)
        {
            foreach (PropertyDescriptor descriptor in this.descriptors)
            {
                descriptor.ResetValue(component);
            }
        }

        public override void SetValue(object component, object value)
        {
            foreach (PropertyDescriptor descriptor in this.descriptors)
            {
                descriptor.SetValue(component, value);
            }
        }

        public override bool ShouldSerializeValue(object component)
        {
            foreach (PropertyDescriptor descriptor in this.descriptors)
            {
                if (!descriptor.ShouldSerializeValue(component))
                {
                    return false;
                }
            }
            return true;
        }

        public override Type ComponentType
        {
            get
            {
                return this.descriptors[0].ComponentType;
            }
        }

        public override TypeConverter Converter
        {
            get
            {
                return this.descriptors[0].Converter;
            }
        }

        public override string DisplayName
        {
            get
            {
                return this.descriptors[0].DisplayName;
            }
        }

        public override bool IsLocalizable
        {
            get
            {
                if (this.localizable == TriState.Unknown)
                {
                    this.localizable = TriState.Yes;
                    foreach (PropertyDescriptor descriptor in this.descriptors)
                    {
                        if (!descriptor.IsLocalizable)
                        {
                            this.localizable = TriState.No;
                            break;
                        }
                    }
                }
                return (this.localizable == TriState.Yes);
            }
        }

        public override bool IsReadOnly
        {
            get
            {
                if (this.readOnly == TriState.Unknown)
                {
                    this.readOnly = TriState.No;
                    foreach (PropertyDescriptor descriptor in this.descriptors)
                    {
                        if (descriptor.IsReadOnly)
                        {
                            this.readOnly = TriState.Yes;
                            break;
                        }
                    }
                }
                return (this.readOnly == TriState.Yes);
            }
        }

        public override Type PropertyType
        {
            get
            {
                return this.descriptors[0].PropertyType;
            }
        }

        private enum TriState
        {
            Unknown,
            Yes,
            No
        }
    }
}

