﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Runtime.InteropServices;

    internal class ContentReferencesFolderNode : HierarchyItem, IVsPersistHierarchyItem2, IVsPersistHierarchyItem, IDisposable
    {
        private IXnaCodeProject codeProject;
        private bool expanded;
        private IVsUIHierarchyWindow hierarchyWindow;
        private int iconIndex;
        private uint itemid;
        private IntPtr itemidAsIntPtr;

        internal ContentReferencesFolderNode(IXnaCodeProject codeProject) : base(null)
        {
            if (codeProject == null)
            {
                throw new ArgumentNullException("codeProject");
            }
            this.codeProject = codeProject;
            this.hierarchyWindow = codeProject.GetVsService(typeof(IVsUIHierarchyWindow)) as IVsUIHierarchyWindow;
            this.itemid = uint.MaxValue;
            base.OverrideProperty(-2042, new HierarchyItem.GetPropertyDelegate(this.GetNextSibling));
            base.OverrideProperty(-1002, new HierarchyItem.GetPropertyDelegate(this.GetNextSibling));
            base.OverrideProperty(-1000, -2);
            base.OverrideProperty(-2006, new HierarchyItem.GetPropertyDelegate(this.GetExpandable));
            base.DisableProperty(-2026);
            Guid guidValue = VSConstants.GUID_ItemType_VirtualFolder;
            base.OverrideGuidProperty(-1004, ref guidValue);
            this.iconIndex = 14;
            base.OverrideProperty(-2005, new HierarchyItem.GetPropertyDelegate(this.GetIconIndex));
            base.OverrideProperty(-2015, this.iconIndex);
            string contentReferencesFolderCaption = XnaProjectResource.ContentReferencesFolderCaption;
            base.OverrideProperty(-2012, contentReferencesFolderCaption);
            base.OverrideProperty(-2003, contentReferencesFolderCaption);
            base.OverrideProperty(-1001, new HierarchyItem.GetPropertyDelegate(this.GetFirstChild));
            base.OverrideProperty(-2041, new HierarchyItem.GetPropertyDelegate(this.GetFirstChild));
            base.OverrideProperty(-2035, false);
            base.OverrideProperty(-2044, true);
        }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if (disposing && (this.itemidAsIntPtr != IntPtr.Zero))
            {
                Marshal.FreeBSTR(this.itemidAsIntPtr);
                this.itemidAsIntPtr = IntPtr.Zero;
            }
        }

        internal override int ExecCommand(uint itemid, ref Guid pguidCmdGroup, uint nCmdID, uint nCmdexecopt, IntPtr pvaIn, IntPtr pvaOut)
        {
            ReferencesFolderNode referencesFolderNode = this.codeProject.HierarchyItemManager.ReferencesFolderNode;
            return referencesFolderNode.ExecCommand(referencesFolderNode.Itemid, ref pguidCmdGroup, nCmdID, nCmdexecopt, pvaIn, pvaOut);
        }

        ~ContentReferencesFolderNode()
        {
            this.Dispose(false);
        }

        private int GetExpandable(uint itemid, int propid, out object property)
        {
            property = this.codeProject.HierarchyItemManager.FirstContentReferenceItem != uint.MaxValue;
            return 0;
        }

        private int GetExpanded(uint itemid, int propid, out object property)
        {
            property = this.expanded;
            return 0;
        }

        private int GetFirstChild(uint itemid, int propid, out object property)
        {
            property = (int) this.codeProject.HierarchyItemManager.FirstContentReferenceItem;
            return 0;
        }

        private int GetIconIndex(uint itemid, int propid, out object property)
        {
            property = this.codeProject.HierarchyItemManager.ProjectNode.ContentReferenceFolderIconIndex;
            return 0;
        }

        internal uint GetItemId()
        {
            if (this.itemid == uint.MaxValue)
            {
                this.itemidAsIntPtr = Marshal.StringToBSTR("");
                this.itemid = (uint) this.itemidAsIntPtr.ToInt32();
            }
            return this.itemid;
        }

        private int GetNextSibling(uint itemid, int propid, out object property)
        {
            property = -1;
            if ((propid != -1002) && (propid != -2042))
            {
                return -2147467263;
            }
            HierarchyItemManager hierarchyItemManager = this.codeProject.HierarchyItemManager;
            uint itemBeforeContentReferencesFolder = hierarchyItemManager.ItemBeforeContentReferencesFolder;
            int num2 = propid;
            if (hierarchyItemManager.IsNestedProjectFirstChildOfRoot)
            {
                itemBeforeContentReferencesFolder = 0xfffffffe;
                if (propid == -1002)
                {
                    num2 = -1001;
                }
                else
                {
                    num2 = -2041;
                }
            }
            return this.codeProject.InnerIVsHierarchy.GetProperty(itemBeforeContentReferencesFolder, num2, out property);
        }

        public int IgnoreItemFileChanges(uint itemid, int fIgnore)
        {
            return 0;
        }

        public int IsItemDirty(uint itemid, IntPtr punkDocData, out int pfDirty)
        {
            pfDirty = 0;
            return 0;
        }

        public int IsItemReloadable(uint itemid, out int pfReloadable)
        {
            pfReloadable = 1;
            return 0;
        }

        internal override int QueryStatusCommand(uint itemid, ref Guid pguidCmdGroup, uint cCmds, OLECMD[] prgCmds, IntPtr pCmdText)
        {
            if ((prgCmds != null) && (prgCmds.Length > 0))
            {
                bool flag = false;
                for (int i = 0; (i < prgCmds.Length) && (i < cCmds); i++)
                {
                    if (((pguidCmdGroup == CmdIDConstants.AddReferenceCmdID.Guid) && (prgCmds[i].cmdID == CmdIDConstants.AddReferenceCmdID.ID)) || ((pguidCmdGroup == CmdIDConstants.AddServiceReferenceCmdID.Guid) && (prgCmds[i].cmdID == CmdIDConstants.AddServiceReferenceCmdID.ID)))
                    {
                        prgCmds[i].cmdf = 0x11;
                        flag = true;
                    }
                }
                if (flag)
                {
                    return 0;
                }
            }
            ReferencesFolderNode referencesFolderNode = this.codeProject.HierarchyItemManager.ReferencesFolderNode;
            return referencesFolderNode.QueryStatusCommand(referencesFolderNode.Itemid, ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
        }

        public int ReloadItem(uint itemid, uint dwReserved)
        {
            return 0;
        }

        public int SaveItem(VSSAVEFLAGS dwSave, string pszSilentSaveAsName, uint itemid, IntPtr punkDocData, out int pfCanceled)
        {
            pfCanceled = 0;
            return 0;
        }

        private int SetExpanded(uint itemid, int propid, object property)
        {
            this.expanded = (bool) property;
            return 0;
        }

        internal IVsHierarchy OuterHierarchy
        {
            get
            {
                return this.codeProject.GetVsHierarchy();
            }
        }
    }
}

