﻿namespace Microsoft.Xna.GameStudio
{
    using EnvDTE;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.IO;

    internal class DisableRenameDocuments : TrackProjectDocumentsEventsSink
    {
        private List<string> documentsNotToBeRenamed;

        internal DisableRenameDocuments(IServiceProvider serviceProvider) : base(serviceProvider)
        {
            this.documentsNotToBeRenamed = new List<string>();
        }

        internal void AddDocument(string mkDocument)
        {
            if (mkDocument == null)
            {
                throw new ArgumentNullException("mkDocument");
            }
            mkDocument = mkDocument.ToLowerInvariant();
            if (!this.documentsNotToBeRenamed.Contains(mkDocument))
            {
                this.documentsNotToBeRenamed.Add(mkDocument);
            }
        }

        private static bool ConflictsWithFileLink(IXnaCodeProject xnaProject, string oldPath, string newPath)
        {
            int num;
            uint num2;
            VSDOCUMENTPRIORITY[] pdwPriority = new VSDOCUMENTPRIORITY[1];
            if (!ErrorHandler.Failed(xnaProject.InnerVsProject.IsDocumentInProject(oldPath, out num, pdwPriority, out num2)))
            {
                object obj2;
                if (ErrorHandler.Failed(xnaProject.InnerIVsHierarchy.GetProperty(num2, -2027, out obj2)))
                {
                    return false;
                }
                EnvDTE.ProjectItem item = obj2 as EnvDTE.ProjectItem;
                if ((item == null) || (item.Collection == null))
                {
                    return false;
                }
                string fileName = Path.GetFileName(newPath);
                foreach (EnvDTE.ProjectItem item2 in item.Collection)
                {
                    if (item != item2)
                    {
                        try
                        {
                            if (((bool) item2.Properties.Item("IsLink").Value) && item2.Name.Equals(fileName, StringComparison.InvariantCultureIgnoreCase))
                            {
                                return true;
                            }
                        }
                        catch (ArgumentException)
                        {
                        }
                    }
                }
            }
            return false;
        }

        public override int OnQueryRenameFiles(IVsProject pProject, int cFiles, string[] rgszMkOldNames, string[] rgszMkNewNames, VSQUERYRENAMEFILEFLAGS[] rgFlags, VSQUERYRENAMEFILERESULTS[] pSummaryResult, VSQUERYRENAMEFILERESULTS[] rgResults)
        {
            if (pProject != null)
            {
                if (rgszMkOldNames == null)
                {
                    throw new ArgumentNullException("rgszMkOldNames");
                }
                if (rgszMkNewNames == null)
                {
                    throw new ArgumentNullException("rgszMkNewNames");
                }
                if (rgFlags == null)
                {
                    throw new ArgumentNullException("rgFlags");
                }
                if (pSummaryResult == null)
                {
                    throw new ArgumentNullException("pSummaryResult");
                }
                if (((cFiles > rgszMkOldNames.Length) || (cFiles > rgszMkNewNames.Length)) || ((cFiles > rgFlags.Length) || (0 >= pSummaryResult.Length)))
                {
                    throw new ArgumentOutOfRangeException("cFiles");
                }
                if (pSummaryResult[0] != VSQUERYRENAMEFILERESULTS.VSQUERYRENAMEFILERESULTS_RenameOK)
                {
                    return base.OnQueryRenameFiles(pProject, cFiles, rgszMkOldNames, rgszMkNewNames, rgFlags, pSummaryResult, rgResults);
                }
                IXnaCodeProject hierarchyObject = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>(pProject as IHierarchyObjectProvider);
                for (int i = 0; i < cFiles; i++)
                {
                    IXnaCodeProject project2 = null;
                    if (hierarchyObject != null)
                    {
                        CrossPlatformGroupManager vsService = hierarchyObject.GetVsService(typeof(CrossPlatformGroupManager)) as CrossPlatformGroupManager;
                        if (vsService == null)
                        {
                            if (ConflictsWithFileLink(hierarchyObject, rgszMkOldNames[i], rgszMkNewNames[i]))
                            {
                                project2 = hierarchyObject;
                            }
                        }
                        else
                        {
                            foreach (KeyValuePair<string, IXnaCodeProject> pair in vsService.GetProjects(hierarchyObject.CrossPlatformGroupID))
                            {
                                if (ConflictsWithFileLink(pair.Value, rgszMkOldNames[i], rgszMkNewNames[i]))
                                {
                                    project2 = pair.Value;
                                    break;
                                }
                            }
                        }
                        if (project2 != null)
                        {
                            IVsUIShell uiShell = hierarchyObject.GetVsService(typeof(SVsUIShell)) as IVsUIShell;
                            if (uiShell != null)
                            {
                                string str = ProjectAssemblyPath.MakeProjectRelativePath(rgszMkNewNames[i], project2.GetVsHierarchy());
                                string projectName = Microsoft.Xna.GameStudio.ProjectUtilities.GetProjectName(project2.GetVsHierarchy());
                                string text = string.Format(XnaProjectResource.FileLinkAlreadyExists, str, projectName);
                                Microsoft.Xna.GameStudio.ProjectUtilities.ShowErrorMessage(uiShell, null, text, null);
                            }
                        }
                    }
                    if ((project2 != null) || this.documentsNotToBeRenamed.Contains(rgszMkOldNames[i].ToLowerInvariant()))
                    {
                        pSummaryResult[0] = VSQUERYRENAMEFILERESULTS.VSQUERYRENAMEFILERESULTS_RenameNotOK;
                        if (rgResults != null)
                        {
                            rgResults[i] = VSQUERYRENAMEFILERESULTS.VSQUERYRENAMEFILERESULTS_RenameNotOK;
                        }
                    }
                }
            }
            return 0;
        }

        internal void RemoveDocument(string mkDocument)
        {
            if (mkDocument == null)
            {
                throw new ArgumentNullException("mkDocument");
            }
            mkDocument = mkDocument.ToLowerInvariant();
            if (this.documentsNotToBeRenamed.Contains(mkDocument))
            {
                this.documentsNotToBeRenamed.Remove(mkDocument);
            }
        }
    }
}

