﻿namespace Microsoft.Xna.GameStudio.FlavoredProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Flavor;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel.Design;
    using System.Diagnostics.CodeAnalysis;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Threading;

    internal abstract class FlavoredProjectBase : Microsoft.Xna.GameStudio.FlavoredProject.IVsAggregatableProjectCorrected, System.IServiceProvider, IVsUIHierarchy, IVsHierarchy, IOleCommandTarget, IVsHierarchyEvents, IHierarchyObjectProvider, IVsFilterAddProjectItemDlg2, IVsFilterAddProjectItemDlg
    {
        private DocumentsEventsSink _documentsEventsSink;
        private bool _hierarchyClosed;
        private bool _impersonateSilverlightProjectSupport;
        private int _inExecCommand;
        private IOleCommandTarget _innerOleCommandTarget;
        private Microsoft.Xna.GameStudio.FlavoredProject.IVsAggregatableProjectCorrected _innerVsAggregatableProject;
        private IVsHierarchy _innerVsHierarchy;
        private IVsUIHierarchy _innerVsUIHierarchy;
        private OleMenuCommandService _menuService;
        private bool _pauseDocumentTracking;
        private uint documentEventsCookie;
        private Dictionary<uint, IVsHierarchyEvents> eventListeners;
        private uint innerHierarchyEventsCookie;
        private uint nextAdviseEventsCookie;
        private System.IServiceProvider serviceProvider;

        public event System.EventHandler<ProjectDocumentsChangeEventArgs> DirectoryAdded;

        public event System.EventHandler<ProjectDocumentsChangeEventArgs> DirectoryRemoved;

        public event System.EventHandler<ProjectDocumentsRenamedEventArgs> DirectoryRenamed;

        public event System.EventHandler<ProjectDocumentsChangeEventArgs> FileAdded;

        public event System.EventHandler<ProjectDocumentsChangeEventArgs> FileRemoved;

        public event System.EventHandler<ProjectDocumentsRenamedEventArgs> FileRenamed;

        public event System.EventHandler<ProjectDocumentsChangeEventArgs> SccStatusChanged;

        public FlavoredProjectBase()
        {
            this._documentsEventsSink = new DocumentsEventsSink(this);
        }

        protected virtual uint AdviseHierarchyEvents(IVsHierarchyEvents eventSink)
        {
            if (this.eventListeners == null)
            {
                this.eventListeners = new Dictionary<uint, IVsHierarchyEvents>();
                ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.AdviseHierarchyEvents(this, out this.innerHierarchyEventsCookie));
            }
            uint num = ++this.nextAdviseEventsCookie;
            this.eventListeners[num] = eventSink;
            return num;
        }

        protected virtual void Close()
        {
            ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.Close());
            if ((this.innerHierarchyEventsCookie != 0) && (this.InnerVsHierarchy != null))
            {
                this.InnerVsHierarchy.UnadviseHierarchyEvents(this.innerHierarchyEventsCookie);
            }
            this.innerHierarchyEventsCookie = 0;
        }

        protected virtual int ExecCommand(uint itemid, ref Guid pguidCmdGroup, uint nCmdID, uint nCmdexecopt, IntPtr pvaIn, IntPtr pvaOut)
        {
            int num;
            try
            {
                if ((pguidCmdGroup == CmdIDConstants.AddServiceReferenceCmdID.Guid) && ((nCmdID == CmdIDConstants.AddServiceReferenceCmdID.ID) || (nCmdID == CmdIDConstants.AddServiceReferenceCtxCmdID.ID)))
                {
                    this._pauseDocumentTracking = true;
                }
                num = this._innerVsUIHierarchy.ExecCommand(itemid, ref pguidCmdGroup, nCmdID, nCmdexecopt, pvaIn, pvaOut);
            }
            finally
            {
                this._pauseDocumentTracking = false;
            }
            return num;
        }

        public int FilterListItemByCategory(ref Guid rguidProjectItemTemplates, string pszCategoryName, out int pfFilter)
        {
            pfFilter = 0;
            return 0;
        }

        public int FilterListItemByLocalizedName(ref Guid rguidProjectItemTemplates, string pszLocalizedName, out int pfFilter)
        {
            pfFilter = 0;
            return 0;
        }

        public int FilterListItemByTemplateFile(ref Guid rguidProjectItemTemplates, string pszTemplateFile, out int pfFilter)
        {
            int num = 0;
            if (!string.IsNullOrWhiteSpace(pszTemplateFile))
            {
                string str = string.Empty;
                string directoryName = Path.GetDirectoryName(pszTemplateFile);
                string fileName = Path.GetFileName(pszTemplateFile);
                if (!string.IsNullOrWhiteSpace(directoryName) && Path.GetExtension(directoryName).Equals(".zip", StringComparison.OrdinalIgnoreCase))
                {
                    fileName = Path.Combine(Path.GetFileName(directoryName), fileName);
                    directoryName = Path.GetDirectoryName(directoryName);
                }
                str = directoryName;
                directoryName = Path.GetFileName(directoryName).Trim();
                if (!string.IsNullOrWhiteSpace(directoryName) && (directoryName.Length == 4))
                {
                    int result = 0;
                    if (int.TryParse(directoryName, out result) && (XnaPackage.Instance.GetProviderLocale() != result))
                    {
                        num = 1;
                    }
                }
                else if (File.Exists(Path.Combine(str, string.Format("{0:0000}", XnaPackage.Instance.GetProviderLocale()), fileName)))
                {
                    num = 1;
                }
            }
            pfFilter = num;
            return 0;
        }

        public int FilterTreeItemByCategory(ref Guid rguidProjectItemTemplates, string pszCategoryName, out int pfFilter)
        {
            pfFilter = 0;
            return 0;
        }

        public int FilterTreeItemByLocalizedName(ref Guid rguidProjectItemTemplates, string pszLocalizedName, out int pfFilter)
        {
            pfFilter = 0;
            return 0;
        }

        public int FilterTreeItemByTemplateDir(ref Guid rguidProjectItemTemplates, string pszTemplateDir, out int pfFilter)
        {
            pfFilter = 0;
            return 0;
        }

        ~FlavoredProjectBase()
        {
        }

        public virtual void FreeInterfaces()
        {
            this._menuService = null;
            this._innerOleCommandTarget = null;
            this._innerVsAggregatableProject = null;
            this._innerVsUIHierarchy = null;
            this._innerVsHierarchy = null;
        }

        [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void GenerateEvents(IVsProject[] projects, int[] firstFiles, string[] mkDocuments, System.EventHandler<ProjectDocumentsChangeEventArgs> eventToGenerate, ProjectDocumentsChangeEventArgs e)
        {
            if (eventToGenerate != null)
            {
                if (projects == null)
                {
                    throw new ArgumentNullException("projects");
                }
                if (firstFiles == null)
                {
                    throw new ArgumentNullException("firstFiles");
                }
                if (mkDocuments == null)
                {
                    throw new ArgumentNullException("mkDocuments");
                }
                if (projects.Length != firstFiles.Length)
                {
                    throw new ArgumentException();
                }
                int num = -1;
                int num2 = mkDocuments.Length - 1;
                for (int i = 0; i < projects.Length; i++)
                {
                    if (num > -1)
                    {
                        num2 = firstFiles[i] - 1;
                        break;
                    }
                    if (ComUtilities.IsSameComObject(projects[i], this))
                    {
                        num = firstFiles[i];
                    }
                }
                if (num2 >= mkDocuments.Length)
                {
                    throw new ArgumentException();
                }
                if (num >= 0)
                {
                    for (int j = num; j <= num2; j++)
                    {
                        try
                        {
                            e.MkDocument = mkDocuments[j];
                            eventToGenerate(this, e);
                        }
                        catch (Exception)
                        {
                        }
                    }
                }
            }
        }

        [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void GenerateRenameEvents(IVsProject[] projects, int[] firstFiles, string[] mkDocumentsOld, string[] mkDocumentsNew, System.EventHandler<ProjectDocumentsRenamedEventArgs> eventToGenerate)
        {
            if (eventToGenerate != null)
            {
                if (projects == null)
                {
                    throw new ArgumentNullException("projects");
                }
                if (firstFiles == null)
                {
                    throw new ArgumentNullException("firstFiles");
                }
                if (mkDocumentsOld == null)
                {
                    throw new ArgumentNullException("mkDocumentsOld");
                }
                if (mkDocumentsNew == null)
                {
                    throw new ArgumentNullException("mkDocumentsNew");
                }
                if (projects.Length != firstFiles.Length)
                {
                    throw new ArgumentException();
                }
                if (mkDocumentsOld.Length != mkDocumentsNew.Length)
                {
                    throw new ArgumentException();
                }
                int num = -1;
                int num2 = mkDocumentsNew.Length - 1;
                for (int i = 0; i < projects.Length; i++)
                {
                    if (num > -1)
                    {
                        num2 = firstFiles[i] - 1;
                        break;
                    }
                    if (ComUtilities.IsSameComObject(projects[i], this))
                    {
                        num = firstFiles[i];
                    }
                }
                if (num2 >= mkDocumentsNew.Length)
                {
                    throw new ArgumentException();
                }
                if (num >= 0)
                {
                    for (int j = num; j <= num2; j++)
                    {
                        try
                        {
                            ProjectDocumentsRenamedEventArgs e = new ProjectDocumentsRenamedEventArgs(mkDocumentsOld[j], mkDocumentsNew[j]);
                            eventToGenerate(this, e);
                        }
                        catch (Exception)
                        {
                        }
                    }
                }
            }
        }

        protected virtual int GetCanonicalName(uint itemId, out string name)
        {
            return this._innerVsHierarchy.GetCanonicalName(itemId, out name);
        }

        protected virtual Guid GetGuidProperty(uint itemId, int propId)
        {
            Guid empty = Guid.Empty;
            if (this._innerVsHierarchy != null)
            {
                ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.GetGuidProperty(itemId, propId, out empty));
                return empty;
            }
            return Guid.Empty;
        }

        protected virtual int GetNestedHierarchy(uint itemId, ref Guid guidHierarchyNested, out IntPtr hierarchyNested, out uint itemIdNested)
        {
            if (this._innerVsHierarchy != null)
            {
                return this._innerVsHierarchy.GetNestedHierarchy(itemId, ref guidHierarchyNested, out hierarchyNested, out itemIdNested);
            }
            hierarchyNested = IntPtr.Zero;
            itemIdNested = uint.MaxValue;
            return -2147467262;
        }

        protected virtual object GetObject(Type objectType)
        {
            return null;
        }

        protected virtual int GetProperty(uint itemId, int propId, out object property)
        {
            if (this._innerVsHierarchy != null)
            {
                return this._innerVsHierarchy.GetProperty(itemId, propId, out property);
            }
            property = null;
            return -2147418113;
        }

        protected virtual Microsoft.VisualStudio.OLE.Interop.IServiceProvider GetSite()
        {
            Microsoft.VisualStudio.OLE.Interop.IServiceProvider ppSP = this as Microsoft.VisualStudio.OLE.Interop.IServiceProvider;
            if (ppSP == null)
            {
                ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.GetSite(out ppSP));
            }
            return ppSP;
        }

        private IVsTrackProjectDocuments2 GetTrackProjectDocuments()
        {
            IVsTrackProjectDocuments2 service = ((System.IServiceProvider) this).GetService(typeof(SVsTrackProjectDocuments)) as IVsTrackProjectDocuments2;
            if (service == null)
            {
                throw new InvalidOperationException();
            }
            return service;
        }

        protected virtual void InitializeForOuter(string fileName, string location, string name, uint flags, ref Guid guidProject, out bool cancel)
        {
            cancel = false;
        }

        int IOleCommandTarget.Exec(ref Guid pguidCmdGroup, uint nCmdID, uint nCmdexecopt, IntPtr pvaIn, IntPtr pvaOut)
        {
            return ((IOleCommandTarget) this._menuService).Exec(ref pguidCmdGroup, nCmdID, nCmdexecopt, pvaIn, pvaOut);
        }

        int IOleCommandTarget.QueryStatus(ref Guid pguidCmdGroup, uint cCmds, OLECMD[] prgCmds, IntPtr pCmdText)
        {
            return ((IOleCommandTarget) this._menuService).QueryStatus(ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
        }

        int IVsHierarchy.AdviseHierarchyEvents(IVsHierarchyEvents eventSink, out uint cookie)
        {
            cookie = this.AdviseHierarchyEvents(eventSink);
            return 0;
        }

        int IVsHierarchy.Close()
        {
            this._hierarchyClosed = true;
            this.Close();
            if (this.documentEventsCookie != 0)
            {
                this.GetTrackProjectDocuments().UnadviseTrackProjectDocumentsEvents(this.documentEventsCookie);
                this.documentEventsCookie = 0;
            }
            this._menuService = null;
            if (this._inExecCommand == 0)
            {
                this.FreeInterfaces();
            }
            return 0;
        }

        int IVsHierarchy.GetCanonicalName(uint itemId, out string name)
        {
            return this.GetCanonicalName(itemId, out name);
        }

        int IVsHierarchy.GetGuidProperty(uint itemId, int propId, out Guid guid)
        {
            guid = this.GetGuidProperty(itemId, propId);
            return 0;
        }

        int IVsHierarchy.GetNestedHierarchy(uint itemId, ref Guid guidHierarchyNested, out IntPtr hierarchyNested, out uint itemIdNested)
        {
            return this.GetNestedHierarchy(itemId, ref guidHierarchyNested, out hierarchyNested, out itemIdNested);
        }

        int IVsHierarchy.GetProperty(uint itemId, int propId, out object property)
        {
            return this.GetProperty(itemId, propId, out property);
        }

        int IVsHierarchy.GetSite(out Microsoft.VisualStudio.OLE.Interop.IServiceProvider site)
        {
            site = this.GetSite();
            return 0;
        }

        int IVsHierarchy.ParseCanonicalName(string name, out uint itemId)
        {
            return this.ParseCanonicalName(name, out itemId);
        }

        int IVsHierarchy.QueryClose(out int canClose)
        {
            canClose = 0;
            if (this.QueryClose())
            {
                canClose = 1;
            }
            return 0;
        }

        int IVsHierarchy.SetGuidProperty(uint itemId, int propId, ref Guid guid)
        {
            this.SetGuidProperty(itemId, propId, ref guid);
            return 0;
        }

        int IVsHierarchy.SetProperty(uint itemId, int propId, object property)
        {
            return this.SetProperty(itemId, propId, property);
        }

        int IVsHierarchy.SetSite(Microsoft.VisualStudio.OLE.Interop.IServiceProvider site)
        {
            this.serviceProvider = new Microsoft.VisualStudio.Shell.ServiceProvider(site);
            ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.SetSite(site));
            return 0;
        }

        int IVsHierarchy.UnadviseHierarchyEvents(uint cookie)
        {
            this.UnadviseHierarchyEvents(cookie);
            return 0;
        }

        int IVsHierarchy.Unused0()
        {
            this.Unused0();
            return 0;
        }

        int IVsHierarchy.Unused1()
        {
            this.Unused1();
            return 0;
        }

        int IVsHierarchy.Unused2()
        {
            this.Unused2();
            return 0;
        }

        int IVsHierarchy.Unused3()
        {
            this.Unused3();
            return 0;
        }

        int IVsHierarchy.Unused4()
        {
            this.Unused4();
            return 0;
        }

        int IVsUIHierarchy.AdviseHierarchyEvents(IVsHierarchyEvents pEventSink, out uint pdwCookie)
        {
            return ((IVsHierarchy) this).AdviseHierarchyEvents(pEventSink, out pdwCookie);
        }

        int IVsUIHierarchy.Close()
        {
            return ((IVsHierarchy) this).Close();
        }

        int IVsUIHierarchy.ExecCommand(uint itemid, ref Guid pguidCmdGroup, uint nCmdID, uint nCmdexecopt, IntPtr pvaIn, IntPtr pvaOut)
        {
            int num = 0;
            try
            {
                this._inExecCommand++;
                num = this.ExecCommand(itemid, ref pguidCmdGroup, nCmdID, nCmdexecopt, pvaIn, pvaOut);
            }
            finally
            {
                this._inExecCommand--;
                if (this._hierarchyClosed && (this._inExecCommand == 0))
                {
                    this.FreeInterfaces();
                }
            }
            return num;
        }

        int IVsUIHierarchy.GetCanonicalName(uint itemid, out string pbstrName)
        {
            return ((IVsHierarchy) this).GetCanonicalName(itemid, out pbstrName);
        }

        int IVsUIHierarchy.GetGuidProperty(uint itemid, int propid, out Guid pguid)
        {
            return ((IVsHierarchy) this).GetGuidProperty(itemid, propid, out pguid);
        }

        int IVsUIHierarchy.GetNestedHierarchy(uint itemid, ref Guid iidHierarchyNested, out IntPtr ppHierarchyNested, out uint pitemidNested)
        {
            return ((IVsHierarchy) this).GetNestedHierarchy(itemid, ref iidHierarchyNested, out ppHierarchyNested, out pitemidNested);
        }

        int IVsUIHierarchy.GetProperty(uint itemid, int propid, out object pvar)
        {
            return ((IVsHierarchy) this).GetProperty(itemid, propid, out pvar);
        }

        int IVsUIHierarchy.GetSite(out Microsoft.VisualStudio.OLE.Interop.IServiceProvider ppSP)
        {
            return ((IVsHierarchy) this).GetSite(out ppSP);
        }

        int IVsUIHierarchy.ParseCanonicalName(string pszName, out uint pitemid)
        {
            return ((IVsHierarchy) this).ParseCanonicalName(pszName, out pitemid);
        }

        int IVsUIHierarchy.QueryClose(out int pfCanClose)
        {
            return ((IVsHierarchy) this).QueryClose(out pfCanClose);
        }

        int IVsUIHierarchy.QueryStatusCommand(uint itemid, ref Guid pguidCmdGroup, uint cCmds, OLECMD[] prgCmds, IntPtr pCmdText)
        {
            return this.QueryStatusCommand(itemid, ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
        }

        int IVsUIHierarchy.SetGuidProperty(uint itemid, int propid, ref Guid rguid)
        {
            return ((IVsHierarchy) this).SetGuidProperty(itemid, propid, ref rguid);
        }

        int IVsUIHierarchy.SetProperty(uint itemid, int propid, object var)
        {
            return ((IVsHierarchy) this).SetProperty(itemid, propid, var);
        }

        int IVsUIHierarchy.SetSite(Microsoft.VisualStudio.OLE.Interop.IServiceProvider psp)
        {
            return ((IVsHierarchy) this).SetSite(psp);
        }

        int IVsUIHierarchy.UnadviseHierarchyEvents(uint dwCookie)
        {
            return ((IVsHierarchy) this).UnadviseHierarchyEvents(dwCookie);
        }

        int IVsUIHierarchy.Unused0()
        {
            return ((IVsHierarchy) this).Unused0();
        }

        int IVsUIHierarchy.Unused1()
        {
            return ((IVsHierarchy) this).Unused1();
        }

        int IVsUIHierarchy.Unused2()
        {
            return ((IVsHierarchy) this).Unused2();
        }

        int IVsUIHierarchy.Unused3()
        {
            return ((IVsHierarchy) this).Unused3();
        }

        int IVsUIHierarchy.Unused4()
        {
            return ((IVsHierarchy) this).Unused4();
        }

        object IHierarchyObjectProvider.GetObject(Type objectType)
        {
            if (objectType == null)
            {
                throw new ArgumentNullException("objectType");
            }
            return this.GetObject(objectType);
        }

        void IHierarchyObjectProvider.SetObject(Type objectType, object instance)
        {
            if (objectType == null)
            {
                throw new ArgumentNullException("objectType");
            }
            this.SetObject(objectType, instance);
        }

        int Microsoft.Xna.GameStudio.FlavoredProject.IVsAggregatableProjectCorrected.GetAggregateProjectTypeGuids(out string projectTypeGuids)
        {
            if (this._innerVsAggregatableProject == null)
            {
                throw new NotSupportedException();
            }
            int aggregateProjectTypeGuids = this._innerVsAggregatableProject.GetAggregateProjectTypeGuids(out projectTypeGuids);
            if (this._impersonateSilverlightProjectSupport)
            {
                projectTypeGuids = projectTypeGuids + ";{A1591282-1198-4647-A2B1-27E5FF5F6F3B}";
            }
            return aggregateProjectTypeGuids;
        }

        int Microsoft.Xna.GameStudio.FlavoredProject.IVsAggregatableProjectCorrected.InitializeForOuter(string fileName, string location, string name, uint flags, ref Guid guidProject, out IntPtr project, out int canceled)
        {
            bool flag;
            int num = 0;
            project = IntPtr.Zero;
            canceled = 0;
            if ((this._innerVsAggregatableProject == null) || (guidProject != VSConstants.IID_IUnknown))
            {
                throw new NotSupportedException();
            }
            IntPtr zero = IntPtr.Zero;
            try
            {
                zero = Marshal.GetIUnknownForObject(this);
                if (zero != IntPtr.Zero)
                {
                    num = Marshal.QueryInterface(zero, ref guidProject, out project);
                }
            }
            finally
            {
                if (zero != IntPtr.Zero)
                {
                    Marshal.Release(zero);
                }
            }
            this.InitializeForOuter(fileName, location, name, flags, ref guidProject, out flag);
            if (flag)
            {
                canceled = 1;
            }
            return num;
        }

        int Microsoft.Xna.GameStudio.FlavoredProject.IVsAggregatableProjectCorrected.OnAggregationComplete()
        {
            this.OnAggregationComplete();
            if (this._innerVsAggregatableProject != null)
            {
                return this._innerVsAggregatableProject.OnAggregationComplete();
            }
            return 0;
        }

        int Microsoft.Xna.GameStudio.FlavoredProject.IVsAggregatableProjectCorrected.SetAggregateProjectTypeGuids(string projectTypeGuids)
        {
            if (this._innerVsAggregatableProject == null)
            {
                throw new NotSupportedException();
            }
            return this._innerVsAggregatableProject.SetAggregateProjectTypeGuids(projectTypeGuids);
        }

        int Microsoft.Xna.GameStudio.FlavoredProject.IVsAggregatableProjectCorrected.SetInnerProject(IntPtr innerIUnknown)
        {
            this.SetInnerProject(innerIUnknown);
            return 0;
        }

        protected virtual void OnAggregationComplete()
        {
            ErrorHandler.ThrowOnFailure(this.GetTrackProjectDocuments().AdviseTrackProjectDocumentsEvents(this._documentsEventsSink, out this.documentEventsCookie));
        }

        public virtual int OnInvalidateIcon(IntPtr hicon)
        {
            if (this.eventListeners != null)
            {
                List<IVsHierarchyEvents> list = new List<IVsHierarchyEvents>(this.eventListeners.Values);
                foreach (IVsHierarchyEvents events in list)
                {
                    events.OnInvalidateIcon(hicon);
                }
            }
            return 0;
        }

        public virtual int OnInvalidateItems(uint itemidParent)
        {
            if (this.eventListeners != null)
            {
                List<IVsHierarchyEvents> list = new List<IVsHierarchyEvents>(this.eventListeners.Values);
                foreach (IVsHierarchyEvents events in list)
                {
                    events.OnInvalidateItems(itemidParent);
                }
            }
            return 0;
        }

        public virtual int OnItemAdded(uint itemidParent, uint itemidSiblingPrev, uint itemidAdded)
        {
            if (this.eventListeners != null)
            {
                List<IVsHierarchyEvents> list = new List<IVsHierarchyEvents>(this.eventListeners.Values);
                foreach (IVsHierarchyEvents events in list)
                {
                    events.OnItemAdded(itemidParent, itemidSiblingPrev, itemidAdded);
                }
            }
            return 0;
        }

        public virtual int OnItemDeleted(uint itemid)
        {
            if (this.eventListeners != null)
            {
                List<IVsHierarchyEvents> list = new List<IVsHierarchyEvents>(this.eventListeners.Values);
                foreach (IVsHierarchyEvents events in list)
                {
                    events.OnItemDeleted(itemid);
                }
            }
            return 0;
        }

        public virtual int OnItemsAppended(uint itemidParent)
        {
            if (this.eventListeners != null)
            {
                List<IVsHierarchyEvents> list = new List<IVsHierarchyEvents>(this.eventListeners.Values);
                foreach (IVsHierarchyEvents events in list)
                {
                    events.OnItemsAppended(itemidParent);
                }
            }
            return 0;
        }

        public virtual int OnPropertyChanged(uint itemid, int propid, uint flags)
        {
            if (this.eventListeners != null)
            {
                List<IVsHierarchyEvents> list = new List<IVsHierarchyEvents>(this.eventListeners.Values);
                foreach (IVsHierarchyEvents events in list)
                {
                    events.OnPropertyChanged(itemid, propid, flags);
                }
            }
            return 0;
        }

        protected virtual int ParseCanonicalName(string name, out uint itemId)
        {
            return this._innerVsHierarchy.ParseCanonicalName(name, out itemId);
        }

        protected virtual bool QueryClose()
        {
            int num;
            ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.QueryClose(out num));
            return (num != 0);
        }

        protected virtual int QueryStatusCommand(uint itemid, ref Guid pguidCmdGroup, uint cCmds, OLECMD[] prgCmds, IntPtr pCmdText)
        {
            return this._innerVsUIHierarchy.QueryStatusCommand(itemid, ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
        }

        protected virtual void SetGuidProperty(uint itemId, int propId, ref Guid guid)
        {
            ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.SetGuidProperty(itemId, propId, ref guid));
        }

        protected virtual void SetInnerProject(IntPtr innerIUnknown)
        {
            object objectForIUnknown = null;
            objectForIUnknown = Marshal.GetObjectForIUnknown(innerIUnknown);
            this._innerVsAggregatableProject = (Microsoft.Xna.GameStudio.FlavoredProject.IVsAggregatableProjectCorrected) objectForIUnknown;
            this._innerVsHierarchy = (IVsHierarchy) objectForIUnknown;
            this._innerVsUIHierarchy = (IVsUIHierarchy) objectForIUnknown;
            this._innerOleCommandTarget = objectForIUnknown as IOleCommandTarget;
            if (this.serviceProvider == null)
            {
                throw new InvalidOperationException(XnaProjectResource.InternalErrorProjectInitialization);
            }
            this._menuService = new OleMenuCommandService(this, this._innerOleCommandTarget);
            IntPtr zero = IntPtr.Zero;
            IXnaProjectAggregator2 aggregator = null;
            try
            {
                zero = Marshal.GetIUnknownForObject(this);
                aggregator = (IXnaProjectAggregator2) Marshal.GetObjectForIUnknown(zero);
                if (aggregator != null)
                {
                    aggregator.SetInner(innerIUnknown);
                }
            }
            finally
            {
                if (zero != IntPtr.Zero)
                {
                    Marshal.Release(zero);
                }
            }
        }

        protected virtual void SetObject(Type objectType, object instance)
        {
        }

        protected virtual int SetProperty(uint itemId, int propId, object property)
        {
            return this._innerVsHierarchy.SetProperty(itemId, propId, property);
        }

        object System.IServiceProvider.GetService(Type serviceType)
        {
            if (serviceType == typeof(IOleCommandTarget))
            {
                return this._menuService;
            }
            if (serviceType == typeof(IMenuCommandService))
            {
                return this._menuService;
            }
            return this.serviceProvider.GetService(serviceType);
        }

        protected virtual void UnadviseHierarchyEvents(uint cookie)
        {
            if ((this.eventListeners != null) && this.eventListeners.ContainsKey(cookie))
            {
                this.eventListeners.Remove(cookie);
            }
        }

        protected virtual void Unused0()
        {
            ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.Unused0());
        }

        protected virtual void Unused1()
        {
            ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.Unused1());
        }

        protected virtual void Unused2()
        {
            ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.Unused2());
        }

        protected virtual void Unused3()
        {
            ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.Unused3());
        }

        protected virtual void Unused4()
        {
            ErrorHandler.ThrowOnFailure(this._innerVsHierarchy.Unused4());
        }

        protected bool ImpersonateSilverlightProject
        {
            get
            {
                return this._impersonateSilverlightProjectSupport;
            }
            set
            {
                this._impersonateSilverlightProjectSupport = value;
            }
        }

        [SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        protected IOleCommandTarget InnerOleCommandTarget
        {
            get
            {
                return this._innerOleCommandTarget;
            }
        }

        [SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        protected Microsoft.Xna.GameStudio.FlavoredProject.IVsAggregatableProjectCorrected InnerVsAggregatableProject
        {
            get
            {
                return this._innerVsAggregatableProject;
            }
        }

        protected IVsHierarchy InnerVsHierarchy
        {
            get
            {
                return this._innerVsHierarchy;
            }
        }

        protected IVsUIHierarchy InnerVsUIHierarchy
        {
            get
            {
                return this._innerVsUIHierarchy;
            }
        }

        protected System.IServiceProvider ServiceProvider
        {
            get
            {
                return this.serviceProvider;
            }
            set
            {
                this.serviceProvider = value;
            }
        }

        internal class DocumentsEventsSink : IVsTrackProjectDocumentsEvents2
        {
            private Microsoft.Xna.GameStudio.FlavoredProject.FlavoredProjectBase _flavoredProjectBase;

            internal DocumentsEventsSink(Microsoft.Xna.GameStudio.FlavoredProject.FlavoredProjectBase flavoredProjectBase)
            {
                this._flavoredProjectBase = flavoredProjectBase;
            }

            int IVsTrackProjectDocumentsEvents2.OnAfterAddDirectoriesEx(int cProjects, int cDirectories, IVsProject[] rgpProjects, int[] rgFirstIndices, string[] rgpszMkDocuments, VSADDDIRECTORYFLAGS[] rgFlags)
            {
                if (!this._flavoredProjectBase._pauseDocumentTracking)
                {
                    this._flavoredProjectBase.GenerateEvents(rgpProjects, rgFirstIndices, rgpszMkDocuments, this._flavoredProjectBase.DirectoryAdded, new ProjectDocumentsChangeEventArgs());
                }
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnAfterAddFilesEx(int cProjects, int cFiles, IVsProject[] rgpProjects, int[] rgFirstIndices, string[] rgpszMkDocuments, VSADDFILEFLAGS[] rgFlags)
            {
                if (!this._flavoredProjectBase._pauseDocumentTracking)
                {
                    this._flavoredProjectBase.GenerateEvents(rgpProjects, rgFirstIndices, rgpszMkDocuments, this._flavoredProjectBase.FileAdded, new ProjectDocumentsChangeEventArgs());
                }
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnAfterRemoveDirectories(int cProjects, int cDirectories, IVsProject[] rgpProjects, int[] rgFirstIndices, string[] rgpszMkDocuments, VSREMOVEDIRECTORYFLAGS[] rgFlags)
            {
                if (!this._flavoredProjectBase._pauseDocumentTracking)
                {
                    this._flavoredProjectBase.GenerateEvents(rgpProjects, rgFirstIndices, rgpszMkDocuments, this._flavoredProjectBase.DirectoryRemoved, new ProjectDocumentsChangeEventArgs());
                }
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnAfterRemoveFiles(int cProjects, int cFiles, IVsProject[] rgpProjects, int[] rgFirstIndices, string[] rgpszMkDocuments, VSREMOVEFILEFLAGS[] rgFlags)
            {
                if (!this._flavoredProjectBase._pauseDocumentTracking)
                {
                    this._flavoredProjectBase.GenerateEvents(rgpProjects, rgFirstIndices, rgpszMkDocuments, this._flavoredProjectBase.FileRemoved, new ProjectDocumentsChangeEventArgs());
                }
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnAfterRenameDirectories(int cProjects, int cDirs, IVsProject[] rgpProjects, int[] rgFirstIndices, string[] rgszMkOldNames, string[] rgszMkNewNames, VSRENAMEDIRECTORYFLAGS[] rgFlags)
            {
                if (!this._flavoredProjectBase._pauseDocumentTracking)
                {
                    this._flavoredProjectBase.GenerateRenameEvents(rgpProjects, rgFirstIndices, rgszMkOldNames, rgszMkNewNames, this._flavoredProjectBase.DirectoryRenamed);
                }
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnAfterRenameFiles(int cProjects, int cFiles, IVsProject[] rgpProjects, int[] rgFirstIndices, string[] rgszMkOldNames, string[] rgszMkNewNames, VSRENAMEFILEFLAGS[] rgFlags)
            {
                if (!this._flavoredProjectBase._pauseDocumentTracking)
                {
                    this._flavoredProjectBase.GenerateRenameEvents(rgpProjects, rgFirstIndices, rgszMkOldNames, rgszMkNewNames, this._flavoredProjectBase.FileRenamed);
                }
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnAfterSccStatusChanged(int cProjects, int cFiles, IVsProject[] rgpProjects, int[] rgFirstIndices, string[] rgpszMkDocuments, uint[] rgdwSccStatus)
            {
                if (!this._flavoredProjectBase._pauseDocumentTracking)
                {
                    this._flavoredProjectBase.GenerateEvents(rgpProjects, rgFirstIndices, rgpszMkDocuments, this._flavoredProjectBase.SccStatusChanged, new ProjectDocumentsChangeEventArgs());
                }
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnQueryAddDirectories(IVsProject pProject, int cDirectories, string[] rgpszMkDocuments, VSQUERYADDDIRECTORYFLAGS[] rgFlags, VSQUERYADDDIRECTORYRESULTS[] pSummaryResult, VSQUERYADDDIRECTORYRESULTS[] rgResults)
            {
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnQueryAddFiles(IVsProject pProject, int cFiles, string[] rgpszMkDocuments, VSQUERYADDFILEFLAGS[] rgFlags, VSQUERYADDFILERESULTS[] pSummaryResult, VSQUERYADDFILERESULTS[] rgResults)
            {
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnQueryRemoveDirectories(IVsProject pProject, int cDirectories, string[] rgpszMkDocuments, VSQUERYREMOVEDIRECTORYFLAGS[] rgFlags, VSQUERYREMOVEDIRECTORYRESULTS[] pSummaryResult, VSQUERYREMOVEDIRECTORYRESULTS[] rgResults)
            {
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnQueryRemoveFiles(IVsProject pProject, int cFiles, string[] rgpszMkDocuments, VSQUERYREMOVEFILEFLAGS[] rgFlags, VSQUERYREMOVEFILERESULTS[] pSummaryResult, VSQUERYREMOVEFILERESULTS[] rgResults)
            {
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnQueryRenameDirectories(IVsProject pProject, int cDirs, string[] rgszMkOldNames, string[] rgszMkNewNames, VSQUERYRENAMEDIRECTORYFLAGS[] rgFlags, VSQUERYRENAMEDIRECTORYRESULTS[] pSummaryResult, VSQUERYRENAMEDIRECTORYRESULTS[] rgResults)
            {
                return 0;
            }

            int IVsTrackProjectDocumentsEvents2.OnQueryRenameFiles(IVsProject pProject, int cFiles, string[] rgszMkOldNames, string[] rgszMkNewNames, VSQUERYRENAMEFILEFLAGS[] rgFlags, VSQUERYRENAMEFILERESULTS[] pSummaryResult, VSQUERYRENAMEFILERESULTS[] rgResults)
            {
                return 0;
            }
        }
    }
}

