﻿namespace Microsoft.Xna.GameStudio.FlavoredProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.Diagnostics.CodeAnalysis;
    using System.Runtime.InteropServices;

    [SuppressMessage("Microsoft.Design", "CA1001:TypesThatOwnDisposableFieldsShouldBeDisposable")]
    internal abstract class FlavoredProjectFactoryBase : IVsAggregatableProjectFactoryCorrected, IVsProjectFactory
    {
        private Microsoft.VisualStudio.Shell.ServiceProvider _serviceProvider;
        private bool disableEditAndContinue;

        protected FlavoredProjectFactoryBase(bool disableEditAndContinue)
        {
            this.disableEditAndContinue = disableEditAndContinue;
        }

        protected virtual bool CanCreateProject(string fileName, uint flags)
        {
            bool flag = !string.IsNullOrEmpty(fileName);
            return (flag | !PackageUtilities.ContainsInvalidFileNameChars(fileName));
        }

        protected virtual void CreateProject(string fileName, string location, string name, uint flags, ref Guid projectGuid, out IntPtr project, out int canceled)
        {
            project = IntPtr.Zero;
            canceled = 0;
        }

        protected virtual void Dispose(bool disposing)
        {
            if (this._serviceProvider != null)
            {
                this._serviceProvider.Dispose();
                this._serviceProvider = null;
            }
        }

        protected virtual void Initialize()
        {
        }

        int IVsProjectFactory.CanCreateProject(string fileName, uint flags, out int canCreate)
        {
            canCreate = this.CanCreateProject(fileName, flags) ? 1 : 0;
            return 0;
        }

        int IVsProjectFactory.Close()
        {
            this.Dispose(true);
            return 0;
        }

        int IVsProjectFactory.CreateProject(string fileName, string location, string name, uint flags, ref Guid projectGuid, out IntPtr project, out int canceled)
        {
            this.CreateProject(fileName, location, name, flags, ref projectGuid, out project, out canceled);
            return 0;
        }

        int IVsProjectFactory.SetSite(Microsoft.VisualStudio.OLE.Interop.IServiceProvider provider)
        {
            this._serviceProvider = new Microsoft.VisualStudio.Shell.ServiceProvider(provider);
            this.Initialize();
            return 0;
        }

        int IVsAggregatableProjectFactoryCorrected.GetAggregateProjectType(string fileName, out string projectTypeGuid)
        {
            projectTypeGuid = this.ProjectTypeGuids(fileName);
            return 0;
        }

        int IVsAggregatableProjectFactoryCorrected.PreCreateForOuter(IntPtr outerProjectIUnknown, out IntPtr projectIUnknown)
        {
            projectIUnknown = IntPtr.Zero;
            object o = this.PreCreateForOuter(outerProjectIUnknown);
            IntPtr zero = IntPtr.Zero;
            ILocalRegistryCorrected globalService = (ILocalRegistryCorrected) Package.GetGlobalService(typeof(SLocalRegistry));
            if (globalService == null)
            {
                throw new InvalidOperationException();
            }
            Guid clsid = new Guid("{139A4080-1B81-40f5-979E-E903DFB4F22F}");
            Guid riid = VSConstants.IID_IUnknown;
            uint dwFlags = 1;
            IntPtr ppvObj = IntPtr.Zero;
            IXnaProjectAggregator2 objectForIUnknown = null;
            try
            {
                ErrorHandler.ThrowOnFailure(globalService.CreateInstance(clsid, outerProjectIUnknown, ref riid, dwFlags, out ppvObj));
                if (outerProjectIUnknown != IntPtr.Zero)
                {
                    zero = Marshal.CreateAggregatedObject(outerProjectIUnknown, o);
                }
                else
                {
                    zero = Marshal.CreateAggregatedObject(ppvObj, o);
                }
                objectForIUnknown = (IXnaProjectAggregator2) Marshal.GetObjectForIUnknown(ppvObj);
                if (objectForIUnknown != null)
                {
                    objectForIUnknown.SetMyProject(zero, this.disableEditAndContinue);
                }
                projectIUnknown = ppvObj;
                ppvObj = IntPtr.Zero;
            }
            finally
            {
                if (zero != IntPtr.Zero)
                {
                    Marshal.Release(zero);
                }
                if (ppvObj != IntPtr.Zero)
                {
                    Marshal.Release(ppvObj);
                }
            }
            if (projectIUnknown == IntPtr.Zero)
            {
                return -2147467259;
            }
            return 0;
        }

        protected abstract object PreCreateForOuter(IntPtr outerProjectIUnknown);
        protected virtual string ProjectTypeGuids(string file)
        {
            throw new NotImplementedException();
        }

        [SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        protected Microsoft.VisualStudio.Shell.ServiceProvider ServiceProvider
        {
            get
            {
                return this._serviceProvider;
            }
        }
    }
}

