﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Globalization;

    internal static class GameStudioWindowPane
    {
        private static bool failedPaneCreation;
        private static IVsOutputWindowPane windowPane;

        internal static void WriteLine(string format, params object[] args)
        {
            if ((windowPane == null) && !failedPaneCreation)
            {
                IVsOutputWindow globalService = Package.GetGlobalService(typeof(SVsOutputWindow)) as IVsOutputWindow;
                Guid gameStudioWindowPane = Microsoft.Xna.GameStudio.ProjectConstants.GameStudioWindowPane;
                if (ErrorHandler.Failed(globalService.GetPane(ref gameStudioWindowPane, out windowPane)))
                {
                    try
                    {
                        ErrorHandler.ThrowOnFailure(globalService.CreatePane(ref gameStudioWindowPane, XnaProjectResource.GameStudioWindowPane, 1, 1));
                        ErrorHandler.ThrowOnFailure(globalService.GetPane(ref gameStudioWindowPane, out windowPane));
                    }
                    catch
                    {
                        failedPaneCreation = true;
                        throw;
                    }
                }
            }
            if (windowPane != null)
            {
                ErrorHandler.ThrowOnFailure(windowPane.Activate());
                ErrorHandler.ThrowOnFailure(windowPane.OutputString(string.Format(CultureInfo.CurrentUICulture, format + "\n", args)));
            }
        }
    }
}

