﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio.Shell;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel.Design;

    internal class MenuCommandFilter
    {
        private CommandID[] commandsToRemove;
        private SortedList<Guid, List<int>> filteredCommands;
        private List<OleMenuCommand> filteredMenuServiceCommands;
        private bool isMenuFilterEnabled;
        private OleMenuCommandService menuService;

        internal MenuCommandFilter(OleMenuCommandService menuService, CommandID[] commandsToRemove)
        {
            if (menuService == null)
            {
                throw new ArgumentNullException("menuService");
            }
            if (commandsToRemove == null)
            {
                throw new ArgumentNullException("commandsToRemove");
            }
            this.menuService = menuService;
            this.commandsToRemove = commandsToRemove;
        }

        internal void AddRange(CommandID[] commandsToAddToFilter)
        {
            if (commandsToAddToFilter == null)
            {
                throw new ArgumentNullException("commandsToAddToFilter");
            }
            if (this.commandsToRemove == null)
            {
                this.commandsToRemove = commandsToAddToFilter;
            }
            else
            {
                CommandID[] array = new CommandID[this.commandsToRemove.Length + commandsToAddToFilter.Length];
                this.commandsToRemove.CopyTo(array, 0);
                commandsToAddToFilter.CopyTo(array, this.commandsToRemove.Length);
                this.commandsToRemove = array;
            }
            bool isMenuFilterEnabled = this.isMenuFilterEnabled;
            this.Enable(false);
            this.filteredCommands = null;
            if (isMenuFilterEnabled)
            {
                this.Enable(true);
            }
        }

        internal void Enable(bool enable)
        {
            this.InitializeCommandFilter();
            if ((this.menuService != null) && (this.isMenuFilterEnabled != enable))
            {
                if (enable)
                {
                    foreach (OleMenuCommand command in this.filteredMenuServiceCommands)
                    {
                        this.menuService.AddCommand(command);
                    }
                }
                else
                {
                    foreach (OleMenuCommand command2 in this.filteredMenuServiceCommands)
                    {
                        this.menuService.RemoveCommand(command2);
                    }
                }
                this.isMenuFilterEnabled = enable;
            }
        }

        private void InitializeCommandFilter()
        {
            if (this.filteredCommands == null)
            {
                this.filteredCommands = new SortedList<Guid, List<int>>();
                CommandID[] commandsToRemove = this.commandsToRemove;
                foreach (CommandID did in commandsToRemove)
                {
                    List<int> list = null;
                    if (!this.filteredCommands.TryGetValue(did.Guid, out list))
                    {
                        list = new List<int>(10);
                        this.filteredCommands.Add(did.Guid, list);
                    }
                    list.Add(did.ID);
                }
                foreach (List<int> list2 in this.filteredCommands.Values)
                {
                    list2.Sort();
                }
                this.filteredMenuServiceCommands = new List<OleMenuCommand>(commandsToRemove.Length);
                EventHandler invokeHandler = delegate (object sender, EventArgs args) {
                };
                foreach (CommandID did2 in commandsToRemove)
                {
                    OleMenuCommand item = new OleMenuCommand(invokeHandler, did2);
                    item.BeforeQueryStatus += new EventHandler(MenuCommandFilter.OnBeforeFilteredCommandQueryStatus);
                    this.filteredMenuServiceCommands.Add(item);
                }
            }
        }

        internal bool IsCommandFiltered(ref Guid commandGroup, uint cmdID)
        {
            this.InitializeCommandFilter();
            List<int> list = null;
            return (this.filteredCommands.TryGetValue(commandGroup, out list) && (list.BinarySearch((int) cmdID) >= 0));
        }

        private static void OnBeforeFilteredCommandQueryStatus(object sender, EventArgs e)
        {
            OleMenuCommand command = sender as OleMenuCommand;
            if (command != null)
            {
                command.Enabled = false;
                command.Visible = false;
            }
        }
    }
}

