﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.IO;

    internal static class ProjectAssemblyPath
    {
        private static Uri GetAbsoluteUri(string path)
        {
            Uri uri;
            if (!Uri.TryCreate(Path.GetFullPath(path), UriKind.Absolute, out uri))
            {
                return null;
            }
            if (!uri.IsFile)
            {
                return null;
            }
            return uri;
        }

        internal static string GetProjectDir(IVsHierarchy hierarchy)
        {
            object obj2;
            ErrorHandler.ThrowOnFailure(hierarchy.GetProperty(0xfffffffe, -2021, out obj2));
            string str = (string) obj2;
            if (str[str.Length - 1] != Path.DirectorySeparatorChar)
            {
                str = str + Path.DirectorySeparatorChar;
            }
            return str;
        }

        private static string GetRelativePath(Uri baseUri, string filename)
        {
            Uri uri;
            if (!Uri.TryCreate(filename, UriKind.Absolute, out uri))
            {
                return filename;
            }
            if (!uri.IsFile)
            {
                return filename;
            }
            if (baseUri != null)
            {
                uri = baseUri.MakeRelativeUri(uri);
            }
            string str = Uri.UnescapeDataString(uri.ToString());
            string[] strArray = new string[] { "file:///", "file:" };
            foreach (string str2 in strArray)
            {
                if (str.StartsWith(str2, StringComparison.OrdinalIgnoreCase))
                {
                    str = str.Substring(str2.Length);
                    break;
                }
            }
            return str.Replace(Path.AltDirectorySeparatorChar, Path.DirectorySeparatorChar);
        }

        internal static string MakeProjectRelativePath(string pathToMakeRelative, IVsHierarchy hierarchy)
        {
            if (hierarchy == null)
            {
                throw new ArgumentNullException("hierarchy");
            }
            if (string.IsNullOrEmpty(pathToMakeRelative))
            {
                return string.Empty;
            }
            return GetRelativePath(GetAbsoluteUri(GetProjectDir(hierarchy)), pathToMakeRelative);
        }
    }
}

