﻿namespace Microsoft.Xna.GameStudio.Projects
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Win32;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Linq;
    using System.Reflection;

    internal class LanguageAdapterFactory
    {
        private static IDictionary<Guid, Type> adapterTypes = new Dictionary<Guid, Type>();

        internal static IProjectLanguageAdapter CreateAdapter(IXnaCodeProject codeProject)
        {
            Guid key = Guid.Parse(GetLanguageGuidString(codeProject));
            if (!adapterTypes.ContainsKey(key))
            {
                adapterTypes.Add(key, GetLanguageAdapter(key, codeProject));
            }
            IProjectLanguageAdapter adapter = (IProjectLanguageAdapter) Activator.CreateInstance(adapterTypes[key]);
            adapter.Initialize(codeProject.InnerIVsHierarchy, codeProject.Adapter, codeProject as IServiceProvider);
            return adapter;
        }

        private static Type GetLanguageAdapter(Guid languageGuid, IXnaCodeProject codeProject)
        {
            string str;
            Func<string, bool> func = null;
            ILocalRegistry3 vsService = codeProject.GetVsService(typeof(SLocalRegistry)) as ILocalRegistry3;
            if (vsService == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.MissingService, new object[] { typeof(SLocalRegistry).FullName }));
            }
            ErrorHandler.ThrowOnFailure(vsService.GetLocalRegistryRoot(out str));
            string name = string.Format(CultureInfo.InvariantCulture, @"{0}\Packages\{1}\CodeLanguages", new object[] { str, typeof(XnaPackage).GUID.ToString("B") });
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(name))
            {
                if (func == null)
                {
                    <>c__DisplayClass2 class2;
                    func = new Func<string, bool>(class2, (IntPtr) this.<GetLanguageAdapter>b__0);
                }
                foreach (string str3 in Enumerable.Where<string>(key.GetSubKeyNames(), func))
                {
                    using (RegistryKey key2 = key.OpenSubKey(str3))
                    {
                        string assemblyString = (string) key2.GetValue("Assembly");
                        string str5 = (string) key2.GetValue("Class");
                        Type extenderType = Assembly.Load(assemblyString).GetType(str5);
                        if (!XnaPackage.CheckExtenderTypeSignature(extenderType))
                        {
                            throw new InvalidOperationException(XnaProjectResource.InvalidPlatformAdapter);
                        }
                        return extenderType;
                    }
                }
            }
            return null;
        }

        private static string GetLanguageGuidString(IXnaCodeProject codeProject)
        {
            return codeProject.GetBuildProperty("ProjectTypeGuids", null).Split(new char[] { ';' }).Last<string>();
        }
    }
}

