﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.Win32;
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.CodeAnalysis;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Security;

    internal static class SdkPaths
    {
        private static readonly string[] versionSdkPathBase = new string[] { @"SOFTWARE\Microsoft\.NETFramework\v2.0.50727{1}\AssemblyFoldersEx\Xna Framework for x86 ({0})", @"SOFTWARE\Microsoft\XNA\AssemblyFolders\{0}\Xbox 360{1}\AssemblyFoldersEx\", @"SOFTWARE\Microsoft\XNA\AssemblyFolders\{0}\Windows Phone{1}\AssemblyFoldersEx\" };

        [SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        internal static string[] GetAllSdkPaths(SdkPathGroup sdkPaths, string version, string refreshLevel = null)
        {
            return GetSdkPaths(string.Format(versionSdkPathBase[(int) sdkPaths], version, (string.IsNullOrWhiteSpace(refreshLevel) || (refreshLevel == "0")) ? string.Empty : (@"\Refresh." + refreshLevel)));
        }

        [SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        private static string[] GetSdkPaths(string path)
        {
            List<string> list = new List<string>();
            list.AddRange(GetSdkPathsList(path));
            return list.ToArray();
        }

        [SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        private static List<string> GetSdkPathsList(string baseKeyName)
        {
            List<string> list = new List<string>();
            RegistryKey key = Registry.LocalMachine.OpenSubKey(baseKeyName);
            if (key != null)
            {
                foreach (string str in key.GetSubKeyNames())
                {
                    try
                    {
                        string item = (string) key.OpenSubKey(str).GetValue(string.Empty, string.Empty);
                        list.Add(item);
                    }
                    catch (SecurityException)
                    {
                    }
                    catch (ObjectDisposedException)
                    {
                    }
                    catch (IOException)
                    {
                    }
                    catch (UnauthorizedAccessException)
                    {
                    }
                }
            }
            return list;
        }
    }
}

