﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.Threading;

    internal class SolutionBuildStatusListener : ISolutionBuildStatusProvider, IVsUpdateSolutionEvents2, IVsUpdateSolutionEvents, IDisposable
    {
        private uint buildEventsCookie;
        private bool isSolutionBuilding;
        private IVsSolutionBuildManager2 solutionBuildManager;

        public event EventHandler BuildStatusChanged
        {
            add
            {
                this.statusChangedHandlers += value;
            }
            remove
            {
                this.statusChangedHandlers -= value;
            }
        }

        private event EventHandler statusChangedHandlers;

        internal SolutionBuildStatusListener(IServiceProvider serviceProvider)
        {
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            this.solutionBuildManager = (IVsSolutionBuildManager2) serviceProvider.GetService(typeof(IVsSolutionBuildManager));
            ErrorHandler.ThrowOnFailure(this.solutionBuildManager.AdviseUpdateSolutionEvents(this, out this.buildEventsCookie));
        }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if ((disposing && (this.solutionBuildManager != null)) && (this.buildEventsCookie != 0))
            {
                this.solutionBuildManager.UnadviseUpdateSolutionEvents(this.buildEventsCookie);
            }
            this.solutionBuildManager = null;
            this.buildEventsCookie = 0;
        }

        ~SolutionBuildStatusListener()
        {
            this.Dispose(false);
        }

        public int OnActiveProjectCfgChange(IVsHierarchy pIVsHierarchy)
        {
            return 0;
        }

        private void RaiseBuildStatusChanged()
        {
            if (this.statusChangedHandlers != null)
            {
                this.statusChangedHandlers(this, EventArgs.Empty);
            }
        }

        public int UpdateProjectCfg_Begin(IVsHierarchy pHierProj, IVsCfg pCfgProj, IVsCfg pCfgSln, uint dwAction, ref int pfCancel)
        {
            return 0;
        }

        public int UpdateProjectCfg_Done(IVsHierarchy pHierProj, IVsCfg pCfgProj, IVsCfg pCfgSln, uint dwAction, int fSuccess, int fCancel)
        {
            return 0;
        }

        public int UpdateSolution_Begin(ref int pfCancelUpdate)
        {
            this.isSolutionBuilding = true;
            this.RaiseBuildStatusChanged();
            return 0;
        }

        public int UpdateSolution_Cancel()
        {
            return 0;
        }

        public int UpdateSolution_Done(int fSucceeded, int fModified, int fCancelCommand)
        {
            this.isSolutionBuilding = false;
            this.RaiseBuildStatusChanged();
            return 0;
        }

        public int UpdateSolution_StartUpdate(ref int pfCancelUpdate)
        {
            return 0;
        }

        public bool IsBuilding
        {
            get
            {
                return this.isSolutionBuilding;
            }
        }
    }
}

