﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Drawing;
    using System.Windows.Forms;

    internal class UpgradeSolutionForm : Form
    {
        private Label backupLabel;
        private Button cancelButton;
        private IContainer components;
        private FlowLayoutPanel flowLayoutPanel1;
        private Label projectsLabel;
        private ListBox projectsListBox;
        private TableLayoutPanel tableLayoutPanel1;
        private Button upgradeButton;

        public UpgradeSolutionForm(IEnumerable<IVsHierarchy> codeProjectsToUpgrade)
        {
            if (codeProjectsToUpgrade == null)
            {
                throw new ArgumentNullException("codeProjectsToUpgrade");
            }
            this.InitializeComponent();
            foreach (IVsHierarchy hierarchy in codeProjectsToUpgrade)
            {
                this.projectsListBox.Items.Add(ProjectUtilities.GetProjectName(hierarchy));
            }
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            base.DialogResult = DialogResult.Cancel;
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void InitializeComponent()
        {
            ComponentResourceManager manager = new ComponentResourceManager(typeof(UpgradeSolutionForm));
            this.projectsListBox = new ListBox();
            this.projectsLabel = new Label();
            this.backupLabel = new Label();
            this.flowLayoutPanel1 = new FlowLayoutPanel();
            this.cancelButton = new Button();
            this.upgradeButton = new Button();
            this.tableLayoutPanel1 = new TableLayoutPanel();
            this.flowLayoutPanel1.SuspendLayout();
            this.tableLayoutPanel1.SuspendLayout();
            base.SuspendLayout();
            manager.ApplyResources(this.projectsListBox, "projectsListBox");
            this.projectsListBox.FormattingEnabled = true;
            this.projectsListBox.Name = "projectsListBox";
            this.projectsListBox.SelectionMode = SelectionMode.None;
            this.projectsListBox.Sorted = true;
            this.projectsListBox.HelpRequested += new HelpEventHandler(this.projectsListBox_HelpRequested);
            manager.ApplyResources(this.projectsLabel, "projectsLabel");
            this.projectsLabel.Name = "projectsLabel";
            manager.ApplyResources(this.backupLabel, "backupLabel");
            this.backupLabel.Name = "backupLabel";
            manager.ApplyResources(this.flowLayoutPanel1, "flowLayoutPanel1");
            this.flowLayoutPanel1.Controls.Add(this.cancelButton);
            this.flowLayoutPanel1.Controls.Add(this.upgradeButton);
            this.flowLayoutPanel1.Name = "flowLayoutPanel1";
            manager.ApplyResources(this.cancelButton, "cancelButton");
            this.cancelButton.DialogResult = DialogResult.Cancel;
            this.cancelButton.Name = "cancelButton";
            this.cancelButton.UseVisualStyleBackColor = true;
            this.cancelButton.Click += new EventHandler(this.cancelButton_Click);
            manager.ApplyResources(this.upgradeButton, "upgradeButton");
            this.upgradeButton.Name = "upgradeButton";
            this.upgradeButton.UseVisualStyleBackColor = true;
            this.upgradeButton.Click += new EventHandler(this.upgradeButton_Click);
            manager.ApplyResources(this.tableLayoutPanel1, "tableLayoutPanel1");
            this.tableLayoutPanel1.Controls.Add(this.projectsLabel, 0, 0);
            this.tableLayoutPanel1.Controls.Add(this.flowLayoutPanel1, 0, 3);
            this.tableLayoutPanel1.Controls.Add(this.projectsListBox, 0, 1);
            this.tableLayoutPanel1.Controls.Add(this.backupLabel, 0, 2);
            this.tableLayoutPanel1.Name = "tableLayoutPanel1";
            manager.ApplyResources(this, "$this");
            base.AutoScaleMode = AutoScaleMode.Font;
            base.Controls.Add(this.tableLayoutPanel1);
            base.HelpButton = true;
            base.MaximizeBox = false;
            base.MinimizeBox = false;
            base.Name = "UpgradeSolutionForm";
            base.ShowIcon = false;
            base.ShowInTaskbar = false;
            base.HelpButtonClicked += new CancelEventHandler(this.UpgradeSolutionForm_HelpButtonClicked);
            this.flowLayoutPanel1.ResumeLayout(false);
            this.flowLayoutPanel1.PerformLayout();
            this.tableLayoutPanel1.ResumeLayout(false);
            this.tableLayoutPanel1.PerformLayout();
            base.ResumeLayout(false);
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            this.Font = SystemFonts.IconTitleFont;
        }

        private void projectsListBox_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            ProjectUtilities.ShowHelp("Microsoft.Xna.GameStudio.UpgradeSolution");
        }

        private void upgradeButton_Click(object sender, EventArgs e)
        {
            base.DialogResult = DialogResult.OK;
        }

        private void UpgradeSolutionForm_HelpButtonClicked(object sender, CancelEventArgs e)
        {
            ProjectUtilities.ShowHelp("Microsoft.Xna.GameStudio.UpgradeSolution");
        }
    }
}

