﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.ComponentModel.Design;
    using System.Runtime.InteropServices;

    internal class VsBuildPropertyStorageSccWrapper : IVsBuildPropertyStorage
    {
        private IVsProject project;
        private IVsBuildPropertyStorage propertyStorage;
        private IVsQueryEditQuerySave2 queryEditQuerySave;

        internal VsBuildPropertyStorageSccWrapper(IVsBuildPropertyStorage propertyStorage, IVsProject project)
        {
            if (propertyStorage == null)
            {
                throw new ArgumentNullException("propertyStorage");
            }
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            this.queryEditQuerySave = Package.GetGlobalService(typeof(SVsQueryEditQuerySave)) as IVsQueryEditQuerySave2;
            this.propertyStorage = propertyStorage;
            this.project = project;
        }

        public int GetItemAttribute(uint item, string pszAttributeName, out string pbstrAttributeValue)
        {
            return this.propertyStorage.GetItemAttribute(item, pszAttributeName, out pbstrAttributeValue);
        }

        public int GetPropertyValue(string pszPropName, string pszConfigName, uint storage, out string pbstrPropValue)
        {
            return this.propertyStorage.GetPropertyValue(pszPropName, pszConfigName, storage, out pbstrPropValue);
        }

        private void QueryEditBuildPropertyStorage()
        {
            if (this.queryEditQuerySave != null)
            {
                string pbstrMkDocument = null;
                ErrorHandler.ThrowOnFailure(this.project.GetMkDocument(0xfffffffe, out pbstrMkDocument));
                string[] rgpszMkDocuments = new string[] { pbstrMkDocument };
                uint pfEditVerdict = 0;
                uint prgfMoreInfo = 0;
                ErrorHandler.ThrowOnFailure(this.queryEditQuerySave.QueryEditFiles(0, rgpszMkDocuments.Length, rgpszMkDocuments, new uint[rgpszMkDocuments.Length], new VSQEQS_FILE_ATTRIBUTE_DATA[rgpszMkDocuments.Length], out pfEditVerdict, out prgfMoreInfo));
                if (pfEditVerdict != 0)
                {
                    throw CheckoutException.Canceled;
                }
            }
        }

        public int RemoveProperty(string pszPropName, string pszConfigName, uint storage)
        {
            this.QueryEditBuildPropertyStorage();
            return this.propertyStorage.RemoveProperty(pszPropName, pszConfigName, storage);
        }

        public int SetItemAttribute(uint item, string pszAttributeName, string pszAttributeValue)
        {
            this.QueryEditBuildPropertyStorage();
            return this.propertyStorage.SetItemAttribute(item, pszAttributeName, pszAttributeValue);
        }

        public int SetPropertyValue(string pszPropName, string pszConfigName, uint storage, string pszPropValue)
        {
            this.QueryEditBuildPropertyStorage();
            return this.propertyStorage.SetPropertyValue(pszPropName, pszConfigName, storage, pszPropValue);
        }
    }
}

