﻿namespace Microsoft.Xna.GameStudio.PlatformTools
{
    using Microsoft.Win32;
    using Microsoft.Xna.GameStudio.PlatformTools.Properties;
    using System;
    using System.AddIn.Hosting;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.IO;
    using System.Windows.Forms;

    public static class PlatformManager
    {
        private static string addInRoot;

        private static Collection<AddInToken> FindAddIns()
        {
            string addInRoot = AddInRoot;
            if (!string.IsNullOrEmpty(addInRoot))
            {
                string message;
                try
                {
                    return AddInStore.FindAddIns(typeof(IPlatform), addInRoot, new string[0]);
                }
                catch (InvalidPipelineStoreException exception)
                {
                    message = exception.Message;
                }
                catch (InvalidOperationException exception2)
                {
                    message = exception2.Message;
                }
                catch (DirectoryNotFoundException exception3)
                {
                    message = exception3.Message;
                }
                MessageBox.Show(string.Format(Resources.PipelineErrorNeedsRepair, message), Resources.ApplicationName, MessageBoxButtons.OK, MessageBoxIcon.Hand);
            }
            return new Collection<AddInToken>();
        }

        private static IList<AddInToken> GetAddInTokensForRuntime(string runtimeToken)
        {
            if (runtimeToken == null)
            {
                throw new ArgumentNullException("runtimeToken");
            }
            List<AddInToken> list = new List<AddInToken>();
            foreach (AddInToken token in FindAddIns())
            {
                string str = token.QualificationData[AddInSegmentType.AddIn]["RuntimeTokens"];
                foreach (string str2 in str.Split(new char[] { ';' }, StringSplitOptions.RemoveEmptyEntries))
                {
                    if (str2 == runtimeToken)
                    {
                        list.Add(token);
                    }
                }
            }
            return list;
        }

        public static IPlatform GetPlatform(string platform, string runtimeToken)
        {
            if (platform == null)
            {
                throw new ArgumentNullException("platform");
            }
            if (runtimeToken == null)
            {
                throw new ArgumentNullException("runtimeToken");
            }
            foreach (AddInToken token in GetAddInTokensForRuntime(runtimeToken))
            {
                if (platform.Equals(token.QualificationData[AddInSegmentType.AddIn]["Platform"], StringComparison.InvariantCultureIgnoreCase))
                {
                    return token.Activate<IPlatform>(AppDomain.CurrentDomain);
                }
            }
            return null;
        }

        public static IList<IPlatform> GetPlatforms()
        {
            List<IPlatform> list = new List<IPlatform>();
            Collection<AddInToken> collection = FindAddIns();
            if (collection != null)
            {
                foreach (AddInToken token in collection)
                {
                    list.Add(token.Activate<IPlatform>(AppDomain.CurrentDomain));
                }
            }
            return list;
        }

        public static IList<IPlatform> GetPlatforms(string runtimeToken)
        {
            if (runtimeToken == null)
            {
                throw new ArgumentNullException("runtimeToken");
            }
            List<IPlatform> list = new List<IPlatform>();
            foreach (AddInToken token in GetAddInTokensForRuntime(runtimeToken))
            {
                list.Add(token.Activate<IPlatform>(AppDomain.CurrentDomain));
            }
            return list;
        }

        public static string[] RefreshCache()
        {
            string addInRoot = AddInRoot;
            if (!string.IsNullOrEmpty(addInRoot))
            {
                return AddInStore.Update(addInRoot);
            }
            return new string[] { Resources.NoPipelineRootPath };
        }

        private static string AddInRoot
        {
            get
            {
                if (addInRoot == null)
                {
                    string pipelineRootDirectory = Settings.Default.PipelineRootDirectory;
                    if (!string.IsNullOrEmpty(pipelineRootDirectory) && Path.IsPathRooted(pipelineRootDirectory))
                    {
                        addInRoot = pipelineRootDirectory;
                    }
                    else
                    {
                        RegistryKey key = Registry.LocalMachine.OpenSubKey(Settings.Default.PlatformToolsRegistryKey);
                        if (key != null)
                        {
                            string str2 = key.GetValue(null, string.Empty) as string;
                            addInRoot = Path.Combine(str2, pipelineRootDirectory);
                        }
                    }
                    if (addInRoot == null)
                    {
                        addInRoot = string.Empty;
                    }
                }
                return addInRoot;
            }
        }
    }
}

