﻿namespace Microsoft.Xna.GameStudio.RegistrationAttributes
{
    using Microsoft.VisualStudio.Shell;
    using System;
    using System.Globalization;
    using System.IO;

    [AttributeUsage(AttributeTargets.Class)]
    public class ProjectLanguageAdapterAttribute : RegistrationAttribute
    {
        private Type adapterType;
        private Guid languageGuid;
        private Type packageType;

        public ProjectLanguageAdapterAttribute(string languageGuid, Type adapterType, Type packageType)
        {
            if (adapterType == null)
            {
                throw new ArgumentNullException("adapterType");
            }
            if (packageType == null)
            {
                throw new ArgumentNullException("packageType");
            }
            if (languageGuid == null)
            {
                throw new ArgumentNullException("languageGuid");
            }
            if (!Guid.TryParse(languageGuid, out this.languageGuid))
            {
                throw new ArgumentException("languageGuid");
            }
            this.adapterType = adapterType;
            this.packageType = packageType;
        }

        private string CodeProjectsKeyPath()
        {
            string str = this.packageType.GUID.ToString("B").ToUpperInvariant();
            return string.Format(CultureInfo.InvariantCulture, @"Packages\{0}\CodeLanguages", new object[] { str });
        }

        public override void Register(RegistrationAttribute.RegistrationContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            using (RegistrationAttribute.Key key = context.CreateKey(this.CodeProjectsKeyPath()))
            {
                using (RegistrationAttribute.Key key2 = key.CreateSubkey(this.AdapterSubkeyPath))
                {
                    key2.SetValue("Assembly", this.adapterType.Assembly.FullName);
                    key2.SetValue("Class", this.adapterType.FullName);
                }
            }
        }

        public override void Unregister(RegistrationAttribute.RegistrationContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            string str = this.CodeProjectsKeyPath();
            context.RemoveKey(Path.Combine(str, this.AdapterSubkeyPath));
            context.RemoveKeyIfEmpty(str);
        }

        private string AdapterSubkeyPath
        {
            get
            {
                return this.languageGuid.ToString("B").ToUpperInvariant();
            }
        }

        public Type AdapterType
        {
            get
            {
                return this.adapterType;
            }
        }

        public Type PackageType
        {
            get
            {
                return this.packageType;
            }
        }
    }
}

