﻿namespace Microsoft.Xna.GameStudio.RegistrationAttributes
{
    using Microsoft.VisualStudio.Shell;
    using System;
    using System.Globalization;
    using System.Runtime.CompilerServices;

    [AttributeUsage(AttributeTargets.Class, AllowMultiple=true, Inherited=true)]
    public sealed class ProvideManagedMenuResourceAttribute : RegistrationAttribute
    {
        public ProvideManagedMenuResourceAttribute(string resourceName, int version)
        {
            if (resourceName == null)
            {
                throw new ArgumentNullException("resourceName");
            }
            this.ResourceName = resourceName;
            this.Version = version;
        }

        public override void Register(RegistrationAttribute.RegistrationContext context)
        {
            using (RegistrationAttribute.Key key = context.CreateKey("Menus"))
            {
                string str = string.Format(CultureInfo.InvariantCulture, ", {0}, {1}", new object[] { this.ResourceName, this.Version });
                key.SetValue(context.ComponentType.GUID.ToString("B"), str);
            }
        }

        public override void Unregister(RegistrationAttribute.RegistrationContext context)
        {
            context.RemoveValue("Menus", context.ComponentType.GUID.ToString("B"));
        }

        public string ResourceName { get; private set; }

        public int Version { get; private set; }
    }
}

