﻿namespace Microsoft.Xna.GameStudio.Wizards
{
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Properties;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Drawing;
    using System.Runtime.CompilerServices;
    using System.Windows.Forms;

    public class FormProjectOptions : Form
    {
        private Button buttonCancel;
        private Button buttonOk;
        private IContainer components;
        private FlowLayoutPanel flowLayoutPanelButtons;
        private FlowLayoutPanel flowLayoutPanelOptionContainer;
        private List<XnaProjectCreationOption> optionsList = new List<XnaProjectCreationOption>();
        private TableLayoutPanel tableLayoutPanelOverall;

        public FormProjectOptions()
        {
            this.InitializeComponent();
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void InitializeComponent()
        {
            this.tableLayoutPanelOverall = new TableLayoutPanel();
            this.flowLayoutPanelOptionContainer = new FlowLayoutPanel();
            this.flowLayoutPanelButtons = new FlowLayoutPanel();
            this.buttonOk = new Button();
            this.buttonCancel = new Button();
            this.tableLayoutPanelOverall.SuspendLayout();
            this.flowLayoutPanelButtons.SuspendLayout();
            base.SuspendLayout();
            this.tableLayoutPanelOverall.AutoSize = true;
            this.tableLayoutPanelOverall.AutoSizeMode = AutoSizeMode.GrowAndShrink;
            this.tableLayoutPanelOverall.ColumnCount = 1;
            this.tableLayoutPanelOverall.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 100f));
            this.tableLayoutPanelOverall.Controls.Add(this.flowLayoutPanelOptionContainer, 0, 0);
            this.tableLayoutPanelOverall.Controls.Add(this.flowLayoutPanelButtons, 0, 1);
            this.tableLayoutPanelOverall.Dock = DockStyle.Fill;
            this.tableLayoutPanelOverall.Location = new Point(0, 0);
            this.tableLayoutPanelOverall.Name = "tableLayoutPanelOverall";
            this.tableLayoutPanelOverall.RowCount = 2;
            this.tableLayoutPanelOverall.RowStyles.Add(new RowStyle(SizeType.Percent, 100f));
            this.tableLayoutPanelOverall.RowStyles.Add(new RowStyle());
            this.tableLayoutPanelOverall.Size = new Size(400, 0x5c);
            this.tableLayoutPanelOverall.TabIndex = 9;
            this.flowLayoutPanelOptionContainer.AutoSize = true;
            this.flowLayoutPanelOptionContainer.AutoSizeMode = AutoSizeMode.GrowAndShrink;
            this.flowLayoutPanelOptionContainer.Dock = DockStyle.Fill;
            this.flowLayoutPanelOptionContainer.Location = new Point(3, 3);
            this.flowLayoutPanelOptionContainer.Name = "flowLayoutPanelOptionContainer";
            this.flowLayoutPanelOptionContainer.Size = new Size(0x18a, 0x2f);
            this.flowLayoutPanelOptionContainer.TabIndex = 10;
            this.flowLayoutPanelButtons.Anchor = AnchorStyles.Right | AnchorStyles.Bottom | AnchorStyles.Top;
            this.flowLayoutPanelButtons.AutoSize = true;
            this.flowLayoutPanelButtons.AutoSizeMode = AutoSizeMode.GrowAndShrink;
            this.flowLayoutPanelButtons.Controls.Add(this.buttonOk);
            this.flowLayoutPanelButtons.Controls.Add(this.buttonCancel);
            this.flowLayoutPanelButtons.Location = new Point(0xc7, 0x38);
            this.flowLayoutPanelButtons.Name = "flowLayoutPanelButtons";
            this.flowLayoutPanelButtons.Size = new Size(0xc6, 0x21);
            this.flowLayoutPanelButtons.TabIndex = 0;
            this.flowLayoutPanelButtons.WrapContents = false;
            this.buttonOk.Anchor = AnchorStyles.Right | AnchorStyles.Bottom;
            this.buttonOk.DialogResult = DialogResult.OK;
            this.buttonOk.Location = new Point(3, 3);
            this.buttonOk.Name = "buttonOk";
            this.buttonOk.Size = new Size(0x5d, 0x1b);
            this.buttonOk.TabIndex = 5;
            this.buttonOk.Text = "OK";
            this.buttonOk.UseVisualStyleBackColor = true;
            this.buttonCancel.Anchor = AnchorStyles.Right | AnchorStyles.Bottom;
            this.buttonCancel.DialogResult = DialogResult.Cancel;
            this.buttonCancel.Location = new Point(0x66, 3);
            this.buttonCancel.Name = "buttonCancel";
            this.buttonCancel.Size = new Size(0x5d, 0x1b);
            this.buttonCancel.TabIndex = 6;
            this.buttonCancel.Text = "Cancel";
            base.AcceptButton = this.buttonOk;
            base.AutoScaleDimensions = new SizeF(6f, 13f);
            base.AutoScaleMode = AutoScaleMode.Font;
            this.AutoSize = true;
            base.CancelButton = this.buttonCancel;
            base.ClientSize = new Size(400, 0x5c);
            base.Controls.Add(this.tableLayoutPanelOverall);
            base.FormBorderStyle = FormBorderStyle.FixedDialog;
            base.MaximizeBox = false;
            base.MinimizeBox = false;
            base.Name = "FormProjectOptions";
            base.ShowIcon = false;
            base.ShowInTaskbar = false;
            base.SizeGripStyle = SizeGripStyle.Hide;
            base.StartPosition = FormStartPosition.CenterParent;
            this.tableLayoutPanelOverall.ResumeLayout(false);
            this.tableLayoutPanelOverall.PerformLayout();
            this.flowLayoutPanelButtons.ResumeLayout(false);
            base.ResumeLayout(false);
            base.PerformLayout();
        }

        public bool IsDialogNeeded { get; private set; }

        public List<XnaProjectCreationOption> Options
        {
            get
            {
                return new List<XnaProjectCreationOption>(this.optionsList);
            }
            set
            {
                this.optionsList = new List<XnaProjectCreationOption>(value);
                this.flowLayoutPanelOptionContainer.SuspendLayout();
                this.flowLayoutPanelOptionContainer.Controls.Clear();
                this.Text = Resources.WizardProjectOptionsDialogTitle;
                this.buttonOk.Text = Resources.WizardProjectOptionsDialogSelectButton;
                this.buttonCancel.Text = Resources.WizardProjectOptionsDialogCancelButton;
                int num = 0;
                foreach (XnaProjectCreationOption option in this.optionsList)
                {
                    Option option2 = new Option {
                        Title = option.Title,
                        Description = option.Description,
                        PropertyName = option.Property,
                        SelectionList = option.Selections
                    };
                    this.flowLayoutPanelOptionContainer.Controls.Add(option2);
                    if (option.Selections.Count > 1)
                    {
                        num++;
                    }
                    else
                    {
                        option2.Visible = false;
                    }
                }
                this.flowLayoutPanelOptionContainer.ResumeLayout();
                this.IsDialogNeeded = num > 0;
            }
        }

        public Dictionary<string, XnaProjectCreationOption.Selection> Selections
        {
            get
            {
                Dictionary<string, XnaProjectCreationOption.Selection> dictionary = new Dictionary<string, XnaProjectCreationOption.Selection>();
                foreach (Control control in this.flowLayoutPanelOptionContainer.Controls)
                {
                    if (control is Option)
                    {
                        Option option = control as Option;
                        dictionary.Add(option.PropertyName, option.Selected);
                    }
                }
                return dictionary;
            }
        }
    }
}

