﻿namespace Microsoft.Xna.GameStudio.Wizards
{
    using EnvDTE;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.ManagedInterfaces.Publish;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.VisualStudio.TemplateWizard;
    using Microsoft.Win32;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.Properties;
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Text;
    using System.Threading;
    using System.Xml;

    [ProgId("Microsoft.Xna.GameStudio.Wizards.ProjectWizard"), ClassInterface(ClassInterfaceType.None), ComVisible(true), Guid("7A28705A-48C9-4967-90F2-423371C341A1")]
    public sealed class ProjectWizard : IWizard
    {
        private const string cultureReplacementToken = "$currentVsCulture$";
        private string defaultSolutionPlatform;
        private const string destDirReplacementToken = "$destinationdirectory$";
        private Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> knownPlatforms;
        private Dictionary<string, string> replacements;
        private List<string> solutionItems = new List<string>();
        private string sourceDirectory;
        private string targetDirectory;
        private IProjectPlatformAdapter targetProjectPlatformAdapter;
        private IProjectPlatformAdapterCaps targetProjectPlatformAdapterCaps;

        public static void AddDependentBootstrapperPackages(Project project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            IClickOnceProjectExtender extender = null;
            try
            {
                extender = project.get_Extender("Microsoft.Xna.GameStudio.CodeProject.ClickOnceProjectExtender") as IClickOnceProjectExtender;
            }
            catch (COMException)
            {
            }
            if (extender != null)
            {
                Property property = null;
                try
                {
                    property = project.Properties.Item("Publish");
                }
                catch (COMException)
                {
                }
                catch (ArgumentException)
                {
                }
                if (property != null)
                {
                    IPublishProperties publishProps = null;
                    try
                    {
                        publishProps = property.Object as IPublishProperties;
                    }
                    catch (COMException)
                    {
                    }
                    if (publishProps != null)
                    {
                        foreach (string str in extender.GetBootstrapperPackages())
                        {
                            SetBootstrapperPackageDependency(publishProps, str, true);
                        }
                    }
                }
            }
        }

        public void BeforeOpeningFile(EnvDTE.ProjectItem projectItem)
        {
        }

        private void CopyFiles(XmlElement wizardData, string relativeFolderPath)
        {
            if (wizardData == null)
            {
                throw new ArgumentNullException("wizardData");
            }
            if (relativeFolderPath == null)
            {
                throw new ArgumentNullException("relativeFolderPath");
            }
            foreach (XmlNode node in wizardData.ChildNodes)
            {
                string str;
                XmlElement projectItemElement = node as XmlElement;
                if (projectItemElement != null)
                {
                    string name = projectItemElement.Name;
                    if (name == null)
                    {
                        goto Label_0127;
                    }
                    if (!(name == "ProjectItem"))
                    {
                        if (name == "Folder")
                        {
                            goto Label_0095;
                        }
                        if (name == "DefaultSolutionPlatform")
                        {
                            goto Label_0119;
                        }
                        goto Label_0127;
                    }
                    this.CopyProjectItem(projectItemElement, relativeFolderPath);
                }
                continue;
            Label_0095:
                str = GetAttributeValue(projectItemElement, "Name");
                string attributeValue = GetAttributeValue(projectItemElement, "TargetFolderName");
                if (string.IsNullOrEmpty(str) || (string.Compare(str, attributeValue, StringComparison.OrdinalIgnoreCase) != 0))
                {
                    throw new ArgumentOutOfRangeException("wizardData", Microsoft.Xna.GameStudio.Properties.Resources.WizardInvalidFolderElement);
                }
                string str3 = Path.Combine(relativeFolderPath, attributeValue);
                string fullPath = Path.GetFullPath(Path.Combine(this.targetDirectory, str3));
                if (!IsValidSubdirectory(this.targetDirectory, fullPath))
                {
                    throw new InvalidOperationException(Microsoft.Xna.GameStudio.Properties.Resources.WizardInvalidDestinationPath);
                }
                Directory.CreateDirectory(fullPath);
                this.CopyFiles(projectItemElement, str3);
                continue;
            Label_0119:
                this.defaultSolutionPlatform = projectItemElement.InnerText;
                continue;
            Label_0127:
                throw new ArgumentOutOfRangeException("wizardData", Microsoft.Xna.GameStudio.Properties.Resources.WizardInvalidProjectItemElement);
            }
        }

        private void CopyFileWithReplacement(string sourceFile, string targetFile)
        {
            if (sourceFile == null)
            {
                throw new ArgumentNullException("sourceFile");
            }
            if (targetFile == null)
            {
                throw new ArgumentNullException("targetFile");
            }
            if (File.Exists(sourceFile))
            {
                using (StreamReader reader = File.OpenText(sourceFile))
                {
                    string str = this.PerformReplacements(reader.ReadToEnd());
                    using (StreamWriter writer = File.CreateText(targetFile))
                    {
                        writer.Write(str);
                    }
                }
            }
        }

        private void CopyProjectItem(XmlElement projectItemElement, string relativeFolderPath)
        {
            if (projectItemElement == null)
            {
                throw new ArgumentNullException("projectItemElement");
            }
            if (relativeFolderPath == null)
            {
                throw new ArgumentNullException("relativeFolderPath");
            }
            if (projectItemElement.Name != "ProjectItem")
            {
                throw new ArgumentException(Microsoft.Xna.GameStudio.Properties.Resources.WizardInvalidProjectItemElement, "projectItemElement");
            }
            string fullPath = Path.GetFullPath(Path.Combine(this.targetDirectory, relativeFolderPath));
            if (!IsValidSubdirectory(this.targetDirectory, fullPath))
            {
                throw new InvalidOperationException(Microsoft.Xna.GameStudio.Properties.Resources.WizardInvalidDestinationPath);
            }
            if (!Directory.Exists(fullPath))
            {
                throw new InvalidOperationException(Microsoft.Xna.GameStudio.Properties.Resources.WizardInvalidDestinationPath);
            }
            string fileName = Path.GetFileName(projectItemElement.InnerText);
            if (string.Compare(fileName, projectItemElement.InnerText, StringComparison.OrdinalIgnoreCase) != 0)
            {
                throw new InvalidOperationException(Microsoft.Xna.GameStudio.Properties.Resources.WizardInvalidFileNameSpecified);
            }
            string attributeValue = GetAttributeValue(projectItemElement, "TargetFileName");
            if (string.IsNullOrEmpty(attributeValue))
            {
                attributeValue = fileName;
            }
            else if (string.Compare(attributeValue, Path.GetFileName(attributeValue), StringComparison.OrdinalIgnoreCase) != 0)
            {
                throw new InvalidOperationException(Microsoft.Xna.GameStudio.Properties.Resources.WizardInvalidFileNameSpecified);
            }
            string sourceFile = Path.Combine(Path.Combine(this.sourceDirectory, relativeFolderPath), fileName);
            string original = Path.Combine(fullPath, attributeValue);
            if (string.Compare(GetAttributeValue(projectItemElement, "ReplaceParameters"), "true", StringComparison.OrdinalIgnoreCase) == 0)
            {
                original = this.PerformReplacements(original);
                this.CopyFileWithReplacement(sourceFile, original);
            }
            else
            {
                File.Copy(sourceFile, original);
            }
            if (string.Compare(GetAttributeValue(projectItemElement, "AddToSolution"), "true", StringComparison.OrdinalIgnoreCase) == 0)
            {
                this.solutionItems.Add(original);
            }
        }

        private static string GetAttributeValue(XmlElement element, string attributeName)
        {
            if (element == null)
            {
                throw new ArgumentNullException("element");
            }
            if (attributeName == null)
            {
                throw new ArgumentNullException("attributeName");
            }
            XmlAttribute namedItem = element.Attributes.GetNamedItem(attributeName) as XmlAttribute;
            if (namedItem != null)
            {
                return namedItem.Value;
            }
            return string.Empty;
        }

        private Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> GetRegisteredProjectPlatformAdapters(object automationObject)
        {
            string str;
            Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> dictionary = new Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter>();
            ServiceProvider provider = new ServiceProvider((Microsoft.VisualStudio.OLE.Interop.IServiceProvider) automationObject);
            ILocalRegistry4 service = provider.GetService(typeof(SLocalRegistry)) as ILocalRegistry4;
            uint pdwRegRootHandle = 0;
            ErrorHandler.ThrowOnFailure(service.GetLocalRegistryRootEx(2, out pdwRegRootHandle, out str));
            string name = string.Format(CultureInfo.InvariantCulture, @"{0}\Packages\{{{1}}}\CodeProjects", new object[] { str, "4098508B-4144-45D3-9801-736DC39D8E8A" });
            using (RegistryKey key = Registry.CurrentUser.OpenSubKey(name))
            {
                if (key == null)
                {
                    return dictionary;
                }
                foreach (string str3 in key.GetSubKeyNames())
                {
                    using (RegistryKey key2 = key.OpenSubKey(str3))
                    {
                        string assemblyName = (string) key2.GetValue("Assembly");
                        string typeName = (string) key2.GetValue("Class");
                        IProjectPlatformAdapter adapter = (IProjectPlatformAdapter) AppDomain.CurrentDomain.CreateInstance(assemblyName, typeName).Unwrap();
                        KeyValuePair<string, string> pair = new KeyValuePair<string, string>(adapter.XnaFrameworkVersion, adapter.XnaPlatform);
                        if (!dictionary.ContainsKey(pair))
                        {
                            dictionary.Add(pair, adapter);
                        }
                    }
                }
            }
            return dictionary;
        }

        private static bool IsValidSubdirectory(string directory, string subdirectory)
        {
            if (directory == null)
            {
                throw new ArgumentNullException("directory");
            }
            if (subdirectory == null)
            {
                throw new ArgumentNullException("subdirectory");
            }
            if (string.Compare(directory, subdirectory, StringComparison.OrdinalIgnoreCase) == 0)
            {
                return true;
            }
            if ((directory.Length > 0) && (directory[directory.Length - 1] != Path.DirectorySeparatorChar))
            {
                directory = directory + Path.DirectorySeparatorChar;
            }
            if ((subdirectory.Length > 0) && (subdirectory[subdirectory.Length - 1] != Path.DirectorySeparatorChar))
            {
                subdirectory = subdirectory + Path.DirectorySeparatorChar;
            }
            return subdirectory.StartsWith(directory, StringComparison.OrdinalIgnoreCase);
        }

        private string PerformReplacements(string original)
        {
            StringBuilder builder = new StringBuilder(original);
            foreach (KeyValuePair<string, string> pair in this.replacements)
            {
                builder.Replace(pair.Key, pair.Value);
            }
            return builder.ToString();
        }

        private void ProcessWizardExtensionData()
        {
            string str = null;
            if (this.replacements.TryGetValue("$wizarddata$", out str) && !string.IsNullOrEmpty(str))
            {
                str = str.Trim();
                XmlDocument document = new XmlDocument();
                XmlElement wizardData = null;
                try
                {
                    XmlElement element2 = document.CreateElement("Temp");
                    element2.InnerXml = str;
                    wizardData = element2;
                }
                catch (XmlException)
                {
                }
                if (wizardData != null)
                {
                    this.CopyFiles(wizardData, string.Empty);
                }
            }
        }

        public void ProjectFinishedGenerating(Project project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            string a = string.Empty;
            XmlDocument document = new XmlDocument();
            document.Load(project.FullName);
            XmlNamespaceManager nsmgr = new XmlNamespaceManager(document.NameTable);
            nsmgr.AddNamespace("msbuild", "http://schemas.microsoft.com/developer/msbuild/2003");
            XmlNode node = document.SelectSingleNode("//msbuild:XnaOutputType", nsmgr);
            if (node != null)
            {
                a = node.InnerText;
            }
            if (string.Equals(a, "Game", StringComparison.InvariantCulture) && (this.targetProjectPlatformAdapterCaps != null))
            {
                bool result = false;
                if (bool.TryParse(this.targetProjectPlatformAdapterCaps.GetCapabilityValue("bootstrapperClickOnce"), out result) && result)
                {
                    this.ProcessWizardExtensionData();
                    AddDependentBootstrapperPackages(project);
                    Solution solution = project.Globals.DTE.Solution;
                    foreach (string str2 in this.solutionItems)
                    {
                        solution.AddFromFile(str2, false);
                    }
                    if (this.defaultSolutionPlatform != null)
                    {
                        project.Globals.DTE.ExecuteCommand("Build.SolutionPlatforms", this.defaultSolutionPlatform);
                    }
                }
            }
        }

        public void ProjectItemFinishedGenerating(EnvDTE.ProjectItem projectItem)
        {
        }

        public void RunFinished()
        {
        }

        public void RunStarted(object automationObject, Dictionary<string, string> replacementsDictionary, WizardRunKind runKind, object[] customParams)
        {
            if (replacementsDictionary == null)
            {
                throw new ArgumentNullException("replacementsDictionary");
            }
            if (customParams == null)
            {
                throw new ArgumentNullException("customParams");
            }
            if (MultiProjectWizard.isMultiProjectActive && MultiProjectWizard.cancelMultiProjWizard)
            {
                throw new WizardCancelledException();
            }
            if (runKind != WizardRunKind.AsNewProject)
            {
                throw new InvalidOperationException("This wizard extension can only be executed in the context of a project template.");
            }
            string str = customParams[0] as string;
            if (string.IsNullOrEmpty(str))
            {
                throw new InvalidOperationException("This wizard extension can only be executed in the context of a project template.");
            }
            this.sourceDirectory = Path.GetFullPath(Path.GetDirectoryName(str));
            if (string.IsNullOrEmpty(this.sourceDirectory))
            {
                throw new InvalidOperationException("This wizard extension can only be executed in the context of a project template.");
            }
            this.replacements = replacementsDictionary;
            this.targetDirectory = replacementsDictionary["$destinationdirectory$"];
            if (this.replacements.ContainsKey("$currentVsCulture$"))
            {
                this.replacements.Remove("$currentVsCulture$");
                this.replacements.Add("$currentVsCulture$", System.Threading.Thread.CurrentThread.CurrentUICulture.Name);
            }
            if (this.replacements.ContainsKey("$XnaPlatform$") && this.replacements.ContainsKey("$XnaFrameworkVersion$"))
            {
                if (this.knownPlatforms == null)
                {
                    this.knownPlatforms = this.GetRegisteredProjectPlatformAdapters(automationObject);
                }
                if (this.knownPlatforms != null)
                {
                    KeyValuePair<string, string> key = new KeyValuePair<string, string>(this.replacements["$XnaFrameworkVersion$"], this.replacements["$XnaPlatform$"]);
                    if (this.knownPlatforms.ContainsKey(key))
                    {
                        this.targetProjectPlatformAdapter = this.knownPlatforms[key];
                        this.targetProjectPlatformAdapterCaps = this.targetProjectPlatformAdapter as IProjectPlatformAdapterCaps;
                        IXnaCreationOptionsFilter targetProjectPlatformAdapter = this.targetProjectPlatformAdapter as IXnaCreationOptionsFilter;
                        if ((targetProjectPlatformAdapter != null) && !WizardUtilities.PresentProjectCreationOptions(automationObject, null, replacementsDictionary, targetProjectPlatformAdapter))
                        {
                            if (MultiProjectWizard.isMultiProjectActive)
                            {
                                MultiProjectWizard.cancelMultiProjWizard = true;
                            }
                            throw new WizardCancelledException();
                        }
                    }
                }
            }
        }

        public static bool SetBootstrapperPackageDependency(IPublishProperties publishProps, string packageName, bool install)
        {
            if (publishProps == null)
            {
                throw new ArgumentNullException("publishProps");
            }
            if (packageName == null)
            {
                throw new ArgumentNullException("packageName");
            }
            bool flag = !install;
            IBootstrapperPackage package = publishProps.BootstrapperPackages[packageName];
            try
            {
                if (package != null)
                {
                    package.Install = install;
                    flag = true;
                }
            }
            catch (NullReferenceException)
            {
            }
            return flag;
        }

        public bool ShouldAddProjectItem(string filePath)
        {
            return true;
        }

        private sealed class AdapterKeyComparer : IEqualityComparer<KeyValuePair<string, string>>
        {
            public bool Equals(KeyValuePair<string, string> x, KeyValuePair<string, string> y)
            {
                return ((x.Key == y.Key) && (x.Value == y.Value));
            }

            public int GetHashCode(KeyValuePair<string, string> obj)
            {
                return (obj.Key + ";" + obj.Value).GetHashCode();
            }
        }
    }
}

