﻿namespace Microsoft.Xna.GameStudio.Wizards
{
    using Microsoft.Build.Construction;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.CodeProject;
    using System;
    using System.Collections.Generic;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    public static class WizardUtilities
    {
        public static bool PresentProjectCreationOptions(object automationObject, string templateName, Dictionary<string, string> replacementsDictionary, IXnaCreationOptionsFilter optionFilter)
        {
            bool userCanceled = false;
            Dictionary<string, string> dictionary = new Dictionary<string, string>(replacementsDictionary.Count);
            foreach (KeyValuePair<string, string> pair in replacementsDictionary)
            {
                string key = pair.Key.Trim("$".ToCharArray());
                dictionary.Add(key, pair.Value);
            }
            Dictionary<string, XnaProjectCreationOption.Selection> dictionary2 = ShowProjectCreationOptionsForm(optionFilter.ProjectCreationOptions(automationObject, templateName, dictionary), out userCanceled);
            foreach (string str in dictionary2.Keys)
            {
                if ((dictionary2[str] != null) && replacementsDictionary.ContainsKey("$" + str + "$"))
                {
                    replacementsDictionary["$" + str + "$"] = dictionary2[str].Value;
                }
            }
            return !userCanceled;
        }

        public static bool PresentProjectMigrationOptions(ProjectRootElement project, IProjectPlatformAdapter targetAdapter)
        {
            bool userCanceled = false;
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            if (targetAdapter == null)
            {
                throw new ArgumentNullException("targetAdapter");
            }
            IXnaCreationOptionsFilter filter = targetAdapter as IXnaCreationOptionsFilter;
            if (filter != null)
            {
                Dictionary<string, string> replacementsDictionary = new Dictionary<string, string>(project.Properties.Count);
                foreach (ProjectPropertyElement element in project.Properties)
                {
                    if (!replacementsDictionary.ContainsKey(element.Name))
                    {
                        replacementsDictionary.Add(element.Name, element.Value);
                    }
                }
                Dictionary<string, XnaProjectCreationOption.Selection> dictionary2 = ShowProjectCreationOptionsForm(filter.ProjectCreationOptions(null, null, replacementsDictionary), out userCanceled);
                foreach (string str in dictionary2.Keys)
                {
                    project.AddProperty(str, dictionary2[str].Value);
                }
            }
            return !userCanceled;
        }

        public static Dictionary<string, XnaProjectCreationOption.Selection> ShowProjectCreationOptionsForm(List<XnaProjectCreationOption> options, out bool userCanceled)
        {
            if (options == null)
            {
                throw new ArgumentNullException("options");
            }
            userCanceled = false;
            if (options.Count <= 0)
            {
                return new Dictionary<string, XnaProjectCreationOption.Selection>();
            }
            FormProjectOptions options2 = new FormProjectOptions {
                Options = options
            };
            if (options2.IsDialogNeeded)
            {
                userCanceled = options2.ShowDialog() != DialogResult.OK;
            }
            return options2.Selections;
        }
    }
}

