#pragma once

#include "Vector3.h"
#include "Matrix.h"

using namespace System;
using namespace System::Diagnostics::CodeAnalysis;
using namespace System::Runtime::InteropServices;

namespace Microsoft {
	namespace Xna {
		namespace Framework {
			value class Vector3;

			public value class Quaternion
			{
			private:
				static Quaternion ^_identity;
				static Quaternion();
			public:
				[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float X;

				[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float Y;

				[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float Z;

				[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float W;

				static property Quaternion^ Identity
				{
					Quaternion^ get()
					{
						return _identity;
					}
				}

				Quaternion(float x, float y, float z, float w);
				Quaternion(Vector3 ^vectorPart, float scalarPart);

				virtual String^ ToString() override;
				bool Equals(Quaternion ^other);
				virtual bool Equals(Object ^obj) override;
				virtual int GetHashCode() override;

				float LengthSquared();
				float Length();

				void Normalize();
				static Quaternion^ Normalize(Quaternion ^quaternion);
				static void Normalize(Quaternion ^%quaternion, [Out] Quaternion ^%result);

				void Conjugate();
				static Quaternion^ Conjugate(Quaternion ^value);
				static void Conjugate(Quaternion ^%value, [Out] Quaternion ^%result);

				static Quaternion^ Inverse(Quaternion ^quaternion);
				static void Inverse(Quaternion ^%quaternion, [Out] Quaternion ^%result);
				
				static Quaternion^ CreateFromAxisAngle(Vector3 ^axis, float angle);
				static void CreateFromAxisAngle(Vector3 ^%axis, float angle, [Out] Quaternion ^%result);
				
				static Quaternion^ CreateFromYawPitchRoll(float yaw, float pitch, float roll);
				static void CreateFromYawPitchRoll(float yaw, float pitch, float roll, [Out] Quaternion ^%result);

				static Quaternion^ CreateFromRotationMatrix(Matrix ^matrix);
				static void CreateFromRotationMatrix(Matrix ^%matrix, [Out] Quaternion ^%result);

				static float Dot(Quaternion ^quaternion1, Quaternion ^quaternion2);
				static void Dot(Quaternion ^%quaternion1, Quaternion ^%quaternion2, [Out] float ^%result);

				static Quaternion^ Slerp(Quaternion ^quaternion1, Quaternion ^quaternion2, float amount);
				static void Slerp(Quaternion ^%quaternion1, Quaternion ^%quaternion2, float amount, [Out] Quaternion ^%result);

				static Quaternion^ Lerp(Quaternion ^quaternion1, Quaternion ^quaternion2, float amount);
				static void Lerp(Quaternion ^%quaternion1, Quaternion ^%quaternion2, float amount, [Out] Quaternion ^%result);

				static Quaternion^ Concatenate(Quaternion ^value1, Quaternion ^value2);
				static void Concatenate(Quaternion ^%value1, Quaternion ^%value2, [Out] Quaternion ^%result);

				static Quaternion^ Negate(Quaternion ^quaternion);
				static void Negate(Quaternion ^%quaternion, [Out] Quaternion ^%result);

				static Quaternion^ Add(Quaternion ^quaternion1, Quaternion ^quaternion2);
				static void Add(Quaternion ^%quaternion1, Quaternion ^%quaternion2, [Out] Quaternion ^%result);

				static Quaternion^ Subtract(Quaternion ^quaternion1, Quaternion ^quaternion2);
				static void Subtract(Quaternion ^%quaternion1, Quaternion ^%quaternion2, [Out] Quaternion ^%result);

				static Quaternion^ Multiply(Quaternion ^quaternion1, Quaternion ^quaternion2);
				static void Multiply(Quaternion ^%quaternion1, Quaternion ^%quaternion2, [Out] Quaternion ^%result);

				static Quaternion^ Multiply(Quaternion ^quaternion1, float scaleFactor);
				static void Multiply(Quaternion ^%quaternion1, float scaleFactor, [Out] Quaternion ^%result);

				static Quaternion^ Divide(Quaternion ^quaternion1, Quaternion ^quaternion2);
				static void Divide(Quaternion ^%quaternion1, Quaternion ^%quaternion2, [Out] Quaternion ^%result);

				static Quaternion^ operator -(Quaternion ^quaternion);
				static bool operator ==(Quaternion ^quaternion1, Quaternion ^quaternion2);
				static bool operator !=(Quaternion ^quaternion1, Quaternion ^quaternion2);
				static Quaternion^ operator +(Quaternion ^quaternion1, Quaternion ^quaternion2);
				static Quaternion^ operator -(Quaternion ^quaternion1, Quaternion ^quaternion2);
				static Quaternion^ operator *(Quaternion ^quaternion1, Quaternion ^quaternion2);
				static Quaternion^ operator *(Quaternion ^quaternion1, float scaleFactor);
				static Quaternion^ operator /(Quaternion ^quaternion1, Quaternion ^quaternion2);
			};
		}
	}
}
