#pragma once

using namespace System;
using namespace System::Diagnostics::CodeAnalysis;
using namespace System::Runtime::InteropServices;

#include "Quaternion.h"
#include "Matrix.h"

namespace Microsoft {
	namespace Xna {
		namespace Framework {
			value class Quaternion;

			public value class Vector2
			{
			private:
				static Vector2 ^_zero;
				static Vector2 ^_one;
				static Vector2 ^_unitX;
				static Vector2 ^_unitY;
				static Vector2();
			public:
				[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float X;

				[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float Y;

				static property Vector2^ Zero
				{
					Vector2^ get()
					{
						return _zero;
					}
				}

				static property Vector2^ One
				{
					Vector2^ get()
					{
						return _one;
					}
				}

				static property Vector2^ UnitX
				{
					Vector2^ get()
					{
						return _unitX;
					}
				}

				static property Vector2^ UnitY
				{
					Vector2^ get()
					{
						return _unitY;
					}
				}

				Vector2(float x, float y);
				Vector2(float value);

				virtual String^ ToString() override;
				bool Equals(Vector2^ other);
				virtual bool Equals(Object^ obj) override;
				virtual int GetHashCode() override;

				float Length();
				float LengthSquared();

				static float Distance(Vector2 ^value1, Vector2 ^value2);
				static void Distance(Vector2 ^%value1,  Vector2 ^%value2, [Out] float ^%result);

				static float DistanceSquared(Vector2 ^value1, Vector2 ^value2);
				static void DistanceSquared(Vector2 ^%value1, Vector2 ^%value2, [Out] float ^%result);

				static float Dot(Vector2 ^value1, Vector2 ^value2);
				static void Dot(Vector2 ^%value1, Vector2 ^%value2, [Out] float ^%result);

				void Normalize();
				static Vector2^ Normalize(Vector2 ^value);
				static void Normalize(Vector2 ^%value, [Out] Vector2 ^%result);

				static Vector2^ Reflect(Vector2 ^vector, Vector2 ^normal);
				static void Reflect(Vector2 ^%vector, Vector2 ^%normal, [Out] Vector2 ^%result);

				static Vector2^ Min(Vector2 ^value1, Vector2 ^value2);
				static void Min(Vector2 ^%value1, Vector2 ^%value2, [Out] Vector2 ^%result);

				static Vector2^ Max(Vector2 ^value1, Vector2 ^value2);
				static void Max(Vector2 ^%value1, Vector2 ^%value2, [Out] Vector2 ^%result);

				static Vector2^ Clamp(Vector2 ^value1, Vector2 ^min, Vector2 ^max);
				static void Clamp(Vector2 ^%value1, Vector2 ^%min, Vector2 ^%max, [Out] Vector2 ^%result);

				static Vector2^ Lerp(Vector2 ^value1, Vector2 ^value2, float amount);
				static void Lerp(Vector2 ^%value1, Vector2 ^%value2, float amount, [Out] Vector2 ^%result);

				static Vector2^ Barycentric(Vector2 ^value1, Vector2 ^value2, Vector2 ^value3, float amount1, float amount2);
				static void Barycentric(Vector2 ^%value1, Vector2 ^%value2, Vector2 ^%value3, float amount1, float amount2, [Out] Vector2 ^%result);

				static Vector2^ SmoothStep(Vector2 ^value1, Vector2 ^value2, float amount);
				static void SmoothStep(Vector2 ^%value1, Vector2 ^%value2, float amount, [Out] Vector2 ^%result);

				static Vector2^ CatmullRom(Vector2 ^value1, Vector2 ^value2, Vector2 ^value3, Vector2 ^value4, float amount);
				static void CatmullRom(Vector2 ^%value1, Vector2 ^%value2, Vector2 ^%value3, Vector2 ^%value4, float amount, [Out] Vector2 ^%result);

				static Vector2^ Hermite(Vector2 ^value1, Vector2 ^tangent1, Vector2 ^value2, Vector2 ^tangent2, float amount);
				static void Hermite(Vector2 ^%value1, Vector2 ^%tangent1, Vector2 ^%value2, Vector2 ^%tangent2, float amount, [Out] Vector2 ^%result);

				static Vector2^ Transform(Vector2 ^position, Matrix ^matrix);
				static void Transform(Vector2 ^%position, Matrix ^%matrix, [Out] Vector2 ^%result);

				static Vector2^ TransformNormal(Vector2 ^normal, Matrix ^matrix);

				static void TransformNormal(Vector2 ^%normal, Matrix ^%matrix, [Out] Vector2 ^%result);

				static Vector2^ Transform(Vector2 ^value, Quaternion ^rotation);

				static void Transform(Vector2 ^%value, Quaternion ^%rotation, [Out] Vector2 ^%result);

				static void Transform(array<Vector2> ^sourceArray, Matrix ^%matrix, array<Vector2> ^destinationArray);

				[SuppressMessage("Microsoft.Usage", "CA2233")]
				static void Transform(array<Vector2> ^sourceArray, int sourceIndex, Matrix ^%matrix, array<Vector2> ^destinationArray, int destinationIndex, int length);

				static void TransformNormal(array<Vector2> ^sourceArray, Matrix ^%matrix, array<Vector2> ^destinationArray);

				[SuppressMessage("Microsoft.Usage", "CA2233")]
				static void TransformNormal(array<Vector2> ^sourceArray, int sourceIndex, Matrix ^%matrix, array<Vector2> ^destinationArray, int destinationIndex, int length);

				static void Transform(array<Vector2> ^sourceArray, Quaternion ^%rotation, array<Vector2> ^destinationArray);

				[SuppressMessage("Microsoft.Usage", "CA2233")]
				static void Transform(array<Vector2> ^sourceArray, int sourceIndex, Quaternion ^%rotation, array<Vector2> ^destinationArray, int destinationIndex, int length);

				static Vector2^ Negate(Vector2 ^value);
				static void Negate(Vector2 ^%value, [Out] Vector2 ^%result);

				static Vector2^ Add(Vector2 ^value1, Vector2 ^value2);
				static void Add(Vector2 ^%value1, Vector2 ^%value2, [Out] Vector2 ^%result);

				static Vector2^ Subtract(Vector2 ^value1, Vector2 ^value2);
				static void Subtract(Vector2 ^%value1, Vector2 ^%value2, [Out] Vector2 ^%result);

				static Vector2^ Multiply(Vector2 ^value1, Vector2 ^value2);
				static void Multiply(Vector2 ^%value1, Vector2 ^%value2, [Out] Vector2 ^%result);

				static Vector2^ Multiply(Vector2 ^value1, float scaleFactor);
				static void Multiply(Vector2 ^%value1, float scaleFactor, [Out] Vector2 ^%result);

				static Vector2^ Divide(Vector2 ^value1, Vector2 ^value2);
				static void Divide(Vector2 ^%value1, Vector2 ^%value2, [Out] Vector2 ^%result);

				static Vector2^ Divide(Vector2 ^value1, float divider);
				static void Divide(Vector2 ^%value1, float divider, [Out] Vector2 ^%result);

				static Vector2^ operator -(Vector2 ^value);
				static bool operator ==(Vector2 ^value1, Vector2 ^value2);
				static bool operator !=(Vector2 ^value1, Vector2 ^value2);
				static Vector2^ operator +(Vector2 ^value1, Vector2 ^value2);
				static Vector2^ operator -(Vector2 ^value1, Vector2 ^value2);
				static Vector2^ operator *(Vector2 ^value1, Vector2 ^value2);
				static Vector2^ operator *(Vector2 ^value, float scaleFactor);
				static Vector2^ operator *(float scaleFactor, Vector2 ^value);
				static Vector2^ operator /(Vector2 ^value1, Vector2 ^value2);
				static Vector2^ operator /(Vector2 ^value1, float divider);
			};
		}
	}
}
