#pragma once

#include "Vector2.h"
#include "Quaternion.h"
#include "Matrix.h"

using namespace System;
using namespace System::Diagnostics::CodeAnalysis;
using namespace System::Runtime::InteropServices;

namespace Microsoft {
	namespace Xna {
		namespace Framework {
			value class Vector2;
			value class Quaternion;

			public value class Vector3
			{
			private:
				static Vector3 ^_zero;
				static Vector3 ^_one;
				static Vector3 ^_unitX;
				static Vector3 ^_unitY;
				static Vector3 ^_unitZ;
				static Vector3 ^_up;
				static Vector3 ^_down;
				static Vector3 ^_right;
				static Vector3 ^_left;
				static Vector3 ^_forward;
				static Vector3 ^_backward;
				static Vector3();
			public:
				[SuppressMessageAttribute("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float X;

				[SuppressMessageAttribute("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float Y;

				[SuppressMessageAttribute("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float Z;

				static property Vector3^ Zero
				{
					Vector3^ get()
					{
						return _zero;
					}
				}

				static property Vector3^ One
				{
					Vector3^ get()
					{
						return _one;
					}
				}

				static property Vector3^ UnitX
				{
					Vector3^ get()
					{
						return _unitX;
					}
				}

				static property Vector3^ UnitY
				{
					Vector3^ get()
					{
						return _unitY;
					}
				}

				static property Vector3^ UnitZ
				{
					Vector3^ get()
					{
						return _unitZ;
					}
				}

				static property Vector3^ Up
				{
					Vector3^ get()
					{
						return _up;
					}
				}

				static property Vector3^ Down
				{
					Vector3^ get()
					{
						return _down;
					}
				}

				static property Vector3^ Right
				{
					Vector3^ get()
					{
						return _right;
					}
				}

				static property Vector3^ Left
				{
					Vector3^ get()
					{
						return _left;
					}
				}

				static property Vector3^ Forward
				{
					Vector3^ get()
					{
						return _forward;
					}
				}

				static property Vector3^ Backward
				{
					Vector3^ get()
					{
						return _backward;
					}
				}

				Vector3(float x, float y, float z);
				Vector3(Vector2 ^value, float z);
				Vector3(float value);

				virtual String^ ToString() override;
				bool Equals(Vector3^ other);
				virtual bool Equals(Object^ obj) override;
				virtual int GetHashCode() override;

				float Length();
				float LengthSquared();
				static float Distance(Vector3 ^value1, Vector3 ^value2);
				static void Distance(Vector3 ^%value1, Vector3 ^%value2, [Out] float ^%result);
				static float DistanceSquared(Vector3 ^value1, Vector3 ^value2);
				static void DistanceSquared(Vector3 ^%value1, Vector3 ^%value2, [Out] float ^%result);
				static float Dot(Vector3 ^vector1, Vector3 ^vector2);
				static void Dot(Vector3 ^%vector1, Vector3 ^%vector2, [Out] float ^%result);
				void Normalize();
				static Vector3^ Normalize(Vector3 ^value);
				static void Normalize(Vector3 ^%value, [Out] Vector3 ^%result);
				static Vector3^ Cross(Vector3 ^vector1, Vector3 ^vector2);
				static void Cross(Vector3 ^%vector1,  Vector3 ^%vector2, [Out] Vector3 ^%result);
				static Vector3^ Reflect(Vector3 ^vector, Vector3 ^normal);
				static void Reflect(Vector3 ^%vector,  Vector3 ^%normal, [Out] Vector3 ^%result);
				static Vector3^ Min(Vector3 ^value1, Vector3 ^value2);
				static void Min( Vector3 ^%value1,  Vector3 ^%value2, [Out] Vector3 ^%result);
				static Vector3^ Max(Vector3 ^value1, Vector3 ^value2);
				static void Max( Vector3 ^%value1,  Vector3 ^%value2, [Out] Vector3 ^%result);
				static Vector3^ Clamp(Vector3 ^value1, Vector3 ^min, Vector3 ^max);
				static void Clamp( Vector3 ^%value1,  Vector3 ^%min,  Vector3 ^%max, [Out] Vector3 ^%result);
				static Vector3^ Lerp(Vector3 ^value1, Vector3 ^value2, float amount);
				static void Lerp( Vector3 ^%value1,  Vector3 ^%value2, float amount, [Out] Vector3 ^%result);
				static Vector3^ Barycentric(Vector3 ^value1, Vector3 ^value2, Vector3 ^value3, float amount1, float amount2);
				static void Barycentric(Vector3 ^%value1, Vector3 ^%value2, Vector3 ^%value3, float amount1, float amount2, [Out] Vector3 ^%result);
				static Vector3^ SmoothStep(Vector3 ^value1, Vector3 ^value2, float amount);
				static void SmoothStep(Vector3 ^%value1, Vector3 ^%value2, float amount, [Out] Vector3 ^%result);
				static Vector3^ CatmullRom(Vector3 ^value1, Vector3 ^value2, Vector3 ^value3, Vector3 ^value4, float amount);
				static void CatmullRom(Vector3 ^%value1, Vector3 ^%value2, Vector3 ^%value3, Vector3 ^%value4, float amount, [Out] Vector3 ^%result);
				static Vector3^ Hermite(Vector3 ^value1, Vector3 ^tangent1, Vector3 ^value2, Vector3 ^tangent2, float amount);
				static void Hermite(Vector3 ^%value1, Vector3 ^%tangent1, Vector3 ^%value2, Vector3 ^%tangent2, float amount, [Out] Vector3 ^%result);

				static Vector3^ Transform(Vector3 position, Matrix matrix);
				static void Transform(Vector3 ^%position, Matrix ^%matrix, [Out] Vector3 ^%result);
				static Vector3^ TransformNormal(Vector3 normal, Matrix matrix);
				static void TransformNormal(Vector3 ^%normal, Matrix ^%matrix, [Out] Vector3 ^%result);
				static Vector3^ Transform(Vector3 value, Quaternion rotation);
				static void Transform(Vector3 ^%value, Quaternion ^%rotation, [Out] Vector3 ^%result);
				static void Transform(array<Vector3> ^sourceArray, Matrix ^%matrix, array<Vector3> ^destinationArray);
				[SuppressMessage("Microsoft.Usage", "CA2233")]
				static void Transform(array<Vector3> ^sourceArray, int sourceIndex, Matrix ^%matrix, array<Vector3> ^destinationArray, int destinationIndex, int length);
				static void TransformNormal(array<Vector3> ^sourceArray, Matrix ^%matrix, array<Vector3> ^destinationArray);
				[SuppressMessage("Microsoft.Usage", "CA2233")]
				static void TransformNormal(array<Vector3> ^sourceArray, int sourceIndex, Matrix ^%matrix, array<Vector3> ^destinationArray, int destinationIndex, int length);
				static void Transform(array<Vector3> ^sourceArray, Quaternion ^%rotation, array<Vector3> ^destinationArray);
				[SuppressMessage("Microsoft.Usage", "CA2233")]
				static void Transform(array<Vector3> ^sourceArray, int sourceIndex, Quaternion ^%rotation, array<Vector3> ^destinationArray, int destinationIndex, int length);

				static Vector3^ Negate(Vector3 ^value);
				static void Negate(Vector3 ^%value, [Out] Vector3 ^%result);
				static Vector3^ Add(Vector3 ^value1, Vector3 ^value2);
				static void Add(Vector3 ^%value1, Vector3 ^%value2, [Out] Vector3 ^%result);
				static Vector3^ Subtract(Vector3 ^value1, Vector3 ^value2);
				static void Subtract(Vector3 ^%value1, Vector3 ^%value2, [Out] Vector3 ^%result);
				static Vector3^ Multiply(Vector3 ^value1, Vector3 ^value2);
				static void Multiply(Vector3 ^%value1, Vector3 ^%value2, [Out] Vector3 ^%result);
				static Vector3^ Multiply(Vector3 ^value1, float scaleFactor);
				static void Multiply(Vector3 ^%value1, float scaleFactor, [Out] Vector3 ^%result);
				static Vector3^ Divide(Vector3 ^value1, Vector3 ^value2);
				static void Divide(Vector3 ^%value1, Vector3 ^%value2, [Out] Vector3 ^%result);
				static Vector3^ Divide(Vector3 ^value1, float divider);
				static void Divide(Vector3 ^%value1, float divider, [Out] Vector3 ^%result);
				static Vector3^ operator -(Vector3 ^value);
				static bool operator ==(Vector3 ^value1, Vector3 ^value2);
				static bool operator !=(Vector3 ^value1, Vector3 ^value2);
				static Vector3^ operator +(Vector3 ^value1, Vector3 ^value2);
				static Vector3^ operator -(Vector3 ^value1, Vector3 ^value2);
				static Vector3^ operator *(Vector3 ^value1, Vector3 ^value2);
				static Vector3^ operator *(Vector3 ^value, float scaleFactor);
				static Vector3^ operator *(float scaleFactor, Vector3 ^value);
				static Vector3^ operator /(Vector3 ^value1, Vector3 ^value2);
				static Vector3^ operator /(Vector3 ^value, float divider);
			};
		}
	}
}
