#pragma once

using namespace System;
using namespace System::Diagnostics::CodeAnalysis;
using namespace System::Runtime::InteropServices;

#include "Vector2.h"
#include "Vector3.h"
#include "Quaternion.h"
#include "Matrix.h"

namespace Microsoft {
	namespace Xna {
		namespace Framework {
			value class Vector2;
			value class Vector3;
			value class Quaternion;

			public value class Vector4
			{
			private:
				static Vector4 ^_zero;
				static Vector4 ^_one;
				static Vector4 ^_unitX;
				static Vector4 ^_unitY;
				static Vector4 ^_unitZ;
				static Vector4 ^_unitW;
				static Vector4();
			public:
				[SuppressMessageAttribute("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float X;
				
				[SuppressMessageAttribute("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float Y;
				
				[SuppressMessageAttribute("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float Z;
				
				[SuppressMessageAttribute("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
				float W;
				
				static property Vector4^ Zero
				{
					Vector4^ get(){ return _zero; }
				}
				
				static property Vector4^ One
				{
					Vector4^ get(){ return _one; }
				}
				
				static property Vector4^ UnitX
				{
					Vector4^ get(){ return _unitX; }
				}
				
				static property Vector4^ UnitY
				{
					Vector4^ get(){ return _unitY; }
				}
				
				static property Vector4^ UnitZ
				{
					Vector4^ get(){ return _unitZ; }
				}

				static property Vector4^ UnitW
				{
					Vector4^ get(){ return _unitW; }
				}

				Vector4(float x, float y, float z, float w);
				Vector4(Vector2 ^value, float z, float w);
				Vector4(Vector3 ^value, float w);
				Vector4(float value);

				virtual String^ ToString() override;
				bool Equals(Vector4 ^other);
				virtual bool Equals(Object ^obj) override;
				virtual int GetHashCode() override;

				float Length();
				float LengthSquared();
				static float Distance(Vector4 ^value1, Vector4 ^value2);
				static void Distance(Vector4 ^%value1, Vector4 ^%value2, [Out] float ^%result);
				static float DistanceSquared(Vector4 ^value1, Vector4 ^value2);
				static void DistanceSquared(Vector4 ^%value1, Vector4 ^%value2, [Out] float ^%result);
				static float Dot(Vector4 ^vector1, Vector4 ^vector2);
				static void Dot(Vector4 ^%vector1, Vector4 ^%vector2, [Out] float ^%result);
				void Normalize();
				static Vector4^ Normalize(Vector4 ^vector);
				static void Normalize(Vector4 ^%vector, [Out] Vector4 ^%result);
				static Vector4^ Min(Vector4 ^value1, Vector4 ^value2);
				static void Min(Vector4 ^%value1, Vector4 ^%value2, [Out] Vector4 ^%result);
				static Vector4^ Max(Vector4 ^value1, Vector4 ^value2);
				static void Max(Vector4 ^%value1, Vector4 ^%value2, [Out] Vector4 ^%result);
				static Vector4^ Clamp(Vector4 ^value1, Vector4 ^min, Vector4 ^max);
				static void Clamp(Vector4 ^%value1, Vector4 ^%min, Vector4 ^%max, [Out] Vector4 ^%result);
				static Vector4^ Lerp(Vector4 ^value1, Vector4 ^value2, float amount);
				static void Lerp(Vector4 ^%value1, Vector4 ^%value2, float amount, [Out] Vector4 ^%result);
				static Vector4^ Barycentric(Vector4 ^value1, Vector4 ^value2, Vector4 ^value3, float amount1, float amount2);
				static void Barycentric(Vector4 ^%value1, Vector4 ^%value2, Vector4 ^%value3, float amount1, float amount2, [Out] Vector4 ^%result);
				static Vector4^ SmoothStep(Vector4 ^value1, Vector4 ^value2, float amount);
				static void SmoothStep(Vector4 ^%value1, Vector4 ^%value2, float amount, [Out] Vector4 ^%result);
				static Vector4^ CatmullRom(Vector4 ^value1, Vector4 ^value2, Vector4 ^value3, Vector4 ^value4, float amount);
				static void CatmullRom(Vector4 ^%value1, Vector4 ^%value2, Vector4 ^%value3, Vector4 ^%value4, float amount, [Out] Vector4 ^%result);
				static Vector4^ Hermite(Vector4 ^value1, Vector4 ^tangent1, Vector4 ^value2, Vector4 ^tangent2, float amount);
				static void Hermite(Vector4 ^%value1, Vector4 ^%tangent1, Vector4 ^%value2, Vector4 ^%tangent2, float amount, [Out] Vector4 ^%result);
				static Vector4^ Transform(Vector2 position, Matrix matrix);
				static void Transform(Vector2 ^%position, Matrix matrix, [Out] Vector4 ^%result);
				static Vector4^ Transform(Vector3 position, Matrix matrix);
				static void Transform(Vector3  ^%position, Matrix  ^%matrix, [Out] Vector4 ^%result);
				static Vector4^ Transform(Vector4 ^vector, Matrix matrix);
				static void Transform(Vector4 ^%vector, Matrix  ^%matrix, [Out] Vector4 ^%result);
				static Vector4^ Transform(Vector2 value, Quaternion rotation);
				static void Transform(Vector2  ^%value, Quaternion  ^%rotation, [Out] Vector4 ^%result);
				static Vector4^ Transform(Vector3 value, Quaternion rotation);
				static void Transform(Vector3  ^%value, Quaternion  ^%rotation, [Out] Vector4 ^%result);
				static Vector4^ Transform(Vector4 ^value, Quaternion rotation);
				static void Transform(Vector4 ^%value, Quaternion  ^%rotation, [Out] Vector4 ^%result);
				static void Transform(array<Vector4> ^sourceArray, Matrix ^%matrix, array<Vector4> ^destinationArray);
				[SuppressMessage("Microsoft.Usage", "CA2233")]
				static void Transform(array<Vector4> ^sourceArray, int sourceIndex, Matrix ^%matrix, array<Vector4> ^destinationArray, int destinationIndex, int length);
				static void Transform(array<Vector4> ^sourceArray, Quaternion ^%rotation, array<Vector4> ^destinationArray);
				[SuppressMessage("Microsoft.Usage", "CA2233")]
				static void Transform(array<Vector4> ^sourceArray, int sourceIndex, Quaternion ^%rotation, array<Vector4> ^destinationArray, int destinationIndex, int length);

				static Vector4^ Negate(Vector4 ^value);
				static void Negate(Vector4 ^%value, [Out] Vector4 ^%result);
				static Vector4^ Add(Vector4 ^value1, Vector4 ^value2);
				static void Add(Vector4 ^%value1, Vector4 ^%value2, [Out] Vector4 ^%result);
				static Vector4^ Subtract(Vector4 ^value1, Vector4 ^value2);
				static void Subtract(Vector4 ^%value1, Vector4 ^%value2, [Out] Vector4 ^%result);
				static Vector4^ Multiply(Vector4 ^value1, Vector4 ^value2);
				static void Multiply(Vector4 ^%value1, Vector4 ^%value2, [Out] Vector4 ^%result);
				static Vector4^ Multiply(Vector4 ^value1, float scaleFactor);
				static void Multiply(Vector4 ^%value1, float scaleFactor, [Out] Vector4 ^%result);
				static Vector4^ Divide(Vector4 ^value1, Vector4 ^value2);
				static void Divide(Vector4 ^%value1, Vector4 ^%value2, [Out] Vector4 ^%result);
				static Vector4^ Divide(Vector4 ^value1, float divider);
				static void Divide(Vector4 ^%value1, float divider, [Out] Vector4 ^%result);
				static Vector4^ operator -(Vector4 ^value);
				static bool operator ==(Vector4 ^value1, Vector4 ^value2);
				static bool operator !=(Vector4 ^value1, Vector4 ^value2);
				static Vector4^ operator +(Vector4 ^value1, Vector4 ^value2);
				static Vector4^ operator -(Vector4 ^value1, Vector4 ^value2);
				static Vector4^ operator *(Vector4 ^value1, Vector4 ^value2);
				static Vector4^ operator *(Vector4 ^value1, float scaleFactor);
				static Vector4^ operator *(float scaleFactor, Vector4 ^value1);
				static Vector4^ operator /(Vector4 ^value1, Vector4 ^value2);
				static Vector4^ operator /(Vector4 ^value1, float divider);
			};
		}
	}
}
